@@ -29,137 +29,138 @@
 
 #include "base/config_file_stream.h"
 
-#ifdef OS_WIN
-#include <windows.h>
-#endif  // OS_WIN
-
 #include <cstring>
-#include <map>
+#include <ios>
+#include <istream>
+#include <iterator>
+#include <memory>
 #include <sstream>
 #include <string>
+#include <utility>
 
+#include "base/strings/zstring_view.h"
+
+#ifdef _WIN32
+#include <windows.h>
+#endif  // _WIN32
+
+#include "absl/container/flat_hash_map.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
+#include "absl/strings/strip.h"
 #include "base/file_stream.h"
 #include "base/file_util.h"
-#include "base/logging.h"
-#include "base/port.h"
 #include "base/singleton.h"
 #include "base/system_util.h"
-#include "absl/strings/match.h"
+
+#ifdef _WIN32
+#include "base/win32/win_sandbox.h"
+#endif  // _WIN32
 
 namespace mozc {
-
 namespace {
 
-static constexpr char kSystemPrefix[] = "system://";
-static constexpr char kUserPrefix[] = "user://";
-static constexpr char kFilePrefix[] = "file://";
-static constexpr char kMemoryPrefix[] = "memory://";
+constexpr absl::string_view kSystemPrefix = "system://";
+constexpr absl::string_view kUserPrefix = "user://";
+constexpr absl::string_view kFilePrefix = "file://";
+constexpr absl::string_view kMemoryPrefix = "memory://";
 
 struct FileData {
-  const char *name;
-  const char *data;
-  size_t size;
+  absl::string_view name;
+  absl::string_view data;
 };
 
-std::string RemovePrefix(const char *prefix, const std::string &filename) {
-  const size_t size = strlen(prefix);
-  if (filename.size() < size) {
-    return "";
-  }
-  return filename.substr(size, filename.size() - size);
-}
+#include "base/config_file_stream_data.inc"
 
 class OnMemoryFileMap {
  public:
-  std::string get(const std::string &key) const {
-    std::map<std::string, std::string>::const_iterator it = map_.find(key);
+  const std::string &get(const absl::string_view key) const {
+    auto it = map_.find(key);
     if (it != map_.end()) {
       return it->second;
     }
-    return std::string("");
+    return empty_string_;
   }
 
-  void set(const std::string &key, const std::string &value) {
-    map_[key] = value;
+  void set(absl::string_view key, zstring_view value) {
+    map_[key] = std::string(value.view());
   }
 
   void clear() { map_.clear(); }
 
  private:
-  std::map<std::string, std::string> map_;
+  absl::flat_hash_map<std::string, std::string> map_;
+  const std::string empty_string_;
 };
-
-#include "base/config_file_stream_data.inc"
 }  // namespace
 
-std::istream *ConfigFileStream::Open(const std::string &filename,
-                                     std::ios_base::openmode mode) {
+std::unique_ptr<std::istream> ConfigFileStream::Open(
+    zstring_view filename, std::ios_base::openmode mode) {
   // system://foo.bar.txt
-  if (absl::StartsWith(filename, kSystemPrefix)) {
-    const std::string new_filename = RemovePrefix(kSystemPrefix, filename);
+  if (filename.view().starts_with(kSystemPrefix)) {
+    absl::string_view new_filename = absl::StripPrefix(filename, kSystemPrefix);
     for (size_t i = 0; i < std::size(kFileData); ++i) {
       if (new_filename == kFileData[i].name) {
-        std::istringstream *ifs = new std::istringstream(
-            std::string(kFileData[i].data, kFileData[i].size), mode);
+        auto ifs = std::make_unique<std::istringstream>(
+            std::string(kFileData[i].data), mode);
         CHECK(ifs);
         if (ifs->good()) {
           return ifs;
         }
-        delete ifs;
         return nullptr;
       }
     }
     // user://foo.bar.txt
-  } else if (absl::StartsWith(filename, kUserPrefix)) {
+  } else if (filename.view().starts_with(kUserPrefix)) {
     const std::string new_filename =
         FileUtil::JoinPath(SystemUtil::GetUserProfileDirectory(),
-                           RemovePrefix(kUserPrefix, filename));
-    InputFileStream *ifs = new InputFileStream(new_filename.c_str(), mode);
+                           absl::StripPrefix(filename, kUserPrefix));
+    auto ifs = std::make_unique<InputFileStream>(new_filename, mode);
     CHECK(ifs);
     if (ifs->good()) {
       return ifs;
     }
-    delete ifs;
     return nullptr;
     // file:///foo.map
-  } else if (absl::StartsWith(filename, kFilePrefix)) {
-    const std::string new_filename = RemovePrefix(kFilePrefix, filename);
-    InputFileStream *ifs = new InputFileStream(new_filename.c_str(), mode);
+  } else if (filename.view().starts_with(kFilePrefix)) {
+    absl::string_view new_filename = absl::StripPrefix(filename, kFilePrefix);
+    auto ifs =
+        std::make_unique<InputFileStream>(std::string(new_filename), mode);
     CHECK(ifs);
     if (ifs->good()) {
       return ifs;
     }
-    delete ifs;
     return nullptr;
-  } else if (absl::StartsWith(filename, kMemoryPrefix)) {
-    std::istringstream *ifs = new std::istringstream(
+  } else if (filename.view().starts_with(kMemoryPrefix)) {
+    auto ifs = std::make_unique<std::istringstream>(
         Singleton<OnMemoryFileMap>::get()->get(filename), mode);
     CHECK(ifs);
     if (ifs->good()) {
       return ifs;
     }
-    delete ifs;
     return nullptr;
   } else {
     LOG(WARNING) << filename << " has no prefix. open from localfile";
-    InputFileStream *ifs = new InputFileStream(filename.c_str(), mode);
+    auto ifs = std::make_unique<InputFileStream>(filename, mode);
     CHECK(ifs);
     if (ifs->good()) {
       return ifs;
     }
-    delete ifs;
     return nullptr;
   }
 
   return nullptr;
 }
 
-bool ConfigFileStream::AtomicUpdate(const std::string &filename,
-                                    const std::string &new_binary_contens) {
-  if (absl::StartsWith(filename, kMemoryPrefix)) {
+bool ConfigFileStream::AtomicUpdate(zstring_view filename,
+                                    zstring_view new_binary_contens) {
+  if (filename.view().starts_with(kMemoryPrefix)) {
     Singleton<OnMemoryFileMap>::get()->set(filename, new_binary_contens);
     return true;
-  } else if (absl::StartsWith(filename, kSystemPrefix)) {
+  } else if (filename->starts_with(kSystemPrefix)) {
     LOG(ERROR) << "Cannot update system:// files.";
     return false;
   }
@@ -171,7 +172,7 @@ bool ConfigFileStream::AtomicUpdate(const std::string &filename,
     return false;
   }
 
-  const std::string tmp_filename = real_filename + ".tmp";
+  const std::string tmp_filename = absl::StrCat(real_filename, ".tmp");
   if (absl::Status s = FileUtil::SetContents(tmp_filename, new_binary_contens);
       !s.ok()) {
     LOG(ERROR) << "Cannot write the contents to " << tmp_filename << ": " << s;
@@ -185,10 +186,10 @@ bool ConfigFileStream::AtomicUpdate(const std::string &filename,
     return false;
   }
 
-#ifdef OS_WIN
+#ifdef _WIN32
   // If file name doesn't end with ".db", the file
   // is more likely a temporary file.
-  if (!absl::EndsWith(real_filename, ".db")) {
+  if (!real_filename.ends_with(".db")) {
     // TODO(yukawa): Provide a way to
     // integrate ::SetFileAttributesTransacted with
     // AtomicRename.
@@ -197,22 +198,22 @@ bool ConfigFileStream::AtomicUpdate(const std::string &filename,
                  << ::GetLastError();
     }
   }
-#endif  // OS_WIN
+#endif  // _WIN32
   return true;
 }
 
-std::string ConfigFileStream::GetFileName(const std::string &filename) {
-  if (absl::StartsWith(filename, kSystemPrefix) ||
-      absl::StartsWith(filename, kMemoryPrefix)) {
+std::string ConfigFileStream::GetFileName(absl::string_view filename) {
+  if (filename.starts_with(kSystemPrefix) ||
+      filename.starts_with(kMemoryPrefix)) {
     return "";
-  } else if (absl::StartsWith(filename, kUserPrefix)) {
+  } else if (filename.starts_with(kUserPrefix)) {
     return FileUtil::JoinPath(SystemUtil::GetUserProfileDirectory(),
-                              RemovePrefix(kUserPrefix, filename));
-  } else if (absl::StartsWith(filename, kFilePrefix)) {
-    return RemovePrefix(kUserPrefix, filename);
+                              absl::StripPrefix(filename, kUserPrefix));
+  } else if (filename.starts_with(kFilePrefix)) {
+    return std::string(absl::StripPrefix(filename, kUserPrefix));
   } else {
     LOG(WARNING) << filename << " has no prefix. open from localfile";
-    return filename;
+    return std::string(filename);
   }
   return "";
 }
@@ -220,4 +221,22 @@ std::string ConfigFileStream::GetFileName(const std::string &filename) {
 void ConfigFileStream::ClearOnMemoryFiles() {
   Singleton<OnMemoryFileMap>::get()->clear();
 }
+
+#ifdef _WIN32
+// Check the file permission of "config1.db" if exists to ensure that
+// "ALL APPLICATION PACKAGES" have read access to it.
+void ConfigFileStream::FixupFilePermission(absl::string_view filename) {
+  const std::string path = ConfigFileStream::GetFileName(filename);
+  if (path.empty()) {
+    return;
+  }
+  const absl::Status status = FileUtil::FileExists(path);
+  if (status.ok()) {
+    WinSandbox::EnsureAllApplicationPackagesPermisssion(
+        win32::Utf8ToWide(path),
+        WinSandbox::AppContainerVisibilityType::kConfigFile);
+  }
+}
+#endif  // _WIN32
+
 }  // namespace mozc
