@@ -29,54 +29,35 @@
 
 #include "base/file_stream.h"
 
-#ifdef OS_WIN
-#include <codecvt>
-#include <locale>
-#endif  // OS_WIN
-
+#include <fstream>
+#include <ios>
 #include <string>
 
+#include "base/strings/pfchar.h"
+#include "base/strings/zstring_view.h"
+
 namespace mozc {
-namespace {
 
-#ifdef OS_WIN
-std::wstring ToPlatformString(const char* filename) {
-  // Since Windows uses UTF-16 for internationalized file names, we should
-  // convert the encoding of the given |filename| from UTF-8 to UTF-16.
-  // NOTE: To avoid circular dependency, |Util::Utf8ToWide| shouldn't be used
-  // here.
-  std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> utf8_to_wide;
-  return utf8_to_wide.from_bytes(filename);
-}
-#else   // OS_WIN
-std::string ToPlatformString(const char* filename) {
-  return std::string(filename);
-}
-#endif  // OS_WIN or not
-
-}  // namespace
-
-InputFileStream::InputFileStream() {}
-
-InputFileStream::InputFileStream(const char* filename,
+InputFileStream::InputFileStream(zstring_view filename,
                                  std::ios_base::openmode mode) {
-  InputFileStream::open(filename, mode);
+  open(filename, mode);
 }
 
-void InputFileStream::open(const char* filename, std::ios_base::openmode mode) {
-  std::ifstream::open(ToPlatformString(filename), mode);
+void InputFileStream::open(zstring_view filename,
+                           std::ios_base::openmode mode) {
+  // to_pfstring() changes encoding to utf-16 on Windows.
+  std::ifstream::open(to_pfstring(filename), mode);
 }
 
-OutputFileStream::OutputFileStream() {}
-
-OutputFileStream::OutputFileStream(const char* filename,
+OutputFileStream::OutputFileStream(zstring_view filename,
                                    std::ios_base::openmode mode) {
-  OutputFileStream::open(filename, mode);
+  open(filename, mode);
 }
 
-void OutputFileStream::open(const char* filename,
+void OutputFileStream::open(zstring_view filename,
                             std::ios_base::openmode mode) {
-  std::ofstream::open(ToPlatformString(filename), mode);
+  // to_pfstring() changes encoding to utf-16 on Windows.
+  std::ofstream::open(to_pfstring(filename), mode);
 }
 
 // Common implementations.
