@@ -29,54 +29,61 @@
 
 #include "base/file_util.h"
 
-#include <iterator>
-
-#ifdef OS_WIN
-#include <KtmW32.h>
-#include <Windows.h>
-#else  // OS_WIN
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
-#endif  // OS_WIN
-
+#include <cerrno>
+#include <cstddef>
+#include <cstdio>
+#include <cstring>
 #include <filesystem>
+#include <ios>
+#include <iterator>
 #include <string>
+#include <system_error>
+#include <utility>
 
-#include "base/file_stream.h"
-#include "base/logging.h"
-#include "base/mmap.h"
-#include "base/scoped_handle.h"
-#include "base/singleton.h"
-#include "base/util.h"
-#include "base/win_util.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_replace.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
+#include "base/file_stream.h"
+#include "base/mmap.h"
+#include "base/port.h"
+#include "base/singleton.h"
+#include "base/strings/zstring_view.h"
+
+#ifdef _WIN32
+#include <wil/resource.h>
+#include <windows.h>
+
+#include "base/util.h"
+#include "base/win32/wide_char.h"
+
+#else  // _WIN32
+#include <sys/stat.h>
+#include <unistd.h>
+#endif  // _WIN32
 
 namespace {
 
-#ifdef OS_WIN
+#ifdef _WIN32
 constexpr char kFileDelimiter = '\\';
-#else   // OS_WIN
+#else   // _WIN32
 constexpr char kFileDelimiter = '/';
-#endif  // OS_WIN
+#endif  // _WIN32
 
 }  // namespace
 
 // Ad-hoc workadound against macro problem on Windows.
-// On Windows, following macros, defined when you include <Windows.h>,
+// On Windows, following macros, defined when you include <windows.h>,
 // should be removed here because they affects the method name definition of
 // Util class.
 // TODO(yukawa): Use different method name if applicable.
-#ifdef CreateDirectory
 #undef CreateDirectory
-#endif  // CreateDirectory
-#ifdef RemoveDirectory
 #undef RemoveDirectory
-#endif  // RemoveDirectory
-#ifdef CopyFile
 #undef CopyFile
-#endif  // CopyFile
 
 namespace mozc {
 namespace {
@@ -85,40 +92,59 @@ class FileUtilImpl : public FileUtilInterface {
   FileUtilImpl() = default;
   ~FileUtilImpl() override = default;
 
-  absl::Status CreateDirectory(const std::string &path) const override;
-  absl::Status RemoveDirectory(const std::string &dirname) const override;
-  absl::Status Unlink(const std::string &filename) const override;
-  absl::Status FileExists(const std::string &filename) const override;
-  absl::Status DirectoryExists(const std::string &dirname) const override;
-  absl::Status CopyFile(const std::string &from,
-                        const std::string &to) const override;
-  absl::StatusOr<bool> IsEqualFile(const std::string &filename1,
-                                   const std::string &filename2) const override;
-  absl::StatusOr<bool> IsEquivalent(
-      const std::string &filename1,
-      const std::string &filename2) const override;
-  absl::Status AtomicRename(const std::string &from,
-                            const std::string &to) const override;
-  absl::Status CreateHardLink(const std::string &from,
-                              const std::string &to) override;
+  absl::Status CreateDirectory(zstring_view path) const override;
+  absl::Status RemoveDirectory(zstring_view dirname) const override;
+  absl::Status Unlink(zstring_view filename) const override;
+  absl::Status FileExists(zstring_view filename) const override;
+  absl::Status DirectoryExists(zstring_view dirname) const override;
+  absl::Status CopyFile(zstring_view from, zstring_view to) const override;
+  absl::StatusOr<bool> IsEqualFile(zstring_view filename1,
+                                   zstring_view filename2) const override;
+  absl::StatusOr<bool> IsEquivalent(zstring_view filename1,
+                                    zstring_view filename2) const override;
+  absl::Status AtomicRename(zstring_view from, zstring_view to) const override;
+  absl::Status CreateHardLink(zstring_view from, zstring_view to) override;
   absl::StatusOr<FileTimeStamp> GetModificationTime(
-      const std::string &filename) const override;
+      zstring_view filename) const override;
 };
 
 using FileUtilSingleton = SingletonMockable<FileUtilInterface, FileUtilImpl>;
 
 }  // namespace
 
-#ifdef OS_WIN
+#ifdef _WIN32
 namespace {
 
+// Converts the Win32 error code to absl::StatusCode.
+absl::StatusCode Win32ErrorToStatusCode(DWORD error_code) {
+  switch (error_code) {
+    case ERROR_SUCCESS:
+      return absl::StatusCode::kOk;
+    case ERROR_FILE_NOT_FOUND:
+    case ERROR_PATH_NOT_FOUND:
+      return absl::StatusCode::kNotFound;
+    case ERROR_ACCESS_DENIED:
+      return absl::StatusCode::kPermissionDenied;
+    case ERROR_ALREADY_EXISTS:
+      return absl::StatusCode::kAlreadyExists;
+    default:
+      return absl::StatusCode::kUnknown;
+  }
+}
+
+// Converts the Win32 error code to absl::Status.
+absl::Status Win32ErrorToStatus(DWORD error_code, absl::string_view message) {
+  return absl::Status(Win32ErrorToStatusCode(error_code),
+                      absl::StrCat(message, ": error_code=", error_code));
+}
+
 absl::StatusOr<DWORD> GetFileAttributes(const std::wstring &filename) {
   if (const DWORD attrs = ::GetFileAttributesW(filename.c_str());
       attrs != INVALID_FILE_ATTRIBUTES) {
     return attrs;
   }
   const DWORD error = ::GetLastError();
-  return WinUtil::ErrorToCanonicalStatus(error, "GetFileAttributesW failed");
+  return Win32ErrorToStatus(error, "GetFileAttributesW failed");
 }
 
 absl::Status SetFileAttributes(const std::wstring &filename, DWORD attrs) {
@@ -126,7 +152,7 @@ absl::Status SetFileAttributes(const std::wstring &filename, DWORD attrs) {
     return absl::OkStatus();
   }
   const DWORD error = ::GetLastError();
-  return WinUtil::ErrorToCanonicalStatus(
+  return Win32ErrorToStatus(
       error, absl::StrCat("SetFileAttributesW failed: attrs = ", attrs));
 }
 
@@ -134,15 +160,13 @@ absl::Status SetFileAttributes(const std::wstring &filename, DWORD attrs) {
 // has some special attribute like read-only. This method tries to strip system,
 // hidden, and read-only attributes from |filename|.
 // This function does nothing if |filename| does not exist.
-absl::Status StripWritePreventingAttributesIfExists(
-    const std::string &filename) {
+absl::Status StripWritePreventingAttributesIfExists(zstring_view filename) {
   if (absl::Status s = FileUtil::FileExists(filename); absl::IsNotFound(s)) {
     return absl::OkStatus();
   } else if (!s.ok()) {
     return s;
   }
-  std::wstring wide_filename;
-  Util::Utf8ToWide(filename, &wide_filename);
+  const std::wstring wide_filename = win32::Utf8ToWide(filename);
   constexpr DWORD kDropAttributes =
       FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_READONLY;
   absl::StatusOr<DWORD> attributes = GetFileAttributes(wide_filename);
@@ -156,64 +180,74 @@ absl::Status StripWritePreventingAttributesIfExists(
           s.code(),
           absl::StrFormat(
               "Cannot drop the write-preventing file attributes of %s: %s",
-              filename, s.message()));
+              filename.view(), s.message()));
     }
   }
   return absl::OkStatus();
 }
 
 }  // namespace
-#endif  // OS_WIN
+#endif  // _WIN32
 
-absl::Status FileUtil::CreateDirectory(const std::string &path) {
+absl::Status FileUtil::CreateDirectory(zstring_view path) {
   return FileUtilSingleton::Get()->CreateDirectory(path);
 }
 
-absl::Status FileUtilImpl::CreateDirectory(const std::string &path) const {
-#if defined(OS_WIN)
-  std::wstring wide;
-  if (Util::Utf8ToWide(path, &wide) <= 0) {
+absl::Status FileUtilImpl::CreateDirectory(zstring_view path) const {
+#if !defined(_WIN32)
+  // On Windows, this check is skipped to avoid freeze of the host application.
+  // This platform dependent behavior is a temporary solution to avoid
+  // freeze of the host application.
+  // https://github.com/google/mozc/issues/1076
+  //
+  // If the path already exists, returns OkStatus and does nothing.
+  if (const absl::Status status = DirectoryExists(path); status.ok()) {
+    return absl::OkStatus();
+  }
+#endif  // !_WIN32
+
+#if defined(_WIN32)
+  const std::wstring wide = win32::Utf8ToWide(path);
+  if (wide.empty()) {
     return absl::InvalidArgumentError("Failed to convert to wstring");
   }
   if (!::CreateDirectoryW(wide.c_str(), nullptr)) {
-    return WinUtil::ErrorToCanonicalStatus(::GetLastError(),
-                                           "CreateDirectoryW failed");
+    return Win32ErrorToStatus(::GetLastError(), "CreateDirectoryW failed");
   }
   return absl::OkStatus();
-#else   // !OS_WIN
+#else   // !_WIN32
   if (::mkdir(path.c_str(), 0700) != 0) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(err, "mkdir failed");
+    return absl::ErrnoToStatus(err, "mkdir failed");
   }
   return absl::OkStatus();
-#endif  // OS_WIN
+#endif  // _WIN32
 }
 
-absl::Status FileUtil::RemoveDirectory(const std::string &dirname) {
+absl::Status FileUtil::RemoveDirectory(zstring_view dirname) {
   return FileUtilSingleton::Get()->RemoveDirectory(dirname);
 }
 
-absl::Status FileUtilImpl::RemoveDirectory(const std::string &dirname) const {
-#ifdef OS_WIN
-  std::wstring wide;
-  if (Util::Utf8ToWide(dirname, &wide) <= 0) {
+absl::Status FileUtilImpl::RemoveDirectory(zstring_view dirname) const {
+#ifdef _WIN32
+  const std::wstring wide = win32::Utf8ToWide(dirname);
+  if (wide.empty()) {
     return absl::InvalidArgumentError("Failed to convert to wstring");
   }
   if (!::RemoveDirectoryW(wide.c_str())) {
-    return WinUtil::ErrorToCanonicalStatus(::GetLastError(),
-                                           "RemoveDirectoryW failed");
+    return Win32ErrorToStatus(::GetLastError(), "RemoveDirectoryW failed");
   }
   return absl::OkStatus();
-#else   // !OS_WIN
+#else   // !_WIN32
   if (::rmdir(dirname.c_str()) != 0) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(err, "rmdir failed");
+    return absl::ErrnoToStatus(err, "rmdir failed");
   }
   return absl::OkStatus();
-#endif  // OS_WIN
+#endif  // _WIN32
 }
 
-absl::Status FileUtil::RemoveDirectoryIfExists(const std::string &dirname) {
+absl::Status FileUtil::RemoveDirectoryIfExists(zstring_view dirname) {
   absl::Status s = FileExists(dirname);
   if (s.ok()) {
     return RemoveDirectory(dirname);
@@ -224,19 +258,19 @@ absl::Status FileUtil::RemoveDirectoryIfExists(const std::string &dirname) {
   return s;
 }
 
-absl::Status FileUtil::Unlink(const std::string &filename) {
+absl::Status FileUtil::Unlink(zstring_view filename) {
   return FileUtilSingleton::Get()->Unlink(filename);
 }
 
-absl::Status FileUtilImpl::Unlink(const std::string &filename) const {
-#ifdef OS_WIN
+absl::Status FileUtilImpl::Unlink(zstring_view filename) const {
+#ifdef _WIN32
   if (absl::Status s = StripWritePreventingAttributesIfExists(filename);
       !s.ok()) {
     return absl::UnknownError(absl::StrFormat(
         "StripWritePreventingAttributesIfExists failed: %s", s.ToString()));
   }
-  std::wstring wide;
-  if (Util::Utf8ToWide(filename, &wide) <= 0) {
+  const std::wstring wide = win32::Utf8ToWide(filename);
+  if (wide.empty()) {
     return absl::InvalidArgumentError("Utf8ToWide failed");
   }
   if (!::DeleteFileW(wide.c_str())) {
@@ -244,17 +278,17 @@ absl::Status FileUtilImpl::Unlink(const std::string &filename) const {
     return absl::UnknownError(absl::StrFormat("DeleteFileW failed: %d", err));
   }
   return absl::OkStatus();
-#else   // !OS_WIN
+#else   // !_WIN32
   if (::unlink(filename.c_str()) != 0) {
     const int err = errno;
     return absl::UnknownError(
         absl::StrFormat("unlink failed: errno = %d", err));
   }
   return absl::OkStatus();
-#endif  // OS_WIN
+#endif  // _WIN32
 }
 
-absl::Status FileUtil::UnlinkIfExists(const std::string &filename) {
+absl::Status FileUtil::UnlinkIfExists(zstring_view filename) {
   absl::Status s = FileExists(filename);
   if (s.ok()) {
     return Unlink(filename);
@@ -265,42 +299,41 @@ absl::Status FileUtil::UnlinkIfExists(const std::string &filename) {
   return s;
 }
 
-void FileUtil::UnlinkOrLogError(const std::string &filename) {
+void FileUtil::UnlinkOrLogError(zstring_view filename) {
   if (absl::Status s = Unlink(filename); !s.ok()) {
     LOG(ERROR) << "Cannot unlink " << filename << ": " << s;
   }
 }
 
-absl::Status FileUtil::FileExists(const std::string &filename) {
+absl::Status FileUtil::FileExists(zstring_view filename) {
   return FileUtilSingleton::Get()->FileExists(filename);
 }
 
-absl::Status FileUtilImpl::FileExists(const std::string &filename) const {
-#ifdef OS_WIN
-  std::wstring wide;
-  if (Util::Utf8ToWide(filename, &wide) <= 0) {
+absl::Status FileUtilImpl::FileExists(zstring_view filename) const {
+#ifdef _WIN32
+  const std::wstring wide = win32::Utf8ToWide(filename);
+  if (wide.empty()) {
     return absl::InvalidArgumentError("Utf8ToWide failed");
   }
   return GetFileAttributes(wide).status();
-#else   // !OS_WIN
+#else   // !_WIN32
   struct stat s;
   if (::stat(filename.c_str(), &s) == 0) {
     return absl::OkStatus();
   }
   const int err = errno;
-  return Util::ErrnoToCanonicalStatus(err,
-                                      absl::StrCat("Cannot stat ", filename));
-#endif  // OS_WIN
+  return absl::ErrnoToStatus(err, absl::StrCat("Cannot stat ", filename));
+#endif  // _WIN32
 }
 
-absl::Status FileUtil::DirectoryExists(const std::string &dirname) {
+absl::Status FileUtil::DirectoryExists(zstring_view dirname) {
   return FileUtilSingleton::Get()->DirectoryExists(dirname);
 }
 
-absl::Status FileUtilImpl::DirectoryExists(const std::string &dirname) const {
-#ifdef OS_WIN
-  std::wstring wide;
-  if (Util::Utf8ToWide(dirname, &wide) <= 0) {
+absl::Status FileUtilImpl::DirectoryExists(zstring_view dirname) const {
+#ifdef _WIN32
+  const std::wstring wide = win32::Utf8ToWide(dirname);
+  if (wide.empty()) {
     return absl::InvalidArgumentError("Utf8ToWide failed");
   }
 
@@ -310,7 +343,7 @@ absl::Status FileUtilImpl::DirectoryExists(const std::string &dirname) const {
   }
   return (*attrs & FILE_ATTRIBUTE_DIRECTORY) ? absl::OkStatus()
                                              : absl::NotFoundError(dirname);
-#else   // !OS_WIN
+#else   // !_WIN32
   struct stat s;
   if (::stat(dirname.c_str(), &s) == 0) {
     return S_ISDIR(s.st_mode)
@@ -318,73 +351,23 @@ absl::Status FileUtilImpl::DirectoryExists(const std::string &dirname) const {
                : absl::NotFoundError("Path exists but it's not a directory");
   }
   const int err = errno;
-  return Util::ErrnoToCanonicalStatus(err,
-                                      absl::StrCat("Cannot stat ", dirname));
-#endif  // OS_WIN
+  return absl::ErrnoToStatus(err, absl::StrCat("Cannot stat ", dirname));
+#endif  // _WIN32
 }
 
-#ifdef OS_WIN
-namespace {
-
-absl::Status TransactionalMoveFile(const std::wstring &from,
-                                   const std::wstring &to) {
-  constexpr DWORD kTimeout = 5000;  // 5 sec.
-  ScopedHandle handle(
-      ::CreateTransaction(nullptr, 0, 0, 0, 0, kTimeout, nullptr));
-  if (handle.get() == 0) {
-    const DWORD create_transaction_error = ::GetLastError();
-    return absl::UnknownError(absl::StrFormat("CreateTransaction failed: %d",
-                                              create_transaction_error));
-  }
-
-  WIN32_FILE_ATTRIBUTE_DATA file_attribute_data = {};
-  if (!::GetFileAttributesTransactedW(from.c_str(), GetFileExInfoStandard,
-                                      &file_attribute_data, handle.get())) {
-    const DWORD get_file_attributes_error = ::GetLastError();
-    return absl::UnknownError(absl::StrFormat(
-        "GetFileAttributesTransactedW failed: %d", get_file_attributes_error));
-  }
-
-  if (!::MoveFileTransactedW(from.c_str(), to.c_str(), nullptr, nullptr,
-                             MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING,
-                             handle.get())) {
-    const DWORD move_file_transacted_error = ::GetLastError();
-    return absl::UnknownError(absl::StrFormat("MoveFileTransactedW failed: %d",
-                                              move_file_transacted_error));
-  }
-
-  if (!::SetFileAttributesTransactedW(
-          to.c_str(), file_attribute_data.dwFileAttributes, handle.get())) {
-    const DWORD set_file_attributes_error = ::GetLastError();
-    return absl::UnknownError(absl::StrFormat(
-        "SetFileAttributesTransactedW failed: %d", set_file_attributes_error));
-  }
-
-  if (!::CommitTransaction(handle.get())) {
-    const DWORD commit_transaction_error = ::GetLastError();
-    return absl::UnknownError(absl::StrFormat("CommitTransaction failed: %d",
-                                              commit_transaction_error));
-  }
-
-  return absl::OkStatus();
-}
-
-}  // namespace
-
-bool FileUtil::HideFile(const std::string &filename) {
+#ifdef _WIN32
+bool FileUtil::HideFile(zstring_view filename) {
   return HideFileWithExtraAttributes(filename, 0);
 }
 
-bool FileUtil::HideFileWithExtraAttributes(const std::string &filename,
+bool FileUtil::HideFileWithExtraAttributes(zstring_view filename,
                                            DWORD extra_attributes) {
   if (absl::Status s = FileUtil::FileExists(filename); !s.ok()) {
     LOG(WARNING) << "File not exists: " << filename << ": " << s;
     return false;
   }
 
-  std::wstring wfilename;
-  Util::Utf8ToWide(filename, &wfilename);
-
+  const std::wstring wfilename = win32::Utf8ToWide(filename);
   const absl::StatusOr<DWORD> original_attributes =
       GetFileAttributes(wfilename);
   if (!original_attributes.ok()) {
@@ -401,36 +384,35 @@ bool FileUtil::HideFileWithExtraAttributes(const std::string &filename,
   }
   return s.ok();
 }
-#endif  // OS_WIN
+#endif  // _WIN32
 
-absl::Status FileUtil::CopyFile(const std::string &from,
-                                const std::string &to) {
+absl::Status FileUtil::CopyFile(zstring_view from, zstring_view to) {
   return FileUtilSingleton::Get()->CopyFile(from, to);
 }
 
-absl::Status FileUtilImpl::CopyFile(const std::string &from,
-                                    const std::string &to) const {
-#ifdef OS_WIN
-  std::wstring wfrom, wto;
-  if (Util::Utf8ToWide(from, &wfrom) <= 0) {
+absl::Status FileUtilImpl::CopyFile(zstring_view from, zstring_view to) const {
+#ifdef _WIN32
+  const std::wstring wfrom = win32::Utf8ToWide(from);
+  if (wfrom.empty()) {
     return absl::InvalidArgumentError(
         absl::StrCat("Cannot convert to wstring: ", from));
   }
-  if (Util::Utf8ToWide(to, &wto) <= 0) {
+  const std::wstring wto = win32::Utf8ToWide(to);
+  if (wto.empty()) {
     return absl::InvalidArgumentError(
         absl::StrCat("Cannot convert to wstring: ", to));
   }
   if (absl::Status s = StripWritePreventingAttributesIfExists(to); !s.ok()) {
     return s;
   }
-#endif  // OS_WIN
+#endif  // _WIN32
 
-  InputFileStream ifs(from.c_str(), std::ios::binary);
+  InputFileStream ifs(from, std::ios::binary);
   if (!ifs) {
     return absl::UnknownError(absl::StrCat("Can't open input file ", from));
   }
 
-  OutputFileStream ofs(to.c_str(), std::ios::binary | std::ios::trunc);
+  OutputFileStream ofs(to, std::ios::binary | std::ios::trunc);
   if (!ofs) {
     return absl::UnknownError(absl::StrCat("Can't open output file ", to));
   }
@@ -442,7 +424,7 @@ absl::Status FileUtilImpl::CopyFile(const std::string &from,
   ifs.close();
   ofs.close();
 
-#ifdef OS_WIN
+#ifdef _WIN32
   absl::StatusOr<DWORD> attrs = GetFileAttributes(wfrom);
   if (attrs.ok()) {
     if (absl::Status s = SetFileAttributes(wto, *attrs); !s.ok()) {
@@ -451,36 +433,36 @@ absl::Status FileUtilImpl::CopyFile(const std::string &from,
   } else {
     LOG(ERROR) << attrs.status();
   }
-#endif  // OS_WIN
+#endif  // _WIN32
 
   return absl::OkStatus();
 }
 
-absl::StatusOr<bool> FileUtil::IsEqualFile(const std::string &filename1,
-                                           const std::string &filename2) {
+absl::StatusOr<bool> FileUtil::IsEqualFile(zstring_view filename1,
+                                           zstring_view filename2) {
   return FileUtilSingleton::Get()->IsEqualFile(filename1, filename2);
 }
 
-absl::StatusOr<bool> FileUtilImpl::IsEqualFile(
-    const std::string &filename1, const std::string &filename2) const {
-  Mmap mmap1, mmap2;
-  if (!mmap1.Open(filename1.c_str(), "r")) {
-    return absl::UnknownError(absl::StrCat("Cannot open by mmap: ", filename1));
+absl::StatusOr<bool> FileUtilImpl::IsEqualFile(zstring_view filename1,
+                                               zstring_view filename2) const {
+  absl::StatusOr<Mmap> mmap1 = Mmap::Map(filename1, Mmap::READ_ONLY);
+  if (!mmap1.ok()) {
+    return std::move(mmap1).status();
   }
-  if (!mmap2.Open(filename2.c_str(), "r")) {
-    return absl::UnknownError(absl::StrCat("Cannot open by mmap: ", filename2));
+  absl::StatusOr<Mmap> mmap2 = Mmap::Map(filename2, Mmap::READ_ONLY);
+  if (!mmap2.ok()) {
+    return std::move(mmap2).status();
   }
-  return mmap1.size() == mmap2.size() &&
-         memcmp(mmap1.begin(), mmap2.begin(), mmap1.size()) == 0;
+  return mmap1->span() == mmap2->span();
 }
 
-absl::StatusOr<bool> FileUtil::IsEquivalent(const std::string &filename1,
-                                            const std::string &filename2) {
+absl::StatusOr<bool> FileUtil::IsEquivalent(zstring_view filename1,
+                                            zstring_view filename2) {
   return FileUtilSingleton::Get()->IsEquivalent(filename1, filename2);
 }
 
-absl::StatusOr<bool> FileUtilImpl::IsEquivalent(
-    const std::string &filename1, const std::string &filename2) const {
+absl::StatusOr<bool> FileUtilImpl::IsEquivalent(zstring_view filename1,
+                                                zstring_view filename2) const {
   // If either of filename1 or filename2 does not exist, an error is returned.
   // Because filesystem::equivalent on some environments returns false instead,
   // that case is checked here to keep the consistency.
@@ -494,8 +476,8 @@ absl::StatusOr<bool> FileUtilImpl::IsEquivalent(
 #else   // __APPLE__
 
   // u8path is deprecated in C++20. The current target is C++17.
-  const std::filesystem::path src = std::filesystem::u8path(filename1);
-  const std::filesystem::path dst = std::filesystem::u8path(filename2);
+  const std::filesystem::path src = std::filesystem::u8path(filename1.c_str());
+  const std::filesystem::path dst = std::filesystem::u8path(filename2.c_str());
 
   std::error_code error_code;
   if (bool is_equiv = std::filesystem::equivalent(src, dst, error_code);
@@ -507,58 +489,42 @@ absl::StatusOr<bool> FileUtilImpl::IsEquivalent(
 #endif  // __APPLE__
 }
 
-absl::Status FileUtil::AtomicRename(const std::string &from,
-                                    const std::string &to) {
+absl::Status FileUtil::AtomicRename(zstring_view from, zstring_view to) {
   return FileUtilSingleton::Get()->AtomicRename(from, to);
 }
 
-absl::Status FileUtilImpl::AtomicRename(const std::string &from,
-                                        const std::string &to) const {
-#ifdef OS_WIN
-  std::wstring fromw, tow;
-  Util::Utf8ToWide(from, &fromw);
-  Util::Utf8ToWide(to, &tow);
-
-  absl::Status move_status = TransactionalMoveFile(fromw, tow);
-  if (move_status.ok()) {
-    return absl::OkStatus();
-  }
-  LOG(WARNING) << "TransactionalMoveFile failed: from: " << from
-               << ", to: " << to << ", status: " << move_status;
+absl::Status FileUtilImpl::AtomicRename(zstring_view from,
+                                        zstring_view to) const {
+#ifdef _WIN32
+  const std::wstring fromw = win32::Utf8ToWide(from);
+  const std::wstring tow = win32::Utf8ToWide(to);
 
   const absl::StatusOr<DWORD> original_attributes = GetFileAttributes(fromw);
   if (!original_attributes.ok()) {
     return absl::Status(
         original_attributes.status().code(),
-        absl::StrFormat(
-            "GetFileAttributes failed: %s; Status of TransactionalMoveFile: %s",
-            original_attributes.status().message(), move_status.ToString()));
+        absl::StrFormat("GetFileAttributes failed: %s",
+                        original_attributes.status().message()));
   }
   if (absl::Status s = StripWritePreventingAttributesIfExists(to); !s.ok()) {
     return absl::Status(
         s.code(),
-        absl::StrFormat("StripWritePreventingAttributesIfExists failed: %s; "
-                        "Status of TransactionalMoveFile: %s",
-                        s.message(), move_status.ToString()));
+        absl::StrFormat("StripWritePreventingAttributesIfExists failed: %s",
+                        s.message()));
   }
   if (!::MoveFileExW(fromw.c_str(), tow.c_str(),
                      MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING)) {
     const DWORD move_file_ex_error = ::GetLastError();
-    return WinUtil::ErrorToCanonicalStatus(
-        move_file_ex_error,
-        absl::StrFormat(
-            "MoveFileExW failed; Status of TransactionalMoveFile: %s",
-            move_status.ToString()));
+    return Win32ErrorToStatus(move_file_ex_error, "MoveFileExW failed");
   }
   if (absl::Status s = SetFileAttributes(tow, *original_attributes); !s.ok()) {
     return absl::Status(
         s.code(),
-        absl::StrFormat("SetFileAttributes failed: original_attrs: %d; Status "
-                        "of TransactionalMoveFile: %s",
-                        *original_attributes, move_status.ToString()));
+        absl::StrFormat("SetFileAttributes failed: original_attrs: %d",
+                        *original_attributes));
   }
   return absl::OkStatus();
-#else   // !OS_WIN
+#else   // !_WIN32
   // Mac OSX: use rename(2), but rename(2) on Mac OSX
   // is not properly implemented, atomic rename is POSIX spec though.
   // http://www.weirdnet.nl/apple/rename.html
@@ -568,24 +534,22 @@ absl::Status FileUtilImpl::AtomicRename(const std::string &from,
         absl::StrFormat("errno(%d): %s", err, std::strerror(err)));
   }
   return absl::OkStatus();
-#endif  // OS_WIN
+#endif  // _WIN32
 }
 
-absl::Status FileUtil::CreateHardLink(const std::string &from,
-                                      const std::string &to) {
+absl::Status FileUtil::CreateHardLink(zstring_view from, zstring_view to) {
   return FileUtilSingleton::Get()->CreateHardLink(from, to);
 }
 
-absl::Status FileUtilImpl::CreateHardLink(const std::string &from,
-                                          const std::string &to) {
+absl::Status FileUtilImpl::CreateHardLink(zstring_view from, zstring_view to) {
 #ifdef __APPLE__
   return absl::UnimplementedError(
       "std::filesystem is only available on macOS 10.15, iOS 13.0, or later.");
 #else   // __APPLE__
 
   // u8path is deprecated in C++20. The current target is C++17.
-  const std::filesystem::path src = std::filesystem::u8path(from);
-  const std::filesystem::path dst = std::filesystem::u8path(to);
+  const std::filesystem::path src = std::filesystem::u8path(from.c_str());
+  const std::filesystem::path dst = std::filesystem::u8path(to.c_str());
 
   std::error_code error_code;
   std::filesystem::create_hard_link(src, dst, error_code);
@@ -598,149 +562,127 @@ absl::Status FileUtilImpl::CreateHardLink(const std::string &from,
 }
 
 std::string FileUtil::JoinPath(
-    const std::vector<absl::string_view> &components) {
+    const absl::Span<const absl::string_view> components) {
   std::string output;
-  JoinPath(components, &output);
+  for (const absl::string_view component : components) {
+    if (component.empty()) {
+      continue;
+    }
+    if (!output.empty() && output.back() != kFileDelimiter) {
+      output.append(1, kFileDelimiter);
+    }
+    absl::StrAppend(&output, component);
+  }
   return output;
 }
 
-void FileUtil::JoinPath(const std::vector<absl::string_view> &components,
-                        std::string *output) {
-  output->clear();
-  for (size_t i = 0; i < components.size(); ++i) {
-    if (components[i].empty()) {
-      continue;
-    }
-    if (!output->empty() && output->back() != kFileDelimiter) {
-      output->append(1, kFileDelimiter);
-    }
-    output->append(components[i].data(), components[i].size());
-  }
-}
-
 // TODO(taku): what happens if filename == '/foo/bar/../bar/..
-std::string FileUtil::Dirname(const std::string &filename) {
-  const std::string::size_type p = filename.find_last_of(kFileDelimiter);
+std::string FileUtil::Dirname(zstring_view filename) {
+  const std::string filename_str(filename.c_str());
+  const std::string::size_type p = filename_str.find_last_of(kFileDelimiter);
   if (p == std::string::npos) {
     return "";
   }
-  return filename.substr(0, p);
+  return filename_str.substr(0, p);
 }
 
-std::string FileUtil::Basename(const std::string &filename) {
-  const std::string::size_type p = filename.find_last_of(kFileDelimiter);
+std::string FileUtil::Basename(zstring_view filename) {
+  const std::string filename_str(filename.c_str());
+  const std::string::size_type p = filename_str.find_last_of(kFileDelimiter);
   if (p == std::string::npos) {
-    return filename;
+    return filename_str;
   }
-  return filename.substr(p + 1, filename.size() - p);
+  return filename_str.substr(p + 1, filename_str.size() - p);
 }
 
-std::string FileUtil::NormalizeDirectorySeparator(const std::string &path) {
-#ifdef OS_WIN
-  constexpr char kFileDelimiterForUnix = '/';
-  constexpr char kFileDelimiterForWindows = '\\';
-  std::string normalized;
-  Util::StringReplace(path, std::string(1, kFileDelimiterForUnix),
-                      std::string(1, kFileDelimiterForWindows), true,
-                      &normalized);
-  return normalized;
-#else   // OS_WIN
-  return path;
-#endif  // OS_WIN
+std::string FileUtil::NormalizeDirectorySeparator(zstring_view path) {
+  if constexpr (TargetIsWindows()) {
+    constexpr absl::string_view kFileDelimiterForUnix = "/";
+    constexpr absl::string_view kFileDelimiterForWindows = "\\";
+    return absl::StrReplaceAll(
+        path, {{kFileDelimiterForUnix, kFileDelimiterForWindows}});
+  }
+  return std::string(path.view());
 }
 
 absl::StatusOr<FileTimeStamp> FileUtil::GetModificationTime(
-    const std::string &filename) {
+    zstring_view filename) {
   return FileUtilSingleton::Get()->GetModificationTime(filename);
 }
 
 absl::StatusOr<FileTimeStamp> FileUtilImpl::GetModificationTime(
-    const std::string &filename) const {
-#if defined(OS_WIN)
-  std::wstring wide;
-  if (!Util::Utf8ToWide(filename, &wide)) {
+    zstring_view filename) const {
+#if defined(_WIN32)
+  const std::wstring wide = win32::Utf8ToWide(filename);
+  if (wide.empty()) {
     return absl::InvalidArgumentError(
         absl::StrCat("Utf8ToWide failed: ", filename));
   }
   WIN32_FILE_ATTRIBUTE_DATA info = {};
   if (!::GetFileAttributesEx(wide.c_str(), GetFileExInfoStandard, &info)) {
     const auto last_error = ::GetLastError();
-    return WinUtil::ErrorToCanonicalStatus(
+    return Win32ErrorToStatus(
         last_error, absl::StrCat("GetFileAttributesEx(", filename, ") failed"));
   }
-  return (static_cast<uint64>(info.ftLastWriteTime.dwHighDateTime) << 32) +
+  return (static_cast<uint64_t>(info.ftLastWriteTime.dwHighDateTime) << 32) +
          info.ftLastWriteTime.dwLowDateTime;
-#else   // !OS_WIN
+#else   // !_WIN32
   struct stat stat_info;
   if (::stat(filename.c_str(), &stat_info)) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(
-        err, absl::StrCat("stat failed: ", filename));
+    return absl::ErrnoToStatus(err, absl::StrCat("stat failed: ", filename));
   }
   return stat_info.st_mtime;
-#endif  // OS_WIN
+#endif  // _WIN32
 }
 
-absl::Status FileUtil::GetContents(const std::string &filename,
-                                   std::string *output,
-                                   std::ios_base::openmode mode) {
-  InputFileStream ifs(filename.c_str(), mode | std::ios::ate);
+absl::StatusOr<std::string> FileUtil::GetContents(
+    zstring_view filename, std::ios_base::openmode mode) {
+  InputFileStream ifs(filename, mode | std::ios::ate);
   if (ifs.fail()) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(err,
-                                        absl::StrCat("Cannot open ", filename));
+    return absl::ErrnoToStatus(err, absl::StrCat("Cannot open ", filename));
   }
   const ptrdiff_t size = ifs.tellg();
   if (size == -1) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(
-        err, absl::StrCat("tellg failed: ", filename));
+    return absl::ErrnoToStatus(err, absl::StrCat("tellg failed: ", filename));
   }
   ifs.seekg(0, std::ios_base::beg);
+  std::string content;
   if (mode & std::ios::binary) {
-    output->resize(size);
-    ifs.read(&(*output)[0], size);
+    content.resize(size);
+    ifs.read(content.data(), size);
   } else {
     // In the text mode, the read size can be smaller than the file size as
     // "\r\n" can be translated to "\n" on Windows. Therefore, we just reserve a
     // buffer size and perform sequential read.
-    output->reserve(size);
-    output->assign(std::istreambuf_iterator<char>(ifs),
+    content.reserve(size);
+    content.assign(std::istreambuf_iterator<char>(ifs),
                    std::istreambuf_iterator<char>());
   }
   ifs.close();
   if (ifs.fail()) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(
-        err,
-        absl::StrCat("Cannot read ", filename, " of size ", size, " bytes"));
-  }
-  return absl::OkStatus();
-}
-
-absl::StatusOr<std::string> FileUtil::GetContents(
-    const std::string &filename, std::ios_base::openmode mode) {
-  std::string content;
-  if (absl::Status s = GetContents(filename, &content, mode); !s.ok()) {
-    return s;
+    return absl::ErrnoToStatus(err, absl::StrCat("Cannot read ", filename,
+                                                 " of size ", size, " bytes"));
   }
   return content;
 }
 
-absl::Status FileUtil::SetContents(const std::string &filename,
+absl::Status FileUtil::SetContents(zstring_view filename,
                                    absl::string_view content,
                                    std::ios_base::openmode mode) {
-  OutputFileStream ofs(filename.c_str(), mode);
+  OutputFileStream ofs(filename, mode);
   if (ofs.fail()) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(err,
-                                        absl::StrCat("Cannot open ", filename));
+    return absl::ErrnoToStatus(err, absl::StrCat("Cannot open ", filename));
   }
   ofs << content;
   ofs.close();
   if (ofs.fail()) {
     const int err = errno;
-    return Util::ErrnoToCanonicalStatus(
+    return absl::ErrnoToStatus(
         err,
         absl::StrCat("Cannot write ", content.size(), " bytes to ", filename));
   }
@@ -751,40 +693,4 @@ void FileUtil::SetMockForUnitTest(FileUtilInterface *mock) {
   FileUtilSingleton::SetMock(mock);
 }
 
-absl::Status FileUtil::LinkOrCopyFile(const std::string &src_path,
-                                      const std::string &dst_path) {
-  absl::StatusOr<bool> is_equiv = FileUtil::IsEquivalent(src_path, dst_path);
-  if (!is_equiv.ok()) {
-    LOG(WARNING) << "Cannot test file equivalence: " << is_equiv.status();
-    // Try hard link or copy.
-  } else if (*is_equiv) {
-    // IsEquivalent() checks if src and dst are the same path or sym/hard link.
-    // This does not check the contents of the files.
-    return absl::OkStatus();
-  }
-
-  const std::string tmp_dst_path = dst_path + ".tmp";
-  FileUtil::UnlinkOrLogError(tmp_dst_path);
-  if (absl::Status s = FileUtil::CreateHardLink(src_path, tmp_dst_path);
-      !s.ok()) {
-    LOG(WARNING) << "Cannot create hardlink from " << src_path << " to "
-                 << tmp_dst_path << ": " << s;
-    // If an error happens, fallback to file copy.
-    if (absl::Status s = FileUtil::CopyFile(src_path, tmp_dst_path); !s.ok()) {
-      return absl::Status(
-          s.code(), absl::StrCat("Cannot copy file. from: ", src_path,
-                                 " to: ", tmp_dst_path, ": ", s.message()));
-    }
-  }
-
-  if (absl::Status s = FileUtil::AtomicRename(tmp_dst_path, dst_path);
-      !s.ok()) {
-    return absl::Status(
-        s.code(), absl::StrCat("AtomicRename failed: ", s.message(),
-                               "; from: ", tmp_dst_path, ", to: ", dst_path));
-  }
-
-  return absl::OkStatus();
-}
-
 }  // namespace mozc
