@@ -29,36 +29,30 @@
 
 #include "base/file_util.h"
 
-#ifdef OS_WIN
-#include <Windows.h>
-#endif  // OS_WIN
-
-#include <fstream>
+#include <ios>
 #include <string>
 
-#include "base/logging.h"
-#include "base/util.h"
-#include "testing/base/public/gmock.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "absl/flags/flag.h"
 #include "absl/status/status.h"
-#include "absl/strings/str_format.h"
+#include "absl/status/statusor.h"
+#include "base/file/temp_dir.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
+
+#ifdef _WIN32
+#include <windows.h>
+
+#include "base/win32/wide_char.h"
+#endif  // _WIN32
 
 // Ad-hoc workadound against macro problem on Windows.
-// On Windows, following macros, defined when you include <Windows.h>,
+// On Windows, following macros, defined when you include <windows.h>,
 // should be removed here because they affects the method name definition of
 // Util class.
 // TODO(yukawa): Use different method name if applicable.
-#ifdef CreateDirectory
 #undef CreateDirectory
-#endif  // CreateDirectory
-#ifdef RemoveDirectory
 #undef RemoveDirectory
-#endif  // RemoveDirectory
-#ifdef CopyFile
 #undef CopyFile
-#endif  // CopyFile
 
 namespace mozc {
 namespace {
@@ -67,10 +61,10 @@ namespace {
   ASSERT_OK(::mozc::FileUtil::SetContents(filename, data))
 
 TEST(FileUtilTest, CreateDirectory) {
-  EXPECT_OK(FileUtil::DirectoryExists(absl::GetFlag(FLAGS_test_tmpdir)));
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  EXPECT_OK(FileUtil::DirectoryExists(temp_dir.path()));
   // dirpath = FLAGS_test_tmpdir/testdir
-  const std::string dirpath =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "testdir");
+  const std::string dirpath = FileUtil::JoinPath(temp_dir.path(), "testdir");
 
   // Delete dirpath, if it exists.
   ASSERT_OK(FileUtil::RemoveDirectoryIfExists(dirpath));
@@ -80,15 +74,24 @@ TEST(FileUtilTest, CreateDirectory) {
   EXPECT_OK(FileUtil::CreateDirectory(dirpath));
   EXPECT_OK(FileUtil::DirectoryExists(dirpath));
 
+#if !defined(_WIN32)
+  // On Windows, CreateDirectory does not return OK if the directory already
+  // exists. See the implementation of CreateDirectory in file_util.cc.
+  // https://github.com/google/mozc/issues/1076
+  //
+  // Create the same directory again.
+  EXPECT_OK(FileUtil::CreateDirectory(dirpath));
+#endif  // !_WIN32
+
   // Delete the directory.
   ASSERT_OK(FileUtil::RemoveDirectory(dirpath));
   ASSERT_FALSE(FileUtil::FileExists(dirpath).ok());
 }
 
 TEST(FileUtilTest, DirectoryExists) {
-  EXPECT_OK(FileUtil::DirectoryExists(absl::GetFlag(FLAGS_test_tmpdir)));
-  const std::string filepath =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "testfile");
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  EXPECT_OK(FileUtil::DirectoryExists(temp_dir.path()));
+  const std::string filepath = FileUtil::JoinPath(temp_dir.path(), "testfile");
 
   // Delete filepath, if it exists.
   ASSERT_OK(FileUtil::UnlinkIfExists(filepath));
@@ -105,8 +108,8 @@ TEST(FileUtilTest, DirectoryExists) {
 }
 
 TEST(FileUtilTest, Unlink) {
-  const std::string filepath =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "testfile");
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string filepath = FileUtil::JoinPath(temp_dir.path(), "testfile");
   ASSERT_OK(FileUtil::UnlinkIfExists(filepath));
   EXPECT_FALSE(FileUtil::FileExists(filepath).ok());
 
@@ -115,7 +118,7 @@ TEST(FileUtilTest, Unlink) {
   EXPECT_OK(FileUtil::Unlink(filepath));
   EXPECT_FALSE(FileUtil::FileExists(filepath).ok());
 
-#ifdef OS_WIN
+#ifdef _WIN32
   constexpr DWORD kTestAttributeList[] = {
       FILE_ATTRIBUTE_ARCHIVE, FILE_ATTRIBUTE_HIDDEN,
       FILE_ATTRIBUTE_NORMAL,  FILE_ATTRIBUTE_NOT_CONTENT_INDEXED,
@@ -123,8 +126,7 @@ TEST(FileUtilTest, Unlink) {
       FILE_ATTRIBUTE_SYSTEM,  FILE_ATTRIBUTE_TEMPORARY,
   };
 
-  std::wstring wfilepath;
-  Util::Utf8ToWide(filepath, &wfilepath);
+  const std::wstring wfilepath = win32::Utf8ToWide(filepath);
   for (size_t i = 0; i < std::size(kTestAttributeList); ++i) {
     SCOPED_TRACE(absl::StrFormat("AttributeTest %zd", i));
     CreateTestFile(filepath, "attribute_test");
@@ -134,21 +136,19 @@ TEST(FileUtilTest, Unlink) {
     EXPECT_OK(FileUtil::Unlink(filepath));
     EXPECT_FALSE(FileUtil::FileExists(filepath).ok());
   }
-#endif  // OS_WIN
+#endif  // _WIN32
 
   EXPECT_OK(FileUtil::UnlinkIfExists(filepath));
 }
 
-#ifdef OS_WIN
+#ifdef _WIN32
 TEST(FileUtilTest, HideFile) {
-  const std::string filename =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "testfile");
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename));
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string filename = FileUtil::JoinPath(temp_dir.path(), "testfile");
 
   EXPECT_FALSE(FileUtil::HideFile(filename));
 
-  std::wstring wfilename;
-  Util::Utf8ToWide(filename.c_str(), &wfilename);
+  const std::wstring wfilename = win32::Utf8ToWide(filename);
 
   CreateTestFile(filename, "test data");
   EXPECT_OK(FileUtil::FileExists(filename));
@@ -156,45 +156,41 @@ TEST(FileUtilTest, HideFile) {
   EXPECT_NE(FALSE,
             ::SetFileAttributesW(wfilename.c_str(), FILE_ATTRIBUTE_NORMAL));
   EXPECT_TRUE(FileUtil::HideFile(filename));
-  EXPECT_EQ(FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
-                FILE_ATTRIBUTE_NOT_CONTENT_INDEXED,
-            ::GetFileAttributesW(wfilename.c_str()));
+  EXPECT_EQ(::GetFileAttributesW(wfilename.c_str()),
+            FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
+                FILE_ATTRIBUTE_NOT_CONTENT_INDEXED);
 
-  EXPECT_NE(FALSE,
-            ::SetFileAttributesW(wfilename.c_str(), FILE_ATTRIBUTE_ARCHIVE));
+  EXPECT_NE(::SetFileAttributesW(wfilename.c_str(), FILE_ATTRIBUTE_ARCHIVE),
+            FALSE);
   EXPECT_TRUE(FileUtil::HideFile(filename));
-  EXPECT_EQ(FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
-                FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_ATTRIBUTE_ARCHIVE,
-            ::GetFileAttributesW(wfilename.c_str()));
+  EXPECT_EQ(::GetFileAttributesW(wfilename.c_str()),
+            FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
+                FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_ATTRIBUTE_ARCHIVE);
 
-  EXPECT_NE(FALSE,
-            ::SetFileAttributesW(wfilename.c_str(), FILE_ATTRIBUTE_NORMAL));
+  EXPECT_NE(::SetFileAttributesW(wfilename.c_str(), FILE_ATTRIBUTE_NORMAL),
+            FALSE);
   EXPECT_TRUE(FileUtil::HideFileWithExtraAttributes(filename,
                                                     FILE_ATTRIBUTE_TEMPORARY));
-  EXPECT_EQ(FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
-                FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_ATTRIBUTE_TEMPORARY,
-            ::GetFileAttributesW(wfilename.c_str()));
+  EXPECT_EQ(::GetFileAttributesW(wfilename.c_str()),
+            FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
+                FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_ATTRIBUTE_TEMPORARY);
 
   EXPECT_NE(FALSE,
             ::SetFileAttributesW(wfilename.c_str(), FILE_ATTRIBUTE_ARCHIVE));
   EXPECT_TRUE(FileUtil::HideFileWithExtraAttributes(filename,
                                                     FILE_ATTRIBUTE_TEMPORARY));
-  EXPECT_EQ(FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
+  EXPECT_EQ(::GetFileAttributesW(wfilename.c_str()),
+            FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM |
                 FILE_ATTRIBUTE_NOT_CONTENT_INDEXED | FILE_ATTRIBUTE_ARCHIVE |
-                FILE_ATTRIBUTE_TEMPORARY,
-            ::GetFileAttributesW(wfilename.c_str()));
-
-  ASSERT_OK(FileUtil::Unlink(filename));
+                FILE_ATTRIBUTE_TEMPORARY);
 }
-#endif  // OS_WIN
+#endif  // _WIN32
 
 TEST(FileUtilTest, IsEqualFile) {
-  const std::string filename1 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test1");
-  const std::string filename2 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test2");
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename1));
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename2));
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string filename1 = FileUtil::JoinPath(temp_dir.path(), "test1");
+  const std::string filename2 = FileUtil::JoinPath(temp_dir.path(), "test2");
+
   EXPECT_FALSE(FileUtil::IsEqualFile(filename1, filename2).ok());
 
   CreateTestFile(filename1, "test data1");
@@ -214,18 +210,12 @@ TEST(FileUtilTest, IsEqualFile) {
   s = FileUtil::IsEqualFile(filename1, filename2);
   EXPECT_OK(s);
   EXPECT_FALSE(*s);
-
-  ASSERT_OK(FileUtil::Unlink(filename1));
-  ASSERT_OK(FileUtil::Unlink(filename2));
 }
 
 TEST(FileUtilTest, IsEquivalent) {
-  const std::string filename1 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test1");
-  const std::string filename2 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test2");
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename1));
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename2));
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string filename1 = FileUtil::JoinPath(temp_dir.path(), "test1");
+  const std::string filename2 = FileUtil::JoinPath(temp_dir.path(), "test2");
   EXPECT_FALSE(FileUtil::IsEquivalent(filename1, filename1).ok());
   EXPECT_FALSE(FileUtil::IsEquivalent(filename1, filename2).ok());
 
@@ -248,13 +238,10 @@ TEST(FileUtilTest, IsEquivalent) {
 }
 
 TEST(FileUtilTest, CopyFile) {
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
   // just test rename operation works as intended
-  const std::string from =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "copy_from");
-  const std::string to =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "copy_to");
-  ASSERT_OK(FileUtil::UnlinkIfExists(from));
-  ASSERT_OK(FileUtil::UnlinkIfExists(to));
+  const std::string from = FileUtil::JoinPath(temp_dir.path(), "copy_from");
+  const std::string to = FileUtil::JoinPath(temp_dir.path(), "copy_to");
 
   CreateTestFile(from, "simple test");
   EXPECT_OK(FileUtil::CopyFile(from, to));
@@ -268,7 +255,7 @@ TEST(FileUtilTest, CopyFile) {
   EXPECT_OK(s);
   EXPECT_TRUE(*s);
 
-#ifdef OS_WIN
+#ifdef _WIN32
   struct TestData {
     TestData(DWORD from, DWORD to) : from_attributes(from), to_attributes(to) {}
     const DWORD from_attributes;
@@ -298,9 +285,8 @@ TEST(FileUtilTest, CopyFile) {
     CreateTestFile(from, test_label);
 
     const TestData &kData = kTestDataList[i];
-    std::wstring wfrom, wto;
-    Util::Utf8ToWide(from.c_str(), &wfrom);
-    Util::Utf8ToWide(to.c_str(), &wto);
+    const std::wstring wfrom = win32::Utf8ToWide(from);
+    const std::wstring wto = win32::Utf8ToWide(to);
     EXPECT_NE(FALSE,
               ::SetFileAttributesW(wfrom.c_str(), kData.from_attributes));
     EXPECT_NE(FALSE, ::SetFileAttributesW(wto.c_str(), kData.to_attributes));
@@ -309,27 +295,23 @@ TEST(FileUtilTest, CopyFile) {
     absl::StatusOr<bool> s = FileUtil::IsEqualFile(from, to);
     EXPECT_OK(s);
     EXPECT_TRUE(*s);
-    EXPECT_EQ(kData.from_attributes, ::GetFileAttributesW(wfrom.c_str()));
-    EXPECT_EQ(kData.from_attributes, ::GetFileAttributesW(wto.c_str()));
+    EXPECT_EQ(::GetFileAttributesW(wfrom.c_str()), kData.from_attributes);
+    EXPECT_EQ(::GetFileAttributesW(wto.c_str()), kData.from_attributes);
 
     EXPECT_NE(FALSE,
               ::SetFileAttributesW(wfrom.c_str(), FILE_ATTRIBUTE_NORMAL));
     EXPECT_NE(FALSE, ::SetFileAttributesW(wto.c_str(), FILE_ATTRIBUTE_NORMAL));
   }
-#endif  // OS_WIN
-
-  ASSERT_OK(FileUtil::Unlink(from));
-  ASSERT_OK(FileUtil::Unlink(to));
+#endif  // _WIN32
 }
 
 TEST(FileUtilTest, AtomicRename) {
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
   // just test rename operation works as intended
-  const std::string from = FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir),
-                                              "atomic_rename_test_from");
-  const std::string to = FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir),
-                                            "atomic_rename_test_to");
-  ASSERT_OK(FileUtil::UnlinkIfExists(from));
-  ASSERT_OK(FileUtil::UnlinkIfExists(to));
+  const std::string from =
+      FileUtil::JoinPath(temp_dir.path(), "atomic_rename_test_from");
+  const std::string to =
+      FileUtil::JoinPath(temp_dir.path(), "atomic_rename_test_to");
 
   // |from| is not found
   EXPECT_FALSE(FileUtil::AtomicRename(from, to).ok());
@@ -343,7 +325,7 @@ TEST(FileUtilTest, AtomicRename) {
   {
     absl::StatusOr<std::string> content = FileUtil::GetContents(to);
     ASSERT_OK(content);
-    EXPECT_EQ("test", *content);
+    EXPECT_EQ(*content, "test");
   }
 
   EXPECT_FALSE(FileUtil::AtomicRename(from, to).ok());
@@ -356,7 +338,7 @@ TEST(FileUtilTest, AtomicRename) {
   CreateTestFile(to, "test");
   EXPECT_OK(FileUtil::AtomicRename(from, to));
 
-#ifdef OS_WIN
+#ifdef _WIN32
   struct TestData {
     TestData(DWORD from, DWORD to) : from_attributes(from), to_attributes(to) {}
     const DWORD from_attributes;
@@ -386,34 +368,27 @@ TEST(FileUtilTest, AtomicRename) {
     CreateTestFile(from, test_label);
 
     const TestData &kData = kTestDataList[i];
-    std::wstring wfrom, wto;
-    Util::Utf8ToWide(from.c_str(), &wfrom);
-    Util::Utf8ToWide(to.c_str(), &wto);
+    const std::wstring wfrom = win32::Utf8ToWide(from);
+    const std::wstring wto = win32::Utf8ToWide(to);
     EXPECT_NE(FALSE,
               ::SetFileAttributesW(wfrom.c_str(), kData.from_attributes));
     EXPECT_NE(FALSE, ::SetFileAttributesW(wto.c_str(), kData.to_attributes));
 
     EXPECT_OK(FileUtil::AtomicRename(from, to));
-    EXPECT_EQ(kData.from_attributes, ::GetFileAttributesW(wto.c_str()));
+    EXPECT_EQ(::GetFileAttributesW(wto.c_str()), kData.from_attributes);
     EXPECT_FALSE(FileUtil::FileExists(from).ok());
     EXPECT_OK(FileUtil::FileExists(to));
 
     ::SetFileAttributesW(wfrom.c_str(), FILE_ATTRIBUTE_NORMAL);
     ::SetFileAttributesW(wto.c_str(), FILE_ATTRIBUTE_NORMAL);
   }
-#endif  // OS_WIN
-
-  ASSERT_OK(FileUtil::UnlinkIfExists(from));
-  ASSERT_OK(FileUtil::UnlinkIfExists(to));
+#endif  // _WIN32
 }
 
 TEST(FileUtilTest, CreateHardLink) {
-  const std::string filename1 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test1");
-  const std::string filename2 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test2");
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename1));
-  ASSERT_OK(FileUtil::UnlinkIfExists(filename2));
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string filename1 = FileUtil::JoinPath(temp_dir.path(), "test1");
+  const std::string filename2 = FileUtil::JoinPath(temp_dir.path(), "test2");
 
   CreateTestFile(filename1, "test data");
   absl::Status s = FileUtil::CreateHardLink(filename1, filename2);
@@ -428,84 +403,86 @@ TEST(FileUtilTest, CreateHardLink) {
   EXPECT_FALSE(FileUtil::CreateHardLink(filename1, filename2).ok());
 }
 
-#ifdef OS_WIN
+#ifdef _WIN32
 #define SP "\\"
-#else  // OS_WIN
+#else  // _WIN32
 #define SP "/"
-#endif  // OS_WIN
+#endif  // _WIN32
 
 TEST(FileUtilTest, JoinPath) {
   EXPECT_TRUE(FileUtil::JoinPath({}).empty());
-  EXPECT_EQ("foo", FileUtil::JoinPath({"foo"}));
-  EXPECT_EQ("foo" SP "bar", FileUtil::JoinPath({"foo", "bar"}));
-  EXPECT_EQ("foo" SP "bar" SP "baz", FileUtil::JoinPath({"foo", "bar", "baz"}));
+  EXPECT_EQ(FileUtil::JoinPath({"foo"}), "foo");
+  EXPECT_EQ(FileUtil::JoinPath({"foo", "bar"}), "foo" SP "bar");
+  EXPECT_EQ(FileUtil::JoinPath({"foo", "bar", "baz"}), "foo" SP "bar" SP "baz");
 
   // Some path components end with delimiter.
-  EXPECT_EQ("foo" SP "bar" SP "baz",
-            FileUtil::JoinPath({"foo" SP, "bar", "baz"}));
-  EXPECT_EQ("foo" SP "bar" SP "baz",
-            FileUtil::JoinPath({"foo", "bar" SP, "baz"}));
-  EXPECT_EQ("foo" SP "bar" SP "baz" SP,
-            FileUtil::JoinPath({"foo", "bar", "baz" SP}));
+  EXPECT_EQ(FileUtil::JoinPath({"foo" SP, "bar", "baz"}),
+            "foo" SP "bar" SP "baz");
+  EXPECT_EQ(FileUtil::JoinPath({"foo", "bar" SP, "baz"}),
+            "foo" SP "bar" SP "baz");
+  EXPECT_EQ(FileUtil::JoinPath({"foo", "bar", "baz" SP}),
+            "foo" SP "bar" SP "baz" SP);
 
   // Containing empty strings.
   EXPECT_TRUE(FileUtil::JoinPath({"", "", ""}).empty());
-  EXPECT_EQ("foo" SP "bar", FileUtil::JoinPath({"", "foo", "bar"}));
-  EXPECT_EQ("foo" SP "bar", FileUtil::JoinPath({"foo", "", "bar"}));
-  EXPECT_EQ("foo" SP "bar", FileUtil::JoinPath({"foo", "bar", ""}));
+  EXPECT_EQ(FileUtil::JoinPath({"", "foo", "bar"}), "foo" SP "bar");
+  EXPECT_EQ(FileUtil::JoinPath({"foo", "", "bar"}), "foo" SP "bar");
+  EXPECT_EQ(FileUtil::JoinPath({"foo", "bar", ""}), "foo" SP "bar");
 }
 
 TEST(FileUtilTest, Dirname) {
-  EXPECT_EQ(SP "foo", FileUtil::Dirname(SP "foo" SP "bar"));
-  EXPECT_EQ(SP "foo" SP "bar",
-            FileUtil::Dirname(SP "foo" SP "bar" SP "foo.txt"));
-  EXPECT_EQ("", FileUtil::Dirname("foo.txt"));
-  EXPECT_EQ("", FileUtil::Dirname(SP));
+  EXPECT_EQ(FileUtil::Dirname(SP "foo" SP "bar"), SP "foo");
+  EXPECT_EQ(FileUtil::Dirname(SP "foo" SP "bar" SP "foo.txt"),
+            SP "foo" SP "bar");
+  EXPECT_EQ(FileUtil::Dirname("foo.txt"), "");
+  EXPECT_EQ(FileUtil::Dirname(SP), "");
 }
 
 TEST(FileUtilTest, Basename) {
-  EXPECT_EQ("bar", FileUtil::Basename(SP "foo" SP "bar"));
-  EXPECT_EQ("foo.txt", FileUtil::Basename(SP "foo" SP "bar" SP "foo.txt"));
-  EXPECT_EQ("foo.txt", FileUtil::Basename("foo.txt"));
-  EXPECT_EQ("foo.txt", FileUtil::Basename("." SP "foo.txt"));
-  EXPECT_EQ(".foo.txt", FileUtil::Basename("." SP ".foo.txt"));
-  EXPECT_EQ("", FileUtil::Basename(SP));
-  EXPECT_EQ("", FileUtil::Basename("foo" SP "bar" SP "buz" SP));
+  EXPECT_EQ(FileUtil::Basename(SP "foo" SP "bar"), "bar");
+  EXPECT_EQ(FileUtil::Basename(SP "foo" SP "bar" SP "foo.txt"), "foo.txt");
+  EXPECT_EQ(FileUtil::Basename("foo.txt"), "foo.txt");
+  EXPECT_EQ(FileUtil::Basename("." SP "foo.txt"), "foo.txt");
+  EXPECT_EQ(FileUtil::Basename("." SP ".foo.txt"), ".foo.txt");
+  EXPECT_EQ(FileUtil::Basename(SP), "");
+  EXPECT_EQ(FileUtil::Basename("foo" SP "bar" SP "buz" SP), "");
 }
 
 #undef SP
 
 TEST(FileUtilTest, NormalizeDirectorySeparator) {
-#ifdef OS_WIN
-  EXPECT_EQ("\\foo\\bar", FileUtil::NormalizeDirectorySeparator("\\foo\\bar"));
-  EXPECT_EQ("\\foo\\bar", FileUtil::NormalizeDirectorySeparator("/foo\\bar"));
-  EXPECT_EQ("\\foo\\bar", FileUtil::NormalizeDirectorySeparator("\\foo/bar"));
-  EXPECT_EQ("\\foo\\bar", FileUtil::NormalizeDirectorySeparator("/foo/bar"));
-  EXPECT_EQ("\\foo\\bar\\",
-            FileUtil::NormalizeDirectorySeparator("\\foo\\bar\\"));
-  EXPECT_EQ("\\foo\\bar\\", FileUtil::NormalizeDirectorySeparator("/foo/bar/"));
-  EXPECT_EQ("", FileUtil::NormalizeDirectorySeparator(""));
-  EXPECT_EQ("\\", FileUtil::NormalizeDirectorySeparator("/"));
-  EXPECT_EQ("\\", FileUtil::NormalizeDirectorySeparator("\\"));
-#else   // OS_WIN
-  EXPECT_EQ("\\foo\\bar", FileUtil::NormalizeDirectorySeparator("\\foo\\bar"));
-  EXPECT_EQ("/foo\\bar", FileUtil::NormalizeDirectorySeparator("/foo\\bar"));
-  EXPECT_EQ("\\foo/bar", FileUtil::NormalizeDirectorySeparator("\\foo/bar"));
-  EXPECT_EQ("/foo/bar", FileUtil::NormalizeDirectorySeparator("/foo/bar"));
-  EXPECT_EQ("\\foo\\bar\\",
-            FileUtil::NormalizeDirectorySeparator("\\foo\\bar\\"));
-  EXPECT_EQ("/foo/bar/", FileUtil::NormalizeDirectorySeparator("/foo/bar/"));
-  EXPECT_EQ("", FileUtil::NormalizeDirectorySeparator(""));
-  EXPECT_EQ("/", FileUtil::NormalizeDirectorySeparator("/"));
-  EXPECT_EQ("\\", FileUtil::NormalizeDirectorySeparator("\\"));
-#endif  // OS_WIN
+#ifdef _WIN32
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\foo\\bar"), "\\foo\\bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/foo\\bar"), "\\foo\\bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\foo/bar"), "\\foo\\bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/foo/bar"), "\\foo\\bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\foo\\bar\\"),
+            "\\foo\\bar\\");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/foo/bar/"), "\\foo\\bar\\");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator(""), "");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/"), "\\");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\"), "\\");
+#else   // _WIN32
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\foo\\bar"), "\\foo\\bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/foo\\bar"), "/foo\\bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\foo/bar"), "\\foo/bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/foo/bar"), "/foo/bar");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\foo\\bar\\"),
+            "\\foo\\bar\\");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/foo/bar/"), "/foo/bar/");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator(""), "");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("/"), "/");
+  EXPECT_EQ(FileUtil::NormalizeDirectorySeparator("\\"), "\\");
+#endif  // _WIN32
 }
 
 TEST(FileUtilTest, GetModificationTime) {
-  EXPECT_FALSE(FileUtil::GetModificationTime("not_existent_file").ok());
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  EXPECT_FALSE(FileUtil::GetModificationTime(
+                   FileUtil::JoinPath(temp_dir.path(), "not_existent_file"))
+                   .ok());
 
-  const std::string &path =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "testfile");
+  const std::string &path = FileUtil::JoinPath(temp_dir.path(), "testfile");
   CreateTestFile(path, "content");
   absl::StatusOr<FileTimeStamp> time_stamp1 =
       FileUtil::GetModificationTime(path);
@@ -516,73 +493,59 @@ TEST(FileUtilTest, GetModificationTime) {
       FileUtil::GetModificationTime(path);
   ASSERT_OK(time_stamp2);
   EXPECT_EQ(*time_stamp1, *time_stamp2);
-
-  // Cleanup
-  ASSERT_OK(FileUtil::Unlink(path));
 }
 
 TEST(FileUtilTest, GetAndSetContents) {
-  const std::string filename =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test.txt");
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string filename = FileUtil::JoinPath(temp_dir.path(), "test.txt");
 
   // File doesn't exist yet.
-  std::string content;
-  EXPECT_TRUE(absl::IsNotFound(FileUtil::GetContents(filename, &content)));
+  EXPECT_TRUE(absl::IsNotFound(FileUtil::GetContents(filename).status()));
 
   // Basic write and read test.
   ASSERT_OK(FileUtil::SetContents(filename, "test"));
   FileUnlinker unlinker(filename);
-  EXPECT_OK(FileUtil::GetContents(filename, &content));
-  EXPECT_EQ("test", content);
+  absl::StatusOr<std::string> content = FileUtil::GetContents(filename);
+  EXPECT_OK(content);
+  EXPECT_EQ(*content, "test");
 
   // Overwrite test.
   ASSERT_OK(FileUtil::SetContents(filename, "more tests!"));
-  EXPECT_OK(FileUtil::GetContents(filename, &content));
-  EXPECT_EQ("more tests!", content);
+  content = FileUtil::GetContents(filename);
+  EXPECT_OK(content);
+  EXPECT_EQ(*content, "more tests!");
 
   // Text mode write.
   ASSERT_OK(FileUtil::SetContents(filename, "test\ntest\n", std::ios::out));
-  EXPECT_OK(FileUtil::GetContents(filename, &content));
-#ifdef OS_WIN
-  EXPECT_EQ("test\r\ntest\r\n", content);
-#else   // OS_WIN
-  EXPECT_EQ("test\ntest\n", content);
-#endif  // OS_WIN
+  content = FileUtil::GetContents(filename);
+  EXPECT_OK(content);
+#ifdef _WIN32
+  EXPECT_EQ(*content, "test\r\ntest\r\n");
+#else   // _WIN32
+  EXPECT_EQ(*content, "test\ntest\n");
+#endif  // _WIN32
 
   // Text mode read.
   ASSERT_OK(FileUtil::SetContents(filename, "test\r\ntest\r\n"));
-  EXPECT_OK(FileUtil::GetContents(filename, &content, std::ios::in));
-#ifdef OS_WIN
-  EXPECT_EQ("test\ntest\n", content);
-#else   // OS_WIN
-  EXPECT_EQ("test\r\ntest\r\n", content);
-#endif  // OS_WIN
+  content = FileUtil::GetContents(filename, std::ios::in);
+  EXPECT_OK(content);
+#ifdef _WIN32
+  EXPECT_EQ(*content, "test\ntest\n");
+#else   // _WIN32
+  EXPECT_EQ(*content, "test\r\ntest\r\n");
+#endif  // _WIN32
 }
 
 TEST(FileUtilTest, FileUnlinker) {
-  const std::string filename =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test.txt");
-  ASSERT_OK(FileUtil::SetContents(filename, "test"));
+  absl::StatusOr<TempFile> temp_file =
+      TempDirectory::Default().CreateTempFile();
+  ASSERT_OK(temp_file);
+  ASSERT_OK(FileUtil::SetContents(temp_file->path(), "test"));
   {
-    FileUnlinker unlinker(filename);
-    EXPECT_OK(FileUtil::FileExists(filename));
+    FileUnlinker unlinker(temp_file->path());
+    EXPECT_OK(FileUtil::FileExists(temp_file->path()));
   }
-  EXPECT_FALSE(FileUtil::FileExists(filename).ok());
-}
-
-TEST(FileUtilTest, LinkOrCopyFile) {
-  const std::string from = FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir),
-                                              "link_or_copy_test_from.txt");
-  const std::string to = FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir),
-                                            "link_or_copy_test_to.txt");
-  ASSERT_OK(FileUtil::UnlinkIfExists(from));
-  ASSERT_OK(FileUtil::UnlinkIfExists(to));
-  EXPECT_TRUE(!FileUtil::LinkOrCopyFile(from, to).ok());
-  ASSERT_OK(FileUtil::SetContents(from, "test"));
-  EXPECT_OK(FileUtil::LinkOrCopyFile(from, to));
-  auto s = FileUtil::IsEqualFile(from, to);
-  EXPECT_OK(s);
-  EXPECT_TRUE(*s);
+  EXPECT_FALSE(FileUtil::FileExists(temp_file->path()).ok());
 }
 
 }  // namespace
