@@ -38,17 +38,21 @@
 #include <utility>
 #include <vector>
 
+#include "absl/strings/string_view.h"
+#include "absl/time/time.h"
 #include "base/run_level.h"
 #include "base/strings/assign.h"
+#include "base/strings/zstring_view.h"
+#include "client/client_interface.h"
 #include "composer/key_event_util.h"
 #include "ipc/ipc.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
-#include "testing/gunit_prod.h"
-#include "absl/strings/string_view.h"
-#include "absl/time/time.h"
-#include "client/client_interface.h"
-// for FRIEND_TEST()
+#include "testing/friend_test.h"
+
+// The obsolete and unmaintained *main.cc files (server_launcher_main.cc and
+// ping_server_main.cc) were removed to decrease maintenance costs and
+// accelerate build times.
 
 namespace mozc {
 namespace client {
@@ -76,7 +80,7 @@ class ServerLauncher : public ServerLauncherInterface {
   }
 
   // return server program
-  const std::string &server_program() const override { return server_program_; }
+  zstring_view server_program() const override { return server_program_; }
 
   void set_restricted(bool restricted) override { restricted_ = restricted; }
 
