@@ -32,27 +32,31 @@
 // events specified by FLAGS_input file or interactive standard input.  Input
 // file format is same as one of session/session_client_main.
 
+#include <cstddef>
 #include <cstdint>
 #include <iostream>
+#include <istream>
 #include <memory>
 #include <string>
 #include <vector>
 
+#include "absl/flags/flag.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/time/clock.h"
+#include "absl/time/time.h"
 #include "base/file_stream.h"
 #include "base/file_util.h"
 #include "base/init_mozc.h"
-#include "base/logging.h"
-#include "base/port.h"
 #include "base/system_util.h"
 #include "base/util.h"
+#include "base/vlog.h"
 #include "client/client.h"
 #include "composer/key_parser.h"
 #include "protocol/commands.pb.h"
 #include "protocol/renderer_command.pb.h"
 #include "renderer/renderer_client.h"
-#include "absl/flags/flag.h"
-#include "absl/status/status.h"
-#include "absl/strings/match.h"
 
 ABSL_FLAG(std::string, input, "", "Input file");
 ABSL_FLAG(int32_t, key_duration, 10, "Key duration (msec)");
@@ -77,7 +81,7 @@ bool ReadKeys(std::istream *input, std::vector<commands::KeyEvent> *keys,
     if (line.size() > 1 && line[0] == '#' && line[1] == '#') {
       continue;
     }
-    if (absl::StartsWith(line, ">> ")) {
+    if (line.starts_with(">> ")) {
       // Answer line
       answer->assign(line, 3, line.size() - 3);
       continue;
@@ -109,22 +113,22 @@ int Loop(std::istream *input) {
   mozc::commands::RendererCommand renderer_command;
 
   if (absl::GetFlag(FLAGS_test_renderer)) {
-#ifdef OS_WIN
+#ifdef _WIN32
     renderer_command.mutable_application_info()->set_process_id(
         ::GetCurrentProcessId());
     renderer_command.mutable_application_info()->set_thread_id(
         ::GetCurrentThreadId());
-#endif  // OS_WIN
-#if defined(OS_WIN) || defined(__APPLE__)
+#endif  // _WIN32
+#if defined(_WIN32) || defined(__APPLE__)
     renderer_command.mutable_preedit_rectangle()->set_left(10);
     renderer_command.mutable_preedit_rectangle()->set_top(10);
     renderer_command.mutable_preedit_rectangle()->set_right(200);
     renderer_command.mutable_preedit_rectangle()->set_bottom(30);
     renderer_client = std::make_unique<renderer::RendererClient>();
     CHECK(renderer_client->Activate());
-#else   // defined(OS_WIN) || defined(__APPLE__)
+#else   // defined(_WIN32) || defined(__APPLE__)
     LOG(FATAL) << "test_renderer is only supported on Windows and Mac";
-#endif  // defined(OS_WIN) || defined(__APPLE__)
+#endif  // defined(_WIN32) || defined(__APPLE__)
   }
 
   commands::Command command;
@@ -139,24 +143,24 @@ int Loop(std::istream *input) {
   while (ReadKeys(input, &keys, &answer)) {
     CHECK(client.NoOperation()) << "Server is not responding";
     for (size_t i = 0; i < keys.size(); ++i) {
-      Util::Sleep(absl::GetFlag(FLAGS_key_duration));
+      absl::SleepFor(absl::Milliseconds(absl::GetFlag(FLAGS_key_duration)));
 
       if (absl::GetFlag(FLAGS_test_testsendkey)) {
-        VLOG(2) << "Sending to Server: " << keys[i].DebugString();
+        MOZC_VLOG(2) << "Sending to Server: " << keys[i];
         client.TestSendKey(keys[i], &output);
-        VLOG(2) << "Output of TestSendKey: " << output.DebugString();
-        Util::Sleep(10);
+        MOZC_VLOG(2) << "Output of TestSendKey: " << output;
+        absl::SleepFor(absl::Milliseconds(10));
       }
 
-      VLOG(2) << "Sending to Server: " << keys[i].DebugString();
+      MOZC_VLOG(2) << "Sending to Server: " << keys[i];
       client.SendKey(keys[i], &output);
-      VLOG(2) << "Output of SendKey: " << output.DebugString();
+      MOZC_VLOG(2) << "Output of SendKey: " << output;
 
       if (renderer_client != nullptr) {
         renderer_command.set_type(commands::RendererCommand::UPDATE);
-        renderer_command.set_visible(output.has_candidates());
+        renderer_command.set_visible(output.has_candidate_window());
         *renderer_command.mutable_output() = output;
-        VLOG(2) << "Sending to Renderer: " << renderer_command.DebugString();
+        MOZC_VLOG(2) << "Sending to Renderer: " << renderer_command;
         renderer_client->ExecCommand(renderer_command);
       }
     }
@@ -190,8 +194,8 @@ int main(int argc, char **argv) {
 
   if (!absl::GetFlag(FLAGS_input).empty()) {
     // Batch mode loading the input file.
-    input_file = std::make_unique<mozc::InputFileStream>(
-        absl::GetFlag(FLAGS_input).c_str());
+    input_file =
+        std::make_unique<mozc::InputFileStream>(absl::GetFlag(FLAGS_input));
     if (input_file->fail()) {
       LOG(ERROR) << "File not opened: " << absl::GetFlag(FLAGS_input);
       return 1;
