@@ -27,33 +27,37 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#include <cstddef>
 #include <cstdint>
 #include <iostream>
+#include <memory>
+#include <ostream>
+#include <string>
+#include <vector>
 
-#include "client/client.h"
-
-#ifdef OS_WIN
+#ifdef _WIN32
 #include <windows.h>
-#else  // OS_WIN
+#else  // _WIN32
 #include <sys/types.h>
 #include <unistd.h>
-#endif  // OS_WIN
+#endif  // _WIN32
 
-#include <memory>
-
-#include "base/file_stream.h"
+#include "absl/flags/flag.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/time/clock.h"
+#include "absl/time/time.h"
 #include "base/init_mozc.h"
-#include "base/logging.h"
-#include "base/port.h"
-#include "base/util.h"
+#include "base/vlog.h"
+#include "client/client.h"
+#include "protocol/commands.pb.h"
 #include "protocol/renderer_command.pb.h"
 #include "renderer/renderer_client.h"
 #include "session/random_keyevents_generator.h"
-#include "absl/flags/flag.h"
 
 // TODO(taku)
 // 1. multi-thread testing
-// 2. change/config the senario
+// 2. change/config the scenario
 
 ABSL_FLAG(int32_t, max_keyevents, 100000,
           "test at most |max_keyevents| key sequences");
@@ -65,8 +69,6 @@ ABSL_FLAG(bool, test_testsendkey, true, "test TestSendKey");
 int main(int argc, char **argv) {
   mozc::InitMozc(argv[0], &argc, &argv);
 
-  absl::SetFlag(&FLAGS_logtostderr, true);
-
   mozc::client::Client client;
   if (!absl::GetFlag(FLAGS_server_path).empty()) {
     client.set_server_program(absl::GetFlag(FLAGS_server_path));
@@ -80,22 +82,22 @@ int main(int argc, char **argv) {
   mozc::commands::RendererCommand renderer_command;
 
   if (absl::GetFlag(FLAGS_test_renderer)) {
-#ifdef OS_WIN
+#ifdef _WIN32
     renderer_command.mutable_application_info()->set_process_id(
         ::GetCurrentProcessId());
     renderer_command.mutable_application_info()->set_thread_id(
         ::GetCurrentThreadId());
-#endif  // OS_WIN
-#if defined(OS_WIN) || defined(__APPLE__)
+#endif  // _WIN32
+#if defined(_WIN32) || defined(__APPLE__)
     renderer_command.mutable_preedit_rectangle()->set_left(10);
     renderer_command.mutable_preedit_rectangle()->set_top(10);
     renderer_command.mutable_preedit_rectangle()->set_right(200);
     renderer_command.mutable_preedit_rectangle()->set_bottom(30);
     renderer_client = std::make_unique<mozc::renderer::RendererClient>();
     CHECK(renderer_client->Activate());
-#else   // OS_WIN || __APPLE__
+#else   // _WIN32 || __APPLE__
     LOG(FATAL) << "test_renderer is only supported on Windows and Mac";
-#endif  // OS_WIN || __APPLE__
+#endif  // _WIN32 || __APPLE__
   }
 
   std::vector<mozc::commands::KeyEvent> keys;
@@ -107,11 +109,12 @@ int main(int argc, char **argv) {
   // Currently, we cannot detect the server crash out of
   // client library, as client automatically re-lahches the server.
 
+  mozc::session::RandomKeyEventsGenerator key_events_generator;
   while (true) {
-    mozc::session::RandomKeyEventsGenerator::GenerateSequence(&keys);
+    key_events_generator.GenerateSequence(&keys);
     CHECK(client.NoOperation()) << "Server is not responding";
     for (size_t i = 0; i < keys.size(); ++i) {
-      mozc::Util::Sleep(absl::GetFlag(FLAGS_key_duration));
+      absl::SleepFor(absl::Milliseconds(absl::GetFlag(FLAGS_key_duration)));
       keyevents_size++;
       if (keyevents_size % 100 == 0) {
         std::cout << keyevents_size << " key events finished" << std::endl;
@@ -122,21 +125,21 @@ int main(int argc, char **argv) {
         return 0;
       }
       if (absl::GetFlag(FLAGS_test_testsendkey)) {
-        VLOG(2) << "Sending to Server: " << keys[i].DebugString();
+        MOZC_VLOG(2) << "Sending to Server: " << keys[i];
         client.TestSendKey(keys[i], &output);
-        VLOG(2) << "Output of TestSendKey: " << output.DebugString();
-        mozc::Util::Sleep(10);
+        MOZC_VLOG(2) << "Output of TestSendKey: " << output;
+        absl::SleepFor(absl::Milliseconds(10));
       }
 
-      VLOG(2) << "Sending to Server: " << keys[i].DebugString();
+      MOZC_VLOG(2) << "Sending to Server: " << keys[i];
       client.SendKey(keys[i], &output);
-      VLOG(2) << "Output of SendKey: " << output.DebugString();
+      MOZC_VLOG(2) << "Output of SendKey: " << output;
 
       if (renderer_client != nullptr) {
         renderer_command.set_type(mozc::commands::RendererCommand::UPDATE);
-        renderer_command.set_visible(output.has_candidates());
+        renderer_command.set_visible(output.has_candidate_window());
         *renderer_command.mutable_output() = output;
-        VLOG(2) << "Sending to Renderer: " << renderer_command.DebugString();
+        MOZC_VLOG(2) << "Sending to Renderer: " << renderer_command;
         renderer_client->ExecCommand(renderer_command);
       }
     }
