@@ -32,28 +32,120 @@
 #ifndef MOZC_COMPOSER_COMPOSER_H_
 #define MOZC_COMPOSER_COMPOSER_H_
 
+#include <cstddef>
 #include <cstdint>
-#include <set>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "base/port.h"
-#include "base/protobuf/repeated_field.h"
-#include "composer/internal/composition.h"
-#include "composer/internal/composition_input.h"
-#include "composer/internal/transliterators.h"
-#include "composer/internal/typing_corrector.h"
+#include "absl/base/attributes.h"
+#include "absl/container/btree_set.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
+#include "composer/composition.h"
+#include "composer/composition_input.h"
 #include "composer/table.h"
-#include "composer/type_corrected_query.h"
+#include "composer/transliterators.h"
 #include "protocol/commands.pb.h"
-// for FRIEND_TEST()
-#include "testing/base/public/gunit_prod.h"
+#include "protocol/config.pb.h"
+#include "testing/friend_test.h"
 #include "transliteration/transliteration.h"
 
 namespace mozc {
 namespace composer {
 
+std::shared_ptr<const commands::Request> GetSharedDefaultRequest();
+
+// ComposerData is a data structure that represents the current state of the
+// composer. It is used by Converter, Predictor and Rewriters as a const
+// object.
+class ComposerData {
+ public:
+  ComposerData() = delete;
+  ComposerData(Composition composition, size_t position,
+               transliteration::TransliterationType input_mode,
+               commands::Context::InputFieldType input_field_type,
+               std::string source_text,
+               std::vector<commands::SessionCommand::CompositionEvent>
+                   compositions_for_handwriting);
+
+  // Copyable and movable.
+  ComposerData(const ComposerData &other);
+  ComposerData &operator=(const ComposerData &);
+  ComposerData(ComposerData &&other) noexcept;
+  ComposerData &operator=(ComposerData &&) noexcept;
+
+  transliteration::TransliterationType GetInputMode() const;
+
+  // Returns a preedit string with user's preferences.
+  std::string GetStringForPreedit() const;
+
+  // Returns a conversion query normalized ascii characters in half width
+  std::string GetQueryForConversion() const;
+
+  // Returns a prediction query trimmed the tail alphabet characters.
+  std::string GetQueryForPrediction() const;
+
+  // Returns a expanded prediction query.
+  std::pair<std::string, absl::btree_set<std::string>> GetQueriesForPrediction()
+      const;
+
+  // Returns a string to be used for type correction.
+  std::string GetStringForTypeCorrection() const;
+
+  size_t GetLength() const;
+  size_t GetCursor() const;
+
+  absl::Span<const commands::SessionCommand::CompositionEvent>
+  GetHandwritingCompositions() const;
+
+  // Returns raw input from a user.
+  // The main purpose is Transliteration.
+  std::string GetRawString() const;
+
+  // Returns substring of raw input.  The position and size is based on the
+  // composed string.  For example, when [さ|sa][し|shi][み|mi] is the
+  // composition, GetRawSubString(0, 2) returns "sashi".
+  std::string GetRawSubString(size_t position, size_t size) const;
+
+  // Generate transliterations.
+  void GetTransliterations(transliteration::Transliterations *t13ns) const;
+
+  // Generate substrings of transliterations.
+  void GetSubTransliterations(size_t position, size_t size,
+                              transliteration::Transliterations *t13ns) const;
+
+  absl::string_view source_text() const { return source_text_; }
+
+ private:
+  // Composition copied from the Composer as a snapshot.
+  Composition composition_;
+
+  // Cursor position.
+  size_t position_ = 0;
+
+  // Input mode of IME (e.g. HIRAGANA, HALF_ASCII, etc.)
+  transliteration::TransliterationType input_mode_ = transliteration::HIRAGANA;
+
+  // Type of the input field to input texts.
+  commands::Context::InputFieldType input_field_type_ =
+      commands::Context::NORMAL;
+
+  // The original text for the composition.
+  // The value is usually empty, and used for reverse conversion.
+  std::string source_text_;
+
+  // Example:
+  //   {{"かん字", 0.99}, {"かlv字", 0.01}}
+  // Please refer to commands.proto
+  std::vector<commands::SessionCommand::CompositionEvent>
+      compositions_for_handwriting_;
+};
+
+// Composer is a class that manages the composing text. It provides methods to
+// edit the text by users. Composer creates ComposerData as the snapshot of
+// the current state of the composer.
 class Composer final {
  public:
   // Pseudo commands in composer.
@@ -63,13 +155,33 @@ class Composer final {
   };
 
   Composer();
-  Composer(const Table *table, const commands::Request *request,
-           const config::Config *config);
+  Composer(std::shared_ptr<const Table> table,
+           std::shared_ptr<const commands::Request> request,
+           std::shared_ptr<const config::Config> config);
 
-  Composer(const Composer &);
-  Composer &operator=(const Composer &);
+  // This constructor is for testing.
+  ABSL_DEPRECATED("Use the constructor with Table")
+  Composer(std::shared_ptr<const commands::Request> request,
+           std::shared_ptr<const config::Config> config);
 
-  ~Composer();
+  // Copies `request` and `config`.
+
+  ABSL_DEPRECATED("Use the constructor with Table")
+  Composer(commands::Request request, config::Config config);
+  Composer(std::shared_ptr<const Table> table, commands::Request request,
+           config::Config config);
+
+  // Copyable and movable.
+  Composer(const Composer &) = default;
+  Composer &operator=(const Composer &) = default;
+  Composer(Composer &&) = default;
+  Composer &operator=(Composer &&) = default;
+
+  // Returns a reference of empty ComposerData used for placeholder.
+  static const ComposerData &EmptyComposerData();
+
+  // Return a ComposerData with the current state of the composer.
+  ComposerData CreateComposerData() const;
 
   // Reset all composing data except table.
   void Reset();
@@ -86,10 +198,10 @@ class Composer final {
   // Check the preedit string is empty or not.
   bool Empty() const;
 
-  void SetTable(const Table *table);
+  void SetTable(std::shared_ptr<const Table> table);
 
-  void SetRequest(const commands::Request *request);
-  void SetConfig(const config::Config *config);
+  void SetRequest(std::shared_ptr<const commands::Request> request);
+  void SetConfig(std::shared_ptr<const config::Config> config);
 
   void SetInputMode(transliteration::TransliterationType mode);
   void SetTemporaryInputMode(transliteration::TransliterationType mode);
@@ -114,25 +226,24 @@ class Composer final {
                   std::string *right) const;
 
   // Returns a preedit string with user's preferences.
-  void GetStringForPreedit(std::string *output) const;
+  std::string GetStringForPreedit() const;
 
   // Returns a submit string with user's preferences.  The difference
   // from the preedit string is the handling of the last 'n'.
-  void GetStringForSubmission(std::string *output) const;
+  std::string GetStringForSubmission() const;
 
   // Returns a conversion query normalized ascii characters in half width
-  void GetQueryForConversion(std::string *output) const;
+  std::string GetQueryForConversion() const;
 
   // Returns a prediction query trimmed the tail alphabet characters.
-  void GetQueryForPrediction(std::string *output) const;
+  std::string GetQueryForPrediction() const;
 
   // Returns a expanded prediction query.
-  void GetQueriesForPrediction(std::string *base,
-                               std::set<std::string> *expanded) const;
+  std::pair<std::string, absl::btree_set<std::string>> GetQueriesForPrediction()
+      const;
 
-  // Returns a type-corrected prediction queries.
-  void GetTypeCorrectedQueriesForPrediction(
-      std::vector<TypeCorrectedQuery> *queries) const;
+  // Returns a string to be used for type correction.
+  std::string GetStringForTypeCorrection() const;
 
   size_t GetLength() const;
   size_t GetCursor() const;
@@ -143,7 +254,7 @@ class Composer final {
   // Delete multiple characters beginning at specified position.
   void DeleteRange(size_t pos, size_t length);
 
-  void InsertCharacter(const std::string &key);
+  void InsertCharacter(std::string key);
 
   // Set preedit text to composer.
   //
@@ -152,7 +263,7 @@ class Composer final {
   // you should use SetPreeditTextForTestOnly().
   // With the current implementation, prediction queries can be transliterated
   // and you will not be able to get right candidates.
-  void InsertCharacterPreedit(const std::string &input);
+  void InsertCharacterPreedit(absl::string_view input);
 
   // TEST ONLY: Set preedit text to composer.
   //
@@ -166,12 +277,21 @@ class Composer final {
   //
   // If the input is ascii characters, input mode will be set as HALF_ASCII.
   // This is useful to test the behavior of alphabet keyboard.
-  void SetPreeditTextForTestOnly(const std::string &input);
+  void SetPreeditTextForTestOnly(absl::string_view input);
 
-  bool InsertCharacterKeyAndPreedit(const std::string &key,
-                                    const std::string &preedit);
+  // Set compositions from handwriting recognition results.
+  // The composition may contain Kana-Kanji mixed string. (ex. "かん字")
+  // Handwriting engine can generate multiple candidates.
+  void SetCompositionsForHandwriting(
+      absl::Span<const commands::SessionCommand::CompositionEvent *const>
+          compositions);
+  absl::Span<const commands::SessionCommand::CompositionEvent>
+  GetHandwritingCompositions() const;
+
+  bool InsertCharacterKeyAndPreedit(absl::string_view key,
+                                    absl::string_view preedit);
   bool InsertCharacterKeyEvent(const commands::KeyEvent &key);
-  void InsertCommandCharacter(const InternalCommand internal_command);
+  void InsertCommandCharacter(InternalCommand internal_command);
   void Delete();
   void Backspace();
 
@@ -184,26 +304,23 @@ class Composer final {
 
   // Returns raw input from a user.
   // The main purpose is Transliteration.
-  void GetRawString(std::string *raw_string) const;
+  std::string GetRawString() const;
 
   // Returns substring of raw input.  The position and size is based on the
   // composed string.  For example, when [さ|sa][し|shi][み|mi] is the
   // composition, GetRawSubString(0, 2) returns "sashi".
-  void GetRawSubString(const size_t position, const size_t size,
-                       std::string *raw_sub_string) const;
+  std::string GetRawSubString(size_t position, size_t size) const;
 
   // Generate transliterations.
   void GetTransliterations(transliteration::Transliterations *t13ns) const;
 
   // Generate substrings of specified transliteration.
-  void GetSubTransliteration(const transliteration::TransliterationType type,
-                             const size_t position, const size_t size,
-                             std::string *transliteration) const;
+  std::string GetSubTransliteration(transliteration::TransliterationType type,
+                                    size_t position, size_t size) const;
 
   // Generate substrings of transliterations.
-  void GetSubTransliterations(
-      size_t position, size_t size,
-      transliteration::Transliterations *transliterations) const;
+  void GetSubTransliterations(size_t position, size_t size,
+                              transliteration::Transliterations *t13ns) const;
 
   // Check if the preedit is can be modified.
   bool EnableInsert() const;
@@ -212,7 +329,7 @@ class Composer final {
   // status and user's settings.
   void AutoSwitchMode();
 
-  // Return true if the composition is adviced to be committed immediately.
+  // Return true if the composition is advised to be committed immediately.
   bool ShouldCommit() const;
 
   // Returns true if characters at the head of the preedit should be committed
@@ -237,33 +354,34 @@ class Composer final {
   // Returns true when the current character at cursor position is toggleable.
   bool IsToggleable() const;
 
-  bool is_new_input() const;
-  size_t shifted_sequence_count() const;
-  const std::string &source_text() const;
-  std::string *mutable_source_text();
-  void set_source_text(const std::string &source_text);
-  size_t max_length() const;
-  void set_max_length(size_t length);
+  bool is_new_input() const { return is_new_input_; }
+  size_t shifted_sequence_count() const { return shifted_sequence_count_; }
+  absl::string_view source_text() const { return source_text_; }
+  std::string *mutable_source_text() { return &source_text_; }
+  void set_source_text(absl::string_view source_text);
+  size_t max_length() const { return max_length_; }
+  void set_max_length(size_t length) { max_length_ = length; }
 
-  int timeout_threshold_msec() const;
-  void set_timeout_threshold_msec(int threshold_msec);
+  int timeout_threshold_msec() const { return timeout_threshold_msec_; }
+  void set_timeout_threshold_msec(int threshold_msec) {
+    timeout_threshold_msec_ = threshold_msec;
+  }
 
  private:
   FRIEND_TEST(ComposerTest, ApplyTemporaryInputMode);
 
-  bool ProcessCompositionInput(const CompositionInput &input);
+  bool ProcessCompositionInput(CompositionInput input);
 
-  // Change input mode temporarily accoding to the current context and
+  // Change input mode temporarily according to the current context and
   // the given input character.
   // This function have a bug when key has characters input with Preedit.
   // Expected behavior: InsertPreedit("A") + InsertKey("a") -> "Aあ"
   // Actual behavior:   InsertPreedit("A") + InsertKey("a") -> "Aa"
-  void ApplyTemporaryInputMode(const std::string &input, bool caps_locked);
+  void ApplyTemporaryInputMode(absl::string_view input, bool caps_locked);
 
   // Generate transliterated substrings.
-  void GetTransliteratedText(Transliterators::Transliterator t12r,
-                             const size_t position, const size_t size,
-                             std::string *result) const;
+  std::string GetTransliteratedText(Transliterators::Transliterator t12r,
+                                    size_t position, size_t size) const;
 
   size_t position_;
   transliteration::TransliterationType input_mode_;
@@ -274,9 +392,6 @@ class Composer final {
   commands::Context::InputFieldType input_field_type_;
 
   size_t shifted_sequence_count_;
-  Composition composition_;
-
-  TypingCorrector typing_corrector_;
 
   // The original text for the composition.  The value is usually
   // empty, and used for reverse conversion.
@@ -284,8 +399,18 @@ class Composer final {
 
   size_t max_length_;
 
-  const commands::Request *request_;
-  const config::Config *config_;
+  std::shared_ptr<const commands::Request> request_;
+  std::shared_ptr<const config::Config> config_;
+
+  // Though we would like to avoid shared object, Table is not copyable so
+  // there is no other way to share them at this moment. The internal data
+  // of the Table is managed by std::unique_ptr, which makes simple copying
+  // impossible. Furthermore, copying the table every time would result in
+  // a significant performance degradation. Style guide says that we prefer
+  // to use std::shared_ptr for shared object.
+  std::shared_ptr<const Table> table_;
+
+  Composition composition_;
 
   // Timestamp of last modified.
   int64_t timestamp_msec_ = 0;
@@ -293,7 +418,7 @@ class Composer final {
   // If the duration between key inputs is more than timeout_threadhols_msec_,
   // the STOP_KEY_TOGGLING event is sent before the next key input.
   // If the value is 0, STOP_KEY_TOGGLING is not sent.
-  int timeout_threshold_msec_ = 0;
+  int timeout_threshold_msec_;
 
   // Whether the next insertion is the beginning of typing after an
   // editing command like SetInputMode or not.  Some conversion rules
@@ -301,6 +426,12 @@ class Composer final {
   // "abc<left-cursor>d", when "a" or "d" is typed, this value should
   // be true.  When "b" or "c" is typed, the value should be false.
   bool is_new_input_;
+
+  // Example:
+  //   {{"かん字", 0.99}, {"かlv字", 0.01}}
+  // Please refer to commands.proto
+  std::vector<commands::SessionCommand::CompositionEvent>
+      compositions_for_handwriting_;
 };
 
 }  // namespace composer
