@@ -35,6 +35,8 @@
 #include <string>
 #include <vector>
 
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "converter/connector.h"
 #include "converter/immutable_converter_interface.h"
 #include "converter/lattice.h"
@@ -45,41 +47,36 @@
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_group.h"
 #include "dictionary/pos_matcher.h"
-#include "dictionary/suppression_dictionary.h"
+#include "engine/modules.h"
 #include "prediction/suggestion_filter.h"
 #include "request/conversion_request.h"
-#include "testing/gunit_prod.h"  //  for FRIEND_TEST()
-#include "absl/base/attributes.h"
-#include "absl/types/span.h"
+#include "testing/friend_test.h"
 
 namespace mozc {
 
-class ImmutableConverterImpl : public ImmutableConverterInterface {
+class ImmutableConverter : public ImmutableConverterInterface {
  public:
-  ImmutableConverterImpl(
-      const dictionary::DictionaryInterface *dictionary,
-      const dictionary::DictionaryInterface *suffix_dictionary,
-      const dictionary::SuppressionDictionary *suppression_dictionary,
-      const Connector &connector, const Segmenter *segmenter,
-      const dictionary::PosMatcher *pos_matcher,
-      const dictionary::PosGroup *pos_group,
-      const SuggestionFilter &suggestion_filter);
-  ImmutableConverterImpl(const ImmutableConverterImpl &) = delete;
-  ImmutableConverterImpl &operator=(const ImmutableConverterImpl &) = delete;
-  ~ImmutableConverterImpl() override = default;
+  explicit ImmutableConverter(const engine::Modules &modules);
+  ImmutableConverter(const ImmutableConverter &) = delete;
+  ImmutableConverter &operator=(const ImmutableConverter &) = delete;
+  ~ImmutableConverter() override = default;
 
-  ABSL_MUST_USE_RESULT bool ConvertForRequest(
-      const ConversionRequest &request, Segments *segments) const override;
+  [[nodiscard]] bool ConvertForRequest(const ConversionRequest &request,
+                                       Segments *segments) const override;
 
  private:
   FRIEND_TEST(ImmutableConverterTest, AddPredictiveNodes);
   FRIEND_TEST(ImmutableConverterTest, DummyCandidatesCost);
   FRIEND_TEST(ImmutableConverterTest, DummyCandidatesInnerSegmentBoundary);
+  FRIEND_TEST(ImmutableConverterTest, MakeLatticeKatakana);
   FRIEND_TEST(ImmutableConverterTest, NotConnectedTest);
   FRIEND_TEST(ImmutableConverterTest, PredictiveNodesOnlyForConversionKey);
   FRIEND_TEST(NBestGeneratorTest, InnerSegmentBoundary);
   FRIEND_TEST(NBestGeneratorTest, MultiSegmentConnectionTest);
   FRIEND_TEST(NBestGeneratorTest, SingleSegmentConnectionTest);
+  FRIEND_TEST(NBestGeneratorTest, NoPartialCandidateBetweenAlphabets);
+  FRIEND_TEST(NBestGeneratorTest, NoAlphabetsConnection2Nodes);
+  FRIEND_TEST(NBestGeneratorTest, NoAlphabetsConnection3Nodes);
   friend class NBestGeneratorTest;
 
   enum InsertCandidatesType {
@@ -94,16 +91,16 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
   };
 
   void ExpandCandidates(const ConversionRequest &request,
-                        const std::string &original_key, NBestGenerator *nbest,
+                        absl::string_view original_key, NBestGenerator *nbest,
                         Segment *segment, size_t expand_size) const;
   void InsertDummyCandidates(Segment *segment, size_t expand_size) const;
-  Node *Lookup(int begin_pos, int end_pos, const ConversionRequest &request,
-               bool is_reverse, bool is_prediction, Lattice *lattice) const;
-  Node *AddCharacterTypeBasedNodes(const char *begin, const char *end,
+  Node *Lookup(int begin_pos, const ConversionRequest &request, bool is_reverse,
+               bool is_prediction, Lattice *lattice) const;
+  Node *AddCharacterTypeBasedNodes(absl::string_view key_substr,
                                    Lattice *lattice, Node *nodes) const;
 
-  void Resegment(const Segments &segments, const std::string &history_key,
-                 const std::string &conversion_key, Lattice *lattice) const;
+  void Resegment(const Segments &segments, absl::string_view history_key,
+                 absl::string_view conversion_key, Lattice *lattice) const;
 
   void ApplyResegmentRules(size_t pos, Lattice *lattice) const;
   // Returns true resegmentation happened
@@ -118,7 +115,7 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
                                           Lattice *lattice) const;
   void MakeLatticeNodesForConversionSegments(const Segments &segments,
                                              const ConversionRequest &request,
-                                             const std::string &history_key,
+                                             absl::string_view history_key,
                                              Lattice *lattice) const;
   void MakeLatticeNodesForPredictiveNodes(const Segments &segments,
                                           const ConversionRequest &request,
@@ -127,7 +124,7 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
   // If the last node ends with "prefix", give an extra
   // wcost penalty. In this case  "無" doesn't tend to appear at
   // user input.
-  void ApplyPrefixSuffixPenalty(const std::string &conversion_key,
+  void ApplyPrefixSuffixPenalty(absl::string_view conversion_key,
                                 Lattice *lattice) const;
 
   bool Viterbi(const Segments &segments, Lattice *lattice) const;
@@ -153,10 +150,9 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
                         size_t max_candidates_size,
                         InsertCandidatesType type) const;
 
-  void InsertCandidatesForRealtime(const ConversionRequest &request,
-                                   const Lattice &lattice,
-                                   absl::Span<const uint16_t> group,
-                                   Segments *segments) const;
+  void InsertCandidatesForRealtimeWithCandidateChecker(
+      const ConversionRequest &request, const Lattice &lattice,
+      absl::Span<const uint16_t> group, Segments *segments) const;
 
   // Helper function for InsertCandidates().
   // Returns true if |node| is valid node for segment end.
@@ -197,13 +193,13 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
                                      absl::Span<const uint16_t> group,
                                      Segments *segments) const;
 
-  const dictionary::DictionaryInterface *dictionary_;
-  const dictionary::DictionaryInterface *suffix_dictionary_;
-  const dictionary::SuppressionDictionary *suppression_dictionary_;
+  const dictionary::DictionaryInterface &dictionary_;
+  const dictionary::DictionaryInterface &suffix_dictionary_;
+  const dictionary::UserDictionaryInterface &user_dictionary_;
   const Connector &connector_;
-  const Segmenter *segmenter_;
-  const dictionary::PosMatcher *pos_matcher_;
-  const dictionary::PosGroup *pos_group_;
+  const Segmenter &segmenter_;
+  const dictionary::PosMatcher &pos_matcher_;
+  const dictionary::PosGroup &pos_group_;
   const SuggestionFilter &suggestion_filter_;
 
   // Cache for POS ids.
