@@ -29,56 +29,40 @@
 
 #include "converter/immutable_converter.h"
 
+#include <cstddef>
 #include <cstdint>
+#include <cstring>
 #include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "base/logging.h"
-#include "base/port.h"
-#include "base/system_util.h"
+#include "absl/log/check.h"
+#include "absl/strings/string_view.h"
 #include "base/util.h"
-#include "config/config_handler.h"
-#include "converter/connector.h"
 #include "converter/lattice.h"
-#include "converter/segmenter.h"
+#include "converter/node.h"
 #include "converter/segments.h"
-#include "data_manager/data_manager_interface.h"
+#include "converter/segments_matchers.h"
 #include "data_manager/testing/mock_data_manager.h"
-#include "dictionary/dictionary_impl.h"
 #include "dictionary/dictionary_interface.h"
-#include "dictionary/pos_group.h"
-#include "dictionary/suffix_dictionary.h"
-#include "dictionary/suppression_dictionary.h"
-#include "dictionary/system/system_dictionary.h"
-#include "dictionary/system/value_dictionary.h"
-#include "dictionary/user_dictionary_stub.h"
-#include "prediction/suggestion_filter.h"
+#include "engine/modules.h"
 #include "protocol/commands.pb.h"
 #include "request/conversion_request.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "absl/strings/match.h"
-#include "absl/strings/string_view.h"
+#include "request/request_test_util.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
 
 namespace mozc {
 namespace {
 
-using dictionary::DictionaryImpl;
 using dictionary::DictionaryInterface;
-using dictionary::PosGroup;
-using dictionary::SuffixDictionary;
-using dictionary::SuppressionDictionary;
-using dictionary::SystemDictionary;
-using dictionary::UserDictionaryStub;
-using dictionary::ValueDictionary;
+using ::testing::StrEq;
 
 void SetCandidate(absl::string_view key, absl::string_view value,
                   Segment *segment) {
   segment->set_key(key);
   Segment::Candidate *candidate = segment->add_candidate();
-  candidate->Init();
 #ifdef ABSL_USES_STD_STRING_VIEW
   candidate->key = key;
   candidate->value = value;
@@ -98,82 +82,31 @@ class MockDataAndImmutableConverter {
   // nullptr is passed, the default mock dictionary is used. This class owns the
   // first argument dictionary but doesn't the second because the same
   // dictionary may be passed to the arguments.
-  explicit MockDataAndImmutableConverter(
-      const DictionaryInterface *dictionary = nullptr,
-      const DictionaryInterface *suffix_dictionary = nullptr) {
-    data_manager_ = std::make_unique<testing::MockDataManager>();
-
-    pos_matcher_.Set(data_manager_->GetPosMatcherData());
-
-    suppression_dictionary_ = std::make_unique<SuppressionDictionary>();
-    CHECK(suppression_dictionary_.get());
-
-    if (dictionary) {
-      dictionary_.reset(dictionary);
-    } else {
-      const char *dictionary_data = nullptr;
-      int dictionary_size = 0;
-      data_manager_->GetSystemDictionaryData(&dictionary_data,
-                                             &dictionary_size);
-      std::unique_ptr<SystemDictionary> sysdic =
-          SystemDictionary::Builder(dictionary_data, dictionary_size)
-              .Build()
-              .value();
-      auto value_dic = std::make_unique<ValueDictionary>(pos_matcher_,
-                                                         &sysdic->value_trie());
-      dictionary_ = std::make_unique<DictionaryImpl>(
-          std::move(sysdic), std::move(value_dic), &user_dictionary_stub_,
-          suppression_dictionary_.get(), &pos_matcher_);
-    }
-    CHECK(dictionary_.get());
-
-    if (!suffix_dictionary) {
-      absl::string_view suffix_key_array_data, suffix_value_array_data;
-      const uint32_t *token_array;
-      data_manager_->GetSuffixDictionaryData(
-          &suffix_key_array_data, &suffix_value_array_data, &token_array);
-      suffix_dictionary_ = std::make_unique<SuffixDictionary>(
-          suffix_key_array_data, suffix_value_array_data, token_array);
-      suffix_dictionary = suffix_dictionary_.get();
-    }
-    CHECK(suffix_dictionary);
-
-    connector_ = Connector::CreateFromDataManager(*data_manager_).value();
-
-    segmenter_.reset(Segmenter::CreateFromDataManager(*data_manager_));
-    CHECK(segmenter_.get());
-
-    pos_group_ = std::make_unique<PosGroup>(data_manager_->GetPosGroupData());
-    CHECK(pos_group_.get());
-
-    {
-      const char *data = nullptr;
-      size_t size = 0;
-      data_manager_->GetSuggestionFilterData(&data, &size);
-      suggestion_filter_ = std::make_unique<SuggestionFilter>(data, size);
-    }
-
-    immutable_converter_ = std::make_unique<ImmutableConverterImpl>(
-        dictionary_.get(), suffix_dictionary, suppression_dictionary_.get(),
-        connector_.get(), segmenter_.get(), &pos_matcher_, pos_group_.get(),
-        suggestion_filter_.get());
-    CHECK(immutable_converter_.get());
+  MockDataAndImmutableConverter() {
+    modules_ =
+        engine::Modules::Create(std::make_unique<testing::MockDataManager>())
+            .value();
+    immutable_converter_ = std::make_unique<ImmutableConverter>(*modules_);
+    CHECK(immutable_converter_);
   }
 
-  ImmutableConverterImpl *GetConverter() { return immutable_converter_.get(); }
+  MockDataAndImmutableConverter(
+      std::unique_ptr<DictionaryInterface> dictionary,
+      std::unique_ptr<DictionaryInterface> suffix_dictionary) {
+    modules_ = engine::ModulesPresetBuilder()
+                   .PresetDictionary(std::move(dictionary))
+                   .PresetSuffixDictionary(std::move(suffix_dictionary))
+                   .Build(std::make_unique<testing::MockDataManager>())
+                   .value();
+    immutable_converter_ = std::make_unique<ImmutableConverter>(*modules_);
+    CHECK(immutable_converter_);
+  }
+
+  ImmutableConverter *GetConverter() { return immutable_converter_.get(); }
 
  private:
-  std::unique_ptr<const DataManagerInterface> data_manager_;
-  std::unique_ptr<const SuppressionDictionary> suppression_dictionary_;
-  std::unique_ptr<const Connector> connector_;
-  std::unique_ptr<const Segmenter> segmenter_;
-  std::unique_ptr<const DictionaryInterface> suffix_dictionary_;
-  std::unique_ptr<const DictionaryInterface> dictionary_;
-  std::unique_ptr<const PosGroup> pos_group_;
-  std::unique_ptr<const SuggestionFilter> suggestion_filter_;
-  std::unique_ptr<ImmutableConverterImpl> immutable_converter_;
-  UserDictionaryStub user_dictionary_stub_;
-  dictionary::PosMatcher pos_matcher_;
+  std::unique_ptr<engine::Modules> modules_;
+  std::unique_ptr<ImmutableConverter> immutable_converter_;
 };
 
 }  // namespace
@@ -182,17 +115,54 @@ TEST(ImmutableConverterTest, KeepKeyForPrediction) {
   std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
       new MockDataAndImmutableConverter);
   Segments segments;
-  ConversionRequest request;
-  request.set_request_type(ConversionRequest::PREDICTION);
-  request.set_max_conversion_candidates_size(10);
+  const ConversionRequest request =
+      ConversionRequestBuilder()
+          .SetOptions({.request_type = ConversionRequest::PREDICTION,
+                       .max_conversion_candidates_size = 10})
+          .Build();
   Segment *segment = segments.add_segment();
   const std::string kRequestKey = "よろしくおねがいしま";
   segment->set_key(kRequestKey);
   EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(request,
                                                                     &segments));
-  EXPECT_EQ(1, segments.segments_size());
+  EXPECT_EQ(segments.segments_size(), 1);
   EXPECT_GT(segments.segment(0).candidates_size(), 0);
-  EXPECT_EQ(kRequestKey, segments.segment(0).key());
+  EXPECT_EQ(segments.segment(0).key(), kRequestKey);
+}
+
+TEST(ImmutableConverterTest, ResegmentTest) {
+  std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
+      new MockDataAndImmutableConverter);
+  Segments segments;
+  const ConversionRequest request =
+      ConversionRequestBuilder()
+          .SetOptions({.request_type = ConversionRequest::CONVERSION,
+                       .max_conversion_candidates_size = 10})
+          .Build();
+
+  {
+    segments.Clear();
+    Segment *segment = segments.add_segment();
+    const std::string kRequestKey = "1ねんせい";
+    segment->set_key(kRequestKey);
+    EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
+        request, &segments));
+    EXPECT_EQ(segments.segments_size(), 2);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "1");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "年生");
+  }
+
+  {
+    segments.Clear();
+    Segment *segment = segments.add_segment();
+    const std::string kRequestKey = "ちゅう2";
+    segment->set_key(kRequestKey);
+    EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
+        request, &segments));
+    EXPECT_EQ(segments.segments_size(), 2);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "中");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "2");
+  }
 }
 
 TEST(ImmutableConverterTest, DummyCandidatesCost) {
@@ -273,27 +243,31 @@ TEST(ImmutableConverterTest, PredictiveNodesOnlyForConversionKey) {
     segment->set_key("いいんじゃな");
     segment->set_segment_type(Segment::HISTORY);
     Segment::Candidate *candidate = segment->add_candidate();
-    candidate->Init();
     candidate->key = "いいんじゃな";
     candidate->value = "いいんじゃな";
 
     segment = segments.add_segment();
     segment->set_key("いか");
 
-    EXPECT_EQ(1, segments.history_segments_size());
-    EXPECT_EQ(1, segments.conversion_segments_size());
+    EXPECT_EQ(segments.history_segments_size(), 1);
+    EXPECT_EQ(segments.conversion_segments_size(), 1);
   }
 
   Lattice lattice;
   lattice.SetKey("いいんじゃないか");
 
-  KeyCheckDictionary *dictionary = new KeyCheckDictionary("ないか");
-  std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
-      new MockDataAndImmutableConverter(dictionary, dictionary));
-  ImmutableConverterImpl *converter = data_and_converter->GetConverter();
+  auto dictionary = std::make_unique<KeyCheckDictionary>("ないか");
+  KeyCheckDictionary *dictionary_ptr = dictionary.get();
+  auto suffix_dictionary = std::make_unique<KeyCheckDictionary>("ないか");
+  KeyCheckDictionary *suffix_dictionary_ptr = dictionary.get();
+
+  auto data_and_converter = std::make_unique<MockDataAndImmutableConverter>(
+      std::move(dictionary), std::move(suffix_dictionary));
+  ImmutableConverter *converter = data_and_converter->GetConverter();
   const ConversionRequest request;
   converter->MakeLatticeNodesForPredictiveNodes(segments, request, &lattice);
-  EXPECT_FALSE(dictionary->received_target_query());
+  EXPECT_FALSE(dictionary_ptr->received_target_query());
+  EXPECT_FALSE(suffix_dictionary_ptr->received_target_query());
 }
 
 TEST(ImmutableConverterTest, AddPredictiveNodes) {
@@ -302,22 +276,29 @@ TEST(ImmutableConverterTest, AddPredictiveNodes) {
     Segment *segment = segments.add_segment();
     segment->set_key("よろしくおねがいしま");
 
-    EXPECT_EQ(1, segments.conversion_segments_size());
+    EXPECT_EQ(segments.conversion_segments_size(), 1);
   }
 
   Lattice lattice;
   lattice.SetKey("よろしくおねがいしま");
 
-  KeyCheckDictionary *dictionary = new KeyCheckDictionary("しま");
-  std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
-      new MockDataAndImmutableConverter(dictionary, dictionary));
-  ImmutableConverterImpl *converter = data_and_converter->GetConverter();
+  auto dictionary = std::make_unique<KeyCheckDictionary>("しま");
+  KeyCheckDictionary *dictionary_ptr = dictionary.get();
+  auto suffix_dictionary = std::make_unique<KeyCheckDictionary>("しま");
+  KeyCheckDictionary *suffix_dictionary_ptr = suffix_dictionary.get();
+
+  auto data_and_converter = std::make_unique<MockDataAndImmutableConverter>(
+      std::move(dictionary), std::move(suffix_dictionary));
+  ImmutableConverter *converter = data_and_converter->GetConverter();
 
   {
-    ConversionRequest request;
-    request.set_request_type(ConversionRequest::CONVERSION);
+    const ConversionRequest request =
+        ConversionRequestBuilder()
+            .SetOptions({.request_type = ConversionRequest::CONVERSION})
+            .Build();
     converter->MakeLatticeNodesForPredictiveNodes(segments, request, &lattice);
-    EXPECT_TRUE(dictionary->received_target_query());
+    EXPECT_FALSE(dictionary_ptr->received_target_query());
+    EXPECT_TRUE(suffix_dictionary_ptr->received_target_query());
   }
 }
 
@@ -328,9 +309,11 @@ TEST(ImmutableConverterTest, InnerSegmenBoundaryForPrediction) {
   Segment *segment = segments.add_segment();
   const std::string kRequestKey = "わたしのなまえはなかのです";
   segment->set_key(kRequestKey);
-  ConversionRequest request;
-  request.set_request_type(ConversionRequest::PREDICTION);
-  request.set_max_conversion_candidates_size(1);
+  const ConversionRequest request =
+      ConversionRequestBuilder()
+          .SetOptions({.request_type = ConversionRequest::PREDICTION,
+                       .max_conversion_candidates_size = 1})
+          .Build();
   EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(request,
                                                                     &segments));
   ASSERT_EQ(1, segments.segments_size());
@@ -338,6 +321,7 @@ TEST(ImmutableConverterTest, InnerSegmenBoundaryForPrediction) {
 
   // Result will be, "私の|名前は|中ノです" with mock dictionary.
   const Segment::Candidate &cand = segments.segment(0).candidate(0);
+  EXPECT_TRUE(cand.IsValid());
   std::vector<absl::string_view> keys, values, content_keys, content_values;
   for (Segment::Candidate::InnerSegmentIterator iter(&cand); !iter.Done();
        iter.Next()) {
@@ -346,25 +330,25 @@ TEST(ImmutableConverterTest, InnerSegmenBoundaryForPrediction) {
     content_keys.push_back(iter.GetContentKey());
     content_values.push_back(iter.GetContentValue());
   }
-  ASSERT_EQ(3, keys.size());
-  EXPECT_EQ("わたしの", keys[0]);
-  EXPECT_EQ("なまえは", keys[1]);
-  EXPECT_EQ("なかのです", keys[2]);
+  ASSERT_EQ(keys.size(), 3);
+  EXPECT_EQ(keys[0], "わたしの");
+  EXPECT_EQ(keys[1], "なまえは");
+  EXPECT_EQ(keys[2], "なかのです");
 
-  ASSERT_EQ(3, values.size());
-  EXPECT_EQ("私の", values[0]);
-  EXPECT_EQ("名前は", values[1]);
-  EXPECT_EQ("中ノです", values[2]);
+  ASSERT_EQ(values.size(), 3);
+  EXPECT_EQ(values[0], "私の");
+  EXPECT_EQ(values[1], "名前は");
+  EXPECT_EQ(values[2], "中ノです");
 
   ASSERT_EQ(3, content_keys.size());
-  EXPECT_EQ("わたし", content_keys[0]);
-  EXPECT_EQ("なまえ", content_keys[1]);
-  EXPECT_EQ("なかの", content_keys[2]);
+  EXPECT_EQ(content_keys[0], "わたし");
+  EXPECT_EQ(content_keys[1], "なまえ");
+  EXPECT_EQ(content_keys[2], "なかの");
 
-  ASSERT_EQ(3, content_values.size());
-  EXPECT_EQ("私", content_values[0]);
-  EXPECT_EQ("名前", content_values[1]);
-  EXPECT_EQ("中ノ", content_values[2]);
+  ASSERT_EQ(content_values.size(), 3);
+  EXPECT_EQ(content_values[0], "私");
+  EXPECT_EQ(content_values[1], "名前");
+  EXPECT_EQ(content_values[2], "中ノ");
 }
 
 TEST(ImmutableConverterTest, NoInnerSegmenBoundaryForConversion) {
@@ -374,7 +358,12 @@ TEST(ImmutableConverterTest, NoInnerSegmenBoundaryForConversion) {
   Segment *segment = segments.add_segment();
   const std::string kRequestKey = "わたしのなまえはなかのです";
   segment->set_key(kRequestKey);
-  EXPECT_TRUE(data_and_converter->GetConverter()->Convert(&segments));
+  const ConversionRequest request =
+      ConversionRequestBuilder()
+          .SetRequestType(ConversionRequest::CONVERSION)
+          .Build();
+  EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(request,
+                                                                    &segments));
   EXPECT_LE(1, segments.segments_size());
   EXPECT_LT(0, segments.segment(0).candidates_size());
   for (size_t i = 0; i < segments.segment(0).candidates_size(); ++i) {
@@ -383,10 +372,33 @@ TEST(ImmutableConverterTest, NoInnerSegmenBoundaryForConversion) {
   }
 }
 
+TEST(ImmutableConverterTest, MakeLatticeKatakana) {
+  std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
+      new MockDataAndImmutableConverter);
+  ImmutableConverter *converter = data_and_converter->GetConverter();
+
+  Segments segments;
+
+  Segment *segment = segments.add_segment();
+  segment->set_segment_type(Segment::FREE);
+  segment->set_key("カタカナです");
+
+  Lattice lattice;
+  lattice.SetKey("カタカナです");
+  const ConversionRequest request;
+  converter->MakeLattice(request, &segments, &lattice);
+
+  // If the first character of a node is `ALPHABET` or `KATAKANA`,
+  // `AddCharacterTypeBasedNodes` should create a node of the character type.
+  Node *node = lattice.begin_nodes(0);
+  EXPECT_EQ(node->key, "カタカナ");
+  EXPECT_EQ(node->value, "カタカナ");
+}
+
 TEST(ImmutableConverterTest, NotConnectedTest) {
   std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
       new MockDataAndImmutableConverter);
-  ImmutableConverterImpl *converter = data_and_converter->GetConverter();
+  ImmutableConverter *converter = data_and_converter->GetConverter();
 
   Segments segments;
 
@@ -437,7 +449,6 @@ TEST(ImmutableConverterTest, HistoryKeyLengthIsVeryLong) {
     segment->set_key(kA100);
     segment->set_segment_type(Segment::HISTORY);
     Segment::Candidate *candidate = segment->add_candidate();
-    candidate->Init();
     candidate->key = kA100;
     candidate->value = kA100;
   }
@@ -451,11 +462,16 @@ TEST(ImmutableConverterTest, HistoryKeyLengthIsVeryLong) {
   // least one candidate is generated.
   std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
       new MockDataAndImmutableConverter);
-  EXPECT_TRUE(data_and_converter->GetConverter()->Convert(&segments));
-  EXPECT_EQ(0, segments.history_segments_size());
-  ASSERT_EQ(1, segments.conversion_segments_size());
+  const ConversionRequest request =
+      ConversionRequestBuilder()
+          .SetRequestType(ConversionRequest::CONVERSION)
+          .Build();
+  EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(request,
+                                                                    &segments));
+  EXPECT_EQ(segments.history_segments_size(), 0);
+  ASSERT_EQ(segments.conversion_segments_size(), 1);
   EXPECT_GT(segments.segment(0).candidates_size(), 0);
-  EXPECT_EQ(kRequestKey, segments.segment(0).key());
+  EXPECT_EQ(segments.segment(0).key(), kRequestKey);
 }
 
 namespace {
@@ -463,22 +479,27 @@ bool AutoPartialSuggestionTestHelper(const ConversionRequest &request) {
   std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
       new MockDataAndImmutableConverter);
   Segments segments;
-  ConversionRequest conversion_request = request;
-  conversion_request.set_request_type(ConversionRequest::PREDICTION);
-  conversion_request.set_max_conversion_candidates_size(10);
+  ConversionRequest::Options options = request.options();
+  options.request_type = ConversionRequest::PREDICTION;
+  options.max_conversion_candidates_size = 10;
+  const ConversionRequest conversion_request =
+      ConversionRequestBuilder()
+          .SetConversionRequest(request)
+          .SetOptions(std::move(options))
+          .Build();
   Segment *segment = segments.add_segment();
   const std::string kRequestKey = "わたしのなまえはなかのです";
   segment->set_key(kRequestKey);
   EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
       conversion_request, &segments));
-  EXPECT_EQ(1, segments.conversion_segments_size());
+  EXPECT_EQ(segments.conversion_segments_size(), 1);
   EXPECT_LT(0, segments.segment(0).candidates_size());
   bool includes_only_first = false;
-  const std::string &segment_key = segments.segment(0).key();
+  absl::string_view segment_key = segments.segment(0).key();
   for (size_t i = 0; i < segments.segment(0).candidates_size(); ++i) {
     const Segment::Candidate &cand = segments.segment(0).candidate(i);
     if (cand.key.size() < segment_key.size() &&
-        absl::StartsWith(segment_key, cand.key)) {
+        segment_key.starts_with(cand.key)) {
       includes_only_first = true;
       break;
     }
@@ -488,63 +509,121 @@ bool AutoPartialSuggestionTestHelper(const ConversionRequest &request) {
 }  // namespace
 
 TEST(ImmutableConverterTest, EnableAutoPartialSuggestion) {
-  const commands::Request request;
-  ConversionRequest conversion_request;
-  conversion_request.set_request(&request);
-  conversion_request.set_create_partial_candidates(true);
-
+  const ConversionRequest conversion_request =
+      ConversionRequestBuilder()
+          .SetOptions({.create_partial_candidates = true})
+          .Build();
+  EXPECT_TRUE(conversion_request.create_partial_candidates());
   EXPECT_TRUE(AutoPartialSuggestionTestHelper(conversion_request));
 }
 
 TEST(ImmutableConverterTest, DisableAutoPartialSuggestion) {
-  const commands::Request request;
-  ConversionRequest conversion_request;
-  conversion_request.set_request(&request);
-  conversion_request.set_create_partial_candidates(false);
-
+  const ConversionRequest conversion_request =
+      ConversionRequestBuilder()
+          .SetOptions({.create_partial_candidates = false})
+          .Build();
   EXPECT_FALSE(AutoPartialSuggestionTestHelper(conversion_request));
 }
 
 TEST(ImmutableConverterTest, AutoPartialSuggestionDefault) {
-  const commands::Request request;
-  ConversionRequest conversion_request;
-  conversion_request.set_request(&request);
-
+  const ConversionRequest conversion_request;
   EXPECT_FALSE(AutoPartialSuggestionTestHelper(conversion_request));
 }
 
-TEST(ImmutableConverterTest, AutoPartialSuggestionForSingleSegment) {
-  const commands::Request request;
-  ConversionRequest conversion_request;
-  conversion_request.set_request_type(ConversionRequest::PREDICTION);
-  conversion_request.set_request(&request);
-  conversion_request.set_create_partial_candidates(true);
-  conversion_request.set_max_conversion_candidates_size(10);
+TEST(ImmutableConverterTest, FirstInnerSegment) {
+  commands::Request request;
+  request_test_util::FillMobileRequest(&request);
+  const ConversionRequest conversion_request =
+      ConversionRequestBuilder()
+          .SetRequest(request)
+          .SetOptions({
+              .request_type = ConversionRequest::PREDICTION,
+              .max_conversion_candidates_size = 100,
+              .create_partial_candidates = true,
+          })
+          .Build();
 
-  std::unique_ptr<MockDataAndImmutableConverter> data_and_converter(
-      new MockDataAndImmutableConverter);
-  const std::string kRequestKeys[] = {
-      "たかまち",
-      "なのは",
-      "まほうしょうじょ",
+  auto data_and_converter = std::make_unique<MockDataAndImmutableConverter>();
+
+  Segments segments;
+  Segment *segment = segments.add_segment();
+  segment->set_key("くるまでこうどうした");
+  EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
+      conversion_request, &segments));
+
+  constexpr auto KeyIs = [](const auto &key) {
+    return Field(&Segment::Candidate::key, StrEq(key));
   };
-  for (size_t testcase = 0; testcase < std::size(kRequestKeys); ++testcase) {
+
+  EXPECT_THAT(*segment, ContainsCandidate(KeyIs("くるまでこうどうした")));
+  EXPECT_THAT(*segment, ContainsCandidate(KeyIs("くるまで")));
+  EXPECT_THAT(*segment, ContainsCandidate(KeyIs("くる")));
+}
+
+TEST(ImmutableConverterTest, FirstInnerSegmentFiltering) {
+  commands::Request request;
+  request_test_util::FillMobileRequest(&request);
+  const ConversionRequest conversion_request =
+      ConversionRequestBuilder()
+          .SetRequest(request)
+          .SetOptions({
+              .request_type = ConversionRequest::PREDICTION,
+              .max_conversion_candidates_size = 100,
+              .create_partial_candidates = true,
+          })
+          .Build();
+
+  auto data_and_converter = std::make_unique<MockDataAndImmutableConverter>();
+  constexpr auto ValueIs = [](const auto &value) {
+    return Field(&Segment::Candidate::value, StrEq(value));
+  };
+
+  {
     Segments segments;
     Segment *segment = segments.add_segment();
-    segment->set_key(kRequestKeys[testcase]);
+    segment->set_key("したとき");
     EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
         conversion_request, &segments));
-    EXPECT_EQ(1, segments.conversion_segments_size());
-    EXPECT_LT(0, segments.segment(0).candidates_size());
-    const std::string &segment_key = segments.segment(0).key();
-    for (size_t i = 0; i < segments.segment(0).candidates_size(); ++i) {
-      const Segment::Candidate &cand = segments.segment(0).candidate(i);
-      if (cand.attributes & Segment::Candidate::PARTIALLY_KEY_CONSUMED) {
-        EXPECT_LT(cand.key.size(), segment_key.size()) << cand.DebugString();
-      } else {
-        EXPECT_GE(cand.key.size(), segment_key.size()) << cand.DebugString();
-      }
-    }
+
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("した時")));
+    // The same segment structure, but included by char coverage rule.
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("したとき")));
+  }
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+    segment->set_key("のとき");
+    EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
+        conversion_request, &segments));
+
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("の時")));
+    // The same segment structure, included by char coverage.
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("のとき")));
+  }
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+    segment->set_key("かえる");
+    EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
+        conversion_request, &segments));
+
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("換える")));
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("代える")));
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("買える")));
+    // Included by cost diff
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("飼える")));
+  }
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+    segment->set_key("くるまでこうどうした");
+    EXPECT_TRUE(data_and_converter->GetConverter()->ConvertForRequest(
+        conversion_request, &segments));
+
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("車で行動した")));
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("車で")));
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("来るまで")));
+    EXPECT_THAT(*segment, ContainsCandidate(ValueIs("くるまで")));
   }
 }
 
