@@ -30,274 +30,79 @@
 #include "converter/segments.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
-#include <limits>
+#include <deque>
+#include <iterator>
+#include <memory>
+#include <numeric>
+#include <ostream>
 #include <sstream>  // For DebugString()
 #include <string>
 #include <utility>
+#include <vector>
 
-#include "base/logging.h"
-#include "base/util.h"
-#include "absl/strings/str_format.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
+#include "absl/types/span.h"
+#include "base/util.h"
+#include "base/vlog.h"
+#include "converter/candidate.h"
 
 namespace mozc {
+
+using ::mozc::converter::Candidate;
+
 namespace {
 constexpr size_t kMaxHistorySize = 32;
 }  // namespace
 
-absl::string_view Segment::Candidate::functional_key() const {
-  return key.size() <= content_key.size()
-             ? absl::string_view()
-             : absl::string_view(key.data() + content_key.size(),
-                                 key.size() - content_key.size());
-}
-
-absl::string_view Segment::Candidate::functional_value() const {
-  return value.size() <= content_value.size()
-             ? absl::string_view()
-             : absl::string_view(value.data() + content_value.size(),
-                                 value.size() - content_value.size());
-}
-
-void Segment::Candidate::Init() {
-  key.clear();
-  value.clear();
-  content_value.clear();
-  content_key.clear();
-  consumed_key_size = 0;
-  prefix.clear();
-  suffix.clear();
-  description.clear();
-  usage_title.clear();
-  usage_description.clear();
-  cost = 0;
-  structure_cost = 0;
-  wcost = 0;
-  lid = 0;
-  rid = 0;
-  usage_id = 0;
-  attributes = 0;
-  source_info = SOURCE_INFO_NONE;
-  style = NumberUtil::NumberString::DEFAULT_STYLE;
-  command = DEFAULT_COMMAND;
-  inner_segment_boundary.clear();
-#ifndef NDEBUG
-  log.clear();
-#endif  // NDEBUG
-}
-
-bool Segment::Candidate::IsValid() const {
-  if (inner_segment_boundary.empty()) {
-    return true;
-  }
-  // The sums of the lengths of key, value, content key and content value
-  // components must coincide with those of key, value, content_key and
-  // content_value, respectively.
-  size_t sum_key_len = 0, sum_value_len = 0;
-  size_t sum_content_key_len = 0, sum_content_value_len = 0;
-  for (InnerSegmentIterator iter(this); !iter.Done(); iter.Next()) {
-    sum_key_len += iter.GetKey().size();
-    sum_value_len += iter.GetValue().size();
-    sum_content_key_len += iter.GetContentKey().size();
-    sum_content_value_len += iter.GetContentValue().size();
-  }
-  if (sum_key_len != key.size() || sum_value_len != value.size() ||
-      sum_content_key_len != content_key.size() ||
-      sum_content_value_len != content_value.size()) {
-    return false;
-  }
-  return true;
-}
-
-bool Segment::Candidate::EncodeLengths(size_t key_len, size_t value_len,
-                                       size_t content_key_len,
-                                       size_t content_value_len,
-                                       uint32_t *result) {
-  if (key_len > std::numeric_limits<uint8_t>::max() ||
-      value_len > std::numeric_limits<uint8_t>::max() ||
-      content_key_len > std::numeric_limits<uint8_t>::max() ||
-      content_value_len > std::numeric_limits<uint8_t>::max()) {
-    return false;
-  }
-  *result = (static_cast<uint32_t>(key_len) << 24) |
-            (static_cast<uint32_t>(value_len) << 16) |
-            (static_cast<uint32_t>(content_key_len) << 8) |
-            static_cast<uint32_t>(content_value_len);
-  return true;
-}
-
-bool Segment::Candidate::PushBackInnerSegmentBoundary(
-    size_t key_len, size_t value_len, size_t content_key_len,
-    size_t content_value_len) {
-  uint32_t encoded;
-  if (EncodeLengths(key_len, value_len, content_key_len, content_value_len,
-                    &encoded)) {
-    inner_segment_boundary.push_back(encoded);
-    return true;
-  }
-  return false;
-}
-
-std::string Segment::Candidate::DebugString() const {
-  std::stringstream os;
-  os << "(key=" << key << " ckey=" << content_key << " val=" << value
-     << " cval=" << content_value << " cost=" << cost
-     << " scost=" << structure_cost << " wcost=" << wcost << " lid=" << lid
-     << " rid=" << rid << " attributes=" << attributes
-     << " consumed_key_size=" << consumed_key_size;
-  if (!prefix.empty()) {
-    os << " prefix=" << prefix;
-  }
-  if (!suffix.empty()) {
-    os << " suffix=" << suffix;
-  }
-  if (!description.empty()) {
-    os << " description=" << description;
-  }
-  if (!inner_segment_boundary.empty()) {
-    os << " segbdd=";
-    for (size_t i = 0; i < inner_segment_boundary.size(); ++i) {
-      const uint32_t encoded_lengths = inner_segment_boundary[i];
-      const int key_len = encoded_lengths >> 24;
-      const int value_len = (encoded_lengths >> 16) & 0xff;
-      const int content_key_len = (encoded_lengths >> 8) & 0xff;
-      const int content_value_len = encoded_lengths & 0xff;
-      os << absl::StrFormat("<%d,%d,%d,%d>", key_len, value_len,
-                            content_key_len, content_value_len);
-    }
-  }
-  os << ")" << std::endl;
-  return os.str();
-}
-
-void Segment::Candidate::InnerSegmentIterator::Next() {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_++];
-  key_offset_ += encoded_lengths >> 24;
-  value_offset_ += (encoded_lengths >> 16) & 0xff;
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetKey() const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(key_offset_, encoded_lengths >> 24);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetValue() const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(value_offset_, (encoded_lengths >> 16) & 0xff);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetContentKey()
-    const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(key_offset_, (encoded_lengths >> 8) & 0xff);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetContentValue()
-    const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(value_offset_, encoded_lengths & 0xff);
-}
-
-constexpr int kCandidatesPoolSize = 16;
-Segment::Segment() : segment_type_(FREE), pool_(kCandidatesPoolSize) {}
-
 Segment::Segment(const Segment &x)
     : removed_candidates_for_debug_(x.removed_candidates_for_debug_),
       segment_type_(x.segment_type_),
       key_(x.key_),
-      meta_candidates_(x.meta_candidates_),
-      pool_(x.pool_.size()) {
-  // Deep-copy candidates.
-  for (const Candidate *cand : x.candidates_) {
-    *add_candidate() = *cand;
-  }
+      key_len_(x.key_len_),
+      meta_candidates_(x.meta_candidates_) {
+  DeepCopyCandidates(x.candidates_);
 }
 
 Segment &Segment::operator=(const Segment &x) {
   removed_candidates_for_debug_ = x.removed_candidates_for_debug_;
   segment_type_ = x.segment_type_;
   key_ = x.key_;
+  key_len_ = x.key_len_;
   meta_candidates_ = x.meta_candidates_;
-  // Deep-copy candidates.
+
   clear_candidates();
-  for (const Candidate *cand : x.candidates_) {
-    *add_candidate() = *cand;
-  }
+  DeepCopyCandidates(x.candidates_);
+
   return *this;
 }
 
-Segment::~Segment() = default;
-
-Segment::SegmentType Segment::segment_type() const { return segment_type_; }
-
-void Segment::set_segment_type(const Segment::SegmentType &segment_type) {
-  segment_type_ = segment_type;
-}
-
-const std::string &Segment::key() const { return key_; }
-
-void Segment::set_key(absl::string_view key) {
-  key_.assign(key.data(), key.size());
-}
-
-bool Segment::is_valid_index(int i) const {
-  if (i < 0) {
-    return (-i - 1 < meta_candidates_.size());
-  } else {
-    return (i < candidates_.size());
-  }
-}
-
-const Segment::Candidate &Segment::candidate(int i) const {
-  if (i < 0) {
-    return meta_candidate(-i - 1);
-  }
-  DCHECK(i < candidates_.size());
-  return *candidates_[i];
-}
-
-Segment::Candidate *Segment::mutable_candidate(int i) {
-  if (i < 0) {
-    const size_t meta_index = -i - 1;
-    DCHECK_LT(meta_index, meta_candidates_.size());
-    return &meta_candidates_[meta_index];
-  }
-  DCHECK_LT(i, candidates_.size());
-  return candidates_[i];
-}
-
-size_t Segment::candidates_size() const { return candidates_.size(); }
-
 void Segment::clear_candidates() {
   pool_.clear();
   candidates_.clear();
 }
 
-Segment::Candidate *Segment::push_back_candidate() {
-  Candidate *candidate =
-      pool_.emplace_back(std::make_unique<Candidate>()).get();
-  candidate->Init();
-  candidates_.push_back(candidate);
-  return candidate;
+Candidate *Segment::push_back_candidate() {
+  auto candidate = std::make_unique<Candidate>();
+  Candidate *ptr = candidate.get();
+  pool_.push_back(std::move(candidate));
+  candidates_.push_back(ptr);
+  return ptr;
 }
 
-Segment::Candidate *Segment::push_front_candidate() {
-  Candidate *candidate =
-      pool_.emplace_back(std::make_unique<Candidate>()).get();
-  candidate->Init();
-  candidates_.push_front(candidate);
-  return candidate;
+Candidate *Segment::push_front_candidate() {
+  auto candidate = std::make_unique<Candidate>();
+  Candidate *ptr = candidate.get();
+  pool_.push_back(std::move(candidate));
+  candidates_.push_front(ptr);
+  return ptr;
 }
 
-Segment::Candidate *Segment::add_candidate() { return push_back_candidate(); }
-
-Segment::Candidate *Segment::insert_candidate(int i) {
+Candidate *Segment::insert_candidate(int i) {
   if (i < 0) {
     LOG(WARNING) << "Invalid insert position [negative]: " << i << " / "
                  << candidates_.size();
@@ -310,7 +115,6 @@ Segment::Candidate *Segment::insert_candidate(int i) {
   }
   Candidate *candidate =
       pool_.emplace_back(std::make_unique<Candidate>()).get();
-  candidate->Init();
   candidates_.insert(candidates_.begin() + i, candidate);
   return candidate;
 }
@@ -327,7 +131,7 @@ void Segment::insert_candidate(int i, std::unique_ptr<Candidate> candidate) {
 }
 
 void Segment::insert_candidates(
-    int i, std::vector<std::unique_ptr<Candidate>> &&candidates) {
+    int i, std::vector<std::unique_ptr<Candidate>> candidates) {
   if (i < 0) {
     i = 0;
   } else if (i > static_cast<int>(candidates_.size())) {
@@ -348,12 +152,14 @@ void Segment::insert_candidates(
 
 void Segment::pop_front_candidate() {
   if (!candidates_.empty()) {
+    // The unique_ptr in pool_ is deleted when the candidate is deleted.
     candidates_.pop_front();
   }
 }
 
 void Segment::pop_back_candidate() {
   if (!candidates_.empty()) {
+    // The unique_ptr in pool_ is deleted when the candidate is deleted.
     candidates_.pop_back();
   }
 }
@@ -376,19 +182,7 @@ void Segment::erase_candidates(int i, size_t size) {
   candidates_.erase(candidates_.begin() + i, candidates_.begin() + end);
 }
 
-size_t Segment::meta_candidates_size() const { return meta_candidates_.size(); }
-
-void Segment::clear_meta_candidates() { meta_candidates_.clear(); }
-
-const std::vector<Segment::Candidate> &Segment::meta_candidates() const {
-  return meta_candidates_;
-}
-
-std::vector<Segment::Candidate> *Segment::mutable_meta_candidates() {
-  return &meta_candidates_;
-}
-
-const Segment::Candidate &Segment::meta_candidate(size_t i) const {
+const Candidate &Segment::meta_candidate(size_t i) const {
   if (i >= meta_candidates_.size()) {
     LOG(ERROR) << "Invalid index number of meta_candidate: " << i;
     i = 0;
@@ -396,7 +190,7 @@ const Segment::Candidate &Segment::meta_candidate(size_t i) const {
   return meta_candidates_[i];
 }
 
-Segment::Candidate *Segment::mutable_meta_candidate(size_t i) {
+Candidate *Segment::mutable_meta_candidate(size_t i) {
   if (i >= meta_candidates_.size()) {
     LOG(ERROR) << "Invalid index number of meta_candidate: " << i;
     i = 0;
@@ -404,11 +198,8 @@ Segment::Candidate *Segment::mutable_meta_candidate(size_t i) {
   return &meta_candidates_[i];
 }
 
-Segment::Candidate *Segment::add_meta_candidate() {
-  Candidate candidate;
-  candidate.Init();
-  meta_candidates_.push_back(candidate);
-  return &meta_candidates_[meta_candidates_size() - 1];
+Candidate *Segment::add_meta_candidate() {
+  return &meta_candidates_.emplace_back();
 }
 
 void Segment::move_candidate(int old_idx, int new_idx) {
@@ -424,7 +215,7 @@ void Segment::move_candidate(int old_idx, int new_idx) {
   // normal segment
   if (old_idx < 0 || old_idx >= static_cast<int>(candidates_size()) ||
       new_idx >= static_cast<int>(candidates_size()) || old_idx == new_idx) {
-    VLOG(1) << "old_idx and new_idx are the same";
+    MOZC_VLOG(1) << "old_idx and new_idx are the same";
     return;
   }
   if (old_idx > new_idx) {  // promotion
@@ -445,10 +236,21 @@ void Segment::move_candidate(int old_idx, int new_idx) {
 void Segment::Clear() {
   clear_candidates();
   key_.clear();
+  key_len_ = 0;
   meta_candidates_.clear();
   segment_type_ = FREE;
 }
 
+void Segment::DeepCopyCandidates(const std::deque<Candidate *> &candidates) {
+  DCHECK(pool_.empty());
+  pool_.reserve(candidates.size());
+  for (const Candidate *cand : candidates) {
+    auto new_cand = std::make_unique<Candidate>(*cand);
+    candidates_.push_back(new_cand.get());
+    pool_.push_back(std::move(new_cand));
+  }
+}
+
 std::string Segment::DebugString() const {
   std::stringstream os;
   os << "[segtype=" << segment_type() << " key=" << key() << std::endl;
@@ -466,18 +268,12 @@ std::string Segment::DebugString() const {
   return os.str();
 }
 
-Segments::Segments()
-    : max_history_segments_size_(0),
-      resized_(false),
-      pool_(32),
-      cached_lattice_(new Lattice()) {}
-
 Segments::Segments(const Segments &x)
     : max_history_segments_size_(x.max_history_segments_size_),
       resized_(x.resized_),
       pool_(32),
       revert_entries_(x.revert_entries_),
-      cached_lattice_(new Lattice()) {
+      cached_lattice_() {
   // Deep-copy segments.
   for (const Segment *segment : x.segments_) {
     *add_segment() = *segment;
@@ -502,28 +298,6 @@ Segments &Segments::operator=(const Segments &x) {
   return *this;
 }
 
-Segments::~Segments() = default;
-
-const Segment &Segments::segment(size_t i) const { return *segments_[i]; }
-
-Segment *Segments::mutable_segment(size_t i) { return segments_[i]; }
-
-const Segment &Segments::history_segment(size_t i) const {
-  return *segments_[i];
-}
-
-Segment *Segments::mutable_history_segment(size_t i) { return segments_[i]; }
-
-const Segment &Segments::conversion_segment(size_t i) const {
-  return *segments_[i + history_segments_size()];
-}
-
-Segment *Segments::mutable_conversion_segment(size_t i) {
-  return segments_[i + history_segments_size()];
-}
-
-Segment *Segments::add_segment() { return push_back_segment(); }
-
 Segment *Segments::insert_segment(size_t i) {
   Segment *segment = pool_.Alloc();
   segment->Clear();
@@ -545,30 +319,63 @@ Segment *Segments::push_front_segment() {
   return segment;
 }
 
-size_t Segments::segments_size() const { return segments_.size(); }
-
-size_t Segments::history_segments_size() const {
-  size_t result = 0;
-  for (size_t i = 0; i < segments_size(); ++i) {
-    if (segment(i).segment_type() != Segment::HISTORY &&
-        segment(i).segment_type() != Segment::SUBMITTED) {
+// Returns an `Iterator` for the end of history segments.
+template <typename Iterator>
+static Iterator history_segments_end_of(Iterator begin, Iterator end) {
+  Iterator it = begin;
+  for (; it != end; ++it) {
+    const Segment &segment = **it;
+    if (segment.segment_type() != Segment::HISTORY &&
+        segment.segment_type() != Segment::SUBMITTED) {
       break;
     }
-    ++result;
   }
-  return result;
+  return it;
 }
 
-size_t Segments::conversion_segments_size() const {
-  return (segments_size() - history_segments_size());
+Segments::const_iterator Segments::history_segments_end() const {
+  return const_iterator{
+      history_segments_end_of<decltype(segments_)::const_iterator>(
+          segments_.begin(), segments_.end())};
+}
+
+Segments::iterator Segments::history_segments_end() {
+  return iterator{history_segments_end_of<decltype(segments_)::iterator>(
+      segments_.begin(), segments_.end())};
+}
+
+size_t Segments::history_segments_size() const {
+  const auto end = history_segments_end_of<decltype(segments_)::const_iterator>(
+      segments_.begin(), segments_.end());
+  return end - segments_.begin();
+}
+
+Segments::range Segments::history_segments() {
+  return make_range(begin(), history_segments_end());
+}
+
+Segments::const_range Segments::history_segments() const {
+  return make_range(begin(), history_segments_end());
+}
+
+Segments::range Segments::conversion_segments() {
+  return make_range(history_segments_end(), end());
+}
+
+Segments::const_range Segments::conversion_segments() const {
+  return make_range(history_segments_end(), end());
 }
 
 void Segments::erase_segment(size_t i) {
   if (i >= segments_size()) {
     return;
   }
-  pool_.Release(mutable_segment(i));
-  segments_.erase(segments_.begin() + i);
+  erase_segment(begin() + i);
+}
+
+Segments::iterator Segments::erase_segment(iterator position) {
+  pool_.Release(&*position);
+  return iterator{segments_.erase(position.iterator_)};
 }
 
 void Segments::erase_segments(size_t i, size_t size) {
@@ -576,10 +383,14 @@ void Segments::erase_segments(size_t i, size_t size) {
   if (i >= segments_size() || end > segments_size()) {
     return;
   }
-  for (size_t j = i; j < end; ++j) {
-    pool_.Release(mutable_segment(j));
+  erase_segments(begin() + i, begin() + end);
+}
+
+Segments::iterator Segments::erase_segments(iterator first, iterator last) {
+  for (auto it = first; it != last; ++it) {
+    pool_.Release(&*it);
   }
-  segments_.erase(segments_.begin() + i, segments_.begin() + end);
+  return iterator{segments_.erase(first.iterator_, last.iterator_)};
 }
 
 void Segments::pop_front_segment() {
@@ -598,11 +409,119 @@ void Segments::pop_back_segment() {
   }
 }
 
+void Segments::InitForConvert(absl::string_view key) {
+  set_max_history_segments_size(4);
+  clear_conversion_segments();
+
+  Segment *segment = add_segment();
+  segment->set_key(key);
+  segment->set_segment_type(mozc::Segment::FREE);
+
+  MOZC_VLOG(2) << DebugString();
+}
+
+void Segments::InitForCommit(absl::string_view key, absl::string_view value) {
+  clear_conversion_segments();
+
+  Segment *segment = add_segment();
+  segment->set_key(key);
+  segment->set_segment_type(Segment::FIXED_VALUE);
+
+  Candidate *candidate = segment->add_candidate();
+  candidate->key = std::string(key);
+  candidate->content_key = std::string(key);
+  candidate->value = std::string(value);
+  candidate->content_value = std::string(value);
+}
+
 void Segments::Clear() {
   clear_segments();
   clear_revert_entries();
 }
 
+void Segments::PrependCandidates(const Segment &previous_segment) {
+  if (conversion_segments_size() == 0) {
+    clear_conversion_segments();
+    Segment *segment = add_segment();
+    segment->set_key(previous_segment.key());
+  }
+
+  DCHECK_EQ(conversion_segments_size(), 1);
+  Segment *segment = mutable_conversion_segment(0);
+  DCHECK(segment);
+
+  const size_t cands_size = previous_segment.candidates_size();
+  for (size_t i = 0; i < cands_size; ++i) {
+    Candidate *candidate = segment->insert_candidate(i);
+    *candidate = previous_segment.candidate(i);
+  }
+  segment->mutable_meta_candidates()->assign(
+      previous_segment.meta_candidates().begin(),
+      previous_segment.meta_candidates().end());
+}
+
+bool Segments::Resize(size_t start_index, absl::Span<const uint8_t> new_sizes) {
+  const size_t total_size =
+      std::accumulate(new_sizes.begin(), new_sizes.end(), 0);
+  if (total_size == 0) {
+    return false;
+  }
+
+  std::string key;
+  size_t key_len = 0;
+  size_t modified_segments_size = 0;
+  for (const Segment &segment : all().drop(start_index)) {
+    absl::StrAppend(&key, segment.key());
+    key_len += segment.key_len();
+    ++modified_segments_size;
+    if (key_len >= total_size) {
+      break;
+    }
+  }
+
+  // If key is empty or less than the total size of new segments, return false.
+  if (key_len == 0 || key_len < total_size) {
+    return false;
+  }
+
+  size_t consumed = 0;
+  std::vector<std::string> new_keys;
+  new_keys.reserve(new_sizes.size());
+
+  for (size_t new_size : new_sizes) {
+    if (new_size != 0 && consumed < key_len) {
+      new_keys.emplace_back(Util::Utf8SubString(key, consumed, new_size));
+      consumed += new_size;
+    }
+  }
+
+  erase_segments(start_index, modified_segments_size);
+
+  for (size_t i = 0; i < new_keys.size(); ++i) {
+    Segment *seg = insert_segment(start_index + i);
+    seg->set_segment_type(Segment::FIXED_BOUNDARY);
+    seg->set_key(std::move(new_keys[i]));
+  }
+
+  // If there is a remaining key, replace the next segment with the new key
+  // prepending the remaining key to the next segment as a FREE type.
+  if (consumed < key_len) {
+    std::string next_segment_key(
+        Util::Utf8SubString(key, consumed, key_len - consumed));
+    const size_t next_segment_index = start_index + new_keys.size();
+    if (next_segment_index < segments_size()) {
+      absl::StrAppend(&next_segment_key, segment(next_segment_index).key());
+      erase_segment(next_segment_index);
+    }
+    Segment *seg = insert_segment(next_segment_index);
+    seg->set_segment_type(Segment::FREE);
+    seg->set_key(next_segment_key);
+  }
+
+  set_resized(true);
+  return true;
+}
+
 void Segments::clear_segments() {
   pool_.Free();
   resized_ = false;
@@ -621,16 +540,8 @@ void Segments::clear_history_segments() {
 }
 
 void Segments::clear_conversion_segments() {
-  const size_t size = history_segments_size();
-  for (size_t i = size; i < segments_size(); ++i) {
-    pool_.Release(mutable_segment(i));
-  }
   resized_ = false;
-  segments_.resize(size);
-}
-
-size_t Segments::max_history_segments_size() const {
-  return max_history_segments_size_;
+  erase_segments(history_segments_end(), end());
 }
 
 void Segments::set_max_history_segments_size(size_t max_history_segments_size) {
@@ -639,14 +550,6 @@ void Segments::set_max_history_segments_size(size_t max_history_segments_size) {
                std::min(max_history_segments_size, kMaxHistorySize));
 }
 
-void Segments::set_resized(bool resized) { resized_ = resized; }
-
-bool Segments::resized() const { return resized_; }
-
-void Segments::clear_revert_entries() { revert_entries_.clear(); }
-
-size_t Segments::revert_entries_size() const { return revert_entries_.size(); }
-
 Segments::RevertEntry *Segments::push_back_revert_entry() {
   revert_entries_.resize(revert_entries_.size() + 1);
   Segments::RevertEntry *entry = &revert_entries_.back();
@@ -657,15 +560,35 @@ Segments::RevertEntry *Segments::push_back_revert_entry() {
   return entry;
 }
 
-const Segments::RevertEntry &Segments::revert_entry(size_t i) const {
-  return revert_entries_[i];
+std::string Segments::history_key(int size) const {
+  const_range segments = history_segments();
+  if (size != -1) {
+    segments = segments.take_last(size);
+  }
+
+  std::string history_key;
+  for (const Segment &seg : segments) {
+    if (seg.candidates_size() == 0) continue;
+    history_key.append(seg.candidate(0).key);
+  }
+
+  return history_key;
 }
 
-Segments::RevertEntry *Segments::mutable_revert_entry(size_t i) {
-  return &revert_entries_[i];
-}
+std::string Segments::history_value(int size) const {
+  const_range segments = history_segments();
+  if (size != -1) {
+    segments = segments.take_last(size);
+  }
 
-Lattice *Segments::mutable_cached_lattice() { return cached_lattice_.get(); }
+  std::string history_value;
+  for (const Segment &seg : segments) {
+    if (seg.candidates_size() == 0) continue;
+    history_value.append(seg.candidate(0).value);
+  }
+
+  return history_value;
+}
 
 std::string Segments::DebugString() const {
   std::stringstream os;
