@@ -30,8 +30,8 @@
 #include "data_manager/oss/oss_data_manager.h"
 
 #include "data_manager/data_manager_test_base.h"
-#include "testing/base/public/gunit.h"
-#include "testing/base/public/mozctest.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 
 namespace mozc {
 namespace oss {
@@ -39,13 +39,6 @@ namespace {
 
 #include "data_manager/oss/segmenter_inl.inc"
 
-std::pair<std::string, std::string> GetTypingModelEntry(
-    const std::string &fname) {
-  return std::pair<std::string, std::string>(
-      fname, mozc::testing::GetSourceFileOrDie(
-                 {"data_manager", "oss", fname + ".data"}));
-}
-
 }  // namespace
 
 class OssDataManagerTest : public DataManagerTestBase {
@@ -53,24 +46,24 @@ class OssDataManagerTest : public DataManagerTestBase {
   OssDataManagerTest()
       : DataManagerTestBase(
             new OssDataManager, kLSize, kRSize, IsBoundaryInternal,
-            mozc::testing::GetSourceFileOrDie(
-                {"data_manager", "oss", "connection_single_column.txt"}),
+            mozc::testing::GetSourceFileOrDie({MOZC_DICT_DIR_COMPONENTS,
+                                               "dictionary_oss",
+                                               "connection_single_column.txt"}),
             1,
             mozc::testing::GetSourceFilesInDirOrDie(
-                {"data", "dictionary_oss"},
+                {MOZC_DICT_DIR_COMPONENTS, "dictionary_oss"},
+#ifdef MOZC_GYP_BUILD
                 {"dictionary00.txt", "dictionary01.txt", "dictionary02.txt",
                  "dictionary03.txt", "dictionary04.txt", "dictionary05.txt",
                  "dictionary06.txt", "dictionary07.txt", "dictionary08.txt",
-                 "dictionary09.txt"}),
-            mozc::testing::GetSourceFilesInDirOrDie({"data", "dictionary_oss"},
-                                                    {"suggestion_filter.txt"}),
-            {
-                GetTypingModelEntry("typing_model_12keys-hiragana.tsv"),
-                GetTypingModelEntry("typing_model_flick-hiragana.tsv"),
-                GetTypingModelEntry("typing_model_godan-hiragana.tsv"),
-                GetTypingModelEntry("typing_model_qwerty_mobile-hiragana.tsv"),
-                GetTypingModelEntry("typing_model_toggle_flick-hiragana.tsv"),
-            }) {}
+                 "dictionary09.txt"}
+#else  // MOZC_GYP_BUILD
+                {"aux_dictionary.txt", "dictionary.txt"}
+#endif  // MOZC_GYP_BUILD
+            ),
+            mozc::testing::GetSourceFilesInDirOrDie(
+                {MOZC_DICT_DIR_COMPONENTS, "dictionary_oss"},
+                {"suggestion_filter.txt"})) {}
 };
 
 TEST_F(OssDataManagerTest, AllTests) { RunAllTests(); }
