@@ -29,9 +29,6 @@
 
 #include "data_manager/testing/mock_data_manager.h"
 
-#include <string>
-#include <utility>
-
 #include "data_manager/data_manager_test_base.h"
 #include "testing/gunit.h"
 #include "testing/mozctest.h"
