@@ -33,6 +33,7 @@
 #include <string>
 #include <vector>
 
+#include "absl/types/span.h"
 #include "base/file/temp_dir.h"
 #include "base/file_util.h"
 #include "data_manager/testing/mock_data_manager.h"
@@ -86,7 +87,7 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
   {
     std::unique_ptr<TextDictionaryLoader> loader = CreateTextDictionaryLoader();
     loader->Load(filename, "");
-    const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
+    absl::Span<const std::unique_ptr<Token>> tokens = loader->tokens();
 
     EXPECT_EQ(tokens.size(), 3);
 
@@ -115,7 +116,7 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
   {
     std::unique_ptr<TextDictionaryLoader> loader = CreateTextDictionaryLoader();
     loader->LoadWithLineLimit(filename, "", 2);
-    const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
+    absl::Span<const std::unique_ptr<Token>> tokens = loader->tokens();
 
     EXPECT_EQ(tokens.size(), 2);
 
@@ -140,7 +141,7 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
     // open twice -- tokens are cleared everytime
     loader->Load(filename, "");
     loader->Load(filename, "");
-    const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
+    absl::Span<const std::unique_ptr<Token>> tokens = loader->tokens();
     EXPECT_EQ(tokens.size(), 3);
   }
 
@@ -234,7 +235,7 @@ TEST_F(TextDictionaryLoaderTest, ReadingCorrectionTest) {
   FileUnlinker reading_correction_unlinker(reading_correction_filename);
 
   loader->Load(dic_filename, reading_correction_filename);
-  const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
+  absl::Span<const std::unique_ptr<Token>> tokens = loader->tokens();
   ASSERT_EQ(tokens.size(), 4);
   EXPECT_EQ(tokens[3]->key, "foobar_error");
   EXPECT_EQ(tokens[3]->value, "foobar");
