@@ -30,32 +30,42 @@
 #include "dictionary/user_dictionary.h"
 
 #include <algorithm>
+#include <atomic>
+#include <cstddef>
 #include <cstdint>
+#include <functional>
 #include <limits>
 #include <memory>
-#include <set>
+#include <optional>
 #include <string>
 #include <utility>
+#include <vector>
 
-#include "base/compiler_specific.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/hash/hash.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/ascii.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 #include "base/file_util.h"
 #include "base/hash.h"
-#include "base/japanese_util.h"
-#include "base/logging.h"
-#include "base/singleton.h"
+#include "base/strings/assign.h"
+#include "base/strings/japanese.h"
+#include "base/strings/unicode.h"
 #include "base/thread.h"
-#include "base/util.h"
+#include "base/vlog.h"
+#include "dictionary/dictionary_interface.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/pos_matcher.h"
-#include "dictionary/suppression_dictionary.h"
 #include "dictionary/user_dictionary_storage.h"
 #include "dictionary/user_dictionary_util.h"
 #include "dictionary/user_pos.h"
 #include "protocol/config.pb.h"
-#include "usage_stats/usage_stats.h"
-#include "absl/strings/match.h"
-#include "absl/strings/string_view.h"
-#include "absl/synchronization/mutex.h"
+#include "protocol/user_dictionary_storage.pb.h"
+#include "request/conversion_request.h"
 
 namespace mozc {
 namespace dictionary {
@@ -88,40 +98,55 @@ struct OrderByKeyThenById {
   }
 };
 
-class UserDictionaryFileManager {
+class SuppressionDictionary {
  public:
-  UserDictionaryFileManager() = default;
-
-  UserDictionaryFileManager(const UserDictionaryFileManager &) = delete;
-  UserDictionaryFileManager &operator=(const UserDictionaryFileManager &) =
-      delete;
-
-  const std::string GetFileName() {
-    absl::MutexLock l(&mutex_);
-    if (filename_.empty()) {
-      return UserDictionaryUtil::GetUserDictionaryFileName();
-    } else {
-      return filename_;
+  bool AddEntry(std::string key, std::string value) {
+    if (key.empty() && value.empty()) {
+      LOG(WARNING) << "Both key and value are empty";
+      return false;
     }
+
+    if (key.empty()) {
+      values_only_.emplace(std::move(value));
+    } else if (value.empty()) {
+      keys_only_.emplace(std::move(key));
+    } else {
+      keys_values_.emplace(std::move(key), std::move(value));
+    }
+
+    return true;
   }
 
-  void SetFileName(const std::string &filename) {
-    absl::MutexLock l(&mutex_);
-    filename_ = filename;
+  bool IsEmpty() const {
+    return keys_only_.empty() && values_only_.empty() && keys_values_.empty();
+  }
+
+  bool IsSuppressedEntry(const absl::string_view key,
+                         const absl::string_view value) const {
+    return !IsEmpty() &&
+           (keys_values_.contains(std::make_pair(key, value)) ||
+            keys_only_.contains(key) || values_only_.contains(value));
   }
 
  private:
-  std::string filename_;
-  absl::Mutex mutex_;
-};
+  using KeyValue = std::pair<std::string, std::string>;
+  using KeyValueView = std::pair<absl::string_view, absl::string_view>;
+  struct KeyValueHash : public absl::Hash<KeyValueView> {
+    using is_transparent = void;
+  };
+  struct KeyValueEq : public std::equal_to<KeyValueView> {
+    using is_transparent = void;
+  };
 
+  absl::flat_hash_set<KeyValue, KeyValueHash, KeyValueEq> keys_values_;
+  absl::flat_hash_set<std::string> keys_only_;
+  absl::flat_hash_set<std::string> values_only_;
+};
 }  // namespace
 
 class UserDictionary::TokensIndex {
  public:
-  TokensIndex(const UserPosInterface *user_pos,
-              SuppressionDictionary *suppression_dictionary)
-      : user_pos_(user_pos), suppression_dictionary_(suppression_dictionary) {}
+  TokensIndex(const UserPos &user_pos) : user_pos_(user_pos) {}
 
   ~TokensIndex() = default;
 
@@ -135,72 +160,82 @@ class UserDictionary::TokensIndex {
     return user_pos_tokens_.end();
   }
 
-  void Load(const user_dictionary::UserDictionaryStorage &storage) {
+  void Load(const user_dictionary::UserDictionaryStorage &storage,
+            std::atomic<bool> *canceled_signal) {
+    DCHECK(canceled_signal);
     user_pos_tokens_.clear();
-    std::set<uint64_t> seen;
+    absl::flat_hash_set<uint64_t> seen;
     std::vector<UserPos::Token> tokens;
 
-    const SuppressionDictionaryLock l(suppression_dictionary_);
-    suppression_dictionary_->Clear();
-
     for (const UserDictionaryStorage::UserDictionary &dic :
          storage.dictionaries()) {
-      if (!dic.enabled() || dic.entries_size() == 0) {
+      if (dic.entries_size() == 0) {
         continue;
       }
-
-      const bool is_shortcuts =
+      const bool is_android_shortcuts =
           (dic.name() == "__auto_imported_android_shortcuts_dictionary");
 
       for (const UserDictionaryStorage::UserDictionaryEntry &entry :
            dic.entries()) {
-        if (!UserDictionaryUtil::IsValidEntry(*user_pos_, entry)) {
+        if (!UserDictionaryUtil::IsValidEntry(user_pos_, entry)) {
           continue;
         }
-
-        std::string tmp, reading;
-        UserDictionaryUtil::NormalizeReading(entry.key(), &tmp);
+        if (canceled_signal->load()) {
+          LOG(INFO) << "User dictionary loading is canceled";
+          return;
+        }
 
         // We cannot call NormalizeVoiceSoundMark inside NormalizeReading,
         // because the normalization is user-visible.
         // http://b/2480844
-        japanese_util::NormalizeVoicedSoundMark(tmp, &reading);
+        std::string reading = japanese::NormalizeVoicedSoundMark(
+            UserDictionaryUtil::NormalizeReading(entry.key()));
 
-        DCHECK_LE(0, entry.pos());
-        MOZC_CLANG_PUSH_WARNING();
-        // clang-format off
-#if MOZC_CLANG_HAS_WARNING(tautological-constant-out-of-range-compare)
-        MOZC_CLANG_DISABLE_WARNING(tautological-constant-out-of-range-compare);
-#endif  // MOZC_CLANG_HAS_WARNING(tautological-constant-out-of-range-compare)
-        // clang-format on
-        DCHECK_LE(entry.pos(), 255);
-        MOZC_CLANG_POP_WARNING();
+        DCHECK(user_dictionary::UserDictionary_PosType_IsValid(entry.pos()));
+        static_assert(user_dictionary::UserDictionary_PosType_PosType_MAX <=
+                      std::numeric_limits<char>::max());
+        const char pos_type_as_char[] = {static_cast<char>(entry.pos())};
         const uint64_t fp =
-            Hash::Fingerprint(reading + "\t" + entry.value() + "\t" +
-                              static_cast<char>(entry.pos()));
+            Fingerprint(absl::StrCat(reading, "\t", entry.value(), "\t",
+                                     absl::string_view(pos_type_as_char, 1)));
         if (!seen.insert(fp).second) {
-          VLOG(1) << "Found dup item";
+          MOZC_VLOG(1) << "Found dup item";
           continue;
         }
 
-        // "抑制単語"
         if (entry.pos() == user_dictionary::UserDictionary::SUPPRESSION_WORD) {
-          suppression_dictionary_->AddEntry(reading, entry.value());
+          // "抑制単語"
+          suppression_dictionary_.AddEntry(std::move(reading), entry.value());
+        } else if (entry.pos() == user_dictionary::UserDictionary::NO_POS) {
+          // In theory NO_POS works without this implementation, as it is
+          // covered in the UserPos::GetTokens function. However, that function
+          // is depending on the user_pos_*.data in the dictionary and there
+          // will not be corresponding POS tag. To avoid invalid behavior, this
+          // special treatment is added here.
+          // "品詞なし"
+          const absl::string_view comment =
+              absl::StripAsciiWhitespace(entry.comment());
+          UserPos::Token token{.key = reading,
+                               .value = entry.value(),
+                               .id = 0,
+                               .attributes = UserPos::Token::SHORTCUT,
+                               .comment = std::string(comment)};
+          // NO_POS has '名詞サ変' id as in user_pos.def
+          user_pos_.GetPosIds("名詞サ変", &token.id);
+          user_pos_tokens_.push_back(std::move(token));
         } else {
           tokens.clear();
-          user_pos_->GetTokens(
-              reading, entry.value(),
-              UserDictionaryUtil::GetStringPosType(entry.pos()), &tokens);
+          user_pos_.GetTokens(reading, entry.value(),
+                              UserDictionaryUtil::GetStringPosType(entry.pos()),
+                              &tokens);
+          const absl::string_view comment =
+              absl::StripAsciiWhitespace(entry.comment());
           for (auto &token : tokens) {
-            Util::StripWhiteSpaces(entry.comment(), &token.comment);
-            if (is_shortcuts &&
+            strings::Assign(token.comment, comment);
+            if (is_android_shortcuts &&
                 token.has_attribute(UserPos::Token::SUGGESTION_ONLY)) {
-              // Words fed by Android shortcut are registered as SUGGESTION_ONLY
-              // POS in order to minimize the side-effect of extremely short
-              // reading. However, user expect that they should appear in the
-              // normal conversion. Here we replace the attribute from
-              // SUGGESTION_ONLY to SHORTCUT, which has more adaptive cost based
-              // on the length of the key.
+              // TODO(b/295964970): This special implementation is planned to be
+              // removed after validating the safety of NO_POS implementation.
               token.remove_attribute(UserPos::Token::SUGGESTION_ONLY);
               token.add_attribute(UserPos::Token::SHORTCUT);
             }
@@ -215,19 +250,24 @@ class UserDictionary::TokensIndex {
     std::sort(user_pos_tokens_.begin(), user_pos_tokens_.end(),
               OrderByKeyThenById());
 
-    VLOG(1) << user_pos_tokens_.size() << " user dic entries loaded";
+    MOZC_VLOG(1) << user_pos_tokens_.size() << " user dic entries loaded";
+  }
 
-    usage_stats::UsageStats::SetInteger(
-        "UserRegisteredWord", static_cast<int>(user_pos_tokens_.size()));
+  bool IsSuppressedEntry(absl::string_view key, absl::string_view value) const {
+    return suppression_dictionary_.IsSuppressedEntry(key, value);
+  }
+
+  bool HasSuppressedEntries() const {
+    return !suppression_dictionary_.IsEmpty();
   }
 
  private:
-  const UserPosInterface *user_pos_;
-  SuppressionDictionary *suppression_dictionary_;
+  const UserPos &user_pos_;
+  SuppressionDictionary suppression_dictionary_;
   std::vector<UserPos::Token> user_pos_tokens_;
 };
 
-class UserDictionary::UserDictionaryReloader : public Thread {
+class UserDictionary::UserDictionaryReloader {
  public:
   explicit UserDictionaryReloader(UserDictionary *dic)
       : modified_at_(0), dic_(dic) {
@@ -237,14 +277,19 @@ class UserDictionary::UserDictionaryReloader : public Thread {
   UserDictionaryReloader(const UserDictionaryReloader &) = delete;
   UserDictionaryReloader &operator=(const UserDictionaryReloader &) = delete;
 
-  ~UserDictionaryReloader() override { Join(); }
+  ~UserDictionaryReloader() { Wait(); }
 
   // When the user dictionary exists AND the modification time has been updated,
   // reloads the dictionary.  Returns true when reloader thread is started.
   bool MaybeStartReload() {
+    if (reload_.has_value() && !reload_->Ready()) {
+      // Previously started reload is still running.
+      // TODO(tomokinat): test this path.
+      return false;
+    }
+
     absl::StatusOr<FileTimeStamp> modification_time =
-        FileUtil::GetModificationTime(
-            Singleton<UserDictionaryFileManager>::get()->GetFileName());
+        FileUtil::GetModificationTime(dic_->GetFileName());
     if (!modification_time.ok()) {
       // If the file doesn't exist, return doing nothing.
       // Therefore if the file is deleted after first reload,
@@ -258,13 +303,20 @@ class UserDictionary::UserDictionaryReloader : public Thread {
       return false;
     }
     modified_at_ = *modification_time;
-    Start("UserDictionaryReloader");
+    // Runs `ThreadMain()` in a background thread.
+    reload_.emplace([this] { ThreadMain(); });
     return true;
   }
 
-  void Run() override {
-    UserDictionaryStorage storage(
-        Singleton<UserDictionaryFileManager>::get()->GetFileName());
+  void Wait() {
+    if (reload_.has_value()) {
+      reload_->Wait();
+    }
+  }
+
+ private:
+  void ThreadMain() {
+    UserDictionaryStorage storage(dic_->GetFileName());
 
     // Load from file
     if (absl::Status s = storage.Load(); !s.ok()) {
@@ -272,43 +324,37 @@ class UserDictionary::UserDictionaryReloader : public Thread {
       return;
     }
 
-    if (storage.ConvertSyncDictionariesToNormalDictionaries()) {
-      LOG(INFO) << "Syncable dictionaries are converted to normal dictionaries";
-      if (storage.Lock()) {
-        if (absl::Status s = storage.Save(); !s.ok()) {
-          LOG(ERROR) << "Failed to save to storage: " << s;
-        }
-        storage.UnLock();
-      }
-    }
-
     dic_->Load(storage.GetProto());
   }
 
- private:
+  std::optional<BackgroundFuture<void>> reload_;
   FileTimeStamp modified_at_;
-  UserDictionary *dic_;
-  std::string key_;
-  std::string value_;
+  UserDictionary *dic_ = nullptr;
 };
 
-UserDictionary::UserDictionary(std::unique_ptr<const UserPosInterface> user_pos,
-                               PosMatcher pos_matcher,
-                               SuppressionDictionary *suppression_dictionary)
-    : ALLOW_THIS_IN_INITIALIZER_LIST(
-          reloader_(new UserDictionaryReloader(this))),
+UserDictionary::UserDictionary(std::unique_ptr<const UserPos> user_pos,
+                               PosMatcher pos_matcher)
+    : UserDictionary::UserDictionary(
+          std::move(user_pos), std::move(pos_matcher),
+          UserDictionaryUtil::GetUserDictionaryFileName()) {}
+
+UserDictionary::UserDictionary(std::unique_ptr<const UserPos> user_pos,
+                               PosMatcher pos_matcher, std::string filename)
+    : reloader_(std::make_unique<UserDictionaryReloader>(this)),
       user_pos_(std::move(user_pos)),
       pos_matcher_(pos_matcher),
-      suppression_dictionary_(suppression_dictionary),
-      tokens_(new TokensIndex(user_pos_.get(), suppression_dictionary)) {
-  DCHECK(user_pos_.get());
-  DCHECK(suppression_dictionary_);
+      tokens_(std::make_shared<TokensIndex>(*user_pos_)),
+      filename_(std::move(filename)) {
+  DCHECK(user_pos_);
+  DCHECK(tokens_);
+  DCHECK(!canceled_signal_);
+  DCHECK(!filename_.empty());
   Reload();
 }
 
 UserDictionary::~UserDictionary() {
-  reloader_->Join();
-  delete tokens_;
+  canceled_signal_.store(true);  // force to finish the thread.
+  WaitForReloader();
 }
 
 bool UserDictionary::HasKey(absl::string_view key) const {
@@ -329,23 +375,24 @@ bool UserDictionary::HasValue(absl::string_view value) const {
 void UserDictionary::LookupPredictive(
     absl::string_view key, const ConversionRequest &conversion_request,
     Callback *callback) const {
-  absl::ReaderMutexLock l(&mutex_);
-
   if (key.empty()) {
-    VLOG(2) << "string of length zero is passed.";
+    MOZC_VLOG(2) << "string of length zero is passed.";
     return;
   }
-  if (tokens_->empty()) {
+
+  std::shared_ptr<const TokensIndex> tokens = GetTokens();
+
+  if (tokens->empty()) {
     return;
   }
-  if (conversion_request.config().incognito_mode()) {
+  if (conversion_request.incognito_mode()) {
     return;
   }
 
   // Find the starting point of iteration over dictionary contents.
   Token token;
-  for (auto [begin, end] = std::equal_range(tokens_->begin(), tokens_->end(),
-                                            key, OrderByKeyPrefix());
+  for (auto [begin, end] = std::equal_range(tokens->begin(), tokens->end(), key,
+                                            OrderByKeyPrefix());
        begin != end; ++begin) {
     const UserPos::Token &user_pos_token = *begin;
     switch (callback->OnKey(user_pos_token.key)) {
@@ -357,6 +404,12 @@ void UserDictionary::LookupPredictive(
       default:
         break;
     }
+    // b/333613472: Make sure not to set the additional penalties.
+    if (callback->OnActualKey(user_pos_token.key, user_pos_token.key,
+                              /* num_expanded= */ 0) ==
+        Callback::TRAVERSE_DONE) {
+      return;
+    }
     PopulateTokenFromUserPosToken(user_pos_token, PREDICTIVE, &token);
     if (callback->OnToken(user_pos_token.key, user_pos_token.key, token) ==
         Callback::TRAVERSE_DONE) {
@@ -369,26 +422,27 @@ void UserDictionary::LookupPredictive(
 void UserDictionary::LookupPrefix(absl::string_view key,
                                   const ConversionRequest &conversion_request,
                                   Callback *callback) const {
-  absl::ReaderMutexLock l(&mutex_);
-
   if (key.empty()) {
     LOG(WARNING) << "string of length zero is passed.";
     return;
   }
-  if (tokens_->empty()) {
+
+  if (conversion_request.incognito_mode()) {
     return;
   }
-  if (conversion_request.config().incognito_mode()) {
+
+  std::shared_ptr<const TokensIndex> tokens = GetTokens();
+
+  if (tokens->empty()) {
     return;
   }
 
   // Find the starting point for iteration over dictionary contents.
-  const absl::string_view first_char =
-      key.substr(0, Util::OneCharLen(key.data()));
+  const absl::string_view first_char = Utf8AsChars(key).front();
   Token token;
-  for (auto it = std::lower_bound(tokens_->begin(), tokens_->end(), first_char,
+  for (auto it = std::lower_bound(tokens->begin(), tokens->end(), first_char,
                                   OrderByKey());
-       it != tokens_->end(); ++it) {
+       it != tokens->end(); ++it) {
     const UserPos::Token &user_pos_token = *it;
     if (user_pos_token.key > key) {
       break;
@@ -396,7 +450,7 @@ void UserDictionary::LookupPrefix(absl::string_view key,
     if (user_pos_token.has_attribute(UserPos::Token::SUGGESTION_ONLY)) {
       continue;
     }
-    if (!absl::StartsWith(key, user_pos_token.key)) {
+    if (!key.starts_with(user_pos_token.key)) {
       continue;
     }
     switch (callback->OnKey(user_pos_token.key)) {
@@ -410,6 +464,11 @@ void UserDictionary::LookupPrefix(absl::string_view key,
       default:
         break;
     }
+    if (callback->OnActualKey(user_pos_token.key, user_pos_token.key,
+                              /* num_expanded= */ 0) ==
+        Callback::TRAVERSE_DONE) {
+      return;
+    }
     PopulateTokenFromUserPosToken(user_pos_token, PREFIX, &token);
     switch (callback->OnToken(user_pos_token.key, user_pos_token.key, token)) {
       case Callback::TRAVERSE_DONE:
@@ -426,19 +485,23 @@ void UserDictionary::LookupPrefix(absl::string_view key,
 void UserDictionary::LookupExact(absl::string_view key,
                                  const ConversionRequest &conversion_request,
                                  Callback *callback) const {
-  absl::ReaderMutexLock l(&mutex_);
-  if (key.empty() || tokens_->empty() ||
-      conversion_request.config().incognito_mode()) {
+  std::shared_ptr<const TokensIndex> tokens = GetTokens();
+
+  if (key.empty() || tokens->empty() || conversion_request.incognito_mode()) {
     return;
   }
   auto [begin, end] =
-      std::equal_range(tokens_->begin(), tokens_->end(), key, OrderByKey());
+      std::equal_range(tokens->begin(), tokens->end(), key, OrderByKey());
   if (begin == end) {
     return;
   }
   if (callback->OnKey(key) != Callback::TRAVERSE_CONTINUE) {
     return;
   }
+  if (callback->OnActualKey(key, key, /* num_expanded= */ 0) !=
+      Callback::TRAVERSE_CONTINUE) {
+    return;
+  }
 
   Token token;
   for (; begin != end; ++begin) {
@@ -461,18 +524,19 @@ bool UserDictionary::LookupComment(absl::string_view key,
                                    absl::string_view value,
                                    const ConversionRequest &conversion_request,
                                    std::string *comment) const {
-  if (key.empty() || conversion_request.config().incognito_mode()) {
+  if (key.empty() || conversion_request.incognito_mode()) {
     return false;
   }
 
-  absl::ReaderMutexLock l(&mutex_);
-  if (tokens_->empty()) {
+  std::shared_ptr<const TokensIndex> tokens = GetTokens();
+
+  if (tokens->empty()) {
     return false;
   }
 
   // Set the comment that was found first.
-  for (auto [begin, end] = std::equal_range(tokens_->begin(), tokens_->end(),
-                                            key, OrderByKey());
+  for (auto [begin, end] =
+           std::equal_range(tokens->begin(), tokens->end(), key, OrderByKey());
        begin != end; ++begin) {
     const UserPos::Token &token = *begin;
     if (token.value == value && !token.comment.empty()) {
@@ -483,95 +547,56 @@ bool UserDictionary::LookupComment(absl::string_view key,
   return false;
 }
 
+bool UserDictionary::IsSuppressedEntry(absl::string_view key,
+                                       absl::string_view value) const {
+  return GetTokens()->IsSuppressedEntry(key, value);
+}
+
+bool UserDictionary::HasSuppressedEntries() const {
+  return GetTokens()->HasSuppressedEntries();
+}
+
 bool UserDictionary::Reload() {
-  if (reloader_->IsRunning()) {
-    return false;
-  }
   if (!reloader_->MaybeStartReload()) {
     LOG(INFO) << "MaybeStartReload() didn't start reloading";
   }
   return true;
 }
 
-namespace {
-
-class FindValueCallback : public DictionaryInterface::Callback {
- public:
-  explicit FindValueCallback(absl::string_view value)
-      : value_(value), found_(false) {}
-
-  ResultType OnToken(absl::string_view,  // key
-                     absl::string_view,  // actual_key
-                     const Token &token) override {
-    if (token.value == value_) {
-      found_ = true;
-      return TRAVERSE_DONE;
-    }
-    return TRAVERSE_CONTINUE;
-  }
-
-  bool found() const { return found_; }
-
- private:
-  const absl::string_view value_;
-  bool found_;
-};
-
-}  // namespace
-
-void UserDictionary::WaitForReloader() { reloader_->Join(); }
-
-void UserDictionary::Swap(TokensIndex *new_tokens) {
-  DCHECK(new_tokens);
-  TokensIndex *old_tokens = tokens_;
-  {
-    absl::WriterMutexLock l(&mutex_);
-    tokens_ = new_tokens;
-  }
-  delete old_tokens;
-}
+void UserDictionary::WaitForReloader() { reloader_->Wait(); }
 
 bool UserDictionary::Load(
     const user_dictionary::UserDictionaryStorage &storage) {
-  size_t size = 0;
-  {
-    absl::ReaderMutexLock l(&mutex_);
-    size = tokens_->size();
-  }
+  const size_t size = GetTokens()->size();
 
   // If UserDictionary is pretty big, we first remove the
   // current dictionary to save memory usage.
-#ifdef OS_ANDROID
+#ifdef __ANDROID__
   constexpr size_t kVeryBigUserDictionarySize = 5000;
-#else   // OS_ANDROID
+#else   // __ANDROID__
   constexpr size_t kVeryBigUserDictionarySize = 100000;
-#endif  // OS_ANDROID
+#endif  // __ANDROID__
 
   if (size >= kVeryBigUserDictionarySize) {
-    TokensIndex *dummy_empty_tokens =
-        new TokensIndex(user_pos_.get(), suppression_dictionary_);
-    Swap(dummy_empty_tokens);
+    auto placeholder_empty_tokens = std::make_shared<TokensIndex>(*user_pos_);
+    SetTokens(std::move(placeholder_empty_tokens));
   }
 
-  TokensIndex *tokens =
-      new TokensIndex(user_pos_.get(), suppression_dictionary_);
-  tokens->Load(storage);
-  Swap(tokens);
+  auto tokens = std::make_shared<TokensIndex>(*user_pos_);
+  tokens->Load(storage, &canceled_signal_);
+
+  SetTokens(tokens);
   return true;
 }
 
 std::vector<std::string> UserDictionary::GetPosList() const {
-  std::vector<std::string> pos_list;
-  user_pos_->GetPosList(&pos_list);
-  return pos_list;
+  return user_pos_->GetPosList();
 }
 
-void UserDictionary::SetUserDictionaryName(const std::string &filename) {
-  Singleton<UserDictionaryFileManager>::get()->SetFileName(filename);
-}
+std::string UserDictionary::GetFileName() const { return filename_; }
 
 void UserDictionary::PopulateTokenFromUserPosToken(
-    const UserPosInterface::Token &user_pos_token, RequestType request_type,
+    const UserPos::Token &user_pos_token, RequestType request_type,
     Token *token) const {
   token->key = user_pos_token.key;
   token->value = user_pos_token.value;
@@ -606,8 +631,8 @@ void UserDictionary::PopulateTokenFromUserPosToken(
   // TODO(taku): Better to apply this cost for all user defined words?
   if (user_pos_token.has_attribute(UserPos::Token::SHORTCUT) &&
       (request_type == PREFIX || request_type == EXACT)) {
-    const int key_length = Util::CharsLen(token->key);
-    token->cost += std::max<int>(0, 4 - key_length) * 2000;
+    const int key_length = strings::AtLeastCharsLen(token->key, 4);
+    token->cost += (4 - key_length) * 2000;
   }
 }
 
