@@ -30,29 +30,27 @@
 #ifndef MOZC_GUI_CONFIG_DIALOG_KEYMAP_EDITOR_H_
 #define MOZC_GUI_CONFIG_DIALOG_KEYMAP_EDITOR_H_
 
+#include <QAbstractButton>
 #include <QWidget>
-#include <map>
+#include <istream>
 #include <memory>
-#include <set>
 #include <string>
 
-#include "base/port.h"
+#include "absl/container/fixed_array.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/container/flat_hash_set.h"
+#include "gui/config_dialog/combobox_delegate.h"
 #include "gui/config_dialog/generic_table_editor.h"
-
-class QAbstractButton;
+#include "gui/config_dialog/keybinding_editor_delegate.h"
 
 namespace mozc {
 namespace gui {
 
-class ComboBoxDelegate;
-class KeyBindingEditorDelegate;
-
 class KeyMapEditorDialog : public GenericTableEditorDialog {
   Q_OBJECT;
 
  public:
   explicit KeyMapEditorDialog(QWidget *parent);
-  ~KeyMapEditorDialog() override;
 
   // show a modal dialog
   static bool Show(QWidget *parent, const std::string &current_keymap,
@@ -63,7 +61,9 @@ class KeyMapEditorDialog : public GenericTableEditorDialog {
   void OnEditMenuAction(QAction *action) override;
 
  protected:
-  std::string GetDefaultFilename() const override { return "keymap.txt"; }
+  QLatin1String GetDefaultFilename() const override {
+    return QLatin1String("keymap.txt");
+  }
   bool LoadFromStream(std::istream *is) override;
   bool Update() override;
 
@@ -71,15 +71,15 @@ class KeyMapEditorDialog : public GenericTableEditorDialog {
   std::string invisible_keymap_table_;
   // This is used for deciding whether the user has changed the settings that
   // are valid only for new applications.
-  std::set<std::string> direct_mode_commands_;
-  std::unique_ptr<QAction *[]> actions_;
-  std::unique_ptr<QAction *[]> import_actions_;
+  absl::flat_hash_set<std::string> direct_mode_commands_;
+  absl::FixedArray<QAction *> actions_;
+  absl::FixedArray<QAction *> import_actions_;
   std::unique_ptr<ComboBoxDelegate> status_delegate_;
   std::unique_ptr<ComboBoxDelegate> commands_delegate_;
   std::unique_ptr<KeyBindingEditorDelegate> keybinding_delegate_;
 
-  std::map<std::string, std::string> normalized_command_map_;
-  std::map<std::string, std::string> normalized_status_map_;
+  absl::flat_hash_map<std::string, std::string> normalized_command_map_;
+  absl::flat_hash_map<std::string, std::string> normalized_status_map_;
 };
 
 }  // namespace gui
