@@ -1,4 +1,4 @@
-// Copyright 2010-2020, Google Inc.
+// Copyright 2010-2021, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,7 @@
 #ifndef MOZC_GUI_DICTIONARY_TOOL_DICTIONARY_CONTENT_TABLE_WIDGET_H_
 #define MOZC_GUI_DICTIONARY_TOOL_DICTIONARY_CONTENT_TABLE_WIDGET_H_
 
-#include <QtWidgets/QTableWidget>
+#include <QTableWidget>
 
 namespace mozc {
 namespace gui {
@@ -56,6 +56,4 @@ class DictionaryContentTableWidget : public QTableWidget {
 }  // namespace gui
 }  // namespace mozc
 
-using mozc::gui::DictionaryContentTableWidget;
-
 #endif  // MOZC_GUI_DICTIONARY_TOOL_DICTIONARY_CONTENT_TABLE_WIDGET_H_
