@@ -30,32 +30,37 @@
 #ifndef MOZC_IPC_IPC_PATH_MANAGER_H_
 #define MOZC_IPC_IPC_PATH_MANAGER_H_
 
-#ifdef OS_WIN
-#include <time.h>  // for time_t
-#else
-#include <sys/time.h>  // for time_t
-#endif                 // OS_WIN
-#ifdef OS_WIN
-#include <map>
-#endif  // OS_WIN
+#include <cstdint>
+#include <ctime>
 #include <memory>
 #include <string>
+#include <utility>
 
-#include "base/mutex.h"
-#include "base/port.h"
-// For FRIEND_TEST
-#include "testing/base/public/gunit_prod.h"
+#include "absl/strings/string_view.h"
+#include "absl/synchronization/mutex.h"
+#include "base/process_mutex.h"
+#include "ipc/ipc.pb.h"
+#include "testing/friend_test.h"
+
+#ifdef _WIN32
+#include "absl/container/flat_hash_map.h"
+#endif  // _WIN32
 
 namespace mozc {
 
-namespace ipc {
-class IPCPathInfo;
-}
-
-class ProcessMutex;
-
 class IPCPathManager {
  public:
+  // do not call the constructor directly.
+  explicit IPCPathManager(std::string name)
+      : ipc_path_info_(),
+        name_(std::move(name)),
+        server_pid_(0),
+        last_modified_(-1) {}
+  virtual ~IPCPathManager() = default;
+
+  // return singleton instance corresponding to "name"
+  static IPCPathManager *GetIPCPathManager(absl::string_view name);
+
   // Brief summary of CreateNew / Save / Load / Get PathName().
   // CreateNewPathName: Generates a pathname and save it into the heap.
   // SavePathName:      Saves a pathname into a file from the heap.
@@ -75,19 +80,19 @@ class IPCPathManager {
   // empty or ipc key file is updated. Returns false if it cannot load.
   bool LoadPathName();
 
-  // Get a pathanem from the heap. If pathanme is empty, returns false.
-  bool GetPathName(std::string *path_name) const;
+  // Get a pathname from the heap. If pathname is empty, returns false.
+  bool GetPathName(std::string *ipc_name) const;
 
   // return protocol version.
   // return 0 if protocol version is not defined.
-  uint32 GetServerProtocolVersion() const;
+  uint32_t GetServerProtocolVersion() const;
 
   // return product version.
   // return "0.0.0.0" if product version is not defined
   const std::string &GetServerProductVersion() const;
 
   // return process id of the server
-  uint32 GetServerProcessId() const;
+  uint32_t GetServerProcessId() const;
 
   // Checks the server pid is the valid server specified with server_path.
   // server pid can be obtained by OS dependent method.
@@ -100,20 +105,13 @@ class IPCPathManager {
   // when pid of 0 is passed, IsValidServer() returns true.
   // when pid of static_cast<size_t>(-1) is passed, IsValidServer()
   // returns false.
-  // To keep backward compatibility and other operationg system
+  // To keep backward compatibility and other operating system
   // having no support of getting peer's pid, you can set 0 pid.
-  bool IsValidServer(uint32 pid, const std::string &server_path);
+  bool IsValidServer(uint32_t pid, absl::string_view server_path);
 
   // clear ipc_key;
   void Clear();
 
-  // return singleton instance corresponding to "name"
-  static IPCPathManager *GetIPCPathManager(const std::string &name);
-
-  // do not call constructor directly.
-  explicit IPCPathManager(const std::string &name);
-  virtual ~IPCPathManager();
-
  private:
   FRIEND_TEST(IPCPathManagerTest, ReloadTest);
   FRIEND_TEST(IPCPathManagerTest, PathNameTest);
@@ -123,19 +121,23 @@ class IPCPathManager {
   // Returns true if the ipc file is updated after it load.
   bool ShouldReload() const;
 
+  bool CreateNewPathNameUnlocked();
+
   // Returns the last modified timestamp of the IPC file.
   time_t GetIPCFileTimeStamp() const;
 
   std::unique_ptr<ProcessMutex> path_mutex_;  // lock ipc path file
-  std::unique_ptr<Mutex> mutex_;              // mutex for methods
-  std::unique_ptr<ipc::IPCPathInfo> ipc_path_info_;
+  mutable absl::Mutex mutex_;                 // mutex for methods
+  ipc::IPCPathInfo ipc_path_info_;
   std::string name_;
   std::string server_path_;  // cache for server_path
-  uint32 server_pid_;   // cache for pid of server_path
+  uint32_t server_pid_;      // cache for pid of server_path
   time_t last_modified_;
-#ifdef OS_WIN
-  std::map<string, std::wstring> expected_server_ntpath_cache_;
-#endif  // OS_WIN
+#ifdef _WIN32
+  // std::less<> is a transparent comparator that's necessary to pass
+  // absl::string_view to map::find(), etc.
+  absl::flat_hash_map<std::string, std::wstring> expected_server_ntpath_cache_;
+#endif  // _WIN32
 };
 
 }  // namespace mozc
