@@ -30,18 +30,26 @@
 #include "prediction/predictor.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
-#include <vector>
 
-#include "base/logging.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
+#include "base/util.h"
+#include "converter/converter_interface.h"
 #include "converter/segments.h"
+#include "prediction/predictor_interface.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
-#include "absl/flags/flag.h"
+#include "request/conversion_request.h"
 
-namespace mozc {
+namespace mozc::prediction {
 namespace {
 
 constexpr int kPredictionSize = 100;
@@ -78,21 +86,39 @@ size_t GetHistoryPredictionSizeFromRequest(const ConversionRequest &request) {
   return 3;
 }
 
+std::optional<std::string> GetReading(const ConverterInterface &converter,
+                                      absl::string_view text) {
+  Segments segments;
+  if (!converter.StartReverseConversion(&segments, text)) {
+    LOG(ERROR) << "Reverse conversion failed to get the reading of " << text;
+    return std::nullopt;
+  }
+  if (segments.conversion_segments_size() != 1 ||
+      segments.conversion_segment(0).candidates_size() == 0) {
+    LOG(ERROR) << "Reverse conversion returned an invalid result for " << text;
+    return std::nullopt;
+  }
+  return std::move(
+      segments.mutable_conversion_segment(0)->mutable_candidate(0)->value);
+}
+
 }  // namespace
 
 BasePredictor::BasePredictor(
     std::unique_ptr<PredictorInterface> dictionary_predictor,
-    std::unique_ptr<PredictorInterface> user_history_predictor)
+    std::unique_ptr<PredictorInterface> user_history_predictor,
+    const ConverterInterface &converter)
     : dictionary_predictor_(std::move(dictionary_predictor)),
-      user_history_predictor_(std::move(user_history_predictor)) {
+      user_history_predictor_(std::move(user_history_predictor)),
+      converter_(converter) {
   DCHECK(dictionary_predictor_);
   DCHECK(user_history_predictor_);
 }
 
-BasePredictor::~BasePredictor() {}
-
 void BasePredictor::Finish(const ConversionRequest &request,
                            Segments *segments) {
+  PopulateReadingOfCommittedCandidateIfMissing(segments);
+
   user_history_predictor_->Finish(request, segments);
   dictionary_predictor_->Finish(request, segments);
 
@@ -125,8 +151,8 @@ bool BasePredictor::ClearUnusedHistory() {
   return user_history_predictor_->ClearUnusedHistory();
 }
 
-bool BasePredictor::ClearHistoryEntry(const std::string &key,
-                                      const std::string &value) {
+bool BasePredictor::ClearHistoryEntry(const absl::string_view key,
+                                      const absl::string_view value) {
   return user_history_predictor_->ClearHistoryEntry(key, value);
 }
 
@@ -136,19 +162,59 @@ bool BasePredictor::Sync() { return user_history_predictor_->Sync(); }
 
 bool BasePredictor::Reload() { return user_history_predictor_->Reload(); }
 
+void BasePredictor::PopulateReadingOfCommittedCandidateIfMissing(
+    Segments *segments) const {
+  if (segments->conversion_segments_size() == 0) return;
+
+  Segment *segment = segments->mutable_conversion_segment(0);
+  if (segment->candidates_size() == 0) return;
+
+  Segment::Candidate *cand = segment->mutable_candidate(0);
+  if (!cand->key.empty() || cand->value.empty()) return;
+
+  if (cand->content_value == cand->value) {
+    if (std::optional<std::string> key = GetReading(converter_, cand->value);
+        key.has_value()) {
+      cand->key = *key;
+      cand->content_key = *std::move(key);
+    }
+    return;
+  }
+
+  if (cand->content_value.empty()) {
+    LOG(ERROR) << "Content value is empty: " << *cand;
+    return;
+  }
+
+  const absl::string_view functional_value = cand->functional_value();
+  if (Util::GetScriptType(functional_value) != Util::HIRAGANA) {
+    LOG(ERROR) << "The functional value is not hiragana: " << *cand;
+    return;
+  }
+  if (std::optional<std::string> content_key =
+          GetReading(converter_, cand->content_value);
+      content_key.has_value()) {
+    cand->key = absl::StrCat(*content_key, functional_value);
+    cand->content_key = *std::move(content_key);
+  }
+}
+
 // static
 std::unique_ptr<PredictorInterface> DefaultPredictor::CreateDefaultPredictor(
     std::unique_ptr<PredictorInterface> dictionary_predictor,
-    std::unique_ptr<PredictorInterface> user_history_predictor) {
+    std::unique_ptr<PredictorInterface> user_history_predictor,
+    const ConverterInterface &converter) {
   return std::make_unique<DefaultPredictor>(std::move(dictionary_predictor),
-                                            std::move(user_history_predictor));
+                                            std::move(user_history_predictor),
+                                            converter);
 }
 
 DefaultPredictor::DefaultPredictor(
     std::unique_ptr<PredictorInterface> dictionary_predictor,
-    std::unique_ptr<PredictorInterface> user_history_predictor)
+    std::unique_ptr<PredictorInterface> user_history_predictor,
+    const ConverterInterface &converter)
     : BasePredictor(std::move(dictionary_predictor),
-                    std::move(user_history_predictor)),
+                    std::move(user_history_predictor), converter),
       predictor_name_("DefaultPredictor") {}
 
 DefaultPredictor::~DefaultPredictor() = default;
@@ -166,15 +232,20 @@ bool DefaultPredictor::PredictForRequest(const ConversionRequest &request,
 
   int size = kPredictionSize;
   if (request.request_type() == ConversionRequest::SUGGESTION) {
-    size = std::min(9, std::max<int>(1, request.config().suggestions_size()));
+    size = std::clamp<int>(request.config().suggestions_size(), 1, 9);
   }
 
   bool result = false;
   int remained_size = size;
-  ConversionRequest request_for_prediction = request;
-  request_for_prediction.set_max_user_history_prediction_candidates_size(size);
-  request_for_prediction
-      .set_max_user_history_prediction_candidates_size_for_zero_query(size);
+  ConversionRequest::Options options = request.options();
+  options.max_user_history_prediction_candidates_size = size;
+  options.max_user_history_prediction_candidates_size_for_zero_query = size;
+  ConversionRequest request_for_prediction =
+      ConversionRequestBuilder()
+          .SetConversionRequestView(request)
+          .SetOptions(std::move(options))
+          .SetHistorySegmentsView(*segments)
+          .Build();
   result |= user_history_predictor_->PredictForRequest(request_for_prediction,
                                                        segments);
   remained_size = size - static_cast<size_t>(GetCandidatesSize(*segments));
@@ -185,9 +256,15 @@ bool DefaultPredictor::PredictForRequest(const ConversionRequest &request,
     return result;
   }
 
-  request_for_prediction.set_max_dictionary_prediction_candidates_size(
-      remained_size);
-  result |= dictionary_predictor_->PredictForRequest(request_for_prediction,
+  ConversionRequest::Options options2 = request_for_prediction.options();
+  options2.max_dictionary_prediction_candidates_size = remained_size;
+  const ConversionRequest request_for_prediction2 =
+      ConversionRequestBuilder()
+          .SetConversionRequestView(request_for_prediction)
+          .SetOptions(std::move(options2))
+          .SetHistorySegmentsView(*segments)
+          .Build();
+  result |= dictionary_predictor_->PredictForRequest(request_for_prediction2,
                                                      segments);
   return result;
 }
@@ -195,53 +272,92 @@ bool DefaultPredictor::PredictForRequest(const ConversionRequest &request,
 // static
 std::unique_ptr<PredictorInterface> MobilePredictor::CreateMobilePredictor(
     std::unique_ptr<PredictorInterface> dictionary_predictor,
-    std::unique_ptr<PredictorInterface> user_history_predictor) {
+    std::unique_ptr<PredictorInterface> user_history_predictor,
+    const ConverterInterface &converter) {
   return std::make_unique<MobilePredictor>(std::move(dictionary_predictor),
-                                           std::move(user_history_predictor));
+                                           std::move(user_history_predictor),
+                                           converter);
 }
 
 MobilePredictor::MobilePredictor(
     std::unique_ptr<PredictorInterface> dictionary_predictor,
-    std::unique_ptr<PredictorInterface> user_history_predictor)
+    std::unique_ptr<PredictorInterface> user_history_predictor,
+    const ConverterInterface &converter)
     : BasePredictor(std::move(dictionary_predictor),
-                    std::move(user_history_predictor)),
+                    std::move(user_history_predictor), converter),
       predictor_name_("MobilePredictor") {}
 
-MobilePredictor::~MobilePredictor() {}
+MobilePredictor::~MobilePredictor() = default;
 
 ConversionRequest MobilePredictor::GetRequestForPredict(
     const ConversionRequest &request, const Segments &segments) {
-  ConversionRequest ret = request;
+  ConversionRequest::Options options = request.options();
   size_t history_prediction_size = GetHistoryPredictionSizeFromRequest(request);
   switch (request.request_type()) {
     case ConversionRequest::SUGGESTION: {
-      ret.set_max_user_history_prediction_candidates_size(
-          history_prediction_size);
-      ret.set_max_user_history_prediction_candidates_size_for_zero_query(4);
-      ret.set_max_dictionary_prediction_candidates_size(20);
+      options.max_user_history_prediction_candidates_size =
+          history_prediction_size;
+      options.max_user_history_prediction_candidates_size_for_zero_query = 4;
+      options.max_dictionary_prediction_candidates_size = 20;
       break;
     }
     case ConversionRequest::PARTIAL_SUGGESTION: {
-      ret.set_max_dictionary_prediction_candidates_size(20);
+      options.max_dictionary_prediction_candidates_size = 20;
       break;
     }
     case ConversionRequest::PARTIAL_PREDICTION: {
-      ret.set_max_dictionary_prediction_candidates_size(kMobilePredictionSize);
+      options.max_dictionary_prediction_candidates_size = kMobilePredictionSize;
       break;
     }
     case ConversionRequest::PREDICTION: {
-      ret.set_max_user_history_prediction_candidates_size(
-          history_prediction_size);
-      ret.set_max_user_history_prediction_candidates_size_for_zero_query(4);
-      ret.set_max_dictionary_prediction_candidates_size(kMobilePredictionSize);
+      options.max_user_history_prediction_candidates_size =
+          history_prediction_size;
+      options.max_user_history_prediction_candidates_size_for_zero_query = 4;
+      options.max_dictionary_prediction_candidates_size = kMobilePredictionSize;
       break;
     }
     default:
-      DLOG(ERROR) << "Never reach here.";
+      DLOG(ERROR) << "Unexpected request type: " << request.request_type();
   }
-  return ret;
+  return ConversionRequestBuilder()
+      .SetConversionRequestView(request)
+      .SetOptions(std::move(options))
+      .SetHistorySegmentsView(segments)
+      .Build();
 }
 
+namespace {
+// Fills empty lid and rid of candidates with the candidates of the same value.
+void MaybeFillFallbackPos(Segments *segments) {
+  for (Segment &segment : segments->conversion_segments()) {
+    absl::flat_hash_map<absl::string_view, Segment::Candidate *> posless_cands;
+    for (size_t ci = 0; ci < segment.candidates_size(); ++ci) {
+      Segment::Candidate *cand = segment.mutable_candidate(ci);
+      // Candidates with empty POS come before candidates with filled POS.
+      if (cand->lid == 0 || cand->rid == 0) {
+        posless_cands[cand->value] = cand;
+        continue;
+      }
+      if (!posless_cands.contains(cand->value)) {
+        continue;
+      }
+
+      Segment::Candidate *posless_cand = posless_cands[cand->value];
+      if (posless_cand->lid == 0) {
+        posless_cand->lid = cand->lid;
+      }
+      if (posless_cand->rid == 0) {
+        posless_cand->rid = cand->rid;
+      }
+      if (posless_cand->lid != 0 && posless_cand->rid != 0) {
+        posless_cands.erase(cand->value);
+      }
+      MOZC_CANDIDATE_LOG(cand, "Fallback POSes were filled.");
+    }
+  }
+}
+}  // namespace
+
 bool MobilePredictor::PredictForRequest(const ConversionRequest &request,
                                         Segments *segments) const {
   DCHECK(request.request_type() == ConversionRequest::PREDICTION ||
@@ -289,11 +405,12 @@ bool MobilePredictor::PredictForRequest(const ConversionRequest &request,
                                                          segments);
       break;
     }
-    default: {
-    }  // Never reach here
+    default:
+      DLOG(FATAL) << "Unexpected request type: " << request.request_type();
   }
 
+  MaybeFillFallbackPos(segments);
   return result;
 }
 
-}  // namespace mozc
+}  // namespace mozc::prediction
