@@ -33,17 +33,20 @@
 #include <memory>
 #include <string>
 
+#include "absl/strings/string_view.h"
+#include "converter/converter_interface.h"
+#include "converter/segments.h"
 #include "prediction/predictor_interface.h"
 #include "request/conversion_request.h"
 
-namespace mozc {
+namespace mozc::prediction {
 
 class BasePredictor : public PredictorInterface {
  public:
   // Initializes the composite of predictor with given sub-predictors.
   BasePredictor(std::unique_ptr<PredictorInterface> dictionary_predictor,
-                std::unique_ptr<PredictorInterface> user_history_predictor);
-  ~BasePredictor() override;
+                std::unique_ptr<PredictorInterface> user_history_predictor,
+                const ConverterInterface &converter);
 
   // Hook(s) for all mutable operations.
   void Finish(const ConversionRequest &request, Segments *segments) override;
@@ -58,13 +61,13 @@ class BasePredictor : public PredictorInterface {
   bool ClearUnusedHistory() override;
 
   // Clears a specific user history data of UserHistoryPredictor.
-  bool ClearHistoryEntry(const std::string &key,
-                         const std::string &value) override;
+  bool ClearHistoryEntry(absl::string_view key,
+                         absl::string_view value) override;
 
   // Syncs user history.
   bool Sync() override;
 
-  // Reloads usre history.
+  // Reloads user history.
   bool Reload() override;
 
   // Waits for syncer to complete.
@@ -78,6 +81,11 @@ class BasePredictor : public PredictorInterface {
  protected:
   std::unique_ptr<PredictorInterface> dictionary_predictor_;
   std::unique_ptr<PredictorInterface> user_history_predictor_;
+
+ private:
+  void PopulateReadingOfCommittedCandidateIfMissing(Segments *segments) const;
+
+  const ConverterInterface &converter_;
 };
 
 // TODO(team): The name should be DesktopPredictor
@@ -85,16 +93,18 @@ class DefaultPredictor : public BasePredictor {
  public:
   static std::unique_ptr<PredictorInterface> CreateDefaultPredictor(
       std::unique_ptr<PredictorInterface> dictionary_predictor,
-      std::unique_ptr<PredictorInterface> user_history_predictor);
+      std::unique_ptr<PredictorInterface> user_history_predictor,
+      const ConverterInterface &converter);
 
   DefaultPredictor(std::unique_ptr<PredictorInterface> dictionary_predictor,
-                   std::unique_ptr<PredictorInterface> user_history_predictor);
+                   std::unique_ptr<PredictorInterface> user_history_predictor,
+                   const ConverterInterface &converter);
   ~DefaultPredictor() override;
 
-  bool PredictForRequest(const ConversionRequest &request,
-                         Segments *segments) const override;
+  [[nodiscard]] bool PredictForRequest(const ConversionRequest &request,
+                                       Segments *segments) const override;
 
-  const std::string &GetPredictorName() const override {
+  absl::string_view GetPredictorName() const override {
     return predictor_name_;
   }
 
@@ -106,16 +116,18 @@ class MobilePredictor : public BasePredictor {
  public:
   static std::unique_ptr<PredictorInterface> CreateMobilePredictor(
       std::unique_ptr<PredictorInterface> dictionary_predictor,
-      std::unique_ptr<PredictorInterface> user_history_predictor);
+      std::unique_ptr<PredictorInterface> user_history_predictor,
+      const ConverterInterface &converter);
 
   MobilePredictor(std::unique_ptr<PredictorInterface> dictionary_predictor,
-                  std::unique_ptr<PredictorInterface> user_history_predictor);
+                  std::unique_ptr<PredictorInterface> user_history_predictor,
+                  const ConverterInterface &converter);
   ~MobilePredictor() override;
 
-  bool PredictForRequest(const ConversionRequest &request,
-                         Segments *segments) const override;
+  [[nodiscard]] bool PredictForRequest(const ConversionRequest &request,
+                                       Segments *segments) const override;
 
-  const std::string &GetPredictorName() const override {
+  absl::string_view GetPredictorName() const override {
     return predictor_name_;
   }
 
@@ -126,6 +138,6 @@ class MobilePredictor : public BasePredictor {
   const std::string predictor_name_;
 };
 
-}  // namespace mozc
+}  // namespace mozc::prediction
 
 #endif  // MOZC_PREDICTION_PREDICTOR_H_
