@@ -33,30 +33,28 @@
 #include <string>
 #include <utility>
 
-#include "base/logging.h"
+#include "absl/log/check.h"
+#include "absl/strings/string_view.h"
 #include "composer/composer.h"
 #include "config/config_handler.h"
 #include "converter/converter_mock.h"
 #include "converter/segments.h"
 #include "data_manager/testing/mock_data_manager.h"
 #include "dictionary/dictionary_mock.h"
-#include "dictionary/pos_matcher.h"
-#include "dictionary/suppression_dictionary.h"
+#include "engine/modules.h"
 #include "prediction/predictor_interface.h"
 #include "prediction/user_history_predictor.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "request/conversion_request.h"
-#include "session/request_test_util.h"
+#include "request/request_test_util.h"
 #include "testing/gmock.h"
 #include "testing/gunit.h"
-#include "absl/strings/string_view.h"
 
 namespace mozc::prediction {
 namespace {
 
 using ::mozc::dictionary::MockDictionary;
-using ::mozc::dictionary::SuppressionDictionary;
 using ::testing::_;
 using ::testing::AtMost;
 using ::testing::DoAll;
@@ -78,7 +76,7 @@ class CheckCandSizeDictionaryPredictor : public PredictorInterface {
     return true;
   }
 
-  const std::string &GetPredictorName() const override {
+  absl::string_view GetPredictorName() const override {
     return predictor_name_;
   }
 
@@ -105,7 +103,7 @@ class CheckCandSizeUserHistoryPredictor : public PredictorInterface {
     return true;
   }
 
-  const std::string &GetPredictorName() const override {
+  absl::string_view GetPredictorName() const override {
     return predictor_name_;
   }
 
@@ -132,7 +130,7 @@ class NullPredictor : public PredictorInterface {
 
   void Clear() { predict_called_ = false; }
 
-  const std::string &GetPredictorName() const override {
+  absl::string_view GetPredictorName() const override {
     return predictor_name_;
   }
 
@@ -149,7 +147,7 @@ class MockPredictor : public PredictorInterface {
   MOCK_METHOD(bool, PredictForRequest,
               (const ConversionRequest &request, Segments *segments),
               (const, override));
-  MOCK_METHOD(const std::string &, GetPredictorName, (), (const, override));
+  MOCK_METHOD(absl::string_view, GetPredictorName, (), (const, override));
 };
 
 }  // namespace
@@ -161,81 +159,92 @@ class MobilePredictorTest : public ::testing::Test {
     config::ConfigHandler::GetDefaultConfig(config_.get());
 
     request_ = std::make_unique<commands::Request>();
-    commands::RequestForUnitTest::FillMobileRequest(request_.get());
-    composer_ = std::make_unique<composer::Composer>(nullptr, request_.get(),
-                                                     config_.get());
+    request_test_util::FillMobileRequest(request_.get());
+    composer_ = std::make_unique<composer::Composer>(*request_, *config_);
+  }
 
-    convreq_ = std::make_unique<ConversionRequest>(
-        composer_.get(), request_.get(), config_.get());
+  ConversionRequest CreateConversionRequest(
+      ConversionRequest::RequestType request_type) const {
+    ConversionRequest::Options options = {.request_type = request_type};
+    return ConversionRequestBuilder()
+        .SetComposer(*composer_)
+        .SetRequestView(*request_)
+        .SetContextView(context_)
+        .SetConfigView(*config_)
+        .SetOptions(std::move(options))
+        .Build();
   }
 
   std::unique_ptr<mozc::composer::Composer> composer_;
   std::unique_ptr<commands::Request> request_;
   std::unique_ptr<config::Config> config_;
-  std::unique_ptr<ConversionRequest> convreq_;
+  commands::Context context_;
 };
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobileSuggestion) {
   MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(20),
-      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4), &converter);
+      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobilePartialSuggestion) {
   MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(20),
-      // We don't call history predictior
-      std::make_unique<CheckCandSizeUserHistoryPredictor>(-1, -1), &converter);
+      // We don't call history predictor
+      std::make_unique<CheckCandSizeUserHistoryPredictor>(-1, -1), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::PARTIAL_SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::PARTIAL_SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobilePrediction) {
   MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(200),
-      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4), &converter);
+      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::PREDICTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::PREDICTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobilePartialPrediction) {
   MockConverter converter;
-  MockDictionary dictionary_mock;
-  testing::MockDataManager data_manager;
-  const dictionary::PosMatcher pos_matcher(data_manager.GetPosMatcherData());
-  const SuppressionDictionary suppression_dictionary;
+  std::unique_ptr<engine::Modules> modules =
+      engine::ModulesPresetBuilder()
+          .PresetDictionary(std::make_unique<MockDictionary>())
+          .Build(std::make_unique<testing::MockDataManager>())
+          .value();
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(200),
-      std::make_unique<UserHistoryPredictor>(&dictionary_mock, &pos_matcher,
-                                             &suppression_dictionary, true),
-      &converter);
+      std::make_unique<UserHistoryPredictor>(*modules, true), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::PARTIAL_PREDICTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::PARTIAL_PREDICTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(MobilePredictorTest, CallPredictForRequestMobile) {
@@ -250,14 +259,15 @@ TEST_F(MobilePredictorTest, CallPredictForRequestMobile) {
 
   MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
-      std::move(predictor1), std::move(predictor2), &converter);
+      std::move(predictor1), std::move(predictor2), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 class PredictorTest : public ::testing::Test {
@@ -267,58 +277,69 @@ class PredictorTest : public ::testing::Test {
     config::ConfigHandler::GetDefaultConfig(config_.get());
 
     request_ = std::make_unique<commands::Request>();
-    composer_ = std::make_unique<composer::Composer>(nullptr, request_.get(),
-                                                     config_.get());
-    convreq_ = std::make_unique<ConversionRequest>(
-        composer_.get(), request_.get(), config_.get());
+    composer_ = std::make_unique<composer::Composer>(*request_, *config_);
+  }
+
+  ConversionRequest CreateConversionRequest(
+      ConversionRequest::RequestType request_type) const {
+    ConversionRequest::Options options = {.request_type = request_type};
+    return ConversionRequestBuilder()
+        .SetComposer(*composer_)
+        .SetRequestView(*request_)
+        .SetContextView(context_)
+        .SetConfigView(*config_)
+        .SetOptions(std::move(options))
+        .Build();
   }
 
   std::unique_ptr<mozc::composer::Composer> composer_;
   std::unique_ptr<commands::Request> request_;
   std::unique_ptr<config::Config> config_;
-  std::unique_ptr<ConversionRequest> convreq_;
+  commands::Context context_;
 };
 
 TEST_F(PredictorTest, AllPredictorsReturnTrue) {
   MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<NullPredictor>(true),
-      std::make_unique<NullPredictor>(true), &converter);
+      std::make_unique<NullPredictor>(true), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
 }
 
 TEST_F(PredictorTest, MixedReturnValue) {
   MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<NullPredictor>(true),
-      std::make_unique<NullPredictor>(false), &converter);
+      std::make_unique<NullPredictor>(false), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(PredictorTest, AllPredictorsReturnFalse) {
   MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<NullPredictor>(false),
-      std::make_unique<NullPredictor>(false), &converter);
+      std::make_unique<NullPredictor>(false), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_FALSE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_FALSE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(PredictorTest, CallPredictorsForSuggestion) {
@@ -329,14 +350,15 @@ TEST_F(PredictorTest, CallPredictorsForSuggestion) {
       std::make_unique<CheckCandSizeDictionaryPredictor>(suggestions_size),
       std::make_unique<CheckCandSizeUserHistoryPredictor>(suggestions_size,
                                                           suggestions_size),
-      &converter);
+      converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(PredictorTest, CallPredictorsForPrediction) {
@@ -346,14 +368,15 @@ TEST_F(PredictorTest, CallPredictorsForPrediction) {
       std::make_unique<CheckCandSizeDictionaryPredictor>(kPredictionSize),
       std::make_unique<CheckCandSizeUserHistoryPredictor>(kPredictionSize,
                                                           kPredictionSize),
-      &converter);
+      converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::PREDICTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::PREDICTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(PredictorTest, CallPredictForRequest) {
@@ -368,14 +391,15 @@ TEST_F(PredictorTest, CallPredictForRequest) {
 
   MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
-      std::move(predictor1), std::move(predictor2), &converter);
+      std::move(predictor1), std::move(predictor2), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 }
 
 TEST_F(PredictorTest, DisableAllSuggestion) {
@@ -385,21 +409,24 @@ TEST_F(PredictorTest, DisableAllSuggestion) {
   const auto *pred2 = predictor2.get();  // Keep the reference
   MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
-      std::move(predictor1), std::move(predictor2), &converter);
+      std::move(predictor1), std::move(predictor2), converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
     CHECK(segment);
   }
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
-
   config_->set_presentation_mode(true);
-  EXPECT_FALSE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq1 =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+
+  EXPECT_FALSE(predictor->PredictForRequest(convreq1, &segments));
   EXPECT_FALSE(pred1->predict_called());
   EXPECT_FALSE(pred2->predict_called());
 
   config_->set_presentation_mode(false);
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  const ConversionRequest convreq2 =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
+  EXPECT_TRUE(predictor->PredictForRequest(convreq2, &segments));
   EXPECT_TRUE(pred1->predict_called());
   EXPECT_TRUE(pred2->predict_called());
 }
@@ -408,7 +435,7 @@ TEST_F(PredictorTest, PopulateReadingOfCommittedCandidateIfMissing) {
   MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<NullPredictor>(true),
-      std::make_unique<NullPredictor>(true), &converter);
+      std::make_unique<NullPredictor>(true), converter);
 
   // Mock reverse conversion adds reading "とうきょう".
   EXPECT_CALL(converter, StartReverseConversion(_, StrEq("東京")))
@@ -434,7 +461,9 @@ TEST_F(PredictorTest, PopulateReadingOfCommittedCandidateIfMissing) {
     cand3->value = "群馬";
     cand3->content_value = "群馬";
 
-    predictor->Finish(*convreq_, &segments);
+    const ConversionRequest convreq =
+        CreateConversionRequest(ConversionRequest::CONVERSION);
+    predictor->Finish(convreq, &segments);
     EXPECT_EQ(cand1->key, "とうきょう");
     EXPECT_EQ(cand1->content_key, "とうきょう");
     EXPECT_TRUE(cand2->key.empty());
@@ -459,7 +488,9 @@ TEST_F(PredictorTest, PopulateReadingOfCommittedCandidateIfMissing) {
     cand3->value = "群馬に";
     cand3->content_value = "群馬";
 
-    predictor->Finish(*convreq_, &segments);
+    const ConversionRequest convreq =
+        CreateConversionRequest(ConversionRequest::CONVERSION);
+    predictor->Finish(convreq, &segments);
     EXPECT_EQ(cand1->key, "とうきょうに");
     EXPECT_EQ(cand1->content_key, "とうきょう");
     EXPECT_TRUE(cand2->key.empty());
@@ -477,7 +508,9 @@ TEST_F(PredictorTest, PopulateReadingOfCommittedCandidateIfMissing) {
     cand1->value = "東京便";
     cand1->content_value = "東京";
 
-    predictor->Finish(*convreq_, &segments);
+    const ConversionRequest convreq =
+        CreateConversionRequest(ConversionRequest::CONVERSION);
+    predictor->Finish(convreq, &segments);
     EXPECT_TRUE(cand1->key.empty());
     EXPECT_TRUE(cand1->content_key.empty());
   }
@@ -490,7 +523,9 @@ TEST_F(PredictorTest, PopulateReadingOfCommittedCandidateIfMissing) {
     cand1->value = "東京";
     cand1->content_value.clear();
 
-    predictor->Finish(*convreq_, &segments);
+    const ConversionRequest convreq =
+        CreateConversionRequest(ConversionRequest::CONVERSION);
+    predictor->Finish(convreq, &segments);
     EXPECT_TRUE(cand1->key.empty());
     EXPECT_TRUE(cand1->content_key.empty());
   }
@@ -531,11 +566,12 @@ TEST_F(MobilePredictorTest, FillPos) {
   MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::move(mock_dictionary_predictor), std::move(mock_history_predictor),
-      &converter);
+      converter);
 
-  convreq_->set_request_type(ConversionRequest::SUGGESTION);
+  const ConversionRequest convreq =
+      CreateConversionRequest(ConversionRequest::SUGGESTION);
   Segments segments;
-  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+  EXPECT_TRUE(predictor->PredictForRequest(convreq, &segments));
 
   EXPECT_EQ(segments.conversion_segments_size(), 1);
   EXPECT_EQ(segments.conversion_segment(0).candidates_size(), 2);
