@@ -31,42 +31,39 @@
 #define MOZC_PREDICTION_USER_HISTORY_PREDICTOR_H_
 
 #include <atomic>
+#include <cstddef>
 #include <cstdint>
 #include <memory>
+#include <optional>
 #include <queue>
-#include <set>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "base/freelist.h"
-#include "base/trie.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/strings/string_view.h"
+#include "base/container/freelist.h"
+#include "base/container/trie.h"
+#include "base/thread.h"
+#include "composer/query.h"
+#include "converter/segments.h"
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_matcher.h"
-#include "dictionary/suppression_dictionary.h"
+#include "engine/modules.h"
 #include "prediction/predictor_interface.h"
 #include "prediction/user_history_predictor.pb.h"
+#include "request/conversion_request.h"
+#include "storage/encrypted_string_storage.h"
 #include "storage/lru_cache.h"
-#include "absl/strings/string_view.h"
-// for FRIEND_TEST
-#include "testing/base/public/gunit_prod.h"
+#include "testing/friend_test.h"  // IWYU pragma: keep
 
-namespace mozc {
-
-namespace storage {
-class StringStorageInterface;
-}  // namespace storage
-
-class ConversionRequest;
-class Segment;
-class Segments;
-class UserHistoryPredictorSyncer;
+namespace mozc::prediction {
 
 // Added serialization method for UserHistory.
 class UserHistoryStorage {
  public:
-  explicit UserHistoryStorage(const std::string &filename);
-  ~UserHistoryStorage();
+  explicit UserHistoryStorage(const absl::string_view filename)
+      : storage_(filename) {}
 
   // Loads from encrypted file.
   bool Load();
@@ -88,7 +85,7 @@ class UserHistoryStorage {
   }
 
  private:
-  std::unique_ptr<storage::StringStorageInterface> storage_;
+  storage::EncryptedStringStorage storage_;
   mozc::user_history_predictor::UserHistory proto_;
 };
 
@@ -99,11 +96,8 @@ class UserHistoryStorage {
 // called by multiple-threads at the same time
 class UserHistoryPredictor : public PredictorInterface {
  public:
-  UserHistoryPredictor(
-      const dictionary::DictionaryInterface *dictionary,
-      const dictionary::PosMatcher *pos_matcher,
-      const dictionary::SuppressionDictionary *suppression_dictionary,
-      bool enable_content_word_learning);
+  UserHistoryPredictor(const engine::Modules &modules,
+                       bool enable_content_word_learning);
   ~UserHistoryPredictor() override;
 
   void set_content_word_learning_enabled(bool value) {
@@ -135,8 +129,8 @@ class UserHistoryPredictor : public PredictorInterface {
   bool ClearUnusedHistory() override;
 
   // Clears a specific history entry.
-  bool ClearHistoryEntry(const std::string &key,
-                         const std::string &value) override;
+  bool ClearHistoryEntry(absl::string_view key,
+                         absl::string_view value) override;
 
   // Implements PredictorInterface.
   bool Wait() override;
@@ -144,18 +138,18 @@ class UserHistoryPredictor : public PredictorInterface {
   // Gets user history filename.
   static std::string GetUserHistoryFileName();
 
-  const std::string &GetPredictorName() const override {
+  absl::string_view GetPredictorName() const override {
     return predictor_name_;
   }
 
   // From user_history_predictor.proto
-  typedef user_history_predictor::UserHistory::Entry Entry;
-  typedef user_history_predictor::UserHistory::NextEntry NextEntry;
-  typedef user_history_predictor::UserHistory::Entry::EntryType EntryType;
+  using Entry = user_history_predictor::UserHistory::Entry;
+  using NextEntry = user_history_predictor::UserHistory::NextEntry;
+  using EntryType = user_history_predictor::UserHistory::Entry::EntryType;
 
   // Returns fingerprints from various object.
-  static uint32_t Fingerprint(const std::string &key, const std::string &value);
-  static uint32_t Fingerprint(const std::string &key, const std::string &value,
+  static uint32_t Fingerprint(absl::string_view key, absl::string_view value);
+  static uint32_t Fingerprint(absl::string_view key, absl::string_view value,
                               EntryType type);
   static uint32_t EntryFingerprint(const Entry &entry);
   static uint32_t SegmentFingerprint(const Segment &segment);
@@ -164,7 +158,7 @@ class UserHistoryPredictor : public PredictorInterface {
   static uint32_t cache_size();
 
   // Returns the size of next entries.
-  static uint32_t max_next_entries_size();
+  uint32_t max_next_entries_size() const;
 
  private:
   struct SegmentForLearning {
@@ -174,54 +168,22 @@ class UserHistoryPredictor : public PredictorInterface {
     std::string content_value;
     std::string description;
   };
+
+  // Fingerprint of key/value.
   static uint32_t LearningSegmentFingerprint(const SegmentForLearning &segment);
 
-  class SegmentsForLearning {
-   public:
-    SegmentsForLearning() {}
+  // Fingerprints of key/value and content_key/content_value.
+  std::vector<uint32_t> LearningSegmentFingerprints(
+      const SegmentForLearning &segment) const;
 
-    virtual ~SegmentsForLearning() {}
+  struct SegmentsForLearning {
+    std::string conversion_segments_key;
+    std::string conversion_segments_value;
 
-    void push_back_history_segment(const SegmentForLearning &val) {
-      history_segments_.push_back(val);
-    }
-
-    void push_back_conversion_segment(const SegmentForLearning &val) {
-      conversion_segments_.push_back(val);
-    }
-
-    size_t history_segments_size() const { return history_segments_.size(); }
-
-    size_t conversion_segments_size() const {
-      return conversion_segments_.size();
-    }
-
-    size_t all_segments_size() const {
-      return history_segments_size() + conversion_segments_size();
-    }
-
-    const SegmentForLearning &history_segment(size_t i) const {
-      return history_segments_[i];
-    }
-
-    const SegmentForLearning &conversion_segment(size_t i) const {
-      return conversion_segments_[i];
-    }
-
-    const SegmentForLearning &all_segment(size_t i) const {
-      if (i < history_segments_size()) {
-        return history_segment(i);
-      } else {
-        return conversion_segments_[i - history_segments_size()];
-      }
-    }
-
-   private:
-    std::vector<SegmentForLearning> history_segments_;
-    std::vector<SegmentForLearning> conversion_segments_;
+    std::vector<SegmentForLearning> history_segments;
+    std::vector<SegmentForLearning> conversion_segments;
   };
 
-  friend class UserHistoryPredictorSyncer;
   friend class UserHistoryPredictorTest;
 
   FRIEND_TEST(UserHistoryPredictorTest, UserHistoryPredictorTestSuggestion);
@@ -230,6 +192,7 @@ class UserHistoryPredictor : public PredictorInterface {
   FRIEND_TEST(UserHistoryPredictorTest, GetScore);
   FRIEND_TEST(UserHistoryPredictorTest, IsValidEntry);
   FRIEND_TEST(UserHistoryPredictorTest, IsValidSuggestion);
+  FRIEND_TEST(UserHistoryPredictorTest, IsValidSuggestionForMixedConversion);
   FRIEND_TEST(UserHistoryPredictorTest, EntryPriorityQueueTest);
   FRIEND_TEST(UserHistoryPredictorTest, RomanFuzzyPrefixMatch);
   FRIEND_TEST(UserHistoryPredictorTest, MaybeRomanMisspelledKey);
@@ -261,6 +224,7 @@ class UserHistoryPredictor : public PredictorInterface {
   FRIEND_TEST(UserHistoryPredictorTest,
               ClearHistoryEntryTrigramDeleteSecondBigram);
   FRIEND_TEST(UserHistoryPredictorTest, 62DayOldEntriesAreDeletedAtSync);
+  FRIEND_TEST(UserHistoryPredictorTest, TypingCorrection);
 
   enum MatchType {
     NO_MATCH,            // no match
@@ -283,6 +247,13 @@ class UserHistoryPredictor : public PredictorInterface {
     NOT_FOUND,
   };
 
+  // Result type for IsValidCandidate() check.
+  enum ResultType {
+    GOOD_RESULT,
+    BAD_RESULT,
+    STOP_ENUMERATION,  // Do not insert and stop enumerations
+  };
+
   // Returns true if this predictor should return results for the input.
   bool ShouldPredict(RequestType request_type, const ConversionRequest &request,
                      const Segments &segments) const;
@@ -310,18 +281,16 @@ class UserHistoryPredictor : public PredictorInterface {
   static uint16_t revert_id();
 
   // Gets match type from two strings
-  static MatchType GetMatchType(const std::string &lstr,
-                                const std::string &rstr);
+  static MatchType GetMatchType(absl::string_view lstr, absl::string_view rstr);
 
   // Gets match type with ambiguity expansion
-  static MatchType GetMatchTypeFromInput(const std::string &input_key,
-                                         const std::string &key_base,
+  static MatchType GetMatchTypeFromInput(absl::string_view input_key,
+                                         absl::string_view key_base,
                                          const Trie<std::string> *key_expanded,
-                                         const std::string &target);
+                                         absl::string_view target);
 
   // Uint32 <=> string conversion
   static std::string Uint32ToString(uint32_t fp);
-  static uint32_t StringToUint32(const std::string &input);
 
   // Returns true if prev_entry has a next_fp link to entry
   static bool HasBigramEntry(const Entry &entry, const Entry &prev_entry);
@@ -331,12 +300,21 @@ class UserHistoryPredictor : public PredictorInterface {
   static bool IsValidSuggestion(RequestType request_type, uint32_t prefix_len,
                                 const Entry &entry);
 
+  // IsValidSuggestion used in mixed conversion (mobile).
+  static bool IsValidSuggestionForMixedConversion(
+      const ConversionRequest &request, uint32_t prefix_len,
+      const Entry &entry);
+
+  static ResultType GetResultType(const ConversionRequest &request,
+                                  RequestType request_type,
+                                  bool is_top_candidate, uint32_t input_key_len,
+                                  const Entry &entry);
+
   // Returns true if entry is DEFAULT_ENTRY, satisfies certain conditions, and
   // doesn't have removed flag.
-  bool IsValidEntry(const Entry &entry, uint32_t available_emoji_carrier) const;
+  bool IsValidEntry(const Entry &entry) const;
   // The same as IsValidEntry except that removed field is ignored.
-  bool IsValidEntryIgnoringRemovedField(const Entry &entry,
-                                        uint32_t available_emoji_carrier) const;
+  bool IsValidEntryIgnoringRemovedField(const Entry &entry) const;
 
   // Returns "tweaked" score of result_entry.
   // the score is basically determined by "last_access_time", (a.k.a,
@@ -348,26 +326,29 @@ class UserHistoryPredictor : public PredictorInterface {
   // Priority Queue class for entry. New item is sorted by
   // |score| internally. By calling Pop() in sequence, you
   // can obtain the list of entry sorted by |score|.
-  class EntryPriorityQueue {
+  class EntryPriorityQueue final {
    public:
-    EntryPriorityQueue();
-    virtual ~EntryPriorityQueue();
+    EntryPriorityQueue() : pool_(kEntryPoolSize) {}
     size_t size() const { return agenda_.size(); }
     bool Push(Entry *entry);
     Entry *Pop();
     Entry *NewEntry();
 
    private:
+    using QueueElement = std::pair<uint32_t, Entry *>;
+    using Agenda = std::priority_queue<QueueElement>;
+
     friend class UserHistoryPredictor;
-    typedef std::pair<uint32_t, Entry *> QueueElement;
-    typedef std::priority_queue<QueueElement> Agenda;
+
+    // Default object pool size for EntryPriorityQueue
+    static constexpr size_t kEntryPoolSize = 16;
     Agenda agenda_;
     FreeList<Entry> pool_;
-    std::set<uint32_t> seen_;
+    absl::flat_hash_set<size_t> seen_;
   };
 
-  typedef mozc::storage::LruCache<uint32_t, Entry> DicCache;
-  typedef DicCache::Element DicElement;
+  using DicCache = mozc::storage::LruCache<uint32_t, Entry>;
+  using DicElement = DicCache::Element;
 
   bool CheckSyncerAndDelete() const;
 
@@ -377,8 +358,8 @@ class UserHistoryPredictor : public PredictorInterface {
   // |prev_entry| is an optional field. If set nullptr, this field is just
   // ignored. This method adds a new result entry with score,
   // pair<score, entry>, to |results|.
-  bool LookupEntry(RequestType request_type, const std::string &input_key,
-                   const std::string &key_base,
+  bool LookupEntry(RequestType request_type, absl::string_view input_key,
+                   absl::string_view key_base,
                    const Trie<std::string> *key_expanded, const Entry *entry,
                    const Entry *prev_entry, EntryPriorityQueue *results) const;
 
@@ -391,20 +372,19 @@ class UserHistoryPredictor : public PredictorInterface {
   // |left_last_access_time| and |left_most_last_access_time| will be updated
   // according to the entry lookup.
   bool GetKeyValueForExactAndRightPrefixMatch(
-      const std::string &input_key, const Entry *entry,
+      absl::string_view input_key, const Entry *entry,
       const Entry **result_last_entry, uint64_t *left_last_access_time,
       uint64_t *left_most_last_access_time, std::string *result_key,
       std::string *result_value) const;
 
-  const Entry *LookupPrevEntry(const Segments &segments,
-                               uint32_t available_emoji_carrier) const;
+  const Entry *LookupPrevEntry(const Segments &segments) const;
 
   // Adds an entry to a priority queue.
   Entry *AddEntry(const Entry &entry, EntryPriorityQueue *results) const;
 
   // Adds the entry whose key and value are modified to a priority queue.
-  Entry *AddEntryWithNewKeyValue(const std::string &key,
-                                 const std::string &value, const Entry &entry,
+  Entry *AddEntryWithNewKeyValue(std::string key, std::string value,
+                                 Entry entry,
                                  EntryPriorityQueue *results) const;
 
   void GetResultsFromHistoryDictionary(RequestType request_type,
@@ -423,58 +403,80 @@ class UserHistoryPredictor : public PredictorInterface {
 
   bool InsertCandidates(RequestType request_type,
                         const ConversionRequest &request,
-                        size_t max_prediction_size, Segments *segments,
+                        size_t max_prediction_size,
+                        size_t max_prediction_char_coverage, Segments *segments,
                         EntryPriorityQueue *results) const;
 
-  void MakeLearningSegments(const Segments &segments,
-                            SegmentsForLearning *learning_segments) const;
+  SegmentsForLearning MakeLearningSegments(const Segments &segments) const;
 
   // Returns true if |prefix| is a fuzzy-prefix of |str|.
   // 'Fuzzy' means that
-  // 1) Allows one character deletation in the |prefix|.
+  // 1) Allows one character deletion in the |prefix|.
   // 2) Allows one character swap in the |prefix|.
-  static bool RomanFuzzyPrefixMatch(const std::string &str,
-                                    const std::string &prefix);
+  static bool RomanFuzzyPrefixMatch(absl::string_view str,
+                                    absl::string_view prefix);
 
   // Returns romanized preedit string if the preedit looks
   // misspelled. It first tries to get the preedit string with
   // composer() if composer is available. If not, use the key
   // directory. It also use MaybeRomanMisspelledKey() defined
-  // below to check the preedit looks missspelled or not.
+  // below to check the preedit looks misspelled or not.
   static std::string GetRomanMisspelledKey(const ConversionRequest &request,
                                            const Segments &segments);
 
+  // Returns the typing corrected queries.
+  std::vector<::mozc::composer::TypeCorrectedQuery> GetTypingCorrectedQueries(
+      const ConversionRequest &request, const Segments &segments) const;
+
   // Returns true if |key| may contain miss spelling.
   // Currently, this function returns true if
   // 1) key contains only one alphabet.
   // 2) other characters of key are all hiragana.
-  static bool MaybeRomanMisspelledKey(const std::string &key);
+  static bool MaybeRomanMisspelledKey(absl::string_view key);
 
   // If roman_input_key can be a target key of entry->key(), creat a new
   // result and insert it to |results|.
   // This method adds a new result entry with score, pair<score, entry>, to
   // |results|.
-  bool RomanFuzzyLookupEntry(const std::string &roman_input_key,
+  bool RomanFuzzyLookupEntry(absl::string_view roman_input_key,
                              const Entry *entry,
                              EntryPriorityQueue *results) const;
 
+  // if `prev_entry` is the prefix of `entry`, add the suffix part as
+  // zero-query suggestion.
+  bool ZeroQueryLookupEntry(RequestType request_type,
+                            absl::string_view input_key, const Entry *entry,
+                            const Entry *prev_entry,
+                            EntryPriorityQueue *results) const;
+
   void InsertHistory(RequestType request_type, bool is_suggestion_selected,
                      uint64_t last_access_time, Segments *segments);
 
+  void InsertHistoryForConversionSegments(
+      RequestType request_type, bool is_suggestion_selected,
+      uint64_t last_access_time, const SegmentsForLearning &learning_segments,
+      Segments *segments);
+
   // Inserts |key,value,description| to the internal dictionary database.
   // |is_suggestion_selected|: key/value is suggestion or conversion.
-  // |next_fp|: fingerprint of the next segment.
-  // |last_access_time|: the time when this entrty was created
-  void Insert(const std::string &key, const std::string &value,
-              const std::string &description, bool is_suggestion_selected,
-              uint32_t next_fp, uint64_t last_access_time, Segments *segments);
+  // |next_fp|: fingerprints of the next segment.
+  // |last_access_time|: the time when this entry was created
+  void Insert(std::string key, std::string value, std::string description,
+              bool is_suggestion_selected, absl::Span<const uint32_t> next_fps,
+              uint64_t last_access_time, Segments *segments);
+
+  // Called by TryInsert to check the Entry to insert.
+  bool ShouldInsert(RequestType request_type, absl::string_view key,
+                    absl::string_view value,
+                    absl::string_view description) const;
 
   // Tries to insert entry.
-  // Entry's contents and request_type will be checked before insersion.
-  void TryInsert(RequestType request_type, const std::string &key,
-                 const std::string &value, const std::string &description,
-                 bool is_suggestion_selected, uint32_t next_fp,
-                 uint64_t last_access_time, Segments *segments);
+  // Entry's contents and request_type will be checked before insertion.
+  void TryInsert(RequestType request_type, absl::string_view key,
+                 absl::string_view value, absl::string_view description,
+                 bool is_suggestion_selected,
+                 absl::Span<const uint32_t> next_fps, uint64_t last_access_time,
+                 Segments *segments);
 
   // Inserts event entry (CLEAN_ALL_EVENT|CLEAN_UNUSED_EVENT).
   void InsertEvent(EntryType type);
@@ -486,9 +488,9 @@ class UserHistoryPredictor : public PredictorInterface {
   static void EraseNextEntries(uint32_t fp, Entry *entry);
 
   // Recursively removes a chain of Entries in |dic_|. See the comment in
-  // implemenetation for details.
+  // implementation for details.
   RemoveNgramChainResult RemoveNgramChain(
-      const std::string &target_key, const std::string &target_value,
+      absl::string_view target_key, absl::string_view target_value,
       Entry *entry, std::vector<absl::string_view> *key_ngrams,
       size_t key_ngrams_len, std::vector<absl::string_view> *value_ngrams,
       size_t value_ngrams_len);
@@ -497,19 +499,25 @@ class UserHistoryPredictor : public PredictorInterface {
   // such like password.
   bool IsPrivacySensitive(const Segments *segments) const;
 
-  void MaybeRecordUsageStats(const Segments &segments) const;
+  // Removes history entries when the selected ratio is under the threshold.
+  // Selected ratio:
+  //  (# of candidate committed) / (# of candidate shown on commit event)
+  void MaybeRemoveUnselectedHistory(const Segments &segments);
 
-  const dictionary::DictionaryInterface *dictionary_;
-  const dictionary::PosMatcher *pos_matcher_;
-  const dictionary::SuppressionDictionary *suppression_dictionary_;
+  const dictionary::DictionaryInterface &dictionary_;
+  const dictionary::PosMatcher &pos_matcher_;
+  const dictionary::UserDictionaryInterface &user_dictionary_;
   const std::string predictor_name_;
 
   bool content_word_learning_enabled_;
   mutable std::atomic<bool> updated_;
   std::unique_ptr<DicCache> dic_;
-  mutable std::unique_ptr<UserHistoryPredictorSyncer> syncer_;
+  mutable std::optional<BackgroundFuture<void>> sync_;
+  const engine::Modules &modules_;
+
+  mutable std::atomic<bool> aggressive_bigram_enabled_ = false;
 };
 
-}  // namespace mozc
+}  // namespace mozc::prediction
 
 #endif  // MOZC_PREDICTION_USER_HISTORY_PREDICTOR_H_
