@@ -33,38 +33,42 @@
 #include <cstddef>
 #include <cstdint>
 #include <memory>
+#include <optional>
 #include <string>
 
+#include "absl/base/thread_annotations.h"
+#include "absl/log/log.h"
+#include "absl/synchronization/mutex.h"
+#include "absl/time/clock.h"
+#include "absl/time/time.h"
 #include "base/clock.h"
-#include "base/logging.h"
 #include "base/process.h"
-#include "base/run_level.h"
 #include "base/system_util.h"
 #include "base/thread.h"
-#include "base/util.h"
 #include "base/version.h"
+#include "base/vlog.h"
 #include "ipc/ipc.h"
 #include "ipc/named_event.h"
 #include "protocol/renderer_command.pb.h"
-#include "absl/synchronization/mutex.h"
 
 #ifdef __APPLE__
-#include "base/mac_util.h"
+#include "base/mac/mac_util.h"
 #endif  // __APPLE__
 
-#ifdef OS_WIN
-#include "base/win_sandbox.h"
-#endif  // OS_WIN
+#ifdef _WIN32
+#include "base/run_level.h"
+#include "base/win32/win_sandbox.h"
+#endif  // _WIN32
 
 namespace mozc {
 namespace renderer {
 namespace {
 
-constexpr int kIPCTimeout = 100;                   // 100 msec
-constexpr int kRendererWaitTimeout = 30 * 1000;    // 30 sec
-constexpr int kRendererWaitSleepTime = 10 * 1000;  // 10 sec
+constexpr absl::Duration kIpcTimeout = absl::Milliseconds(100);
+constexpr absl::Duration kRendererWaitTimeout = absl::Seconds(30);
+constexpr absl::Duration kRendererWaitSleepTime = absl::Seconds(10);
 constexpr size_t kMaxErrorTimes = 5;
-constexpr uint64_t kRetryIntervalTime = 30;  // 30 sec
+constexpr absl::Duration kRetryIntervalTime = absl::Seconds(30);
 constexpr char kServiceName[] = "renderer";
 
 inline void CallCommand(IPCClientInterface *client,
@@ -73,139 +77,51 @@ inline void CallCommand(IPCClientInterface *client,
   command.SerializeToString(&buf);
 
   // basically, we don't need to get the result
-  char result[32];
-  size_t result_size = sizeof(result);
+  std::string result;
 
-  if (!client->Call(buf.data(), buf.size(), result, &result_size,
-                    kIPCTimeout)) {
+  if (!client->Call(buf, &result, kIpcTimeout)) {
     LOG(ERROR) << "Cannot send the request: ";
   }
 }
 }  // namespace
 
-class RendererLauncher : public RendererLauncherInterface, public Thread {
+class RendererLauncher : public RendererLauncherInterface {
  public:
-  bool CanConnect() const override {
-    switch (renderer_status_) {
-      case RendererLauncher::RENDERER_UNKNOWN:
-      case RendererLauncher::RENDERER_READY:
-        return true;
-      case RendererLauncher::RENDERER_LAUNCHING:
-        VLOG(1) << "now starting renderer";
-        return false;
-      case RendererLauncher::RENDERER_TIMEOUT:
-      case RendererLauncher::RENDERER_TERMINATED:
-        if (error_times_ <= kMaxErrorTimes &&
-            Clock::GetTime() - last_launch_time_ >= kRetryIntervalTime) {
-          return true;
-        }
-        VLOG(1) << "never re-launch renderer";
-        return false;
-      case RendererLauncher::RENDERER_FATAL:
-        VLOG(1) << "never re-launch renderer";
-        return false;
-      default:
-        LOG(ERROR) << "Unknown status";
-        return false;
+  RendererLauncher() = default;
+
+  ~RendererLauncher() override {
+    if (!launcher_.has_value()) {
+      // StartRenderer has never been called.
+      return;
     }
 
-    return false;
-  }
-
-  bool IsAvailable() const override {
-    return (renderer_status_ == RendererLauncher::RENDERER_READY);
+    if (!launcher_->Ready()) {
+      NamedEventNotifier notify(name_.c_str());
+      notify.Notify();
+    }
+    launcher_->Wait();
   }
 
   void StartRenderer(
       const std::string &name, const std::string &path,
       bool disable_renderer_path_check,
       IPCClientFactoryInterface *ipc_client_factory_interface) override {
-    renderer_status_ = RendererLauncher::RENDERER_LAUNCHING;
+    if (Status() == RendererStatus::RENDERER_LAUNCHING ||
+        Status() == RendererStatus::RENDERER_READY ||
+        Status() == RendererStatus::RENDERER_TIMEOUT) {
+      // Renderer is already launching.
+      // The renderer is still up and running when in the pending command state.
+      return;
+    }
+    SetStatus(RendererStatus::RENDERER_LAUNCHING);
     name_ = name;
     path_ = path;
     disable_renderer_path_check_ = disable_renderer_path_check;
     ipc_client_factory_interface_ = ipc_client_factory_interface;
-    Thread::Start("Renderer");
-  }
-
-  void Run() override {
-    last_launch_time_ = Clock::GetTime();
-
-    NamedEventListener listener(name_.c_str());
-    const bool listener_is_available = listener.IsAvailable();
-
-#ifdef OS_WIN
-    DWORD pid = 0;
-    const bool process_in_job = RunLevel::IsProcessInJob();
-    const std::string arg = process_in_job ? "--restricted" : "";
-
-    WinSandbox::SecurityInfo info;
-    info.primary_level = WinSandbox::USER_INTERACTIVE;
-    info.impersonation_level = WinSandbox::USER_RESTRICTED_SAME_ACCESS;
-    info.integrity_level = WinSandbox::INTEGRITY_LEVEL_LOW;
-    // If the current process is in a job, you cannot use
-    // CREATE_BREAKAWAY_FROM_JOB. b/1571395
-    info.use_locked_down_job = !process_in_job;
-    info.allow_ui_operation = true;  // skip UI protection
-    info.in_system_dir = true;  // use system dir not to lock current directory
-    info.creation_flags = CREATE_DEFAULT_ERROR_MODE;
-
-    // start renreder process
-    const bool result =
-        WinSandbox::SpawnSandboxedProcess(path_, arg, info, &pid);
-#elif defined(__APPLE__)  // OS_WIN
-    // Start renderer process by using launch_msg API.
-    pid_t pid = 0;
-    const bool result = MacUtil::StartLaunchdService("Renderer", &pid);
-#else                     // OS_WIN, __APPLE__
-    size_t tmp = 0;
-    const bool result = Process::SpawnProcess(path_, "", &tmp);
-    uint32_t pid = static_cast<uint32_t>(tmp);
-#endif                    // OS_WIN, __APPLE__
-
-    if (!result) {
-      LOG(ERROR) << "Can't start process";
-      renderer_status_ = RendererLauncher::RENDERER_FATAL;
-      return;
-    }
-
-    if (listener_is_available) {
-      const int ret = listener.WaitEventOrProcess(kRendererWaitTimeout, pid);
-      switch (ret) {
-        case NamedEventListener::TIMEOUT:
-          LOG(ERROR) << "seems that mozc_renderer is not ready within "
-                     << kRendererWaitTimeout << " msec";
-          renderer_status_ = RendererLauncher::RENDERER_TIMEOUT;
-          ++error_times_;
-          break;
-        case NamedEventListener::EVENT_SIGNALED:
-          VLOG(1) << "mozc_renderer is launched successfully within "
-                  << kRendererWaitTimeout << " msec";
-          FlushPendingCommand();
-          renderer_status_ = RendererLauncher::RENDERER_READY;
-          error_times_ = 0;
-          break;
-        case NamedEventListener::PROCESS_SIGNALED:
-          LOG(ERROR) << "Mozc renderer is terminated";
-          renderer_status_ = RendererLauncher::RENDERER_TERMINATED;
-          ++error_times_;
-          break;
-        default:
-          LOG(ERROR) << "Unknown status";
-          renderer_status_ = RendererLauncher::RENDERER_FATAL;
-          ++error_times_;
-      }
-    } else {
-      LOG(ERROR) << "cannot make NamedEventListener ";
-      Util::Sleep(kRendererWaitSleepTime);
-      FlushPendingCommand();
-      renderer_status_ = RendererLauncher::RENDERER_READY;
-      error_times_ = 0;
-    }
-
-    if (renderer_status_ == RendererLauncher::RENDERER_FATAL) {
-      OnFatal(RendererLauncherInterface::RENDERER_FATAL);
+    if (launcher_.has_value()) {
+      launcher_->Wait();
     }
+    launcher_.emplace([this] { ThreadMain(); });
   }
 
   bool ForceTerminateRenderer(const std::string &name) override {
@@ -233,14 +149,45 @@ class RendererLauncher : public RendererLauncherInterface, public Thread {
     }
   }
 
-  void SetPendingCommand(const commands::RendererCommand &command) override {
+  bool IsAvailable() const override {
+    return (Status() == RendererStatus::RENDERER_READY);
+  }
+
+  bool CanConnect() const override {
+    switch (Status()) {
+      case RendererStatus::RENDERER_UNKNOWN:
+      case RendererStatus::RENDERER_READY:
+        return true;
+      case RendererStatus::RENDERER_LAUNCHING:
+        MOZC_VLOG(1) << "now starting renderer";
+        return false;
+      case RendererStatus::RENDERER_TIMEOUT:
+      case RendererStatus::RENDERER_TERMINATED:
+        if (error_times_ <= kMaxErrorTimes &&
+            Clock::GetAbslTime() - last_launch_time_ >= kRetryIntervalTime) {
+          return true;
+        }
+        MOZC_VLOG(1) << "never re-launch renderer";
+        return false;
+      case RendererStatus::RENDERER_FATAL:
+        MOZC_VLOG(1) << "never re-launch renderer";
+        return false;
+      default:
+        LOG(ERROR) << "Unknown status";
+        return false;
+    }
+
+    return false;
+  }
+
+  void SetPendingCommand(const commands::RendererCommand &command) override
+      ABSL_LOCKS_EXCLUDED(mu_) {
     // ignore NOOP|SHUTDOWN
     if (command.type() == commands::RendererCommand::UPDATE) {
-      absl::MutexLock l(&pending_command_mutex_);
-      if (!pending_command_) {
-        pending_command_ = std::make_unique<commands::RendererCommand>();
+      absl::MutexLock l(&mu_);
+      if (!pending_command_.has_value()) {
+        pending_command_ = command;
       }
-      *pending_command_ = command;
     }
   }
 
@@ -248,25 +195,8 @@ class RendererLauncher : public RendererLauncherInterface, public Thread {
     suppress_error_dialog_ = suppress;
   }
 
-  RendererLauncher()
-      : last_launch_time_(0),
-        error_times_(0),
-        ipc_client_factory_interface_(nullptr),
-        renderer_status_(RendererLauncher::RENDERER_UNKNOWN),
-        disable_renderer_path_check_(false),
-        suppress_error_dialog_(false) {}
-
-  ~RendererLauncher() override {
-    if (!IsRunning()) {
-      return;
-    }
-    NamedEventNotifier notify(name_.c_str());
-    notify.Notify();
-    Join();
-  }
-
  private:
-  enum RendererStatus {
+  enum class RendererStatus {
     RENDERER_UNKNOWN = 0,
     RENDERER_LAUNCHING = 1,
     RENDERER_READY = 2,
@@ -275,24 +205,113 @@ class RendererLauncher : public RendererLauncherInterface, public Thread {
     RENDERER_FATAL = 5,
   };
 
-  void FlushPendingCommand() {
-    absl::MutexLock l(&pending_command_mutex_);
-    if (ipc_client_factory_interface_ != nullptr && pending_command_) {
+  RendererStatus Status() const ABSL_LOCKS_EXCLUDED(mu_) {
+    absl::MutexLock l(&mu_);
+    return renderer_status_;
+  }
+
+  void SetStatus(RendererStatus status) ABSL_LOCKS_EXCLUDED(mu_) {
+    absl::MutexLock l(&mu_);
+    renderer_status_ = status;
+  }
+
+  void ThreadMain() {
+    last_launch_time_ = Clock::GetAbslTime();
+
+    NamedEventListener listener(name_.c_str());
+    const bool listener_is_available = listener.IsAvailable();
+
+#ifdef _WIN32
+    DWORD pid = 0;
+    const bool process_in_job = RunLevel::IsProcessInJob();
+    const std::string arg = process_in_job ? "--restricted" : "";
+
+    WinSandbox::SecurityInfo info;
+    info.primary_level = WinSandbox::USER_INTERACTIVE;
+    info.impersonation_level = WinSandbox::USER_RESTRICTED_SAME_ACCESS;
+    info.integrity_level = WinSandbox::INTEGRITY_LEVEL_LOW;
+    // If the current process is in a job, you cannot use
+    // CREATE_BREAKAWAY_FROM_JOB. b/1571395
+    info.use_locked_down_job = !process_in_job;
+    info.allow_ui_operation = true;  // skip UI protection
+    info.in_system_dir = true;  // use system dir not to lock current directory
+    info.creation_flags = CREATE_DEFAULT_ERROR_MODE;
+
+    // start renderer process
+    const bool result =
+        WinSandbox::SpawnSandboxedProcess(path_, arg, info, &pid);
+#elif defined(__APPLE__)  // _WIN32
+    // Start renderer process by using launch_msg API.
+    pid_t pid = 0;
+    const bool result = MacUtil::StartLaunchdService("Renderer", &pid);
+#else                     // _WIN32, __APPLE__
+    size_t tmp = 0;
+    const bool result = Process::SpawnProcess(path_, "", &tmp);
+    uint32_t pid = static_cast<uint32_t>(tmp);
+#endif                    // _WIN32, __APPLE__
+
+    if (!result) {
+      LOG(ERROR) << "Can't start process";
+      SetStatus(RendererStatus::RENDERER_FATAL);
+      return;
+    }
+
+    if (listener_is_available) {
+      const int ret = listener.WaitEventOrProcess(kRendererWaitTimeout, pid);
+      switch (ret) {
+        case NamedEventListener::TIMEOUT:
+          LOG(ERROR) << "seems that mozc_renderer is not ready within "
+                     << kRendererWaitTimeout << " msec";
+          SetStatus(RendererStatus::RENDERER_TIMEOUT);
+          ++error_times_;
+          break;
+        case NamedEventListener::EVENT_SIGNALED:
+          MOZC_VLOG(1) << "mozc_renderer is launched successfully within "
+                       << kRendererWaitTimeout << " msec";
+          FlushPendingCommand();
+          error_times_ = 0;
+          break;
+        case NamedEventListener::PROCESS_SIGNALED:
+          LOG(ERROR) << "Mozc renderer is terminated";
+          SetStatus(RendererStatus::RENDERER_TERMINATED);
+          ++error_times_;
+          break;
+        default:
+          LOG(ERROR) << "Unknown status";
+          SetStatus(RendererStatus::RENDERER_FATAL);
+          ++error_times_;
+      }
+    } else {
+      LOG(ERROR) << "cannot make NamedEventListener ";
+      absl::SleepFor(kRendererWaitSleepTime);
+      FlushPendingCommand();
+      error_times_ = 0;
+    }
+
+    if (Status() == RendererStatus::RENDERER_FATAL) {
+      OnFatal(RendererLauncherInterface::RENDERER_FATAL);
+    }
+  }
+
+  void FlushPendingCommand() ABSL_LOCKS_EXCLUDED(mu_) {
+    absl::MutexLock l(&mu_);
+    if (ipc_client_factory_interface_ != nullptr &&
+        pending_command_.has_value()) {
       std::unique_ptr<IPCClientInterface> client(CreateIPCClient());
-      if (!client) {
+      if (client != nullptr) {
         CallCommand(client.get(), *pending_command_);
       }
     }
     pending_command_.reset();
 
     // |renderer_status_| is also protected by mutex.
-    // Until this method finsihs, SetPendingCommand is blocked.
+    // Until this method finishes, SetPendingCommand is blocked.
     // RendererClint checks the status AGAIN after SetPendingCommand.
-    renderer_status_ = RendererLauncher::RENDERER_READY;
+    renderer_status_ = RendererStatus::RENDERER_READY;
     error_times_ = 0;
   }
 
-  IPCClientInterface *CreateIPCClient() const {
+  std::unique_ptr<IPCClientInterface> CreateIPCClient() const {
     if (ipc_client_factory_interface_ == nullptr) {
       return nullptr;
     }
@@ -304,14 +323,17 @@ class RendererLauncher : public RendererLauncherInterface, public Thread {
 
   std::string name_;
   std::string path_;
-  volatile uint64_t last_launch_time_;
-  volatile size_t error_times_;
-  IPCClientFactoryInterface *ipc_client_factory_interface_;
-  std::unique_ptr<commands::RendererCommand> pending_command_;
-  absl::Mutex pending_command_mutex_;
-  volatile RendererStatus renderer_status_;
-  bool disable_renderer_path_check_;
-  bool suppress_error_dialog_;
+  absl::Time last_launch_time_ = absl::UnixEpoch();
+  volatile size_t error_times_ = 0;
+  IPCClientFactoryInterface *ipc_client_factory_interface_ = nullptr;
+  mutable absl::Mutex mu_;
+  std::optional<commands::RendererCommand> pending_command_
+      ABSL_GUARDED_BY(mu_);
+  volatile RendererStatus renderer_status_ ABSL_GUARDED_BY(mu_) =
+      RendererStatus::RENDERER_UNKNOWN;
+  bool disable_renderer_path_check_ = false;
+  bool suppress_error_dialog_ = false;
+  std::optional<BackgroundFuture<void>> launcher_;
 };
 
 RendererClient::RendererClient()
@@ -379,7 +401,7 @@ bool RendererClient::Shutdown(bool force) {
   }
 
   if (!client->Connected()) {
-    VLOG(1) << "renderer is not running.";
+    MOZC_VLOG(1) << "renderer is not running.";
     return true;
   }
 
@@ -425,7 +447,7 @@ bool RendererClient::ExecCommand(const commands::RendererCommand &command) {
     // Check CanConnect() again, as the status might be changed
     // after SetPendingCommand().
     if (!renderer_launcher_interface_->CanConnect()) {
-      VLOG(1) << "renderer_launcher::CanConnect() return false";
+      MOZC_VLOG(1) << "renderer_launcher::CanConnect() return false";
       return true;
     }
   }
@@ -436,7 +458,7 @@ bool RendererClient::ExecCommand(const commands::RendererCommand &command) {
     return true;
   }
 
-  VLOG(2) << "Sending: " << command.DebugString();
+  MOZC_VLOG(2) << "Sending: " << command;
 
   std::unique_ptr<IPCClientInterface> client(CreateIPCClient());
 
@@ -501,7 +523,7 @@ bool RendererClient::ExecCommand(const commands::RendererCommand &command) {
   return true;
 }
 
-IPCClientInterface *RendererClient::CreateIPCClient() const {
+std::unique_ptr<IPCClientInterface> RendererClient::CreateIPCClient() const {
   if (ipc_client_factory_interface_ == nullptr) {
     return nullptr;
   }
