@@ -32,16 +32,17 @@
 #include <memory>
 #include <string>
 
-#include "base/logging.h"
-#include "ipc/ipc_test_util.h"
-#include "protocol/renderer_command.pb.h"
-#include "testing/gunit.h"
-#include "testing/mozctest.h"
+#include "absl/log/log.h"
 #include "absl/strings/string_view.h"
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
+#include "ipc/ipc.h"
+#include "ipc/ipc_test_util.h"
+#include "protocol/renderer_command.pb.h"
 #include "renderer/renderer_client.h"
 #include "renderer/renderer_interface.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 
 namespace mozc {
 namespace renderer {
