@@ -29,16 +29,10 @@
 
 #include "renderer/renderer_style_handler.h"
 
-// clang-format off
-#if defined(OS_WIN)
+#if defined(_WIN32)
 #include <windows.h>
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
-#include <atlbase.h>
-#include <atlapp.h>
-#include <atlgdi.h>
-#endif  // OS_WIN
-// clang-format on
+#include <wil/resource.h>
+#endif  // _WIN32
 
 #include "base/singleton.h"
 #include "protocol/renderer_style.pb.h"
@@ -46,9 +40,9 @@
 namespace mozc {
 namespace renderer {
 namespace {
-#if defined(OS_WIN)
+#if defined(_WIN32)
 constexpr int kDefaultDPI = 96;
-#endif  // OS_WIN
+#endif  // _WIN32
 
 class RendererStyleHandlerImpl {
  public:
@@ -206,24 +200,24 @@ void RendererStyleHandler::GetDefaultRendererStyle(RendererStyle *style) {
 }
 
 void RendererStyleHandler::GetDPIScalingFactor(double *x, double *y) {
-#if defined OS_WIN
-  WTL::CDC desktop_dc(::GetDC(nullptr));
-  const int dpi_x = desktop_dc.GetDeviceCaps(LOGPIXELSX);
-  const int dpi_y = desktop_dc.GetDeviceCaps(LOGPIXELSY);
+#ifdef _WIN32
+  wil::unique_hdc_window desktop_dc(::GetDC(nullptr));
+  const int dpi_x = ::GetDeviceCaps(desktop_dc.get(), LOGPIXELSX);
+  const int dpi_y = ::GetDeviceCaps(desktop_dc.get(), LOGPIXELSY);
   if (x != nullptr) {
     *x = static_cast<double>(dpi_x) / kDefaultDPI;
   }
   if (y != nullptr) {
     *y = static_cast<double>(dpi_y) / kDefaultDPI;
   }
-#else
+#else   // _WIN32
   if (x != nullptr) {
     *x = 1.0;
   }
   if (y != nullptr) {
     *y = 1.0;
   }
-#endif
+#endif  // !_WIN32
 }
 
 }  // namespace renderer
