@@ -32,13 +32,15 @@
 #include <algorithm>
 #include <numeric>
 
-#include "base/logging.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "base/coordinates.h"
 
 namespace mozc {
 namespace renderer {
 namespace {
 // The minimum height of the indicator in the VScrollBar.
-const int kMinimumIndicatorHeight = 1;
+constexpr int kMinimumIndicatorHeight = 1;
 }  // namespace
 
 // ------------------------------------------------------------------------
