@@ -29,30 +29,32 @@
 
 #include "renderer/win32/win32_image_util.h"
 
-// clang-format off
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
 #include <atlbase.h>
-#include <atlapp.h>
-#include <atlgdi.h>
-#include <atlmisc.h>
-// clang-format on
+#include <atltypes.h>
+#include <wil/resource.h>
 
-#include <algorithm>
-#include <fstream>
-#include <list>
+#include <cstdlib>
+
+#undef StrCat
+
+#include <cstddef>
+#include <cstdint>
 #include <memory>
+#include <string>
+#include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_cat.h"
 #include "base/file_stream.h"
 #include "base/file_util.h"
-#include "base/logging.h"
-#include "base/mmap.h"
-#include "base/util.h"
-#include "base/win_font_test_helper.h"
-#include "net/jsoncpp.h"
-#include "testing/base/public/gmock.h"
-#include "testing/base/public/gunit.h"
-#include "testing/base/public/mozctest.h"
+#include "base/protobuf/text_format.h"
+#include "base/win32/wide_char.h"
+#include "base/win32/win_font_test_helper.h"
+#include "data/test/renderer/win32/test_spec.pb.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 
 using ::std::max;
 using ::std::min;
@@ -62,24 +64,22 @@ using ::std::min;
 // TODO(yukawa): Use WIC (Windows Imaging Component) instead of GDI+.
 #include <gdiplus.h>  // NOLINT
 
-using ::mozc::renderer::win32::internal::GaussianBlur;
-using ::mozc::renderer::win32::internal::SafeFrameBuffer;
-using ::mozc::renderer::win32::internal::SubdivisionalPixel;
-using ::mozc::renderer::win32::internal::TextLabel;
-
-using ::WTL::CBitmap;
-using ::WTL::CDC;
-using ::WTL::CLogFont;
-using ::WTL::CPoint;
-using ::WTL::CSize;
-
 namespace mozc {
 namespace renderer {
 namespace win32 {
 namespace {
 
-typedef SubdivisionalPixel::SubdivisionalPixelIterator
-    SubdivisionalPixelIterator;
+using ::mozc::renderer::win32::internal::GaussianBlur;
+using ::mozc::renderer::win32::internal::SafeFrameBuffer;
+using ::mozc::renderer::win32::internal::SubdivisionalPixel;
+using ::mozc::renderer::win32::internal::TextLabel;
+using ::mozc::win32::Utf8ToWide;
+using ::mozc::win32::WideToUtf8;
+
+using BalloonImageInfo = BalloonImage::BalloonImageInfo;
+using SubdivisionalPixelIterator =
+    SubdivisionalPixel::SubdivisionalPixelIterator;
+using TestSpec = ::mozc::test::renderer::win32::TestSpec;
 
 class BalloonImageTest : public ::testing::Test,
                          public ::testing::WithParamInterface<const char *> {
@@ -107,19 +107,18 @@ class BalloonImageTest : public ::testing::Test,
     using BalloonImage::CreateInternal;
   };
 
-  static void BalloonImageTest::SaveTestImage(
-      const TestableBalloonImage::BalloonImageInfo &info,
-      const std::wstring filename) {
+  static void SaveTestImage(const BalloonImageInfo &info,
+                            const std::wstring &filename) {
     CPoint tail_offset;
     CSize size;
     std::vector<ARGBColor> buffer;
-    CBitmap dib = TestableBalloonImage::CreateInternal(info, &tail_offset,
-                                                       &size, &buffer);
+    wil::unique_hbitmap dib(TestableBalloonImage::CreateInternal(
+        info, &tail_offset, &size, &buffer));
 
-    Json::Value tail;
-    BalloonInfoToJson(info, &tail);
-    tail["output"]["tail_offset_x"] = tail_offset.x;
-    tail["output"]["tail_offset_y"] = tail_offset.y;
+    TestSpec spec = TestSpec();
+    BalloonInfoToTextProto(info, &spec);
+    spec.mutable_output()->set_tail_offset_x(tail_offset.x);
+    spec.mutable_output()->set_tail_offset_y(tail_offset.y);
 
     Gdiplus::Bitmap bitmap(size.cx, size.cy);
     for (size_t y = 0; y < size.cy; ++y) {
@@ -132,64 +131,57 @@ class BalloonImageTest : public ::testing::Test,
 
     bitmap.Save(filename.c_str(), &clsid_png_);
 
-    std::string utf8_filename;
-    Util::WideToUtf8(filename + L".json", &utf8_filename);
-    OutputFileStream os(utf8_filename.c_str());
-    Json::StyledWriter writer;
-    os << writer.write(tail);
+    OutputFileStream os(absl::StrCat(WideToUtf8(filename), ".textproto"));
+    std::string textproto;
+    mozc::protobuf::TextFormat::PrintToString(spec, &textproto);
+    os << textproto;
   }
 
-  static void BalloonInfoToJson(
-      const TestableBalloonImage::BalloonImageInfo &info, Json::Value *tail) {
-    Json::Value input(Json::objectValue);
-    input["frame_color"] = Json::Value(ColorToInteger(info.frame_color));
-    input["inside_color"] = Json::Value(ColorToInteger(info.inside_color));
-    input["label_color"] = Json::Value(ColorToInteger(info.label_color));
-    input["blur_color"] = Json::Value(ColorToInteger(info.blur_color));
-    input["blur_alpha"] = Json::Value(info.blur_alpha);
-    input["label_size"] = Json::Value(info.label_size);
-    input["label_font"] = Json::Value(info.label_font);
-    input["label"] = Json::Value(info.label);
-    input["rect_width"] = Json::Value(info.rect_width);
-    input["rect_height"] = Json::Value(info.rect_height);
-    input["frame_thickness"] = Json::Value(info.frame_thickness);
-    input["corner_radius"] = Json::Value(info.corner_radius);
-    input["tail_height"] = Json::Value(info.tail_height);
-    input["tail_width"] = Json::Value(info.tail_width);
-    input["tail_direction"] =
-        Json::Value(static_cast<int>(info.tail_direction));
-    input["blur_sigma"] = Json::Value(info.blur_sigma);
-    input["blur_offset_x"] = Json::Value(info.blur_offset_x);
-    input["blur_offset_y"] = Json::Value(info.blur_offset_y);
-
-    (*tail)["input"] = input;
+  static void BalloonInfoToTextProto(const BalloonImageInfo &info,
+                                     TestSpec *spec) {
+    TestSpec::Input *input = spec->mutable_input();
+    input->set_frame_color(ColorToInteger(info.frame_color));
+    input->set_inside_color(ColorToInteger(info.inside_color));
+    input->set_label_color(ColorToInteger(info.label_color));
+    input->set_blur_color(ColorToInteger(info.blur_color));
+    input->set_blur_alpha(info.blur_alpha);
+    input->set_label_size(info.label_size);
+    input->set_label_font(info.label_font);
+    input->set_label(info.label);
+    input->set_rect_width(info.rect_width);
+    input->set_rect_height(info.rect_height);
+    input->set_frame_thickness(info.frame_thickness);
+    input->set_corner_radius(info.corner_radius);
+    input->set_tail_height(info.tail_height);
+    input->set_tail_width(info.tail_width);
+    input->set_tail_direction(TranslateEnum(info.tail_direction));
+    input->set_blur_sigma(info.blur_sigma);
+    input->set_blur_offset_x(info.blur_offset_x);
+    input->set_blur_offset_y(info.blur_offset_y);
   }
 
-  static void JsonToBalloonInfo(const Json::Value &tail,
-                                TestableBalloonImage::BalloonImageInfo *info) {
-    const Json::Value &input = tail["input"];
-
-    *info = TestableBalloonImage::BalloonImageInfo();
-    info->frame_color = IntegerToColor(input["frame_color"].asInt());
-    info->inside_color = IntegerToColor(input["inside_color"].asInt());
-    info->label_color = IntegerToColor(input["label_color"].asInt());
-    info->blur_color = IntegerToColor(input["blur_color"].asInt());
-    info->blur_alpha = input["blur_alpha"].asDouble();
-    info->label_size = input["label_size"].asInt();
-    info->label_font = input["label_font"].asString();
-    info->label = input["label"].asString();
-    info->rect_width = input["rect_width"].asDouble();
-    info->rect_height = input["rect_height"].asDouble();
-    info->frame_thickness = input["frame_thickness"].asDouble();
-    info->corner_radius = input["corner_radius"].asDouble();
-    info->tail_height = input["tail_height"].asDouble();
-    info->tail_width = input["tail_width"].asDouble();
-    info->tail_direction =
-        static_cast<TestableBalloonImage::BalloonImageInfo::TailDirection>(
-            input["tail_direction"].asInt());
-    info->blur_sigma = input["blur_sigma"].asDouble();
-    info->blur_offset_x = input["blur_offset_x"].asInt();
-    info->blur_offset_y = input["blur_offset_y"].asInt();
+  static void TextProtoToBalloonInfo(const TestSpec &spec,
+                                     BalloonImageInfo *info) {
+    const TestSpec::Input &input = spec.input();
+    *info = BalloonImageInfo();
+    info->frame_color = IntegerToColor(input.frame_color());
+    info->inside_color = IntegerToColor(input.inside_color());
+    info->label_color = IntegerToColor(input.label_color());
+    info->blur_color = IntegerToColor(input.blur_color());
+    info->blur_alpha = input.blur_alpha();
+    info->label_size = input.label_size();
+    info->label_font = input.label_font();
+    info->label = input.label();
+    info->rect_width = input.rect_width();
+    info->rect_height = input.rect_height();
+    info->frame_thickness = input.frame_thickness();
+    info->corner_radius = input.corner_radius();
+    info->tail_height = input.tail_height();
+    info->tail_width = input.tail_width();
+    info->tail_direction = TranslateEnum(input.tail_direction());
+    info->blur_sigma = input.blur_sigma();
+    info->blur_offset_x = input.blur_offset_x();
+    info->blur_offset_y = input.blur_offset_y();
   }
 
  private:
@@ -201,7 +193,7 @@ class BalloonImageTest : public ::testing::Test,
       return false;
     }
 
-    std::unique_ptr<uint8[]> codesc_buffer(new uint8[codecs_buffer_size]);
+    std::unique_ptr<uint8_t[]> codesc_buffer(new uint8_t[codecs_buffer_size]);
     Gdiplus::ImageCodecInfo *codecs =
         reinterpret_cast<Gdiplus::ImageCodecInfo *>(codesc_buffer.get());
 
@@ -230,101 +222,131 @@ class BalloonImageTest : public ::testing::Test,
 
   static void UninitGdiplus() { Gdiplus::GdiplusShutdown(gdiplus_token_); }
 
-  static int32 ColorToInteger(RGBColor color) {
-    return static_cast<int32>(color.r) << 16 |
-           static_cast<int32>(color.g) << 8 | static_cast<int32>(color.b);
+  static uint32_t ColorToInteger(RGBColor color) {
+    return static_cast<uint32_t>(color.r) << 16 |
+           static_cast<uint32_t>(color.g) << 8 | static_cast<uint32_t>(color.b);
   }
 
-  static RGBColor IntegerToColor(int32 color) {
+  static RGBColor IntegerToColor(uint32_t color) {
     return RGBColor((color >> 16) & 0xff, (color >> 8) & 0xff, color & 0xff);
   }
 
+  static TestSpec::TailDirection TranslateEnum(
+      BalloonImageInfo::TailDirection direction) {
+    switch (direction) {
+      case BalloonImageInfo::kTop:
+        return TestSpec::TOP;
+      case BalloonImageInfo::kBottom:
+        return TestSpec::BOTTOM;
+      case BalloonImageInfo::kLeft:
+        return TestSpec::LEFT;
+      case BalloonImageInfo::kRight:
+        return TestSpec::RIGHT;
+      default:
+        LOG(FATAL) << "Unexpected direction: " << direction;
+    }
+  }
+
+  static BalloonImageInfo::TailDirection TranslateEnum(
+      TestSpec::TailDirection direction) {
+    switch (direction) {
+      case TestSpec::UNSPECIFIED:
+        LOG(FATAL) << "TailDirection must be set.";
+      case TestSpec::TOP:
+        return BalloonImageInfo::TailDirection::kTop;
+      case TestSpec::BOTTOM:
+        return BalloonImageInfo::TailDirection::kBottom;
+      case TestSpec::LEFT:
+        return BalloonImageInfo::TailDirection::kLeft;
+      case TestSpec::RIGHT:
+        return BalloonImageInfo::TailDirection::kRight;
+      default:
+        LOG(FATAL) << "Unexpected direction: " << direction;
+    }
+  }
+
   static CLSID clsid_png_;
   static CLSID clsid_bmp_;
 
-  static HANDLE font_handle_;
   static ULONG_PTR gdiplus_token_;
 };
 
-HANDLE BalloonImageTest::font_handle_;
 CLSID BalloonImageTest::clsid_png_;
 CLSID BalloonImageTest::clsid_bmp_;
 ULONG_PTR BalloonImageTest::gdiplus_token_;
 
 // Tests should be passed.
 const char *kRenderingResultList[] = {
-    "data/test/renderer/win32/balloon_blur_alpha_-1.png",
-    "data/test/renderer/win32/balloon_blur_alpha_0.png",
-    "data/test/renderer/win32/balloon_blur_alpha_10.png",
-    "data/test/renderer/win32/balloon_blur_color_32_64_128.png",
-    "data/test/renderer/win32/balloon_blur_offset_-20_-10.png",
-    "data/test/renderer/win32/balloon_blur_offset_0_0.png",
-    "data/test/renderer/win32/balloon_blur_offset_20_5.png",
-    "data/test/renderer/win32/balloon_blur_sigma_0.0.png",
-    "data/test/renderer/win32/balloon_blur_sigma_0.5.png",
-    "data/test/renderer/win32/balloon_blur_sigma_1.0.png",
-    "data/test/renderer/win32/balloon_blur_sigma_2.0.png",
-    "data/test/renderer/win32/balloon_frame_thickness_-1.png",
-    "data/test/renderer/win32/balloon_frame_thickness_0.png",
-    "data/test/renderer/win32/balloon_frame_thickness_1.5.png",
-    "data/test/renderer/win32/balloon_frame_thickness_3.png",
-    "data/test/renderer/win32/balloon_inside_color_32_64_128.png",
-    "data/test/renderer/win32/balloon_no_label.png",
-    "data/test/renderer/win32/balloon_tail_bottom.png",
-    "data/test/renderer/win32/balloon_tail_left.png",
-    "data/test/renderer/win32/balloon_tail_right.png",
-    "data/test/renderer/win32/balloon_tail_top.png",
-    "data/test/renderer/win32/balloon_tail_width_height_-10_-10.png",
-    "data/test/renderer/win32/balloon_tail_width_height_0_0.png",
-    "data/test/renderer/win32/balloon_tail_width_height_10_20.png",
-    "data/test/renderer/win32/balloon_width_height_40_30.png",
+    "balloon_blur_alpha_-1.png",
+    "balloon_blur_alpha_0.png",
+    "balloon_blur_alpha_10.png",
+    "balloon_blur_color_32_64_128.png",
+    "balloon_blur_offset_-20_-10.png",
+    "balloon_blur_offset_0_0.png",
+    "balloon_blur_offset_20_5.png",
+    "balloon_blur_sigma_0.0.png",
+    "balloon_blur_sigma_0.5.png",
+    "balloon_blur_sigma_1.0.png",
+    "balloon_blur_sigma_2.0.png",
+    "balloon_frame_thickness_-1.png",
+    "balloon_frame_thickness_0.png",
+    "balloon_frame_thickness_1.5.png",
+    "balloon_frame_thickness_3.png",
+    "balloon_inside_color_32_64_128.png",
+    "balloon_no_label.png",
+    "balloon_tail_bottom.png",
+    "balloon_tail_left.png",
+    "balloon_tail_right.png",
+    "balloon_tail_top.png",
+    "balloon_tail_width_height_-10_-10.png",
+    "balloon_tail_width_height_0_0.png",
+    "balloon_tail_width_height_10_20.png",
+    "balloon_width_height_40_30.png",
 };
 
 INSTANTIATE_TEST_CASE_P(BalloonImageParameters, BalloonImageTest,
                         ::testing::ValuesIn(kRenderingResultList));
 
 TEST_P(BalloonImageTest, TestImpl) {
-  const std::string &expected_image_path =
-      mozc::testing::GetSourceFileOrDie({GetParam()});
-  const std::string json_path = expected_image_path + ".json";
-  ASSERT_OK(FileUtil::FileExists(json_path))
-      << "Manifest file is not found: " << json_path;
+  const std::string &expected_image_path = mozc::testing::GetSourceFileOrDie(
+      {MOZC_DICT_DIR_COMPONENTS, "test", "renderer", "win32", GetParam()});
+  const std::string textproto_path = expected_image_path + ".textproto";
+  ASSERT_OK(FileUtil::FileExists(textproto_path))
+      << "Manifest file is not found: " << textproto_path;
 
-  Json::Value tail;
+  TestSpec spec;
   {
-    InputFileStream fs(json_path.c_str());
-    ASSERT_TRUE(fs.good());
-    fs >> tail;
+    absl::StatusOr<std::string> data = FileUtil::GetContents(textproto_path);
+    ASSERT_OK(data);
+    ASSERT_TRUE(
+        mozc::protobuf::TextFormat::ParseFromString(data.value(), &spec));
   }
-  TestableBalloonImage::BalloonImageInfo info;
-  JsonToBalloonInfo(tail, &info);
+  BalloonImageInfo info;
+  TextProtoToBalloonInfo(spec, &info);
 
   CPoint actual_tail_offset;
   CSize actual_size;
   std::vector<ARGBColor> actual_buffer;
-  CBitmap dib = TestableBalloonImage::CreateInternal(
-      info, &actual_tail_offset, &actual_size, &actual_buffer);
+  wil::unique_hbitmap dib(TestableBalloonImage::CreateInternal(
+      info, &actual_tail_offset, &actual_size, &actual_buffer));
 
-  EXPECT_EQ(tail["output"]["tail_offset_x"].asInt(), actual_tail_offset.x);
-  EXPECT_EQ(tail["output"]["tail_offset_y"].asInt(), actual_tail_offset.y);
+  EXPECT_EQ(actual_tail_offset.x, spec.output().tail_offset_x());
+  EXPECT_EQ(actual_tail_offset.y, spec.output().tail_offset_y());
 
-  std::wstring wide_path;
-  Util::Utf8ToWide(expected_image_path, &wide_path);
+  Gdiplus::Bitmap bitmap(Utf8ToWide(expected_image_path).c_str());
 
-  Gdiplus::Bitmap bitmap(wide_path.c_str());
-
-  ASSERT_EQ(bitmap.GetWidth(), actual_size.cx);
-  ASSERT_EQ(bitmap.GetHeight(), actual_size.cy);
+  ASSERT_EQ(actual_size.cx, bitmap.GetWidth());
+  ASSERT_EQ(actual_size.cy, bitmap.GetHeight());
 
   for (size_t y = 0; y < actual_size.cy; ++y) {
     for (size_t x = 0; x < actual_size.cx; ++x) {
       ARGBColor argb = actual_buffer[y * actual_size.cx + x];
       Gdiplus::Color color;
-      ASSERT_EQ(Gdiplus::Ok, bitmap.GetPixel(x, y, &color));
-      EXPECT_EQ(color.GetA(), argb.a) << "(x, y): (" << x << ", " << y << ")";
-      EXPECT_EQ(color.GetR(), argb.r) << "(x, y): (" << x << ", " << y << ")";
-      EXPECT_EQ(color.GetG(), argb.g) << "(x, y): (" << x << ", " << y << ")";
-      EXPECT_EQ(color.GetB(), argb.b) << "(x, y): (" << x << ", " << y << ")";
+      ASSERT_EQ(bitmap.GetPixel(x, y, &color), Gdiplus::Ok);
+      EXPECT_EQ(argb.a, color.GetA()) << "(x, y): (" << x << ", " << y << ")";
+      EXPECT_EQ(argb.r, color.GetR()) << "(x, y): (" << x << ", " << y << ")";
+      EXPECT_EQ(argb.g, color.GetG()) << "(x, y): (" << x << ", " << y << ")";
+      EXPECT_EQ(argb.b, color.GetB()) << "(x, y): (" << x << ", " << y << ")";
     }
   }
 }
@@ -344,34 +366,34 @@ TEST(SubdivisionalPixelTest, BasicTest) {
   const RGBColor kGreen(0, 255, 0);
 
   SubdivisionalPixel sub_pixel;
-  EXPECT_EQ(0.0, sub_pixel.GetCoverage())
+  EXPECT_EQ(sub_pixel.GetCoverage(), 0.0)
       << "Should be zero for an empty pixel";
-  EXPECT_EQ(RGBColor::kBlack, sub_pixel.GetPixelColor())
+  EXPECT_EQ(sub_pixel.GetPixelColor(), RGBColor::kBlack)
       << "Should be black for an empty pixel";
 
   // SetSubdivisionalPixel sets only sub-pixel specified.
   sub_pixel.SetSubdivisionalPixel(SubdivisionalPixel::Fraction2D(0, 0),
                                   RGBColor::kWhite);
   EXPECT_NEAR(1.0 / 255.0, sub_pixel.GetCoverage(), 0.1);
-  EXPECT_EQ(RGBColor::kWhite, sub_pixel.GetPixelColor());
+  EXPECT_EQ(sub_pixel.GetPixelColor(), RGBColor::kWhite);
 
   sub_pixel.SetColorToFilledPixels(kGreen);
   EXPECT_NEAR(1.0 / 255.0, sub_pixel.GetCoverage(), 0.1);
-  EXPECT_EQ(kGreen, sub_pixel.GetPixelColor());
+  EXPECT_EQ(sub_pixel.GetPixelColor(), kGreen);
 
   // SetPixel sets all the sub-pixels.
   sub_pixel.SetPixel(kBlue);
   EXPECT_NEAR(1.0, sub_pixel.GetCoverage(), 0.01);
-  EXPECT_EQ(kBlue, sub_pixel.GetPixelColor());
+  EXPECT_EQ(sub_pixel.GetPixelColor(), kBlue);
 
   sub_pixel.SetSubdivisionalPixel(SubdivisionalPixel::Fraction2D(0, 0),
                                   RGBColor::kWhite);
   EXPECT_NEAR(1.0, sub_pixel.GetCoverage(), 0.01);
-  EXPECT_EQ(1, sub_pixel.GetPixelColor().r);
+  EXPECT_EQ(sub_pixel.GetPixelColor().r, 1);
 
   sub_pixel.SetColorToFilledPixels(kBlue);
   EXPECT_NEAR(1.0, sub_pixel.GetCoverage(), 0.01);
-  EXPECT_EQ(kBlue, sub_pixel.GetPixelColor());
+  EXPECT_EQ(sub_pixel.GetPixelColor(), kBlue);
 }
 
 TEST(SubdivisionalPixelTest, IteratorTest) {
@@ -390,7 +412,7 @@ TEST(SubdivisionalPixelTest, IteratorTest) {
     EXPECT_GE(1.0, it.GetY());
     ++count;
   }
-  EXPECT_EQ(SubdivisionalPixel::kTotalPixels, count);
+  EXPECT_EQ(count, SubdivisionalPixel::kTotalPixels);
 }
 
 TEST(GaussianBlurTest, NoBlurTest) {
@@ -400,8 +422,8 @@ TEST(GaussianBlurTest, NoBlurTest) {
   struct Source {
     Source() : call_count_(0) {}
     double operator()(int x, int y) const {
-      EXPECT_EQ(0, x);
-      EXPECT_EQ(0, y);
+      EXPECT_EQ(x, 0);
+      EXPECT_EQ(y, 0);
       ++call_count_;
       return 1.0;
     }
@@ -409,8 +431,8 @@ TEST(GaussianBlurTest, NoBlurTest) {
   };
 
   Source source;
-  EXPECT_EQ(1.0, blur.Apply(0, 0, source));
-  EXPECT_EQ(1, source.call_count_);
+  EXPECT_EQ(blur.Apply(0, 0, source), 1.0);
+  EXPECT_EQ(source.call_count_, 1);
 }
 
 TEST(GaussianBlurTest, InvalidParamBlurTest) {
@@ -421,8 +443,8 @@ TEST(GaussianBlurTest, InvalidParamBlurTest) {
   struct Source {
     Source() : call_count_(0) {}
     double operator()(int x, int y) const {
-      EXPECT_EQ(0, x);
-      EXPECT_EQ(0, y);
+      EXPECT_EQ(x, 0);
+      EXPECT_EQ(y, 0);
       ++call_count_;
       return 1.0;
     }
@@ -430,8 +452,8 @@ TEST(GaussianBlurTest, InvalidParamBlurTest) {
   };
 
   Source source;
-  EXPECT_EQ(1.0, blur.Apply(0, 0, source));
-  EXPECT_EQ(1, source.call_count_);
+  EXPECT_EQ(blur.Apply(0, 0, source), 1.0);
+  EXPECT_EQ(source.call_count_, 1);
 }
 
 TEST(GaussianBlurTest, NormalBlurTest) {
@@ -452,7 +474,7 @@ TEST(GaussianBlurTest, NormalBlurTest) {
   Source source(blur.cutoff_length());
   EXPECT_NEAR(1.0, blur.Apply(0, 0, source), 0.1);
   const size_t matrix_length = blur.cutoff_length() * 2 + 1;
-  EXPECT_EQ(matrix_length * matrix_length, source.call_count_);
+  EXPECT_EQ(source.call_count_, matrix_length * matrix_length);
 }
 
 TEST(SafeFrameBufferTest, BasicTest) {
@@ -463,31 +485,31 @@ TEST(SafeFrameBufferTest, BasicTest) {
   constexpr int kHeight = 100;
   SafeFrameBuffer buffer(Rect(kLeft, kTop, kWidth, kHeight));
 
-  EXPECT_EQ(kTransparent, buffer.GetPixel(kLeft, kTop))
+  EXPECT_EQ(buffer.GetPixel(kLeft, kTop), kTransparent)
       << "Initial color should be transparent";
   buffer.SetPixel(kLeft, kTop, ARGBColor::kWhite);
-  EXPECT_EQ(ARGBColor::kWhite, buffer.GetPixel(kLeft, kTop));
+  EXPECT_EQ(buffer.GetPixel(kLeft, kTop), ARGBColor::kWhite);
 
   buffer.SetPixel(kLeft + kWidth, kTop, ARGBColor::kWhite);
-  EXPECT_EQ(kTransparent, buffer.GetPixel(kLeft + kWidth, kTop))
+  EXPECT_EQ(buffer.GetPixel(kLeft + kWidth, kTop), kTransparent)
       << "(left + width) is outside.";
 
   buffer.SetPixel(kLeft, kTop + kHeight, ARGBColor::kWhite);
-  EXPECT_EQ(kTransparent, buffer.GetPixel(kLeft, kTop + kHeight))
+  EXPECT_EQ(buffer.GetPixel(kLeft, kTop + kHeight), kTransparent)
       << "(top + height) is outside.";
 
   buffer.SetPixel(kLeft - 10, kTop - 10, ARGBColor::kWhite);
-  EXPECT_EQ(kTransparent, buffer.GetPixel(kLeft - 10, kTop - 10))
+  EXPECT_EQ(buffer.GetPixel(kLeft - 10, kTop - 10), kTransparent)
       << "Outside pixel should be kept as transparent.";
 }
 
 TEST(TextLabelTest, BoundingBoxTest) {
   const TextLabel label(-10.5, -5.1, 10.5, 5.0, "text", "font name", 10,
                         RGBColor::kWhite);
-  EXPECT_EQ(-11, label.bounding_rect().Left());
-  EXPECT_EQ(-6, label.bounding_rect().Top());
-  EXPECT_EQ(0, label.bounding_rect().Right());
-  EXPECT_EQ(0, label.bounding_rect().Bottom());
+  EXPECT_EQ(label.bounding_rect().Left(), -11);
+  EXPECT_EQ(label.bounding_rect().Top(), -6);
+  EXPECT_EQ(label.bounding_rect().Right(), 0);
+  EXPECT_EQ(label.bounding_rect().Bottom(), 0);
 }
 
 }  // namespace
