@@ -32,18 +32,21 @@
 
 #include <string>
 #include <vector>
+
+#include "absl/strings/string_view.h"
 #include "converter/segments.h"
+#include "dictionary/pos_matcher.h"
 #include "rewriter/rewriter_interface.h"
-#include "testing/base/public/gunit_prod.h"
-// for FRIEND_TEST()
+#include "testing/friend_test.h"
 
 namespace mozc {
 class ConversionRequest;
 
 class EnglishVariantsRewriter : public RewriterInterface {
  public:
-  EnglishVariantsRewriter();
-  ~EnglishVariantsRewriter() override;
+  explicit EnglishVariantsRewriter(dictionary::PosMatcher pos_matcher)
+      : pos_matcher_(pos_matcher) {}
+  ~EnglishVariantsRewriter() override = default;
 
   int capability(const ConversionRequest &request) const override;
 
@@ -52,11 +55,17 @@ class EnglishVariantsRewriter : public RewriterInterface {
 
  private:
   FRIEND_TEST(EnglishVariantsRewriterTest, ExpandEnglishVariants);
+  FRIEND_TEST(EnglishVariantsRewriterTest, ExpandSpacePrefixedVariants);
   bool IsT13NCandidate(Segment::Candidate *candidate) const;
   bool IsEnglishCandidate(Segment::Candidate *candidate) const;
-  bool ExpandEnglishVariants(const std::string &input,
+  bool ExpandEnglishVariants(absl::string_view input,
                              std::vector<std::string> *variants) const;
-  bool ExpandEnglishVariantsWithSegment(Segment *seg) const;
+  bool ExpandSpacePrefixedVariants(absl::string_view input,
+                                   std::vector<std::string> *variants) const;
+  bool ExpandEnglishVariantsWithSegment(bool need_space_prefix,
+                                        Segment *seg) const;
+
+  const dictionary::PosMatcher pos_matcher_;
 };
 
 }  // namespace mozc
