@@ -30,11 +30,13 @@
 #ifndef MOZC_REWRITER_MERGER_REWRITER_H_
 #define MOZC_REWRITER_MERGER_REWRITER_H_
 
+#include <cstddef>
 #include <memory>
+#include <optional>
 #include <utility>
 #include <vector>
 
-#include "config/config_handler.h"
+#include "absl/log/check.h"
 #include "converter/segments.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
@@ -51,44 +53,53 @@ class MergerRewriter : public RewriterInterface {
   MergerRewriter(const MergerRewriter &) = delete;
   MergerRewriter &operator=(const MergerRewriter &) = delete;
 
-  // Returns true if rewriter can be called with the segments.
-  bool CheckCapability(const ConversionRequest &request,
-                       const Segments *segments,
-                       const RewriterInterface &rewriter) const {
-    if (segments == nullptr) {
-      return false;
-    }
-    switch (request.request_type()) {
-      case ConversionRequest::CONVERSION:
-        return ((rewriter.capability(request) &
-                 RewriterInterface::CONVERSION) != 0);
-
-      case ConversionRequest::PREDICTION:
-      case ConversionRequest::PARTIAL_PREDICTION:
-        return ((rewriter.capability(request) &
-                 RewriterInterface::PREDICTION) != 0);
-
-      case ConversionRequest::SUGGESTION:
-      case ConversionRequest::PARTIAL_SUGGESTION:
-        return ((rewriter.capability(request) &
-                 RewriterInterface::SUGGESTION) != 0);
-
-      case ConversionRequest::REVERSE_CONVERSION:
-      default:
-        return false;
-    }
+  void AddRewriter(std::unique_ptr<RewriterInterface> rewriter) {
+    DCHECK(rewriter);
+    rewriters_.push_back(std::move(rewriter));
   }
 
-  void AddRewriter(std::unique_ptr<RewriterInterface> rewriter) {
-    rewriters_.push_back(std::move(rewriter));
+  std::optional<ResizeSegmentsRequest> CheckResizeSegmentsRequest(
+      const ConversionRequest &request, const Segments &segments) const {
+    if (segments.resized()) {
+      return std::nullopt;
+    }
+
+    for (const std::unique_ptr<RewriterInterface> &rewriter : rewriters_) {
+      std::optional<ResizeSegmentsRequest> resize_request =
+          rewriter->CheckResizeSegmentsRequest(request, segments);
+      if (resize_request.has_value()) {
+        return resize_request;
+      }
+    }
+    return std::nullopt;
   }
 
   bool Rewrite(const ConversionRequest &request,
                Segments *segments) const override {
-    bool result = false;
+    if (segments == nullptr) {
+      return false;
+    }
+
+    const CapabilityType capability_type = [&request]() {
+      switch (request.request_type()) {
+        case ConversionRequest::CONVERSION:
+          return RewriterInterface::CONVERSION;
+        case ConversionRequest::PREDICTION:
+        case ConversionRequest::PARTIAL_PREDICTION:
+          return RewriterInterface::PREDICTION;
+        case ConversionRequest::SUGGESTION:
+        case ConversionRequest::PARTIAL_SUGGESTION:
+          return RewriterInterface::SUGGESTION;
+        case ConversionRequest::REVERSE_CONVERSION:
+        default:
+          return RewriterInterface::NOT_AVAILABLE;
+      }
+    }();
+
+    bool is_updated = false;
     for (const std::unique_ptr<RewriterInterface> &rewriter : rewriters_) {
-      if (CheckCapability(request, segments, *rewriter)) {
-        result |= rewriter->Rewrite(request, segments);
+      if (rewriter->capability(request) & capability_type) {
+        is_updated |= rewriter->Rewrite(request, segments);
       }
     }
 
@@ -103,7 +114,7 @@ class MergerRewriter : public RewriterInterface {
                                   candidate_size - max_suggestions);
       }
     }
-    return result;
+    return is_updated;
   }
 
   // This method is mainly called when user puts SPACE key
@@ -127,6 +138,22 @@ class MergerRewriter : public RewriterInterface {
     }
   }
 
+  void Revert(Segments *segments) override {
+    for (const std::unique_ptr<RewriterInterface> &rewriter : rewriters_) {
+      rewriter->Revert(segments);
+    }
+  }
+
+  bool ClearHistoryEntry(const Segments &segments, size_t segment_index,
+                         int candidate_index) override {
+    bool result = false;
+    for (const std::unique_ptr<RewriterInterface> &rewriter : rewriters_) {
+      result |=
+          rewriter->ClearHistoryEntry(segments, segment_index, candidate_index);
+    }
+    return result;
+  }
+
   // Syncs internal data to local file system.
   bool Sync() override {
     bool result = false;
