@@ -29,17 +29,18 @@
 
 #include "rewriter/merger_rewriter.h"
 
+#include <cstddef>
 #include <memory>
 #include <string>
 
-#include "base/system_util.h"
-#include "config/config_handler.h"
+#include "absl/strings/string_view.h"
 #include "converter/segments.h"
+#include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "request/conversion_request.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "absl/flags/flag.h"
+#include "rewriter/rewriter_interface.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 
 namespace mozc {
 namespace {
@@ -48,14 +49,15 @@ namespace {
 // and what value should be returned.
 class TestRewriter : public RewriterInterface {
  public:
-  TestRewriter(std::string *buffer, const std::string &name, bool return_value)
+  TestRewriter(std::string *buffer, const absl::string_view name,
+               bool return_value)
       : buffer_(buffer),
         name_(name),
         return_value_(return_value),
         capability_(RewriterInterface::CONVERSION) {}
 
-  TestRewriter(std::string *buffer, const std::string &name, bool return_value,
-               int capability)
+  TestRewriter(std::string *buffer, const absl::string_view name,
+               bool return_value, int capability)
       : buffer_(buffer),
         name_(name),
         return_value_(return_value),
@@ -83,6 +85,10 @@ class TestRewriter : public RewriterInterface {
     buffer_->append(name_ + ".Finish();");
   }
 
+  void Revert(Segments *segments) override {
+    buffer_->append(name_ + ".Revert();");
+  }
+
   bool Sync() override {
     buffer_->append(name_ + ".Sync();");
     return return_value_;
@@ -102,12 +108,11 @@ class TestRewriter : public RewriterInterface {
   int capability_;
 };
 
-class MergerRewriterTest : public testing::Test {
- protected:
-  void SetUp() override {
-    SystemUtil::SetUserProfileDirectory(absl::GetFlag(FLAGS_test_tmpdir));
-  }
-};
+class MergerRewriterTest : public testing::TestWithTempUserProfile {};
+
+ConversionRequest ConvReq(ConversionRequest::RequestType request_type) {
+  return ConversionRequestBuilder().SetRequestType(request_type).Build();
+}
 
 TEST_F(MergerRewriterTest, Rewrite) {
   std::string call_result;
@@ -119,48 +124,45 @@ TEST_F(MergerRewriterTest, Rewrite) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
   EXPECT_FALSE(merger.Rewrite(request, &segments));
-  EXPECT_EQ(
-      "a.Rewrite();"
-      "b.Rewrite();"
-      "c.Rewrite();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Rewrite();"
+            "b.Rewrite();"
+            "c.Rewrite();");
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "d", true));
   call_result.clear();
   EXPECT_TRUE(merger.Rewrite(request, &segments));
-  EXPECT_EQ(
-      "a.Rewrite();"
-      "b.Rewrite();"
-      "c.Rewrite();"
-      "d.Rewrite();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Rewrite();"
+            "b.Rewrite();"
+            "c.Rewrite();"
+            "d.Rewrite();");
 }
 
 TEST_F(MergerRewriterTest, RewriteSuggestion) {
   std::string call_result;
   MergerRewriter merger;
   Segments segments;
-  ConversionRequest request;
-  request.set_request_type(ConversionRequest::SUGGESTION);
+  const ConversionRequest request = ConvReq(ConversionRequest::SUGGESTION);
 
   merger.AddRewriter(std::make_unique<TestRewriter>(
       &call_result, "a", true, RewriterInterface::SUGGESTION));
 
-  EXPECT_EQ(0, segments.conversion_segments_size());
+  EXPECT_EQ(segments.conversion_segments_size(), 0);
   Segment *segment = segments.push_back_segment();
-  EXPECT_EQ(1, segments.conversion_segments_size());
+  EXPECT_EQ(segments.conversion_segments_size(), 1);
 
-  EXPECT_EQ(0, segment->candidates_size());
+  EXPECT_EQ(segment->candidates_size(), 0);
   segment->push_back_candidate();
   segment->push_back_candidate();
   segment->push_back_candidate();
   segment->push_back_candidate();
-  EXPECT_EQ(4, segment->candidates_size());
-  EXPECT_EQ(3, request.config().suggestions_size());
+  EXPECT_EQ(segment->candidates_size(), 4);
+  EXPECT_EQ(request.config().suggestions_size(), 3);
 
   EXPECT_TRUE(merger.Rewrite(request, &segments));
-  EXPECT_EQ("a.Rewrite();", call_result);
+  EXPECT_EQ(call_result, "a.Rewrite();");
 
-  EXPECT_EQ(3, segment->candidates_size());
+  EXPECT_EQ(segment->candidates_size(), 3);
 }
 
 TEST_F(MergerRewriterTest, RewriteSuggestionWithMixedConversion) {
@@ -172,38 +174,39 @@ TEST_F(MergerRewriterTest, RewriteSuggestionWithMixedConversion) {
   // should result that the merger rewriter does not trim exceeded suggestions.
   commands::Request commands_request;
   commands_request.set_mixed_conversion(true);
-  ConversionRequest request;
-  request.set_request(&commands_request);
-  request.set_request_type(ConversionRequest::SUGGESTION);
+  const ConversionRequest request =
+      ConversionRequestBuilder()
+          .SetRequest(commands_request)
+          .SetRequestType(ConversionRequest::SUGGESTION)
+          .Build();
   EXPECT_TRUE(request.request().mixed_conversion());
 
   merger.AddRewriter(std::make_unique<TestRewriter>(
       &call_result, "a", true, RewriterInterface::SUGGESTION));
 
-  EXPECT_EQ(0, segments.conversion_segments_size());
+  EXPECT_EQ(segments.conversion_segments_size(), 0);
   Segment *segment = segments.push_back_segment();
-  EXPECT_EQ(1, segments.conversion_segments_size());
+  EXPECT_EQ(segments.conversion_segments_size(), 1);
 
-  EXPECT_EQ(0, segment->candidates_size());
+  EXPECT_EQ(segment->candidates_size(), 0);
   segment->push_back_candidate();
   segment->push_back_candidate();
   segment->push_back_candidate();
   segment->push_back_candidate();
-  EXPECT_EQ(4, segment->candidates_size());
-  EXPECT_EQ(3, request.config().suggestions_size());
+  EXPECT_EQ(segment->candidates_size(), 4);
+  EXPECT_EQ(request.config().suggestions_size(), 3);
 
   EXPECT_TRUE(merger.Rewrite(request, &segments));
-  EXPECT_EQ("a.Rewrite();", call_result);
+  EXPECT_EQ(call_result, "a.Rewrite();");
 
   // If mixed_conversion is true, the suggestions are not deleted.
-  EXPECT_EQ(4, segment->candidates_size());
+  EXPECT_EQ(segment->candidates_size(), 4);
 }
 
 TEST_F(MergerRewriterTest, RewriteCheckTest) {
   std::string call_result;
   MergerRewriter merger;
   Segments segments;
-  ConversionRequest request;
   merger.AddRewriter(std::make_unique<TestRewriter>(
       &call_result, "a", false, RewriterInterface::CONVERSION));
   merger.AddRewriter(std::make_unique<TestRewriter>(
@@ -216,8 +219,9 @@ TEST_F(MergerRewriterTest, RewriteCheckTest) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "e", false,
                                                     RewriterInterface::ALL));
 
-  request.set_request_type(ConversionRequest::CONVERSION);
-  EXPECT_FALSE(merger.Rewrite(request, &segments));
+  const ConversionRequest request_conversion =
+      ConvReq(ConversionRequest::CONVERSION);
+  EXPECT_FALSE(merger.Rewrite(request_conversion, &segments));
   EXPECT_EQ(
       "a.Rewrite();"
       "d.Rewrite();"
@@ -225,38 +229,38 @@ TEST_F(MergerRewriterTest, RewriteCheckTest) {
       call_result);
   call_result.clear();
 
-  request.set_request_type(ConversionRequest::PREDICTION);
-  EXPECT_FALSE(merger.Rewrite(request, &segments));
-  EXPECT_EQ(
-      "c.Rewrite();"
-      "d.Rewrite();"
-      "e.Rewrite();",
-      call_result);
+  const ConversionRequest request_prediction =
+      ConvReq(ConversionRequest::PREDICTION);
+  EXPECT_FALSE(merger.Rewrite(request_prediction, &segments));
+  EXPECT_EQ(call_result,
+            "c.Rewrite();"
+            "d.Rewrite();"
+            "e.Rewrite();");
   call_result.clear();
 
-  request.set_request_type(ConversionRequest::SUGGESTION);
-  EXPECT_FALSE(merger.Rewrite(request, &segments));
-  EXPECT_EQ(
-      "b.Rewrite();"
-      "e.Rewrite();",
-      call_result);
+  const ConversionRequest request_suggestion =
+      ConvReq(ConversionRequest::SUGGESTION);
+  EXPECT_FALSE(merger.Rewrite(request_suggestion, &segments));
+  EXPECT_EQ(call_result,
+            "b.Rewrite();"
+            "e.Rewrite();");
   call_result.clear();
 
-  request.set_request_type(ConversionRequest::PARTIAL_SUGGESTION);
-  EXPECT_FALSE(merger.Rewrite(request, &segments));
-  EXPECT_EQ(
-      "b.Rewrite();"
-      "e.Rewrite();",
-      call_result);
+  const ConversionRequest request_partial_suggestion =
+      ConvReq(ConversionRequest::PARTIAL_SUGGESTION);
+  EXPECT_FALSE(merger.Rewrite(request_partial_suggestion, &segments));
+  EXPECT_EQ(call_result,
+            "b.Rewrite();"
+            "e.Rewrite();");
   call_result.clear();
 
-  request.set_request_type(ConversionRequest::PARTIAL_PREDICTION);
-  EXPECT_FALSE(merger.Rewrite(request, &segments));
-  EXPECT_EQ(
-      "c.Rewrite();"
-      "d.Rewrite();"
-      "e.Rewrite();",
-      call_result);
+  const ConversionRequest request_partial_prediction =
+      ConvReq(ConversionRequest::PARTIAL_PREDICTION);
+  EXPECT_FALSE(merger.Rewrite(request_partial_prediction, &segments));
+  EXPECT_EQ(call_result,
+            "c.Rewrite();"
+            "d.Rewrite();"
+            "e.Rewrite();");
   call_result.clear();
 }
 
@@ -267,20 +271,18 @@ TEST_F(MergerRewriterTest, Focus) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
   EXPECT_FALSE(merger.Focus(nullptr, 0, 0));
-  EXPECT_EQ(
-      "a.Focus();"
-      "b.Focus();"
-      "c.Focus();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Focus();"
+            "b.Focus();"
+            "c.Focus();");
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "d", true));
   call_result.clear();
   EXPECT_TRUE(merger.Focus(nullptr, 0, 0));
-  EXPECT_EQ(
-      "a.Focus();"
-      "b.Focus();"
-      "c.Focus();"
-      "d.Focus();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Focus();"
+            "b.Focus();"
+            "c.Focus();"
+            "d.Focus();");
 }
 
 TEST_F(MergerRewriterTest, Finish) {
@@ -291,11 +293,24 @@ TEST_F(MergerRewriterTest, Finish) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
   merger.Finish(request, nullptr);
-  EXPECT_EQ(
-      "a.Finish();"
-      "b.Finish();"
-      "c.Finish();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Finish();"
+            "b.Finish();"
+            "c.Finish();");
+}
+
+TEST_F(MergerRewriterTest, Revert) {
+  std::string call_result;
+  const ConversionRequest request;
+  MergerRewriter merger;
+  merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "a", false));
+  merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
+  merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
+  merger.Revert(nullptr);
+  EXPECT_EQ(call_result,
+            "a.Revert();"
+            "b.Revert();"
+            "c.Revert();");
 }
 
 TEST_F(MergerRewriterTest, Sync) {
@@ -305,20 +320,18 @@ TEST_F(MergerRewriterTest, Sync) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
   EXPECT_FALSE(merger.Sync());
-  EXPECT_EQ(
-      "a.Sync();"
-      "b.Sync();"
-      "c.Sync();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Sync();"
+            "b.Sync();"
+            "c.Sync();");
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "d", true));
   call_result.clear();
   EXPECT_TRUE(merger.Sync());
-  EXPECT_EQ(
-      "a.Sync();"
-      "b.Sync();"
-      "c.Sync();"
-      "d.Sync();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Sync();"
+            "b.Sync();"
+            "c.Sync();"
+            "d.Sync();");
 }
 
 TEST_F(MergerRewriterTest, Reload) {
@@ -328,20 +341,18 @@ TEST_F(MergerRewriterTest, Reload) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
   EXPECT_FALSE(merger.Reload());
-  EXPECT_EQ(
-      "a.Reload();"
-      "b.Reload();"
-      "c.Reload();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Reload();"
+            "b.Reload();"
+            "c.Reload();");
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "d", true));
   call_result.clear();
   EXPECT_TRUE(merger.Reload());
-  EXPECT_EQ(
-      "a.Reload();"
-      "b.Reload();"
-      "c.Reload();"
-      "d.Reload();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Reload();"
+            "b.Reload();"
+            "c.Reload();"
+            "d.Reload();");
 }
 
 TEST_F(MergerRewriterTest, Clear) {
@@ -351,20 +362,18 @@ TEST_F(MergerRewriterTest, Clear) {
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "b", false));
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "c", false));
   merger.Clear();
-  EXPECT_EQ(
-      "a.Clear();"
-      "b.Clear();"
-      "c.Clear();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Clear();"
+            "b.Clear();"
+            "c.Clear();");
   merger.AddRewriter(std::make_unique<TestRewriter>(&call_result, "d", true));
   call_result.clear();
   merger.Clear();
-  EXPECT_EQ(
-      "a.Clear();"
-      "b.Clear();"
-      "c.Clear();"
-      "d.Clear();",
-      call_result);
+  EXPECT_EQ(call_result,
+            "a.Clear();"
+            "b.Clear();"
+            "c.Clear();"
+            "d.Clear();");
 }
 
 }  // namespace
