@@ -29,50 +29,50 @@
 
 #include "rewriter/symbol_rewriter.h"
 
+#include <cstddef>
 #include <memory>
+#include <optional>
 #include <string>
 
-#include "base/logging.h"
-#include "base/system_util.h"
+#include "absl/log/check.h"
+#include "absl/strings/string_view.h"
 #include "base/util.h"
-#include "config/config_handler.h"
+#include "converter/converter_mock.h"
 #include "converter/segments.h"
 #include "data_manager/testing/mock_data_manager.h"
-#include "engine/engine_interface.h"
+#include "engine/engine.h"
 #include "engine/mock_data_engine_factory.h"
 #include "protocol/commands.pb.h"
 #include "request/conversion_request.h"
-#include "session/request_test_util.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "absl/flags/flag.h"
-#include "absl/memory/memory.h"
+#include "request/request_test_util.h"
+#include "rewriter/rewriter_interface.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 
 namespace mozc {
-
 namespace {
-void AddSegment(const std::string &key, const std::string &value,
+
+void AddSegment(const absl::string_view key, const absl::string_view value,
                 Segments *segments) {
   Segment *seg = segments->push_back_segment();
   seg->set_key(key);
   Segment::Candidate *candidate = seg->add_candidate();
-  candidate->Init();
-  candidate->value = value;
-  candidate->content_key = key;
-  candidate->content_value = value;
+  candidate->value = std::string(value);
+  candidate->content_key = std::string(key);
+  candidate->content_value = std::string(value);
 }
 
-void AddCandidate(const std::string &value, Segment *segment) {
+void AddCandidate(const absl::string_view value, Segment *segment) {
   Segment::Candidate *candidate = segment->add_candidate();
-  candidate->Init();
-  candidate->value = value;
+  candidate->value = std::string(value);
   candidate->content_key = segment->key();
-  candidate->content_value = value;
+  candidate->content_value = std::string(value);
 }
 
 bool HasCandidateAndDescription(const Segments &segments, int index,
-                                const std::string &key,
-                                const std::string &description) {
+                                const absl::string_view key,
+                                const absl::string_view description) {
   CHECK_GT(segments.segments_size(), index);
   bool check_description = !description.empty();
 
@@ -91,31 +91,18 @@ bool HasCandidateAndDescription(const Segments &segments, int index,
 }
 
 bool HasCandidate(const Segments &segments, int index,
-                  const std::string &value) {
+                  const absl::string_view value) {
   return HasCandidateAndDescription(segments, index, value, "");
 }
+
 }  // namespace
 
-class SymbolRewriterTest : public ::testing::Test {
+class SymbolRewriterTest : public testing::TestWithTempUserProfile {
  protected:
-  SymbolRewriterTest() = default;
-  ~SymbolRewriterTest() override = default;
-
   void SetUp() override {
-    SystemUtil::SetUserProfileDirectory(absl::GetFlag(FLAGS_test_tmpdir));
-
-    // We cannot use mock converter here because SymbolRewriter uses
-    // ResizeSegment of converter implementation. However, SymbolRewriter is
-    // independent of underlying dictionary and, hence, we can use a converter
-    // with mock data.
-    engine_.reset(MockDataEngineFactory::Create());
-    converter_ = engine_->GetConverter();
-
-    data_manager_ = absl::make_unique<testing::MockDataManager>();
+    data_manager_ = std::make_unique<testing::MockDataManager>();
   }
 
-  std::unique_ptr<EngineInterface> engine_;
-  const ConverterInterface *converter_;
   std::unique_ptr<testing::MockDataManager> data_manager_;
 };
 
@@ -123,75 +110,95 @@ class SymbolRewriterTest : public ::testing::Test {
 // Test result can be changed if symbol dictionary is modified.
 // TODO(toshiyuki): Modify symbol rewriter so that we can use symbol dictionary
 // for testing.
-TEST_F(SymbolRewriterTest, TriggerRewriteTest) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+TEST_F(SymbolRewriterTest, CheckResizeSegmentsRequestTest) {
+  SymbolRewriter symbol_rewriter(*data_manager_);
   const ConversionRequest request;
 
   {
+    // Two segments should be resized to one segment (i.e. "ー>").
     Segments segments;
     AddSegment("ー", "test", &segments);
     AddSegment(">", "test", &segments);
-    EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
-    EXPECT_TRUE(HasCandidate(segments, 0, "→"));
+    std::optional<RewriterInterface::ResizeSegmentsRequest> resize_request =
+        symbol_rewriter.CheckResizeSegmentsRequest(request, segments);
+    ASSERT_TRUE(resize_request.has_value());
+    EXPECT_EQ(resize_request->segment_index, 0);
+    EXPECT_EQ(resize_request->segment_sizes[0], 2);
   }
   {
+    // Already resized.
+    Segments segments;
+    AddSegment("ー>", "test", &segments);
+    std::optional<RewriterInterface::ResizeSegmentsRequest> resize_request =
+        symbol_rewriter.CheckResizeSegmentsRequest(request, segments);
+    EXPECT_FALSE(resize_request.has_value());
+  }
+  {
+    // No applicable symbols.
     Segments segments;
     AddSegment("ー", "test", &segments);
     AddSegment("ー", "test", &segments);
-    EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
-    EXPECT_TRUE(HasCandidate(segments, 0, "―"));
-    EXPECT_TRUE(HasCandidate(segments, 1, "―"));
-  }
-}
-
-TEST_F(SymbolRewriterTest, TriggerRewriteEntireTest) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
-  const ConversionRequest request;
-  {
-    Segments segments;
-    AddSegment("ー", "test", &segments);
-    AddSegment(">", "test", &segments);
-    EXPECT_TRUE(symbol_rewriter.RewriteEntireCandidate(request, &segments));
-    EXPECT_TRUE(HasCandidate(segments, 0, "→"));
-  }
-  {
-    Segments segments;
-    AddSegment("ー", "test", &segments);
-    AddSegment("ー", "test", &segments);
-    EXPECT_FALSE(symbol_rewriter.RewriteEntireCandidate(request, &segments));
+    std::optional<RewriterInterface::ResizeSegmentsRequest> resize_request =
+        symbol_rewriter.CheckResizeSegmentsRequest(request, segments);
+    EXPECT_FALSE(resize_request.has_value());
   }
 }
 
 TEST_F(SymbolRewriterTest, TriggerRewriteEachTest) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+  SymbolRewriter symbol_rewriter(*data_manager_);
   const ConversionRequest request;
   {
     Segments segments;
     AddSegment("ー", "test", &segments);
     AddSegment(">", "test", &segments);
     EXPECT_TRUE(symbol_rewriter.RewriteEachCandidate(request, &segments));
-    EXPECT_EQ(2, segments.segments_size());
+    EXPECT_EQ(segments.segments_size(), 2);
     EXPECT_TRUE(HasCandidate(segments, 0, "―"));
     EXPECT_FALSE(HasCandidate(segments, 0, "→"));
     EXPECT_TRUE(HasCandidate(segments, 1, "〉"));
   }
 }
 
+TEST_F(SymbolRewriterTest, HentaiganaSymbolTest) {
+  SymbolRewriter symbol_rewriter(*data_manager_);
+  const ConversionRequest request;
+  {
+    Segments segments;
+    AddSegment("あ", {"あ"}, &segments);
+    EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
+    EXPECT_TRUE(
+        HasCandidateAndDescription(segments, 0, "\U0001B002", "安の変体仮名"));
+    EXPECT_TRUE(
+        HasCandidateAndDescription(segments, 0, "\U0001B003", "愛の変体仮名"));
+    EXPECT_FALSE(
+        HasCandidateAndDescription(segments, 0, "\U0001B007", "伊の変体仮名"));
+  }
+  {
+    Segments segments;
+    AddSegment("いぇ", {"いぇ"}, &segments);
+    EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
+    EXPECT_TRUE(
+        HasCandidateAndDescription(segments, 0, "\U0001B001", "江の変体仮名"));
+    EXPECT_TRUE(
+        HasCandidateAndDescription(segments, 0, "\U0001B121", "変体仮名"));
+  }
+}
+
 TEST_F(SymbolRewriterTest, TriggerRewriteDescriptionTest) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+  SymbolRewriter symbol_rewriter(*data_manager_);
   const ConversionRequest request;
   {
     Segments segments;
     AddSegment("したつき", "test", &segments);
     EXPECT_TRUE(symbol_rewriter.RewriteEachCandidate(request, &segments));
-    EXPECT_EQ(1, segments.segments_size());
+    EXPECT_EQ(segments.segments_size(), 1);
     EXPECT_TRUE(
         HasCandidateAndDescription(segments, 0, "₍", "下付き文字(始め丸括弧)"));
   }
 }
 
 TEST_F(SymbolRewriterTest, InsertAfterSingleKanjiAndT13n) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+  SymbolRewriter symbol_rewriter(*data_manager_);
   const ConversionRequest request;
   {
     Segments segments;
@@ -224,12 +231,12 @@ TEST_F(SymbolRewriterTest, InsertAfterSingleKanjiAndT13n) {
   }
 }
 
-TEST_F(SymbolRewriterTest, InsertSymbolsPosition_MobileSymbolKey) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+TEST_F(SymbolRewriterTest, InsertSymbolsPositionMobileSymbolKey) {
+  SymbolRewriter symbol_rewriter(*data_manager_);
   commands::Request command_request;
-  commands::RequestForUnitTest::FillMobileRequest(&command_request);
-  ConversionRequest request;
-  request.set_request(&command_request);
+  request_test_util::FillMobileRequest(&command_request);
+  const ConversionRequest request =
+      ConversionRequestBuilder().SetRequest(command_request).Build();
 
   {
     Segments segments;
@@ -244,16 +251,16 @@ TEST_F(SymbolRewriterTest, InsertSymbolsPosition_MobileSymbolKey) {
     EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
     EXPECT_GT(segments.segment(0).candidates_size(), 5);
     // Full width should be inserted with high ranking.
-    EXPECT_EQ("％", segments.segment(0).candidate(1).value);
+    EXPECT_EQ(segments.segment(0).candidate(1).value, "％");
   }
 }
 
-TEST_F(SymbolRewriterTest, InsertSymbolsPosition_MobileAlphabetKey) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+TEST_F(SymbolRewriterTest, InsertSymbolsPositionMobileAlphabetKey) {
+  SymbolRewriter symbol_rewriter(*data_manager_);
   commands::Request command_request;
-  commands::RequestForUnitTest::FillMobileRequest(&command_request);
-  ConversionRequest request;
-  request.set_request(&command_request);
+  request_test_util::FillMobileRequest(&command_request);
+  const ConversionRequest request =
+      ConversionRequestBuilder().SetRequest(command_request).Build();
 
   {
     Segments segments;
@@ -268,14 +275,14 @@ TEST_F(SymbolRewriterTest, InsertSymbolsPosition_MobileAlphabetKey) {
     EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
     EXPECT_GT(segments.segment(0).candidates_size(), 5);  // Symbols were added
     // Should keep top candidates.
-    EXPECT_EQ("app", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("apple", segments.segment(0).candidate(1).value);
-    EXPECT_EQ("align", segments.segment(0).candidate(2).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "app");
+    EXPECT_EQ(segments.segment(0).candidate(1).value, "apple");
+    EXPECT_EQ(segments.segment(0).candidate(2).value, "align");
   }
 }
 
 TEST_F(SymbolRewriterTest, SetKey) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+  SymbolRewriter symbol_rewriter(*data_manager_);
   Segments segments;
   const ConversionRequest request;
 
@@ -283,45 +290,45 @@ TEST_F(SymbolRewriterTest, SetKey) {
   const std::string kKey = "てん";
   segment->set_key(kKey);
   Segment::Candidate *candidate = segment->add_candidate();
-  candidate->Init();
   candidate->key = "strange key";
   candidate->value = "strange value";
   candidate->content_key = "strange key";
   candidate->content_value = "strange value";
-  EXPECT_EQ(1, segment->candidates_size());
+  EXPECT_EQ(segment->candidates_size(), 1);
   EXPECT_TRUE(symbol_rewriter.Rewrite(request, &segments));
   EXPECT_GT(segment->candidates_size(), 1);
   for (size_t i = 1; i < segment->candidates_size(); ++i) {
-    EXPECT_EQ(kKey, segment->candidate(i).key);
+    EXPECT_EQ(segment->candidate(i).key, kKey);
   }
 }
 
 TEST_F(SymbolRewriterTest, MobileEnvironmentTest) {
-  ConversionRequest convreq;
   commands::Request request;
-  convreq.set_request(&request);
-  SymbolRewriter rewriter(converter_, data_manager_.get());
+  SymbolRewriter rewriter(*data_manager_);
 
   {
     request.set_mixed_conversion(true);
-    EXPECT_EQ(RewriterInterface::ALL, rewriter.capability(convreq));
+    const ConversionRequest convreq =
+        ConversionRequestBuilder().SetRequest(request).Build();
+    EXPECT_EQ(rewriter.capability(convreq), RewriterInterface::ALL);
   }
 
   {
     request.set_mixed_conversion(false);
-    EXPECT_EQ(RewriterInterface::CONVERSION, rewriter.capability(convreq));
+    const ConversionRequest convreq =
+        ConversionRequestBuilder().SetRequest(request).Build();
+    EXPECT_EQ(rewriter.capability(convreq), RewriterInterface::CONVERSION);
   }
 }
 
 TEST_F(SymbolRewriterTest, ExpandSpace) {
-  SymbolRewriter symbol_rewriter(converter_, data_manager_.get());
+  SymbolRewriter symbol_rewriter(*data_manager_);
   Segments segments;
   const ConversionRequest request;
 
   Segment *segment = segments.push_back_segment();
   segment->set_key(" ");
   Segment::Candidate *candidate = segment->add_candidate();
-  candidate->Init();
   candidate->key = " ";
   candidate->value = " ";
   candidate->content_key = " ";
@@ -332,21 +339,63 @@ TEST_F(SymbolRewriterTest, ExpandSpace) {
   EXPECT_LE(2, segment->candidates_size());
 
   const Segment::Candidate &cand0 = segment->candidate(0);
-  EXPECT_EQ(" ", cand0.key);
-  EXPECT_EQ(" ", cand0.value);
-  EXPECT_EQ(" ", cand0.content_key);
-  EXPECT_EQ(" ", cand0.content_value);
-  ASSERT_EQ(1, cand0.inner_segment_boundary.size());
-  EXPECT_EQ(Segment::Candidate::EncodeLengths(1, 1, 1, 1),
-            cand0.inner_segment_boundary[0]);
+  EXPECT_EQ(cand0.key, " ");
+  EXPECT_EQ(cand0.value, " ");
+  EXPECT_EQ(cand0.content_key, " ");
+  EXPECT_EQ(cand0.content_value, " ");
+  ASSERT_EQ(cand0.inner_segment_boundary.size(), 1);
+  EXPECT_EQ(cand0.inner_segment_boundary[0],
+            Segment::Candidate::EncodeLengths(1, 1, 1, 1));
 
   const char *kFullWidthSpace = "　";
   const Segment::Candidate &cand1 = segment->candidate(1);
-  EXPECT_EQ(" ", cand1.key);
-  EXPECT_EQ(kFullWidthSpace, cand1.value);
-  EXPECT_EQ(" ", cand1.content_key);
-  EXPECT_EQ(kFullWidthSpace, cand1.content_value);
+  EXPECT_EQ(cand1.key, " ");
+  EXPECT_EQ(cand1.value, kFullWidthSpace);
+  EXPECT_EQ(cand1.content_key, " ");
+  EXPECT_EQ(cand1.content_value, kFullWidthSpace);
   EXPECT_TRUE(cand1.inner_segment_boundary.empty());
 }
 
+TEST_F(SymbolRewriterTest, InvalidSizeOfSegments) {
+  const SymbolRewriter rewriter(*data_manager_);
+
+  // Valid case: segment size is 1.
+  {
+    Segments segments;
+    const ConversionRequest request;
+
+    // 1 segment. There are symbols assigned to "おんがく".
+    AddSegment("ぎりしゃ", "test", &segments);
+    EXPECT_TRUE(rewriter.Rewrite(request, &segments));
+  }
+
+  // Invalid case: segment size is not 1.
+  {
+    Segments segments;
+    const ConversionRequest request;
+
+    // 0 segments.
+    EXPECT_FALSE(rewriter.Rewrite(request, &segments));
+
+    // 2 segments. There are no symbols assigned to "おん" or "がく".
+    AddSegment("おん", "test", &segments);
+    AddSegment("がく", "test", &segments);
+    EXPECT_FALSE(rewriter.Rewrite(request, &segments));
+  }
+}
+
+TEST_F(SymbolRewriterTest, ResizeSegmentFailureIsNotFatal) {
+  const SymbolRewriter rewriter(*data_manager_);
+
+  Segments segments;
+  const ConversionRequest request;
+  AddSegment("ー", "test", &segments);
+  AddSegment(">", "test", &segments);
+  std::optional<RewriterInterface::ResizeSegmentsRequest> resize_request =
+      rewriter.CheckResizeSegmentsRequest(request, segments);
+  ASSERT_TRUE(resize_request.has_value());
+  EXPECT_EQ(resize_request->segment_index, 0);
+  EXPECT_EQ(resize_request->segment_sizes[0], 2);
+}
+
 }  // namespace mozc
