@@ -32,11 +32,12 @@
 
 #include <cstdint>
 #include <string>
-#include <vector>
 
-#include "base/port.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "converter/segments.h"
 #include "dictionary/pos_matcher.h"
+#include "request/conversion_request.h"
 #include "rewriter/rewriter_interface.h"
 
 namespace mozc {
@@ -44,7 +45,9 @@ namespace mozc {
 class TransliterationRewriter : public RewriterInterface {
  public:
   explicit TransliterationRewriter(const dictionary::PosMatcher &pos_matcher);
-  ~TransliterationRewriter() override;
+  TransliterationRewriter(const TransliterationRewriter &) = delete;
+  TransliterationRewriter &operator=(const TransliterationRewriter &) = delete;
+  ~TransliterationRewriter() override = default;
 
   int capability(const ConversionRequest &request) const override;
 
@@ -54,13 +57,13 @@ class TransliterationRewriter : public RewriterInterface {
   void Finish(const ConversionRequest &request, Segments *segments) override {}
 
  private:
-  void InitT13nCandidate(const std::string &key, const std::string &value,
+  void InitT13nCandidate(absl::string_view key, absl::string_view value,
                          uint16_t lid, uint16_t rid,
                          Segment::Candidate *cand) const;
   // Sets transliteration values into segment.  If t13ns is invalid,
   // false is returned.
-  bool SetTransliterations(const std::vector<std::string> &t13ns,
-                           const std::string &key, Segment *segment) const;
+  bool SetTransliterations(absl::Span<const std::string> t13ns,
+                           absl::string_view key, Segment *segment) const;
   bool FillT13nsFromComposer(const ConversionRequest &request,
                              Segments *segments) const;
   bool FillT13nsFromKey(Segments *segments) const;
@@ -68,8 +71,6 @@ class TransliterationRewriter : public RewriterInterface {
                                   Segments *segments) const;
 
   const uint16_t unknown_id_;
-
-  DISALLOW_COPY_AND_ASSIGN(TransliterationRewriter);
 };
 
 }  // namespace mozc
