@@ -30,32 +30,27 @@
 #ifndef MOZC_REWRITER_USER_SEGMENT_HISTORY_REWRITER_H_
 #define MOZC_REWRITER_USER_SEGMENT_HISTORY_REWRITER_H_
 
+#include <algorithm>
+#include <cstddef>
 #include <cstdint>
 #include <memory>
-#include <string>
 #include <vector>
 
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "converter/segments.h"
 #include "dictionary/pos_group.h"
 #include "dictionary/pos_matcher.h"
+#include "request/conversion_request.h"
 #include "rewriter/rewriter_interface.h"
+#include "storage/lru_storage.h"
 
 namespace mozc {
-namespace storage {
-class LruStorage;
-}  // namespace storage
 
 class UserSegmentHistoryRewriter : public RewriterInterface {
  public:
-  struct ScoreType {
-    uint32_t last_access_time;
-    uint32_t score;
-    const Segment::Candidate *candidate;
-  };
-
-  UserSegmentHistoryRewriter(const dictionary::PosMatcher *pos_matcher,
-                             const dictionary::PosGroup *pos_group);
-  ~UserSegmentHistoryRewriter() override;
+  UserSegmentHistoryRewriter(const dictionary::PosMatcher &pos_matcher,
+                             const dictionary::PosGroup &pos_group);
 
   bool Rewrite(const ConversionRequest &request,
                Segments *segments) const override;
@@ -64,30 +59,72 @@ class UserSegmentHistoryRewriter : public RewriterInterface {
   bool Sync() override;
   bool Reload() override;
   void Clear() override;
+  void Revert(Segments *segments) override;
+  bool ClearHistoryEntry(const Segments &segments, size_t segment_index,
+                         int candidate_index) override;
 
  private:
+  friend class UserSegmentHistoryRewriterTestPeer;
+
+  struct Score {
+    constexpr void Update(const Score other) {
+      score = std::max(score, other.score);
+      last_access_time = std::max(last_access_time, other.last_access_time);
+    }
+
+    friend constexpr bool operator>(const Score a, const Score b) {
+      if (a.score == b.score) {
+        return a.last_access_time > b.last_access_time;
+      }
+      return a.score > b.score;
+    }
+
+    uint32_t score, last_access_time;
+  };
+
+  struct ScoreCandidate : public Score {
+    ScoreCandidate(const Score s, const Segment::Candidate *candidate)
+        : Score(s), candidate(candidate) {}
+
+    const Segment::Candidate *candidate;
+  };
+
+  static Segments MakeLearningSegmentsFromInnerSegments(
+      const ConversionRequest &request, const Segments &segments);
+
+  // Returns id for RevertEntry
+  static uint16_t revert_id();
+
   bool IsAvailable(const ConversionRequest &request,
                    const Segments &segments) const;
-  bool GetScore(const Segments &segments, size_t segment_index,
-                int candidate_index, uint32_t *score,
-                uint32_t *last_access_time) const;
-  bool Replaceable(const Segment::Candidate &lhs,
-                   const Segment::Candidate &rhs) const;
-  void RememberFirstCandidate(const Segments &segments, size_t segment_index);
-  void RememberNumberPreference(const Segment &segment);
+  Score GetScore(const ConversionRequest &request, const Segments &segments,
+                 size_t segment_index, int candidate_index) const;
+  bool Replaceable(const ConversionRequest &request,
+                   const Segment::Candidate &best_candidate,
+                   const Segment::Candidate &target_candidate) const;
+  // |revert_entries| will be stored to Segments and used to revert last
+  // Finish() operation in Revert().
+  void RememberFirstCandidate(
+      const ConversionRequest &request, const Segments &segments,
+      size_t segment_index, std::vector<Segments::RevertEntry> &revert_entries);
+  void RememberNumberPreference(
+      const Segment &segment,
+      std::vector<Segments::RevertEntry> &revert_entries);
   bool RewriteNumber(Segment *segment) const;
   bool ShouldRewrite(const Segment &segment, size_t *max_candidates_size) const;
   void InsertTriggerKey(const Segment &segment);
   bool IsPunctuation(const Segment &seg,
                      const Segment::Candidate &candidate) const;
-  bool GetFeatureLN(const Segments &segments, size_t i,
-                    const std::string &base_key, const std::string &base_value,
-                    std::string *value) const;
-  bool GetFeatureRN(const Segments &segments, size_t i,
-                    const std::string &base_key, const std::string &base_value,
-                    std::string *value) const;
-  bool SortCandidates(const std::vector<ScoreType> &sorted_scores,
+  bool SortCandidates(absl::Span<const ScoreCandidate> sorted_scores,
                       Segment *segment) const;
+  Score Fetch(absl::string_view key, uint32_t weight) const;
+  void Insert(absl::string_view key, bool force,
+              std::vector<Segments::RevertEntry> &revert_entries);
+  void MaybeInsertRevertEntry(
+      absl::string_view key,
+      std::vector<Segments::RevertEntry> &revert_entries);
+  // Returns true if deletion succeeded.
+  bool DeleteEntry(absl::string_view key);
 
   std::unique_ptr<storage::LruStorage> storage_;
   const dictionary::PosMatcher *pos_matcher_;
