@@ -32,46 +32,39 @@
 #ifndef MOZC_SESSION_SESSION_H_
 #define MOZC_SESSION_SESSION_H_
 
+#include <cstddef>
+#include <deque>
 #include <memory>
 #include <string>
 
-#include "base/port.h"
+#include "absl/strings/string_view.h"
+#include "absl/time/time.h"
 #include "composer/composer.h"
-#include "session/session_interface.h"
-// for FRIEND_TEST()
-#include "testing/base/public/gunit_prod.h"
+#include "composer/table.h"
+#include "engine/engine_interface.h"
+#include "protocol/commands.pb.h"
+#include "protocol/config.pb.h"
+#include "session/ime_context.h"
+#include "session/keymap.h"
+#include "testing/friend_test.h"
 #include "transliteration/transliteration.h"
 
 namespace mozc {
-namespace commands {
-class ApplicationInfo;
-class Capability;
-class Command;
-class Input;
-class KeyEvent;
-}  // namespace commands
-
-namespace composer {
-class Table;
-}  // namespace composer
-
-class EngineInterface;
-
 namespace session {
-class ImeContext;
 
-class Session : public SessionInterface {
+class Session {
  public:
-  explicit Session(EngineInterface *engine);
-  ~Session() override;
+  explicit Session(const EngineInterface &engine);
+  Session(const Session &) = delete;
+  Session &operator=(const Session &) = delete;
 
-  bool SendKey(mozc::commands::Command *command) override;
+  bool SendKey(mozc::commands::Command *command);
 
   // Check if the input key event will be consumed by the session.
-  bool TestSendKey(mozc::commands::Command *command) override;
+  bool TestSendKey(mozc::commands::Command *command);
 
   // Perform the SEND_COMMAND command defined commands.proto.
-  bool SendCommand(mozc::commands::Command *command) override;
+  bool SendCommand(mozc::commands::Command *command);
 
   // Turn on IME. Do nothing (but the keyevent is consumed) when IME is already
   // turned on.
@@ -81,7 +74,7 @@ class Session : public SessionInterface {
   // turned off.
   bool IMEOff(mozc::commands::Command *command);
 
-  // Unlike IMEOn/IMEOff, these commands 1) can update compositioin mode, and
+  // Unlike IMEOn/IMEOff, these commands 1) can update composition mode, and
   // 2) are functional even when IME is already turned on/off.
   // TODO(team): Merge these into IMEOn/Off once b/10250883 is fixed.
   bool MakeSureIMEOn(mozc::commands::Command *command);
@@ -139,6 +132,8 @@ class Session : public SessionInterface {
   bool InsertSpaceHalfWidth(mozc::commands::Command *command);
   bool InsertSpaceFullWidth(mozc::commands::Command *command);
   bool InsertCharacter(mozc::commands::Command *command);
+  bool UpdateComposition(mozc::commands::Command *command);
+  bool UpdateCompositionInternal(mozc::commands::Command *command);
   bool Delete(mozc::commands::Command *command);
   bool Backspace(mozc::commands::Command *command);
   bool EditCancel(mozc::commands::Command *command);
@@ -168,9 +163,6 @@ class Session : public SessionInterface {
   // Select a candidate located by input.command.id and commit.
   bool CommitCandidate(mozc::commands::Command *command);
 
-  // Expands suggestion candidates.
-  bool ExpandSuggestion(mozc::commands::Command *command);
-
   // Commits only the first segment.
   bool CommitSegment(mozc::commands::Command *command);
   // Commits some characters at the head of the preedit.
@@ -230,41 +222,46 @@ class Session : public SessionInterface {
   // Let client launch word register dialog
   bool LaunchWordRegisterDialog(mozc::commands::Command *command);
 
-  // Undo if pre-composition is empty. Rewind KANA cycle othrewise.
+  // Undo if pre-composition is empty. Rewind KANA cycle otherwise.
   bool UndoOrRewind(mozc::commands::Command *command);
 
   // Stops key toggling in the composer.
   bool StopKeyToggling(mozc::commands::Command *command);
 
-  // Send a command to the composer to append a special string.
-  bool SendComposerCommand(
-      const mozc::composer::Composer::InternalCommand composer_command,
-      mozc::commands::Command *command);
-
   bool ReportBug(mozc::commands::Command *command);
 
-  void SetConfig(mozc::config::Config *config) override;
+  void SetConfig(std::shared_ptr<const mozc::config::Config> config);
 
-  void SetRequest(const mozc::commands::Request *request) override;
+  void SetKeyMapManager(
+      std::shared_ptr<const mozc::keymap::KeyMapManager> key_map_manager);
 
-  void SetTable(const mozc::composer::Table *table) override;
+  void SetRequest(std::shared_ptr<const mozc::commands::Request> request);
+
+  void SetConfig(mozc::config::Config config) {
+    SetConfig(std::make_shared<const config::Config>(std::move(config)));
+  }
+
+  void SetRequest(mozc::commands::Request request) {
+    SetRequest(
+        std::make_shared<const mozc::commands::Request>(std::move(request)));
+  }
+
+  void SetTable(std::shared_ptr<const mozc::composer::Table> table);
 
   // Set client capability for this session.  Used by unittest.
-  void set_client_capability(
-      const mozc::commands::Capability &capability) override;
+  void set_client_capability(mozc::commands::Capability capability);
 
   // Set application information for this session.
-  void set_application_info(
-      const mozc::commands::ApplicationInfo &application_info) override;
+  void set_application_info(mozc::commands::ApplicationInfo application_info);
 
   // Get application information
-  const mozc::commands::ApplicationInfo &application_info() const override;
+  const mozc::commands::ApplicationInfo &application_info() const;
 
   // Return the time when this instance was created.
-  uint64 create_session_time() const override;
+  absl::Time create_session_time() const;
 
   // return 0 (default value) if no command is executed in this session.
-  uint64 last_command_time() const override;
+  absl::Time last_command_time() const;
 
   // TODO(komatsu): delete this function.
   // For unittest only
@@ -276,21 +273,29 @@ class Session : public SessionInterface {
   FRIEND_TEST(SessionTest, OutputInitialComposition);
   FRIEND_TEST(SessionTest, IsFullWidthInsertSpace);
   FRIEND_TEST(SessionTest, RequestUndo);
-
-  // Underlying conversion engine for this session. Please note that:
-  //   i) Session doesn't own the pointer.
-  //  ii) The state of underlying converter will change because it manages user
-  //      history, user dictionary, etc.
-  mozc::EngineInterface *engine_;
+  FRIEND_TEST(SessionTest, SetConfig);
 
   std::unique_ptr<ImeContext> context_;
-  std::unique_ptr<ImeContext> prev_context_;
 
-  void InitContext(ImeContext *context) const;
+  // Undo stack. *begin is the oldest, and *back is the newest.
+  std::deque<std::unique_ptr<ImeContext>> undo_contexts_;
+
+  std::unique_ptr<ImeContext> CreateContext(
+      const EngineInterface &engine) const;
 
   void PushUndoContext();
   void PopUndoContext();
+  // Clear the undo context.
+  // This should be called when the composer's preedit or cursor position
+  // is updated by non-undo related operations. This achieves intuitive
+  // behavior, which clears the undo context on the user's edit operation.
   void ClearUndoContext();
+  bool HasUndoContext() const;
+  // Set command.output.status.undo_available to true if HasUndoContext()==true.
+  // Otherwise no-op.
+  // Some code treats empty Status and default Status differently so
+  // we don't want to create a new Status instance if not required.
+  void MaybeSetUndoStatus(commands::Command *command) const;
 
   // Return true if full width space is preferred in the given new input
   // state than half width space. When |input| does not have new input mode,
@@ -308,27 +313,23 @@ class Session : public SessionInterface {
   // it requires the argument id.
   bool SelectCandidate(mozc::commands::Command *command);
 
-  // Calls SessionConverter::ConmmitFirstSegment() and deletes characters
-  // from the composer.
-  void CommitFirstSegmentInternal(const commands::Context &context);
-
-  // Calls SessionConverter::ConmmitHeadToFocusedSegments()
+  // Calls EngineConverter::ConmmitHeadToFocusedSegments()
   // and deletes characters from the composer.
   void CommitHeadToFocusedSegmentsInternal(const commands::Context &context);
 
-  // Commits without SessionConverter.
+  // Commits without EngineConverter.
   void CommitCompositionDirectly(commands::Command *command);
   void CommitSourceTextDirectly(commands::Command *command);
   void CommitRawTextDirectly(commands::Command *command);
-  void CommitStringDirectly(const std::string &key, const std::string &preedit,
+  void CommitStringDirectly(absl::string_view key, absl::string_view preedit,
                             commands::Command *command);
   bool CommitInternal(commands::Command *command,
                       bool trigger_zero_query_suggest);
 
-  // Calls SessionConverter::Suggest if the condition is applicable to
-  // call it.  True is returned when SessionConverter::Suggest is
+  // Calls EngineConverter::Suggest if the condition is applicable to
+  // call it.  True is returned when EngineConverter::Suggest is
   // called and results exist.  False is returned when
-  // SessionConverter::Suggest is not called or no results exist.
+  // EngineConverter::Suggest is not called or no results exist.
   bool Suggest(const mozc::commands::Input &input);
 
   // Commands like EditCancel should restore the original string used for
@@ -364,6 +365,9 @@ class Session : public SessionInterface {
   bool SendKeyCompositionState(mozc::commands::Command *command);
   bool SendKeyConversionState(mozc::commands::Command *command);
 
+  bool MoveCursorToEndInternal(mozc::commands::Command *command,
+                               bool clear_undo);
+
   // update last_command_time;
   void UpdateTime();
 
@@ -386,8 +390,6 @@ class Session : public SessionInterface {
 
   // Commits the raw text of the composition.
   bool CommitRawText(commands::Command *command);
-
-  DISALLOW_COPY_AND_ASSIGN(Session);
 };
 
 }  // namespace session
