@@ -29,103 +29,75 @@
 
 #include "session/session_handler_tool.h"
 
+#include <cstddef>
 #include <cstdint>
 #include <memory>
+#include <optional>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "absl/base/no_destructor.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_join.h"
+#include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "base/config_file_stream.h"
-#include "base/file_stream.h"
 #include "base/file_util.h"
 #include "base/number_util.h"
 #include "base/protobuf/descriptor.h"
 #include "base/protobuf/message.h"
 #include "base/protobuf/text_format.h"
+#include "base/strings/assign.h"
+#include "base/text_normalizer.h"
 #include "base/util.h"
 #include "composer/key_parser.h"
 #include "config/character_form_manager.h"
 #include "config/config_handler.h"
-#include "converter/converter_interface.h"
 #include "engine/engine_factory.h"
-#include "engine/user_data_manager_interface.h"
+#include "engine/engine_interface.h"
 #include "prediction/user_history_predictor.h"
-#include "protocol/candidates.pb.h"
+#include "protocol/candidate_window.pb.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
-#include "session/request_test_util.h"
+#include "request/request_test_util.h"
 #include "session/session_handler.h"
-#include "session/session_handler_interface.h"
-#include "session/session_usage_observer.h"
-#include "storage/registry.h"
-#include "usage_stats/usage_stats.h"
-#include "absl/status/status.h"
-#include "absl/strings/str_cat.h"
-#include "absl/strings/str_split.h"
-#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace session {
+namespace {
 
-using commands::CandidateList;
-using commands::CandidateWord;
-using commands::Command;
-using commands::CompositionMode;
-using commands::Input;
-using commands::KeyEvent;
-using commands::Output;
-using commands::Request;
-using commands::RequestForUnitTest;
-using config::CharacterFormManager;
-using config::Config;
-using config::ConfigHandler;
-using protobuf::FieldDescriptor;
-using protobuf::Message;
-using protobuf::TextFormat;
-using session::SessionHandlerTool;
+using ::mozc::commands::CandidateWord;
+using ::mozc::commands::Command;
+using ::mozc::commands::CompositionMode;
+using ::mozc::commands::Input;
+using ::mozc::commands::KeyEvent;
+using ::mozc::commands::Output;
+using ::mozc::commands::Request;
+using ::mozc::config::CharacterFormManager;
+using ::mozc::config::Config;
+using ::mozc::config::ConfigHandler;
+using ::mozc::protobuf::FieldDescriptor;
+using ::mozc::protobuf::Message;
+using ::mozc::protobuf::TextFormat;
+using ::mozc::session::SessionHandlerTool;
 
-bool CreateSession(SessionHandlerInterface *handler, uint64_t *id) {
-  Command command;
-  command.mutable_input()->set_type(commands::Input::CREATE_SESSION);
-  command.mutable_input()->mutable_capability()->set_text_deletion(
-      commands::Capability::DELETE_PRECEDING_TEXT);
-  handler->EvalCommand(&command);
-  if (id != nullptr) {
-    *id = command.has_output() ? command.output().id() : 0;
-  }
-  return (command.output().error_code() == commands::Output::SESSION_SUCCESS);
+std::string ToTextFormat(const Message &proto) {
+  std::string str;
+  TextFormat::PrintToString(proto, &str);
+  return str;
 }
 
-bool DeleteSession(SessionHandlerInterface *handler, uint64_t id) {
-  Command command;
-  command.mutable_input()->set_id(id);
-  command.mutable_input()->set_type(commands::Input::DELETE_SESSION);
-  return handler->EvalCommand(&command);
-}
-
-bool CleanUp(SessionHandlerInterface *handler, uint64_t id) {
-  Command command;
-  command.mutable_input()->set_id(id);
-  command.mutable_input()->set_type(commands::Input::CLEANUP);
-  return handler->EvalCommand(&command);
-}
-
-bool IsGoodSession(SessionHandlerInterface *handler, uint64_t id) {
-  Command command;
-  command.mutable_input()->set_id(id);
-  command.mutable_input()->set_type(commands::Input::SEND_KEY);
-  command.mutable_input()->mutable_key()->set_special_key(
-      commands::KeyEvent::SPACE);
-  handler->EvalCommand(&command);
-  return (command.output().error_code() == commands::Output::SESSION_SUCCESS);
-}
+}  // namespace
 
 SessionHandlerTool::SessionHandlerTool(std::unique_ptr<EngineInterface> engine)
     : id_(0),
-      usage_observer_(new SessionUsageObserver),
-      data_manager_(engine->GetUserDataManager()),
-      handler_(new SessionHandler(std::move(engine))) {
-  handler_->AddObserver(usage_observer_.get());
-}
-
-SessionHandlerTool::~SessionHandlerTool() {}
+      engine_(engine.get()),
+      handler_(std::make_unique<SessionHandler>(std::move(engine))) {}
 
 bool SessionHandlerTool::CreateSession() {
   Command command;
@@ -158,6 +130,13 @@ bool SessionHandlerTool::ClearUserPrediction() {
   return handler_->EvalCommand(&command);
 }
 
+bool SessionHandlerTool::ClearUserHistory() {
+  Command command;
+  command.mutable_input()->set_id(id_);
+  command.mutable_input()->set_type(commands::Input::CLEAR_USER_HISTORY);
+  return handler_->EvalCommand(&command);
+}
+
 bool SessionHandlerTool::SendKeyWithOption(const commands::KeyEvent &key,
                                            const commands::Input &option,
                                            commands::Output *output) {
@@ -178,6 +157,25 @@ bool SessionHandlerTool::TestSendKeyWithOption(const commands::KeyEvent &key,
   return EvalCommand(&input, output);
 }
 
+bool SessionHandlerTool::UpdateComposition(absl::Span<const std::string> args,
+                                           commands::Output *output) {
+  DCHECK_EQ(0, args.size() % 2);
+  commands::Input input;
+  //  input.set_type(commands::Input::UPDATE_COMPOSITION);
+  input.set_type(commands::Input::SEND_COMMAND);
+  input.mutable_command()->set_type(
+      commands::SessionCommand::UPDATE_COMPOSITION);
+  for (int i = 0; i < args.size(); i += 2) {
+    commands::SessionCommand::CompositionEvent *composition_event =
+        input.mutable_command()->add_composition_events();
+    composition_event->set_composition_string(args[i]);
+    if (double value = 0.0; NumberUtil::SafeStrToDouble(args[i + 1], &value)) {
+      composition_event->set_probability(value);
+    }
+  }
+  return EvalCommand(&input, output);
+}
+
 bool SessionHandlerTool::SelectCandidate(uint32_t id,
                                          commands::Output *output) {
   commands::Input input;
@@ -196,14 +194,6 @@ bool SessionHandlerTool::SubmitCandidate(uint32_t id,
   return EvalCommand(&input, output);
 }
 
-bool SessionHandlerTool::ExpandSuggestion(commands::Output *output) {
-  commands::Input input;
-  input.set_type(commands::Input::SEND_COMMAND);
-  input.mutable_command()->set_type(
-      commands::SessionCommand::EXPAND_SUGGESTION);
-  return EvalCommand(&input, output);
-}
-
 bool SessionHandlerTool::Reload() {
   commands::Input input;
   input.set_type(commands::Input::RELOAD);
@@ -224,6 +214,18 @@ bool SessionHandlerTool::UndoOrRewind(commands::Output *output) {
   return EvalCommand(&input, output);
 }
 
+bool SessionHandlerTool::DeleteCandidateFromHistory(std::optional<int> id,
+                                                    commands::Output *output) {
+  commands::Input input;
+  input.set_type(commands::Input::SEND_COMMAND);
+  input.mutable_command()->set_type(
+      commands::SessionCommand::DELETE_CANDIDATE_FROM_HISTORY);
+  if (id.has_value()) {
+    input.mutable_command()->set_id(*id);
+  }
+  return EvalCommand(&input, output);
+}
+
 bool SessionHandlerTool::SwitchInputMode(
     commands::CompositionMode composition_mode) {
   commands::Input input;
@@ -250,10 +252,22 @@ bool SessionHandlerTool::SetConfig(const config::Config &config,
   return EvalCommand(&input, output);
 }
 
-bool SessionHandlerTool::SyncData() { return data_manager_->Wait(); }
+bool SessionHandlerTool::SyncData() {
+  engine_->Sync();
+  engine_->Wait();
+  return true;
+}
 
-void SessionHandlerTool::SetCallbackText(const std::string &text) {
-  callback_text_ = text;
+void SessionHandlerTool::SetCallbackText(const absl::string_view text) {
+  strings::Assign(callback_text_, text);
+}
+
+bool SessionHandlerTool::ReloadSupplementalModel(absl::string_view model_path) {
+  commands::Input input;
+  input.mutable_engine_reload_request()->set_file_path(model_path);
+  input.set_type(commands::Input::RELOAD_SUPPLEMENTAL_MODEL);
+  commands::Output output;
+  return EvalCommand(&input, &output);
 }
 
 bool SessionHandlerTool::EvalCommandInternal(commands::Input *input,
@@ -293,11 +307,9 @@ SessionHandlerInterpreter::SessionHandlerInterpreter()
 SessionHandlerInterpreter::SessionHandlerInterpreter(
     std::unique_ptr<EngineInterface> engine) {
   client_ = std::make_unique<SessionHandlerTool>(std::move(engine));
-  config_ = std::make_unique<Config>();
   last_output_ = std::make_unique<Output>();
   request_ = std::make_unique<Request>();
-
-  ConfigHandler::GetConfig(config_.get());
+  config_ = ConfigHandler::GetCopiedConfig();
 
   // Set up session.
   CHECK(client_->CreateSession()) << "Client initialization is failed.";
@@ -308,15 +320,10 @@ SessionHandlerInterpreter::~SessionHandlerInterpreter() {
   CHECK(client_->DeleteSession());
 
   ClearState();
-  request_.reset();
-  last_output_.reset();
-  config_.reset();
-  client_.reset();
 }
 
 void SessionHandlerInterpreter::ClearState() {
-  Config config;
-  ConfigHandler::GetDefaultConfig(&config);
+  const Config &config = ConfigHandler::DefaultConfig();
   ConfigHandler::SetConfig(config);
 
   // CharacterFormManager is not automatically updated when the config is
@@ -325,19 +332,17 @@ void SessionHandlerInterpreter::ClearState() {
 
   // Some destructors may save the state on storages. To clear the state, we
   // explicitly call destructors before clearing storages.
-  mozc::storage::Registry::Clear();
   FileUtil::UnlinkOrLogError(
-      mozc::ConfigFileStream::GetFileName("user://boundary.db"));
+      ConfigFileStream::GetFileName("user://boundary.db"));
   FileUtil::UnlinkOrLogError(
-      mozc::ConfigFileStream::GetFileName("user://segment.db"));
+      ConfigFileStream::GetFileName("user://segment.db"));
   FileUtil::UnlinkOrLogError(
-      mozc::UserHistoryPredictor::GetUserHistoryFileName());
+      prediction::UserHistoryPredictor::GetUserHistoryFileName());
 }
 
 void SessionHandlerInterpreter::ClearAll() {
   ResetContext();
   ClearUserPrediction();
-  ClearUsageStats();
 }
 
 void SessionHandlerInterpreter::ResetContext() {
@@ -351,19 +356,16 @@ void SessionHandlerInterpreter::SyncDataToStorage() {
 
 void SessionHandlerInterpreter::ClearUserPrediction() {
   CHECK(client_->ClearUserPrediction());
+  CHECK(client_->ClearUserHistory());
   SyncDataToStorage();
 }
 
-void SessionHandlerInterpreter::ClearUsageStats() {
-  mozc::usage_stats::UsageStats::ClearAllStatsForTest();
-}
-
 const Output &SessionHandlerInterpreter::LastOutput() const {
   return *last_output_;
 }
 
 const CandidateWord &SessionHandlerInterpreter::GetCandidateByValue(
-    const absl::string_view value) {
+    const absl::string_view value) const {
   const Output &output = LastOutput();
 
   for (const CandidateWord &candidate :
@@ -380,40 +382,64 @@ const CandidateWord &SessionHandlerInterpreter::GetCandidateByValue(
     }
   }
 
-  static CandidateWord *fallback_candidate = new CandidateWord;
+  static absl::NoDestructor<CandidateWord> fallback_candidate;
   return *fallback_candidate;
 }
 
 bool SessionHandlerInterpreter::GetCandidateIdByValue(
-    const absl::string_view value, uint32_t *id) {
+    const absl::string_view value, uint32_t *id) const {
   const Output &output = LastOutput();
-
-  auto find_id = [&value](const CandidateList &candidate_list,
-                          uint32 *id) -> bool {
-    for (const CandidateWord &candidate : candidate_list.candidates()) {
-      if (candidate.has_value() && candidate.value() == value) {
-        *id = candidate.id();
-        return true;
-      }
-    }
+  if (!output.has_all_candidate_words()) {
     return false;
-  };
-
-  if (output.has_all_candidate_words() &&
-      find_id(output.all_candidate_words(), id)) {
-    return true;
   }
 
-  if (output.has_removed_candidate_words_for_debug() &&
-      find_id(output.removed_candidate_words_for_debug(), id)) {
-    return true;
+  for (const CandidateWord &candidate :
+       output.all_candidate_words().candidates()) {
+    if (candidate.has_value() && candidate.value() == value) {
+      *id = candidate.id();
+      return true;
+    }
   }
-
   return false;
 }
 
-bool SetOrAddFieldValueFromString(const std::string &name,
-                                  const std::string &value, Message *message) {
+std::vector<uint32_t> SessionHandlerInterpreter::GetCandidateIdsByValue(
+    absl::string_view value) const {
+  const Output &output = LastOutput();
+  if (!output.has_all_candidate_words()) {
+    return {};
+  }
+
+  std::vector<uint32_t> ids;
+  for (const CandidateWord &candidate :
+       output.all_candidate_words().candidates()) {
+    if (candidate.has_value() && candidate.value() == value) {
+      ids.push_back(candidate.id());
+    }
+  }
+  return ids;
+}
+
+std::vector<uint32_t> SessionHandlerInterpreter::GetRemovedCandidateIdsByValue(
+    absl::string_view value) const {
+  const Output &output = LastOutput();
+  if (!output.has_removed_candidate_words_for_debug()) {
+    return {};
+  }
+
+  std::vector<uint32_t> ids;
+  for (const CandidateWord &candidate :
+       output.removed_candidate_words_for_debug().candidates()) {
+    if (candidate.has_value() && candidate.value() == value) {
+      ids.push_back(candidate.id());
+    }
+  }
+  return ids;
+}
+
+bool SetOrAddFieldValueFromString(const absl::string_view name,
+                                  const absl::string_view value,
+                                  Message *message) {
   const FieldDescriptor *field =
       message->GetDescriptor()->FindFieldByName(name);
   if (!field) {
@@ -429,15 +455,33 @@ bool SetOrAddFieldValueFromString(const std::string &name,
   return TextFormat::ParseFieldValueFromString(value, field, message);
 }
 
+bool SetOrAddFieldValueFromString(const absl::Span<const std::string> names,
+                                  const absl::string_view value,
+                                  Message *message) {
+  if (names.empty()) {
+    LOG(ERROR) << "Empty names is passed";
+    return false;
+  }
+  const std::string &first = names[0];
+  if (names.size() == 1) {
+    return SetOrAddFieldValueFromString(first, value, message);
+  }
+  const FieldDescriptor *field =
+      message->GetDescriptor()->FindFieldByName(first);
+  Message *field_message =
+      message->GetReflection()->MutableMessage(message, field);
+  return SetOrAddFieldValueFromString(names.subspan(1), value, field_message);
+}
+
 // Parses protobuf from string without validation.
 // input sample: context.experimental_features="chrome_omnibox"
 // We cannot use TextFormat::ParseFromString since it doesn't allow invalid
 // protobuf. (e.g. lack of required field)
-bool ParseProtobufFromString(const std::string &text, Message *message) {
+bool ParseProtobufFromString(const absl::string_view text, Message *message) {
   const size_t separator_pos = text.find('=');
-  const std::string full_name = text.substr(0, separator_pos);
-  const std::string value = text.substr(separator_pos + 1);
-  std::vector<std::string> names =
+  const absl::string_view full_name = text.substr(0, separator_pos);
+  const absl::string_view value = text.substr(separator_pos + 1);
+  std::vector<absl::string_view> names =
       absl::StrSplit(full_name, '.', absl::SkipEmpty());
 
   Message *msg = message;
@@ -451,11 +495,11 @@ bool ParseProtobufFromString(const std::string &text, Message *message) {
     msg = msg->GetReflection()->MutableMessage(msg, field);
   }
 
-  return SetOrAddFieldValueFromString(names[names.size() - 1], value, msg);
+  return SetOrAddFieldValueFromString(names.back(), value, msg);
 }
 
 std::vector<std::string> SessionHandlerInterpreter::Parse(
-    const std::string &line) {
+    const absl::string_view line) {
   std::vector<std::string> args;
   if (line.empty() || line.front() == '#') {
     return args;
@@ -463,7 +507,7 @@ std::vector<std::string> SessionHandlerInterpreter::Parse(
   std::vector<absl::string_view> columns = absl::StrSplit(line, '\t');
   for (absl::string_view column : columns) {
     if (column.empty()) {
-      args.push_back("");
+      args.emplace_back();
       continue;
     }
     // If the first and last characters are doublequotes, trim them.
@@ -471,7 +515,7 @@ std::vector<std::string> SessionHandlerInterpreter::Parse(
       column.remove_prefix(1);
       column.remove_suffix(1);
     }
-    args.push_back(std::string(column));
+    args.emplace_back(column);
   }
   return args;
 }
@@ -493,8 +537,44 @@ std::vector<std::string> SessionHandlerInterpreter::Parse(
     return absl::InvalidArgumentError(""); \
   }
 
+#define MOZC_EXPECT_EQ_MSG(expected, actual, message) \
+  if ((expected) != (actual)) {                       \
+    return absl::InternalError(message);              \
+  }
+#define MOZC_EXPECT_EQ(expected, actual) \
+  if ((expected) != (actual)) {          \
+    return absl::InternalError("");      \
+  }
+
+#define MOZC_EXPECT_TRUE_MSG(result, message) \
+  if (!(result)) {                            \
+    return absl::InternalError(message);      \
+  }
+#define MOZC_EXPECT_TRUE(result)    \
+  if (!(result)) {                  \
+    return absl::InternalError(""); \
+  }
+
+// Placeholders
+#define MOZC_EXPECT_STATS_NOT_EXIST(name) \
+  while (false) {                         \
+  }
+#define MOZC_EXPECT_COUNT_STATS(name, value) \
+  while (false) {                            \
+  }
+#define MOZC_EXPECT_INTEGER_STATS(name, value) \
+  while (false) {                              \
+  }
+#define MOZC_EXPECT_BOOLEAN_STATS(name, value) \
+  while (false) {                              \
+  }
+// Uses args to suppress compiler warnings.
+#define MOZC_EXPECT_TIMING_STATS(name, total, num, min, max) \
+  while (false && total && num && min && max) {              \
+  }
+
 absl::Status SessionHandlerInterpreter::Eval(
-    const std::vector<std::string> &args) {
+    const absl::Span<const std::string> args) {
   if (args.empty()) {
     // Skip empty args
     return absl::Status();
@@ -567,6 +647,10 @@ absl::Status SessionHandlerInterpreter::Eval(
     }
     MOZC_ASSERT_TRUE(
         client_->TestSendKeyWithOption(key_event, option, last_output_.get()));
+  } else if (command == "UPDATE_COMPOSITION") {
+    MOZC_ASSERT_EQ(1, args.size() % 2);
+    MOZC_ASSERT_TRUE(
+        client_->UpdateComposition(args.subspan(1), last_output_.get()));
   } else if (command == "SELECT_CANDIDATE") {
     MOZC_ASSERT_EQ(2, args.size());
     MOZC_ASSERT_TRUE(client_->SelectCandidate(NumberUtil::SimpleAtoi(args[1]),
@@ -585,10 +669,16 @@ absl::Status SessionHandlerInterpreter::Eval(
     uint32_t id;
     MOZC_ASSERT_TRUE(GetCandidateIdByValue(args[1], &id));
     MOZC_ASSERT_TRUE(client_->SubmitCandidate(id, last_output_.get()));
-  } else if (command == "EXPAND_SUGGESTION") {
-    MOZC_ASSERT_TRUE(client_->ExpandSuggestion(last_output_.get()));
   } else if (command == "UNDO_OR_REWIND") {
     MOZC_ASSERT_TRUE(client_->UndoOrRewind(last_output_.get()));
+  } else if (command == "DELETE_CANDIDATE_FROM_HISTORY") {
+    MOZC_ASSERT_TRUE(args.size() == 1 || args.size() == 2);
+    std::optional<int> id = std::nullopt;
+    if (args.size() == 2) {
+      id = NumberUtil::SimpleAtoi(args[1]);
+    }
+    MOZC_ASSERT_TRUE(
+        client_->DeleteCandidateFromHistory(id, last_output_.get()));
   } else if (command == "SWITCH_INPUT_MODE") {
     MOZC_ASSERT_EQ(2, args.size());
     CompositionMode composition_mode;
@@ -599,18 +689,34 @@ absl::Status SessionHandlerInterpreter::Eval(
     *request_ = Request::default_instance();
     MOZC_ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
   } else if (command == "SET_MOBILE_REQUEST") {
-    RequestForUnitTest::FillMobileRequest(request_.get());
+    request_test_util::FillMobileRequest(request_.get());
+    MOZC_ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
+  } else if (command == "SET_HANDWRITING_REQUEST") {
+    request_test_util::FillMobileRequestForHandwriting(request_.get());
     MOZC_ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
   } else if (command == "SET_REQUEST") {
-    MOZC_ASSERT_EQ(3, args.size());
-    MOZC_ASSERT_TRUE(
-        SetOrAddFieldValueFromString(args[1], args[2], request_.get()));
+    MOZC_ASSERT_TRUE(args.size() >= 3);
+    MOZC_ASSERT_TRUE(SetOrAddFieldValueFromString(
+        std::vector<std::string>(args.begin() + 1, args.end() - 1),
+        *(args.end() - 1), request_.get()));
     MOZC_ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
   } else if (command == "SET_CONFIG") {
-    MOZC_ASSERT_EQ(3, args.size());
-    MOZC_ASSERT_TRUE(
-        SetOrAddFieldValueFromString(args[1], args[2], config_.get()));
-    MOZC_ASSERT_TRUE(client_->SetConfig(*config_, last_output_.get()));
+    MOZC_ASSERT_TRUE(args.size() >= 3);
+    MOZC_ASSERT_TRUE(SetOrAddFieldValueFromString(
+        std::vector<std::string>(args.begin() + 1, args.end() - 1),
+        *(args.end() - 1), &config_));
+    MOZC_ASSERT_TRUE(client_->SetConfig(config_, last_output_.get()));
+  } else if (command == "MERGE_DECODER_EXPERIMENT_PARAMS") {
+    MOZC_ASSERT_EQ(2, args.size());
+    if (const std::string &textproto = args[1]; !textproto.empty()) {
+      mozc::commands::DecoderExperimentParams params;
+      CHECK(mozc::protobuf::TextFormat::ParseFromString(textproto, &params))
+          << "Invalid DecoderExperimentParams: " << textproto;
+      request_->mutable_decoder_experiment_params()->MergeFrom(params);
+      LOG(INFO) << "DecoderExperimentParams was set:\n"
+                << request_->decoder_experiment_params();
+      MOZC_ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
+    }
   } else if (command == "SET_SELECTION_TEXT") {
     MOZC_ASSERT_EQ(2, args.size());
     client_->SetCallbackText(args[1]);
@@ -633,14 +739,127 @@ absl::Status SessionHandlerInterpreter::Eval(
   } else if (command == "CLEAR_USER_PREDICTION") {
     MOZC_ASSERT_EQ(1, args.size());
     ClearUserPrediction();
-  } else if (command == "CLEAR_USAGE_STATS") {
-    MOZC_ASSERT_EQ(1, args.size());
-    ClearUsageStats();
+  } else if (command == "EXPECT_CONSUMED") {
+    MOZC_ASSERT_EQ(args.size(), 2);
+    MOZC_ASSERT_TRUE(last_output_->has_consumed());
+    MOZC_EXPECT_EQ(last_output_->consumed(), args[1] == "true");
+  } else if (command == "EXPECT_PREEDIT") {
+    // Concat preedit segments and assert.
+    const std::string &expected_preedit =
+        TextNormalizer::NormalizeText(args.size() == 1 ? "" : args[1]);
+    std::string preedit_string;
+    const mozc::commands::Preedit &preedit = last_output_->preedit();
+    for (int i = 0; i < preedit.segment_size(); ++i) {
+      preedit_string += preedit.segment(i).value();
+    }
+    MOZC_EXPECT_EQ_MSG(
+        preedit_string, expected_preedit,
+        absl::StrCat("Expected preedit: ", expected_preedit, "\n",
+                     "Actual preedit: ", ToTextFormat(preedit)));
+  } else if (command == "EXPECT_PREEDIT_IN_DETAIL") {
+    MOZC_ASSERT_TRUE(!args.empty());
+    const mozc::commands::Preedit &preedit = last_output_->preedit();
+    MOZC_ASSERT_EQ(preedit.segment_size(), args.size() - 1);
+    for (int i = 0; i < preedit.segment_size(); ++i) {
+      MOZC_EXPECT_EQ_MSG(preedit.segment(i).value(),
+                         TextNormalizer::NormalizeText(args[i + 1]),
+                         absl::StrCat("Segment index = ", i));
+    }
+  } else if (command == "EXPECT_PREEDIT_CURSOR_POS") {
+    // Concat preedit segments and assert.
+    MOZC_ASSERT_EQ(args.size(), 2);
+    const size_t expected_pos = NumberUtil::SimpleAtoi(args[1]);
+    const mozc::commands::Preedit &preedit = last_output_->preedit();
+    MOZC_EXPECT_EQ_MSG(preedit.cursor(), expected_pos, ToTextFormat(preedit));
+  } else if (command == "EXPECT_CANDIDATE") {
+    MOZC_ASSERT_EQ(args.size(), 3);
+    uint32_t candidate_id = 0;
+    const bool has_result = GetCandidateIdByValue(args[2], &candidate_id);
+    MOZC_EXPECT_TRUE_MSG(
+        has_result,
+        absl::StrCat(args[2], " is not found\n",
+                     ToTextFormat(last_output_->candidate_window())));
+    if (has_result) {
+      MOZC_EXPECT_EQ_MSG(
+          candidate_id, NumberUtil::SimpleAtoi(args[1]),
+          absl::StrCat(args[1], " is not found\n",
+                       ToTextFormat(last_output_->candidate_window())));
+    }
+  } else if (command == "EXPECT_CANDIDATE_DESCRIPTION") {
+    MOZC_ASSERT_EQ(args.size(), 3);
+    const CandidateWord &cand = GetCandidateByValue(args[1]);
+    const bool has_cand = !cand.value().empty();
+    MOZC_EXPECT_TRUE_MSG(
+        has_cand, absl::StrCat(args[1], " is not found\n",
+                               ToTextFormat(last_output_->candidate_window())));
+    MOZC_EXPECT_TRUE(has_cand);
+    MOZC_EXPECT_EQ_MSG(cand.annotation().description(), args[2],
+                       ToTextFormat(cand));
+  } else if (command == "EXPECT_RESULT") {
+    if (args.size() == 2 && !args[1].empty()) {
+      MOZC_ASSERT_TRUE(last_output_->has_result());
+      const mozc::commands::Result &result = last_output_->result();
+      MOZC_EXPECT_EQ_MSG(result.value(), TextNormalizer::NormalizeText(args[1]),
+                         ToTextFormat(result));
+    } else {
+      MOZC_EXPECT_TRUE_MSG(!last_output_->has_result(),
+                           ToTextFormat(last_output_->result()));
+    }
+  } else if (command == "EXPECT_IN_ALL_CANDIDATE_WORDS") {
+    MOZC_ASSERT_EQ(args.size(), 2);
+    uint32_t candidate_id = 0;
+    const bool has_result = GetCandidateIdByValue(args[1], &candidate_id);
+    MOZC_EXPECT_TRUE_MSG(has_result, absl::StrCat(args[1], " is not found.\n",
+                                                  ToTextFormat(*last_output_)));
+  } else if (command == "EXPECT_NOT_IN_ALL_CANDIDATE_WORDS") {
+    MOZC_ASSERT_EQ(args.size(), 2);
+    uint32_t candidate_id = 0;
+    const bool has_result = GetCandidateIdByValue(args[1], &candidate_id);
+    MOZC_EXPECT_TRUE_MSG(
+        !has_result,
+        absl::StrCat(args[1], " is found.\n", ToTextFormat(*last_output_)));
+  } else if (command == "EXPECT_HAS_CANDIDATES") {
+    if (args.size() == 2 && !args[1].empty()) {
+      MOZC_ASSERT_TRUE(last_output_->has_candidate_window());
+      MOZC_ASSERT_TRUE_MSG(last_output_->candidate_window().size() >
+                               NumberUtil::SimpleAtoi(args[1]),
+                           ToTextFormat(*last_output_));
+    } else {
+      MOZC_ASSERT_TRUE(last_output_->has_candidate_window());
+    }
+  } else if (command == "EXPECT_NO_CANDIDATES") {
+    MOZC_ASSERT_TRUE(!last_output_->has_candidate_window());
+  } else if (command == "EXPECT_SEGMENTS_SIZE") {
+    MOZC_ASSERT_EQ(args.size(), 2);
+    MOZC_ASSERT_EQ(last_output_->preedit().segment_size(),
+                   NumberUtil::SimpleAtoi(args[1]));
+  } else if (command == "EXPECT_HIGHLIGHTED_SEGMENT_INDEX") {
+    MOZC_ASSERT_EQ(args.size(), 2);
+    MOZC_ASSERT_TRUE(last_output_->has_preedit());
+    const mozc::commands::Preedit &preedit = last_output_->preedit();
+    int index = -1;
+    for (int i = 0; i < preedit.segment_size(); ++i) {
+      if (preedit.segment(i).annotation() ==
+          mozc::commands::Preedit::Segment::HIGHLIGHT) {
+        index = i;
+        break;
+      }
+    }
+    MOZC_ASSERT_EQ(index, NumberUtil::SimpleAtoi(args[1]));
   } else {
     return absl::Status(absl::StatusCode::kUnimplemented, "");
   }
 
-  return absl::Status();
+  return absl::OkStatus();
+}
+
+void SessionHandlerInterpreter::SetRequest(const commands::Request &request) {
+  *request_ = request;
+}
+
+void SessionHandlerInterpreter::ReloadSupplementalModel(
+    absl::string_view model_path) {
+  client_->ReloadSupplementalModel(model_path);
 }
 
 }  // namespace session
