@@ -32,14 +32,14 @@
 
 #include <string>
 
-#include "base/port.h"
+#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace storage {
 
 class StringStorageInterface {
  public:
-  virtual ~StringStorageInterface() {}
+  virtual ~StringStorageInterface() = default;
 
   virtual bool Load(std::string *output) const = 0;
   virtual bool Save(const std::string &input) const = 0;
@@ -47,8 +47,10 @@ class StringStorageInterface {
 
 class EncryptedStringStorage : public StringStorageInterface {
  public:
-  explicit EncryptedStringStorage(const std::string &filename);
-  ~EncryptedStringStorage() override;
+  explicit EncryptedStringStorage(const absl::string_view filename)
+      : filename_(filename) {}
+  EncryptedStringStorage(const EncryptedStringStorage &) = delete;
+  EncryptedStringStorage &operator=(const EncryptedStringStorage &) = delete;
 
   bool Load(std::string *output) const override;
   bool Save(const std::string &input) const override;
@@ -59,8 +61,6 @@ class EncryptedStringStorage : public StringStorageInterface {
 
  private:
   std::string filename_;
-
-  DISALLOW_COPY_AND_ASSIGN(EncryptedStringStorage);
 };
 
 }  // namespace storage
