@@ -38,50 +38,49 @@
 #include <sstream>
 #include <string>
 
+#include "absl/flags/flag.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "absl/time/time.h"
 #include "base/clock.h"
 #include "base/const.h"
 #include "base/file_util.h"
-#include "base/logging.h"
 #include "base/singleton.h"
 #include "base/system_util.h"
 #include "base/util.h"
+#include "base/vlog.h"
 #include "client/client.h"
+#include "protocol/candidate_window.pb.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
-#include "session/ime_switch_util.h"
+#include "renderer/renderer_client.h"
+#include "unix/ibus/candidate_window_handler.h"
 #include "unix/ibus/engine_registrar.h"
 #include "unix/ibus/ibus_candidate_window_handler.h"
+#include "unix/ibus/ibus_wrapper.h"
 #include "unix/ibus/key_event_handler.h"
 #include "unix/ibus/message_translator.h"
-#include "unix/ibus/mozc_engine_property.h"
 #include "unix/ibus/path_util.h"
 #include "unix/ibus/preedit_handler.h"
 #include "unix/ibus/property_handler.h"
-#include "unix/ibus/selection_monitor.h"
 #include "unix/ibus/surrounding_text_util.h"
-#include "absl/flags/flag.h"
 
-#ifdef ENABLE_GTK_RENDERER
-#include "renderer/renderer_client.h"
-#include "unix/ibus/gtk_candidate_window_handler.h"
-#endif  // ENABLE_GTK_RENDERER
-
-
-#ifdef ENABLE_GTK_RENDERER
 ABSL_FLAG(bool, use_mozc_renderer, true,
           "The engine tries to use mozc_renderer if available.");
-#endif  // ENABLE_GTK_RENDERER
 
+namespace mozc {
+namespace ibus {
 namespace {
 
 // The ID for candidates which are not associated with texts.
-const int32 kBadCandidateId = -1;
+const int32_t kBadCandidateId = -1;
 
 // Default UI locale
 constexpr char kMozcDefaultUILocale[] = "en_US.UTF-8";
 
 // for every 5 minutes, call SyncData
-const uint64 kSyncDataInterval = 5 * 60;
+const absl::Duration kSyncDataInterval = absl::Minutes(5);
 
 const char *kUILocaleEnvNames[] = {
     "LC_ALL",
@@ -104,90 +103,24 @@ std::string GetMessageLocale() {
   return kMozcDefaultUILocale;
 }
 
-struct IBusMozcEngineClass {
-  IBusEngineClass parent;
-};
-
-struct IBusMozcEngine {
-  IBusEngine parent;
-  mozc::ibus::MozcEngine *engine;
-};
-
-IBusEngineClass *g_parent_class = nullptr;
-
-GObject *MozcEngineClassConstructor(
-    GType type, guint n_construct_properties,
-    GObjectConstructParam *construct_properties) {
-  return G_OBJECT_CLASS(g_parent_class)
-      ->constructor(type, n_construct_properties, construct_properties);
-}
-
-void MozcEngineClassDestroy(IBusObject *engine) {
-  IBUS_OBJECT_CLASS(g_parent_class)->destroy(engine);
-}
-
-void MozcEngineClassInit(gpointer klass, gpointer class_data) {
-  IBusEngineClass *engine_class = IBUS_ENGINE_CLASS(klass);
-
-  VLOG(2) << "MozcEngineClassInit is called";
-  mozc::ibus::EngineRegistrar::Register(
-      mozc::Singleton<mozc::ibus::MozcEngine>::get(), engine_class);
-
-  g_parent_class =
-      reinterpret_cast<IBusEngineClass *>(g_type_class_peek_parent(klass));
-
-  GObjectClass *object_class = G_OBJECT_CLASS(klass);
-  object_class->constructor = MozcEngineClassConstructor;
-  IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS(klass);
-  ibus_object_class->destroy = MozcEngineClassDestroy;
-}
-
-void MozcEngineInstanceInit(GTypeInstance *instance, gpointer klass) {
-  IBusMozcEngine *engine = reinterpret_cast<IBusMozcEngine *>(instance);
-  engine->engine = mozc::Singleton<mozc::ibus::MozcEngine>::get();
-}
-
-}  // namespace
-
-namespace mozc {
-namespace ibus {
-
-namespace {
 struct SurroundingTextInfo {
   SurroundingTextInfo() : relative_selected_length(0) {}
-  int32 relative_selected_length;
+  int32_t relative_selected_length;
   std::string preceding_text;
   std::string selection_text;
   std::string following_text;
 };
 
-bool GetSurroundingText(IBusEngine *engine,
-                        SelectionMonitorInterface *selection_monitor,
-                        SurroundingTextInfo *info) {
-  if (!(engine->client_capabilities & IBUS_CAP_SURROUNDING_TEXT)) {
-    VLOG(1) << "Give up CONVERT_REVERSE due to client_capabilities: "
-            << engine->client_capabilities;
+bool GetSurroundingText(IbusEngineWrapper *engine, SurroundingTextInfo *info) {
+  if (!(engine->CheckCapabilities(IBUS_CAP_SURROUNDING_TEXT))) {
+    MOZC_VLOG(1) << "Give up CONVERT_REVERSE due to client_capabilities: "
+                 << engine->GetCapabilities();
     return false;
   }
-  guint cursor_pos = 0;
-  guint anchor_pos = 0;
-  // DO NOT call g_object_unref against this.
-  // http://developer.gnome.org/gobject/stable/gobject-The-Base-Object-Type.html#gobject-The-Base-Object-Type.description
-  IBusText *text = nullptr;
-  ibus_engine_get_surrounding_text(engine, &text, &cursor_pos, &anchor_pos);
-  const std::string surrounding_text(ibus_text_get_text(text));
-
-#ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
-  if (cursor_pos == anchor_pos && selection_monitor != nullptr) {
-    const SelectionInfo &info = selection_monitor->GetSelectionInfo();
-    guint new_anchor_pos = 0;
-    if (SurroundingTextUtil::GetAnchorPosFromSelection(
-            surrounding_text, info.selected_text, cursor_pos,
-            &new_anchor_pos)) {
-      anchor_pos = new_anchor_pos;
-    }
-  }
-#endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
+  uint cursor_pos = 0;
+  uint anchor_pos = 0;
+  const absl::string_view surrounding_text =
+      engine->GetSurroundingText(&cursor_pos, &anchor_pos);
 
   if (!SurroundingTextUtil::GetSafeDelta(cursor_pos, anchor_pos,
                                          &info->relative_selected_length)) {
@@ -236,47 +169,78 @@ std::unique_ptr<client::ClientInterface> CreateAndConfigureClient() {
   return client;
 }
 
-#if defined(ENABLE_GTK_RENDERER) || defined(ENABLE_QT_RENDERER)
-CandidateWindowHandlerInterface *createGtkCandidateWindowHandler(
-    ::mozc::renderer::RendererClient *renderer_client) {
-  if (!absl::GetFlag(FLAGS_use_mozc_renderer)) {
-    return nullptr;
+std::optional<absl::string_view> GetMapValue(
+    const absl::flat_hash_map<std::string, std::string> &map,
+    absl::string_view key) {
+  absl::flat_hash_map<std::string, std::string>::const_iterator it =
+      map.find(key);
+  if (it == map.end()) {
+    return std::nullopt;
   }
+  return absl::string_view(it->second);
+}
 
-#ifndef ENABLE_QT_RENDERER
-  if (GetEnv("XDG_SESSION_TYPE") == "wayland") {
-    // mozc_renderer is not supported on wayland session.
-    return nullptr;
+bool IsWaylandSession(
+    const absl::flat_hash_map<std::string, std::string> &env) {
+  return env.contains("WAYLAND_DISPLAY");
+}
+
+std::vector<std::string> GetCurrentDesktops(
+    const absl::flat_hash_map<std::string, std::string> &env) {
+  const std::optional<absl::string_view> env_xdg_current_desktop =
+      GetMapValue(env, "XDG_CURRENT_DESKTOP");
+  if (!env_xdg_current_desktop.has_value()) {
+    return {};
+  }
+  // $XDG_CURRENT_DESKTOP is a colon-separated list.
+  // https://specifications.freedesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html#recognized-keys
+  return absl::StrSplit(env_xdg_current_desktop.value(), ':');
+}
+
+void UpdateEnvironMap(absl::flat_hash_map<std::string, std::string> &env,
+                      const char *envname) {
+  const char *result = ::getenv(envname);
+  if (result == nullptr) {
+    return;
+  }
+  env.insert_or_assign(envname, result);
+}
+
+bool UseMozcCandidateWindow(const IbusConfig &ibus_config) {
+  if (!absl::GetFlag(FLAGS_use_mozc_renderer)) {
+    return false;
+  }
+
+  const std::string renderer_path =
+      FileUtil::JoinPath(SystemUtil::GetServerDirectory(), "mozc_renderer");
+  if (absl::Status s = FileUtil::FileExists(renderer_path); !s.ok()) {
+    LOG(ERROR) << s;
+    return false;
   }
-#endif  // !ENABLE_QT_RENDERER
+
+  absl::flat_hash_map<std::string, std::string> env;
+  UpdateEnvironMap(env, "MOZC_IBUS_CANDIDATE_WINDOW");
+  UpdateEnvironMap(env, "XDG_CURRENT_DESKTOP");
+  UpdateEnvironMap(env, "WAYLAND_DISPLAY");
 
-  auto *handler = new GtkCandidateWindowHandler(renderer_client);
-  handler->RegisterGSettingsObserver();
-  return handler;
+  return CanUseMozcCandidateWindow(ibus_config, env);
 }
-#endif  // ENABLE_GTK_RENDERER || ENABLE_QT_RENDERER
 
 }  // namespace
 
 MozcEngine::MozcEngine()
-    : last_sync_time_(Clock::GetTime()),
+    : last_sync_time_(Clock::GetAbslTime()),
       key_event_handler_(new KeyEventHandler),
       client_(CreateAndConfigureClient()),
-#ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
-      selection_monitor_(SelectionMonitorFactory::Create(1024)),
-#endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
       preedit_handler_(new PreeditHandler()),
-#if defined(ENABLE_GTK_RENDERER) || defined(ENABLE_QT_RENDERER)
-      gtk_candidate_window_handler_(
-          createGtkCandidateWindowHandler(new renderer::RendererClient())),
-#endif  // ENABLE_GTK_RENDERER || ENABLE_QT_RENDERER
-      ibus_candidate_window_handler_(new IBusCandidateWindowHandler()),
+      use_mozc_candidate_window_(false),
+      mozc_candidate_window_handler_(new renderer::RendererClient()),
       preedit_method_(config::Config::ROMAN) {
-  if (selection_monitor_ != nullptr) {
-    selection_monitor_->StartMonitoring();
-  }
-
   ibus_config_.Initialize();
+  use_mozc_candidate_window_ = UseMozcCandidateWindow(ibus_config_);
+  if (use_mozc_candidate_window_) {
+    mozc_candidate_window_handler_.RegisterGSettingsObserver();
+  }
   property_handler_ = std::make_unique<PropertyHandler>(
       std::make_unique<LocaleBasedMessageTranslator>(GetMessageLocale()),
       ibus_config_.IsActiveOnLaunch(), client_.get());
@@ -287,12 +251,12 @@ MozcEngine::MozcEngine()
 
 MozcEngine::~MozcEngine() { SyncData(true); }
 
-void MozcEngine::CandidateClicked(IBusEngine *engine, guint index, guint button,
-                                  guint state) {
+void MozcEngine::CandidateClicked(IbusEngineWrapper *engine, uint index,
+                                  uint button, uint state) {
   if (index >= unique_candidate_ids_.size()) {
     return;
   }
-  const int32 id = unique_candidate_ids_[index];
+  const int32_t id = unique_candidate_ids_[index];
   if (id == kBadCandidateId) {
     return;
   }
@@ -304,21 +268,43 @@ void MozcEngine::CandidateClicked(IBusEngine *engine, guint index, guint button,
   UpdateAll(engine, output);
 }
 
-void MozcEngine::CursorDown(IBusEngine *engine) {
+void MozcEngine::CursorDown(IbusEngineWrapper *engine) {
   // TODO(mazda): Implement this.
 }
 
-void MozcEngine::CursorUp(IBusEngine *engine) {
+void MozcEngine::CursorUp(IbusEngineWrapper *engine) {
   // TODO(mazda): Implement this.
 }
 
-void MozcEngine::Disable(IBusEngine *engine) {
+void MozcEngine::Disable(IbusEngineWrapper *engine) {
   RevertSession(engine);
   GetCandidateWindowHandler(engine)->Hide(engine);
   key_event_handler_->Clear();
 }
 
-void MozcEngine::Enable(IBusEngine *engine) {
+namespace {
+commands::CompositionMode ConvertCompositionMode(
+    ibus::Engine::CompositionMode mode) {
+  switch (mode) {
+    case ibus::Engine::DIRECT:
+      return commands::DIRECT;
+    case ibus::Engine::HIRAGANA:
+      return commands::HIRAGANA;
+    case ibus::Engine::FULL_KATAKANA:
+      return commands::FULL_KATAKANA;
+    case ibus::Engine::HALF_ASCII:
+      return commands::HALF_ASCII;
+    case ibus::Engine::FULL_ASCII:
+      return commands::FULL_ASCII;
+    case ibus::Engine::HALF_KATAKANA:
+      return commands::HALF_KATAKANA;
+    default:
+      return commands::NUM_OF_COMPOSITIONS;
+  }
+}
+}  // namespace
+
+void MozcEngine::Enable(IbusEngineWrapper *engine) {
   // Launch mozc_server
   client_->EnsureConnection();
   UpdatePreeditMethod();
@@ -329,17 +315,35 @@ void MozcEngine::Enable(IBusEngine *engine) {
   // (crosbug.com/4596).
   RevertSession(engine);
 
-  // If engine wants to use surrounding text, we should call
-  // ibus_engine_get_surrounding_text once when the engine enabled.
-  ibus_engine_get_surrounding_text(engine, nullptr, nullptr, nullptr);
+  engine->EnableSurroundingText();
+
+  commands::CompositionMode mode = ConvertCompositionMode(
+      ibus_config_.GetCompositionMode(engine->GetName()));
+
+  if (mode == commands::NUM_OF_COMPOSITIONS) {
+    // Do nothing.
+  } else {
+    commands::SessionCommand command;
+    if (mode == commands::DIRECT) {
+      command.set_type(commands::SessionCommand::TURN_OFF_IME);
+    } else {
+      command.set_type(commands::SessionCommand::TURN_ON_IME);
+      command.set_composition_mode(mode);
+    }
+    commands::Output output;
+    if (!client_->SendCommand(command, &output)) {
+      LOG(ERROR) << "SendCommand failed";
+    }
+    property_handler_->Update(engine, output);
+  }
 }
 
-void MozcEngine::FocusIn(IBusEngine *engine) {
+void MozcEngine::FocusIn(IbusEngineWrapper *engine) {
   property_handler_->Register(engine);
   UpdatePreeditMethod();
 }
 
-void MozcEngine::FocusOut(IBusEngine *engine) {
+void MozcEngine::FocusOut(IbusEngineWrapper *engine) {
   GetCandidateWindowHandler(engine)->Hide(engine);
   property_handler_->ResetContentType(engine);
 
@@ -354,40 +358,38 @@ void MozcEngine::FocusOut(IBusEngine *engine) {
   SyncData(false);
 }
 
-void MozcEngine::PageDown(IBusEngine *engine) {
+void MozcEngine::PageDown(IbusEngineWrapper *engine) {
   // TODO(mazda,yusukes): Implement this to support arrow icons inside the Gtk+
   // candidate window.
 }
 
-void MozcEngine::PageUp(IBusEngine *engine) {
+void MozcEngine::PageUp(IbusEngineWrapper *engine) {
   // TODO(mazda,yusukes): Implement this to support arrow icons inside the Gtk+
   // candidate window.
 }
 
-gboolean MozcEngine::ProcessKeyEvent(IBusEngine *engine, guint keyval,
-                                     guint keycode, guint modifiers) {
-  VLOG(2) << "keyval: " << keyval << ", keycode: " << keycode
-          << ", modifiers: " << modifiers;
+bool MozcEngine::ProcessKeyEvent(IbusEngineWrapper *engine, uint keyval,
+                                 uint keycode, uint modifiers) {
+  MOZC_VLOG(2) << "keyval: " << keyval << ", keycode: " << keycode
+               << ", modifiers: " << modifiers;
   if (property_handler_->IsDisabled()) {
-    return FALSE;
+    return false;
   }
 
   // layout_is_jp is only used determine Kana input with US layout.
-  const std::string &layout =
-      ibus_config_.GetLayout(ibus_engine_get_name(engine));
+  const absl::string_view layout = ibus_config_.GetLayout(engine->GetName());
   const bool layout_is_jp = (layout != "us");
 
   commands::KeyEvent key;
   if (!key_event_handler_->GetKeyEvent(keyval, keycode, modifiers,
                                        preedit_method_, layout_is_jp, &key)) {
     // Doesn't send a key event to mozc_server.
-    return FALSE;
+    return false;
   }
 
-  VLOG(2) << key.DebugString();
-  if (!property_handler_->IsActivated() &&
-      !config::ImeSwitchUtil::IsDirectModeCommand(key)) {
-    return FALSE;
+  MOZC_VLOG(2) << key;
+  if (!property_handler_->IsActivated() && !client_->IsDirectModeCommand(key)) {
+    return false;
   }
 
   key.set_activated(property_handler_->IsActivated());
@@ -395,54 +397,53 @@ gboolean MozcEngine::ProcessKeyEvent(IBusEngine *engine, guint keyval,
 
   commands::Context context;
   SurroundingTextInfo surrounding_text_info;
-  if (GetSurroundingText(engine,
-                         selection_monitor_.get(),
-                         &surrounding_text_info)) {
+  if (GetSurroundingText(engine, &surrounding_text_info)) {
     context.set_preceding_text(surrounding_text_info.preceding_text);
     context.set_following_text(surrounding_text_info.following_text);
   }
   commands::Output output;
   if (!client_->SendKeyWithContext(key, context, &output)) {
     LOG(ERROR) << "SendKey failed";
-    return FALSE;
+    return false;
   }
 
-  VLOG(2) << output.DebugString();
+  MOZC_VLOG(2) << output;
 
   UpdateAll(engine, output);
 
-  const bool consumed = output.consumed();
-  return consumed ? TRUE : FALSE;
+  return output.consumed();
 }
 
-void MozcEngine::PropertyActivate(IBusEngine *engine,
-                                  const gchar *property_name,
-                                  guint property_state) {
+void MozcEngine::PropertyActivate(IbusEngineWrapper *engine,
+                                  const char *property_name,
+                                  uint property_state) {
   property_handler_->ProcessPropertyActivate(engine, property_name,
                                              property_state);
 }
 
-void MozcEngine::PropertyHide(IBusEngine *engine, const gchar *property_name) {
+void MozcEngine::PropertyHide(IbusEngineWrapper *engine,
+                              const char *property_name) {
   // We can ignore the signal.
 }
 
-void MozcEngine::PropertyShow(IBusEngine *engine, const gchar *property_name) {
+void MozcEngine::PropertyShow(IbusEngineWrapper *engine,
+                              const char *property_name) {
   // We can ignore the signal.
 }
 
-void MozcEngine::Reset(IBusEngine *engine) { RevertSession(engine); }
+void MozcEngine::Reset(IbusEngineWrapper *engine) { RevertSession(engine); }
 
-void MozcEngine::SetCapabilities(IBusEngine *engine, guint capabilities) {
+void MozcEngine::SetCapabilities(IbusEngineWrapper *engine, uint capabilities) {
   // Do nothing.
 }
 
-void MozcEngine::SetCursorLocation(IBusEngine *engine, gint x, gint y, gint w,
-                                   gint h) {
+void MozcEngine::SetCursorLocation(IbusEngineWrapper *engine, int x, int y,
+                                   int w, int h) {
   GetCandidateWindowHandler(engine)->UpdateCursorRect(engine);
 }
 
-void MozcEngine::SetContentType(IBusEngine *engine, guint purpose,
-                                guint hints) {
+void MozcEngine::SetContentType(IbusEngineWrapper *engine, uint purpose,
+                                uint hints) {
   const bool prev_disabled = property_handler_->IsDisabled();
   property_handler_->UpdateContentType(engine);
   if (!prev_disabled && property_handler_->IsDisabled()) {
@@ -451,28 +452,8 @@ void MozcEngine::SetContentType(IBusEngine *engine, guint purpose,
   }
 }
 
-GType MozcEngine::GetType() {
-  static GType type = 0;
-
-  static const GTypeInfo type_info = {
-      sizeof(IBusMozcEngineClass), nullptr, nullptr,
-      MozcEngineClassInit,         nullptr, nullptr,
-      sizeof(IBusMozcEngine),      0,       MozcEngineInstanceInit,
-  };
-
-  if (type == 0) {
-    type = g_type_register_static(IBUS_TYPE_ENGINE, "IBusMozcEngine",
-                                  &type_info, static_cast<GTypeFlags>(0));
-    DCHECK_NE(type, 0) << "g_type_register_static failed";
-  }
-
-  return type;
-}
-
-// static
-void MozcEngine::Disconnected(IBusBus *bus, gpointer user_data) { ibus_quit(); }
-
-bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
+bool MozcEngine::UpdateAll(IbusEngineWrapper *engine,
+                           const commands::Output &output) {
   UpdateDeletionRange(engine, output);
   UpdateResult(engine, output);
   preedit_handler_->Update(engine, output);
@@ -486,44 +467,39 @@ bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
   return true;
 }
 
-bool MozcEngine::UpdateDeletionRange(IBusEngine *engine,
+bool MozcEngine::UpdateDeletionRange(IbusEngineWrapper *engine,
                                      const commands::Output &output) {
   if (output.has_deletion_range() && output.deletion_range().offset() < 0 &&
       output.deletion_range().offset() + output.deletion_range().length() >=
           0) {
-    // Nowadays 'ibus_engine_delete_surrounding_text' becomes functional on
-    // many of the major applications.  Confirmed that it works on
-    // Firefox 10.0, LibreOffice 3.3.4 and GEdit 3.2.3.
-    ibus_engine_delete_surrounding_text(engine,
-                                        output.deletion_range().offset(),
-                                        output.deletion_range().length());
+    engine->DeleteSurroundingText(output.deletion_range().offset(),
+                                  output.deletion_range().length());
   }
   return true;
 }
 
-bool MozcEngine::UpdateResult(IBusEngine *engine,
+bool MozcEngine::UpdateResult(IbusEngineWrapper *engine,
                               const commands::Output &output) const {
   if (!output.has_result()) {
-    VLOG(2) << "output doesn't contain result";
+    MOZC_VLOG(2) << "output doesn't contain result";
     return true;
   }
 
-  IBusText *text = ibus_text_new_from_string(output.result().value().c_str());
-  ibus_engine_commit_text(engine, text);
-  // |text| is released by ibus_engine_commit_text.
+  engine->CommitText(output.result().value());
   return true;
 }
 
 bool MozcEngine::UpdateCandidateIDMapping(const commands::Output &output) {
-  if (!output.has_candidates() || output.candidates().candidate_size() == 0) {
+  if (!output.has_candidate_window() ||
+      output.candidate_window().candidate_size() == 0) {
     return true;
   }
 
   unique_candidate_ids_.clear();
-  const commands::Candidates &candidates = output.candidates();
-  for (int i = 0; i < candidates.candidate_size(); ++i) {
-    if (candidates.candidate(i).has_id()) {
-      const int32 id = candidates.candidate(i).id();
+  const commands::CandidateWindow &candidate_window = output.candidate_window();
+  for (int i = 0; i < candidate_window.candidate_size(); ++i) {
+    if (candidate_window.candidate(i).has_id()) {
+      const int32_t id = candidate_window.candidate(i).id();
       unique_candidate_ids_.push_back(id);
     } else {
       // The parent node of the cascading window does not have an id since the
@@ -549,10 +525,10 @@ void MozcEngine::SyncData(bool force) {
     return;
   }
 
-  const uint64 current_time = Clock::GetTime();
+  const absl::Time current_time = Clock::GetAbslTime();
   if (force || (current_time >= last_sync_time_ &&
                 current_time - last_sync_time_ >= kSyncDataInterval)) {
-    VLOG(1) << "Syncing data";
+    MOZC_VLOG(1) << "Syncing data";
     client_->SyncData();
     last_sync_time_ = current_time;
   }
@@ -560,16 +536,16 @@ void MozcEngine::SyncData(bool force) {
 
 bool MozcEngine::LaunchTool(const commands::Output &output) const {
   if (!client_->LaunchToolWithProtoBuf(output)) {
-    VLOG(2) << output.DebugString() << " Launch Failed";
+    MOZC_VLOG(2) << output << " Launch Failed";
     return false;
   }
 
   return true;
 }
 
-void MozcEngine::RevertSession(IBusEngine *engine) {
+void MozcEngine::RevertSession(IbusEngineWrapper *engine) {
   // TODO(team): We should skip following actions when there is no on-going
-  // omposition.
+  // composition.
   commands::SessionCommand command;
   command.set_type(commands::SessionCommand::REVERT);
   commands::Output output;
@@ -580,7 +556,7 @@ void MozcEngine::RevertSession(IBusEngine *engine) {
   UpdateAll(engine, output);
 }
 
-bool MozcEngine::ExecuteCallback(IBusEngine *engine,
+bool MozcEngine::ExecuteCallback(IbusEngineWrapper *engine,
                                  const commands::Output &output) {
   if (!output.has_callback()) {
     return false;
@@ -618,14 +594,12 @@ bool MozcEngine::ExecuteCallback(IBusEngine *engine,
       // Having |IBUS_CAP_SURROUNDING_TEXT| does not necessarily mean that the
       // client supports |ibus_engine_delete_surrounding_text()|, but there is
       // no other good criteria.
-      if (!(engine->client_capabilities & IBUS_CAP_SURROUNDING_TEXT)) {
+      if (!(engine->CheckCapabilities(IBUS_CAP_SURROUNDING_TEXT))) {
         return false;
       }
       break;
     case commands::SessionCommand::CONVERT_REVERSE: {
-      if (!GetSurroundingText(engine,
-                              selection_monitor_.get(),
-                              &surrounding_text_info)) {
+      if (!GetSurroundingText(engine, &surrounding_text_info)) {
         return false;
       }
       session_command.set_text(surrounding_text_info.selection_text);
@@ -649,7 +623,7 @@ bool MozcEngine::ExecuteCallback(IBusEngine *engine,
     // offset should be a negative value to delete preceding text.
     // For backward selection (that is, |relative_selected_length < 0|),
     // IBus and/or some applications seem to expect |offset == 0| somehow.
-    const int32 offset =
+    const int32_t offset =
         surrounding_text_info.relative_selected_length > 0
             ? -surrounding_text_info
                    .relative_selected_length  // forward selection
@@ -665,29 +639,44 @@ bool MozcEngine::ExecuteCallback(IBusEngine *engine,
 }
 
 CandidateWindowHandlerInterface *MozcEngine::GetCandidateWindowHandler(
-    IBusEngine *engine) {
-#ifndef ENABLE_GTK_RENDERER
-  return ibus_candidate_window_handler_.get();
-#else   // ENABLE_GTK_RENDERER
-  if (!gtk_candidate_window_handler_) {
-    return ibus_candidate_window_handler_.get();
+    IbusEngineWrapper *engine) {
+  if (use_mozc_candidate_window_ &&
+      engine->CheckCapabilities(IBUS_CAP_PREEDIT_TEXT)) {
+    return &mozc_candidate_window_handler_;
+  }
+  return &ibus_candidate_window_handler_;
+}
+
+bool CanUseMozcCandidateWindow(
+    const IbusConfig &ibus_config,
+    const absl::flat_hash_map<std::string, std::string> &env) {
+  if (!ibus_config.IsMozcRendererEnabled()) {
+    return false;
   }
 
-  if (!(engine->client_capabilities & IBUS_CAP_PREEDIT_TEXT)) {
-    return ibus_candidate_window_handler_.get();
+  const std::optional<absl::string_view> env_candidate_window =
+      GetMapValue(env, "MOZC_IBUS_CANDIDATE_WINDOW");
+  if (env_candidate_window.has_value() &&
+      env_candidate_window.value() == "ibus") {
+    return false;
   }
 
-  // TODO(nona): integrate with renderer/renderer_client.cc
-  const std::string renderer_path =
-      FileUtil::JoinPath(SystemUtil::GetServerDirectory(), "mozc_renderer");
-  if (absl::Status s = FileUtil::FileExists(renderer_path); !s.ok()) {
-    LOG(ERROR) << s;
-    return ibus_candidate_window_handler_.get();
+  if (!IsWaylandSession(env)) {
+    return true;
   }
 
-  // TODO(nona): Check executable bit for renderer.
-  return gtk_candidate_window_handler_.get();
-#endif  // not ENABLE_GTK_RENDERER
+  const std::vector<std::string> current_desktops = GetCurrentDesktops(env);
+  if (current_desktops.empty()) {
+    return false;
+  }
+  for (const std::string &compatible_desktop :
+       ibus_config.GetMozcRendererCompatibleWaylandDesktopNames()) {
+    if (std::find(current_desktops.begin(), current_desktops.end(),
+                  compatible_desktop) != current_desktops.end()) {
+      return true;
+    }
+  }
+  return false;
 }
 
 }  // namespace ibus
