@@ -30,15 +30,23 @@
 #ifndef MOZC_UNIX_IBUS_MOZC_ENGINE_H_
 #define MOZC_UNIX_IBUS_MOZC_ENGINE_H_
 
+#include <cstdint>
 #include <memory>
 #include <set>
 #include <vector>
 
+#include "absl/container/flat_hash_map.h"
 #include "base/port.h"
 #include "protocol/commands.pb.h"
-#include "testing/base/public/gunit_prod.h"
+#include "protocol/config.pb.h"
+#include "testing/friend_test.h"
+#include "unix/ibus/candidate_window_handler.h"
 #include "unix/ibus/engine_interface.h"
+#include "unix/ibus/ibus_candidate_window_handler.h"
 #include "unix/ibus/ibus_config.h"
+#include "unix/ibus/ibus_wrapper.h"
+#include "unix/ibus/preedit_handler.h"
+#include "unix/ibus/property_handler.h"
 
 namespace mozc {
 
@@ -51,10 +59,6 @@ namespace ibus {
 class CandidateWindowHandlerInterface;
 class KeyEventHandler;
 class LaunchToolTest;
-class MessageTranslatorInterface;
-class PreeditHandlerInterface;
-class PropertyHandlerInterface;
-class SelectionMonitorInterface;
 
 // Implements EngineInterface and handles signals from IBus daemon.
 // This class mainly does the two things:
@@ -65,48 +69,52 @@ class SelectionMonitorInterface;
 class MozcEngine : public EngineInterface {
  public:
   MozcEngine();
+  MozcEngine(const MozcEngine &) = delete;
+  MozcEngine &operator=(const MozcEngine &) = delete;
   virtual ~MozcEngine();
 
   // EngineInterface functions
-  void CandidateClicked(IBusEngine *engine, guint index, guint button,
-                        guint state);
-  void CursorDown(IBusEngine *engine);
-  void CursorUp(IBusEngine *engine);
-  void Disable(IBusEngine *engine);
-  void Enable(IBusEngine *engine);
-  void FocusIn(IBusEngine *engine);
-  void FocusOut(IBusEngine *engine);
-  void PageDown(IBusEngine *engine);
-  void PageUp(IBusEngine *engine);
-  gboolean ProcessKeyEvent(IBusEngine *engine, guint keyval, guint keycode,
-                           guint state);
-  void PropertyActivate(IBusEngine *engine, const gchar *property_name,
-                        guint property_state);
-  void PropertyHide(IBusEngine *engine, const gchar *property_name);
-  void PropertyShow(IBusEngine *engine, const gchar *property_name);
-  void Reset(IBusEngine *engine);
-  void SetCapabilities(IBusEngine *engine, guint capabilities);
-  void SetCursorLocation(IBusEngine *engine, gint x, gint y, gint w, gint h);
-  void SetContentType(IBusEngine *engine, guint purpose, guint hints);
-
-  // Returns the GType which this class represents.
-  static GType GetType();
-  // The callback function to the "disconnected" signal to the bus object.
-  static void Disconnected(IBusBus *bus, gpointer user_data);
+  void CandidateClicked(IbusEngineWrapper *engine, uint index, uint button,
+                        uint state) override;
+  void CursorDown(IbusEngineWrapper *engine) override;
+  void CursorUp(IbusEngineWrapper *engine) override;
+  void Disable(IbusEngineWrapper *engine) override;
+  void Enable(IbusEngineWrapper *engine) override;
+  void FocusIn(IbusEngineWrapper *engine) override;
+  void FocusOut(IbusEngineWrapper *engine) override;
+  void PageDown(IbusEngineWrapper *engine) override;
+  void PageUp(IbusEngineWrapper *engine) override;
+  bool ProcessKeyEvent(IbusEngineWrapper *engine, uint keyval, uint keycode,
+                       uint state) override;
+  void PropertyActivate(IbusEngineWrapper *engine, const char *property_name,
+                        uint property_state) override;
+  void PropertyHide(IbusEngineWrapper *engine,
+                    const char *property_name) override;
+  void PropertyShow(IbusEngineWrapper *engine,
+                    const char *property_name) override;
+  void Reset(IbusEngineWrapper *engine) override;
+  void SetCapabilities(IbusEngineWrapper *engine, uint capabilities) override;
+  void SetCursorLocation(IbusEngineWrapper *engine, int x, int y, int w,
+                         int h) override;
+  void SetContentType(IbusEngineWrapper *engine, uint purpose,
+                      uint hints) override;
 
  private:
   // Updates the preedit text and the candidate window and inserts result
   // based on the content of |output|.
-  bool UpdateAll(IBusEngine *engine, const commands::Output &output);
+  bool UpdateAll(IbusEngineWrapper *engine, const commands::Output &output);
   // Inserts a result text based on the content of |output|.
-  bool UpdateResult(IBusEngine *engine, const commands::Output &output) const;
+  bool UpdateResult(IbusEngineWrapper *engine,
+                    const commands::Output &output) const;
   // Updates |unique_candidate_ids_|.
   bool UpdateCandidateIDMapping(const commands::Output &output);
   // Updates the deletion range message based on the content of |output|.
-  bool UpdateDeletionRange(IBusEngine *engine, const commands::Output &output);
+  bool UpdateDeletionRange(IbusEngineWrapper *engine,
+                           const commands::Output &output);
 
   // Updates the callback message based on the content of |output|.
-  bool ExecuteCallback(IBusEngine *engine, const commands::Output &output);
+  bool ExecuteCallback(IbusEngineWrapper *engine,
+                       const commands::Output &output);
 
   // Launches Mozc tool with appropriate arguments.
   bool LaunchTool(const commands::Output &output) const;
@@ -121,38 +129,37 @@ class MozcEngine : public EngineInterface {
 
   // Reverts internal state of mozc_server by sending SessionCommand::REVERT IPC
   // message, then hides a preedit string and the candidate window.
-  void RevertSession(IBusEngine *engine);
+  void RevertSession(IbusEngineWrapper *engine);
 
   CandidateWindowHandlerInterface *GetCandidateWindowHandler(
-      IBusEngine *engine);
+      IbusEngineWrapper *engine);
 
-  uint64 last_sync_time_;
+  absl::Time last_sync_time_;
   std::unique_ptr<KeyEventHandler> key_event_handler_;
   std::unique_ptr<client::ClientInterface> client_;
-  std::unique_ptr<SelectionMonitorInterface> selection_monitor_;
 
-  std::unique_ptr<PropertyHandlerInterface> property_handler_;
-  std::unique_ptr<PreeditHandlerInterface> preedit_handler_;
+  std::unique_ptr<PropertyHandler> property_handler_;
+  std::unique_ptr<PreeditHandler> preedit_handler_;
 
-#if defined(ENABLE_GTK_RENDERER) || defined(ENABLE_QT_RENDERER)
+  // If true, uses Mozc candidate window instead of IBus default one.
+  bool use_mozc_candidate_window_;
   // TODO(nona): Introduce CandidateWindowHandlerManager to avoid direct access.
-  std::unique_ptr<CandidateWindowHandlerInterface>
-      gtk_candidate_window_handler_;
-#endif  // ENABLE_GTK_RENDERER || ENABLE_QT_RENDERER
-  std::unique_ptr<CandidateWindowHandlerInterface>
-      ibus_candidate_window_handler_;
+  CandidateWindowHandler mozc_candidate_window_handler_;
+  IBusCandidateWindowHandler ibus_candidate_window_handler_;
   config::Config::PreeditMethod preedit_method_;
 
   // Unique IDs of candidates that are currently shown.
-  std::vector<int32> unique_candidate_ids_;
+  std::vector<int32_t> unique_candidate_ids_;
   IbusConfig ibus_config_;
 
   friend class LaunchToolTest;
   FRIEND_TEST(LaunchToolTest, LaunchToolTest);
-
-  DISALLOW_COPY_AND_ASSIGN(MozcEngine);
 };
 
+bool CanUseMozcCandidateWindow(
+    const IbusConfig &ibus_config,
+    const absl::flat_hash_map<std::string, std::string> &env);
+
 }  // namespace ibus
 }  // namespace mozc
 
