@@ -32,9 +32,7 @@
 
 #include <windows.h>
 
-#include <memory>
-
-#include "base/port.h"
+#include "base/stopwatch.h"
 #include "win32/base/keyboard.h"
 
 namespace mozc {
@@ -46,8 +44,6 @@ class IndicatorVisibilityTracker {
     kNothing,   // The caller has nothing to do.
     kUpdateUI,  // The caller must update UI for the indicator.
   };
-  IndicatorVisibilityTracker();
-  ~IndicatorVisibilityTracker();
 
   // Event call back endpoints.
   Action OnDissociateContext();
@@ -57,12 +53,11 @@ class IndicatorVisibilityTracker {
   Action OnChangeInputMode();
 
   // Returns if the indicator should be displayed or not.
-  bool IsVisible() const;
+  constexpr bool IsVisible() const { return visible_; }
 
  private:
-  struct InternalState;
-  std::unique_ptr<InternalState> state_;
-  DISALLOW_COPY_AND_ASSIGN(IndicatorVisibilityTracker);
+  bool visible_ = false;
+  Stopwatch mode_changed_;
 };
 
 }  // namespace win32
