@@ -27,15 +27,15 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#include "win32/broker/prelauncher.h"
+
 #include <memory>
 
-#include "base/logging.h"
 #include "base/run_level.h"
-#include "base/system_util.h"
-#include "base/win_util.h"
+#include "base/win32/win_util.h"
+#include "client/client.h"
 #include "client/client_interface.h"
 #include "renderer/renderer_client.h"
-#include "win32/base/imm_util.h"
 
 namespace mozc {
 namespace win32 {
@@ -60,11 +60,6 @@ int RunPrelaunchProcesses(int argc, char *argv[]) {
     return kErrorLevelGeneralError;
   }
 
-  if (!SystemUtil::IsWindows8OrLater() && !ImeUtil::IsDefault()) {
-    // If Mozc is not default on Windows 7 or former, do nothing.
-    return kErrorLevelSuccess;
-  }
-
   {
     std::unique_ptr<client::ClientInterface> converter_client(
         client::ClientFactory::NewClient());
