@@ -31,39 +31,46 @@
 
 // clang-format off
 #include <windows.h>
-#include <IEPMapi.h>
 #include <atlbase.h>
-#if !defined(MOZC_NO_LOGGING)
-#include <atlstr.h>
-#endif  // !MOZC_NO_LOGGING
 #include <msiquery.h>
 // clang-format on
 
+#undef StrCat  // NOLINT: TODO: triggers clang-tidy, defined by windows.h.
+
+#include <cstdarg>
+#include <cstddef>
 #include <memory>
 #include <string>
 #include <vector>
 
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 #include "base/const.h"
 #include "base/process.h"
+#include "base/strings/zstring_view.h"
 #include "base/system_util.h"
 #include "base/url.h"
-#include "base/util.h"
 #include "base/version.h"
-#include "base/win_sandbox.h"
-#include "base/win_util.h"
+#include "base/win32/scoped_com.h"
+#include "base/win32/wide_char.h"
+#include "base/win32/win_sandbox.h"
+#include "base/win32/win_util.h"
+#include "client/client.h"
 #include "client/client_interface.h"
-#include "config/stats_config_util.h"
-#include "protocol/commands.pb.h"
+#include "config/config_handler.h"
 #include "renderer/renderer_client.h"
-#include "server/cache_service_manager.h"
-#include "win32/base/imm_registrar.h"
-#include "win32/base/imm_util.h"
-#include "win32/base/keyboard_layout_id.h"
+#include "win32/base/input_dll.h"
 #include "win32/base/omaha_util.h"
+#include "win32/base/tsf_profile.h"
 #include "win32/base/tsf_registrar.h"
 #include "win32/base/uninstall_helper.h"
+#include "win32/cache_service/cache_service_manager.h"
 #include "win32/custom_action/resource.h"
 
+#if !defined(NDEBUG)
+#include <atlstr.h>
+#endif  // !NDEBUG
+
 #ifdef _DEBUG
 #define DEBUG_BREAK_FOR_DEBUGGER()                                      \
   ::OutputDebugStringA(                                                 \
@@ -71,7 +78,7 @@
   if (::IsDebuggerPresent()) {                                          \
     __debugbreak();                                                     \
   }
-#else   // _DEBUG
+#else  // _DEBUG
 #define DEBUG_BREAK_FOR_DEBUGGER()
 #endif  // _DEBUG
 
@@ -79,70 +86,45 @@ namespace {
 
 using mozc::win32::OmahaUtil;
 
-constexpr char kIEFrameDll[] = "ieframe.dll";
-const wchar_t kSystemSharedKey[] = L"Software\\Microsoft\\CTF\\SystemShared";
-
 HMODULE g_module = nullptr;
 
-std::wstring GetMozcComponentPath(const std::string &filename) {
-  const std::string path =
-      mozc::SystemUtil::GetServerDirectory() + "\\" + filename;
-  std::wstring wpath;
-  mozc::Util::Utf8ToWide(path, &wpath);
-  return wpath;
+std::wstring GetMozcComponentPath(const absl::string_view filename) {
+  return mozc::win32::Utf8ToWide(
+      absl::StrCat(mozc::SystemUtil::GetServerDirectory(), "\\", filename));
 }
 
 // Retrieves the value for an installer property.
 // Returns an empty string if a property corresponding to |name| is not found or
 // error occurs.
-std::wstring GetProperty(MSIHANDLE msi, const std::wstring &name) {
+std::wstring GetProperty(MSIHANDLE msi, const mozc::zwstring_view name) {
   DWORD num_buf = 0;
   // Obtains the size of the property's string, without null termination.
   // Note: |MsiGetProperty()| requires non-null writable buffer.
-  wchar_t dummy_buffer[1] = {L'\0'};
-  UINT result = MsiGetProperty(msi, name.c_str(), dummy_buffer, &num_buf);
-  if (ERROR_MORE_DATA != result) {
+  std::wstring buf;
+  UINT result = MsiGetProperty(msi, name.c_str(), buf.data(), &num_buf);
+  if (result != ERROR_MORE_DATA) {
     return L"";
   }
 
+  buf.resize(num_buf);
   // add 1 for null termination
-  ++num_buf;
-  std::unique_ptr<wchar_t[]> buf(new wchar_t[num_buf]);
-  result = MsiGetProperty(msi, name.c_str(), buf.get(), &num_buf);
-  if (ERROR_SUCCESS != result) {
+  num_buf += 1;
+  result = MsiGetProperty(msi, name.c_str(), buf.data(), &num_buf);
+  if (result != ERROR_SUCCESS) {
     return L"";
   }
 
-  return std::wstring(buf.get());
+  return buf;
 }
 
-bool SetProperty(MSIHANDLE msi, const std::wstring &name,
-                 const std::wstring &value) {
+bool SetProperty(MSIHANDLE msi, const mozc::zwstring_view name,
+                 const mozc::zwstring_view value) {
   if (MsiSetProperty(msi, name.c_str(), value.c_str()) != ERROR_SUCCESS) {
     return false;
   }
   return true;
 }
 
-#ifndef MOZC_NO_LOGGING
-bool DisableErrorReportingInternal(const wchar_t *key_name,
-                                   const wchar_t *value_name, DWORD value,
-                                   REGSAM additional_sam_desired) {
-  CRegKey key;
-  LONG result =
-      key.Create(HKEY_LOCAL_MACHINE, key_name, REG_NONE,
-                 REG_OPTION_NON_VOLATILE, KEY_WRITE | additional_sam_desired);
-  if (ERROR_SUCCESS != result) {
-    return false;
-  }
-  result = key.SetDWORDValue(value_name, value);
-  if (ERROR_SUCCESS != result) {
-    return false;
-  }
-  return true;
-}
-#endif  // MOZC_NO_LOGGING
-
 std::wstring FormatMessageByResourceId(int resourceID, ...) {
   wchar_t format_message[4096];
   {
@@ -189,12 +171,12 @@ bool WriteOmahaErrorById(int resource_id) {
 
 template <size_t num_elements>
 bool WriteOmahaError(const wchar_t (&function)[num_elements], int line) {
-#if !defined(MOZC_NO_LOGGING)
-  CStringW log;
+#if !defined(NDEBUG)
+  ATL::CStringW log;
   log.Format(L"%s: %s; %s(%d)", L"WriteOmahaError: ",
              mozc::Version::GetMozcVersionW().c_str(), function, line);
   ::OutputDebugStringW(log);
-#endif  // !defined(MOZC_NO_LOGGING)
+#endif  // !defined(NDEBUG)
   const std::wstring &message =
       FormatMessageByResourceId(IDS_FORMAT_FUNCTION_AND_LINE, function, line);
   return OmahaUtil::WriteOmahaError(message, GetVersionHeader());
@@ -205,18 +187,6 @@ bool WriteOmahaError(const wchar_t (&function)[num_elements], int line) {
 // dialog.
 #define LOG_ERROR_FOR_OMAHA() WriteOmahaError(_T(__FUNCTION__), __LINE__)
 
-UINT RemovePreloadKeyByKLID(const mozc::win32::KeyboardLayoutID &klid) {
-  if (!klid.has_id()) {
-    // already removed.
-    return ERROR_SUCCESS;
-  }
-  const mozc::win32::KeyboardLayoutID default_klid(
-      mozc::kDefaultKeyboardLayout);
-  const HRESULT result =
-      mozc::win32::ImmRegistrar::RemoveKeyFromPreload(klid, default_klid);
-  return SUCCEEDED(result) ? ERROR_SUCCESS : ERROR_INSTALL_FAILURE;
-}
-
 }  // namespace
 
 BOOL APIENTRY DllMain(HMODULE module, DWORD ul_reason_for_call,
@@ -240,31 +210,23 @@ BOOL APIENTRY DllMain(HMODULE module, DWORD ul_reason_for_call,
 UINT __stdcall EnsureAllApplicationPackagesPermisssions(MSIHANDLE msi_handle) {
   DEBUG_BREAK_FOR_DEBUGGER();
   if (!mozc::WinSandbox::EnsureAllApplicationPackagesPermisssion(
-          GetMozcComponentPath(mozc::kMozcServerName))) {
+          GetMozcComponentPath(mozc::kMozcServerName),
+          mozc::WinSandbox::AppContainerVisibilityType::kProgramFiles)) {
     return ERROR_INSTALL_FAILURE;
   }
   if (!mozc::WinSandbox::EnsureAllApplicationPackagesPermisssion(
-          GetMozcComponentPath(mozc::kMozcRenderer))) {
+          GetMozcComponentPath(mozc::kMozcRenderer),
+          mozc::WinSandbox::AppContainerVisibilityType::kProgramFiles)) {
     return ERROR_INSTALL_FAILURE;
   }
   if (!mozc::WinSandbox::EnsureAllApplicationPackagesPermisssion(
-          GetMozcComponentPath(mozc::kMozcTIP32))) {
+          GetMozcComponentPath(mozc::kMozcTIP32),
+          mozc::WinSandbox::AppContainerVisibilityType::kProgramFiles)) {
     return ERROR_INSTALL_FAILURE;
   }
-  if (mozc::SystemUtil::IsWindowsX64()) {
-    if (!mozc::WinSandbox::EnsureAllApplicationPackagesPermisssion(
-            GetMozcComponentPath(mozc::kMozcTIP64))) {
-      return ERROR_INSTALL_FAILURE;
-    }
-  }
-  return ERROR_SUCCESS;
-}
-
-UINT __stdcall CallIERefreshElevationPolicy(MSIHANDLE msi_handle) {
-  DEBUG_BREAK_FOR_DEBUGGER();
-  HRESULT result = ::IERefreshElevationPolicy();
-  if (FAILED(result)) {
-    LOG_ERROR_FOR_OMAHA();
+  if (!mozc::WinSandbox::EnsureAllApplicationPackagesPermisssion(
+          GetMozcComponentPath(mozc::kMozcTIP64),
+          mozc::WinSandbox::AppContainerVisibilityType::kProgramFiles)) {
     return ERROR_INSTALL_FAILURE;
   }
   return ERROR_SUCCESS;
@@ -273,9 +235,8 @@ UINT __stdcall CallIERefreshElevationPolicy(MSIHANDLE msi_handle) {
 // [Return='ignore']
 UINT __stdcall OpenUninstallSurveyPage(MSIHANDLE msi_handle) {
   DEBUG_BREAK_FOR_DEBUGGER();
-  std::string url;
-  mozc::Url::GetUninstallationSurveyUrl(mozc::Version::GetMozcVersion(), &url);
-  mozc::Process::OpenBrowser(url);
+  mozc::Process::OpenBrowser(
+      mozc::url::GetUninstallationSurveyUrl(mozc::Version::GetMozcVersion()));
   return ERROR_SUCCESS;
 }
 
@@ -283,21 +244,19 @@ UINT __stdcall ShutdownServer(MSIHANDLE msi_handle) {
   DEBUG_BREAK_FOR_DEBUGGER();
   std::unique_ptr<mozc::client::ClientInterface> server_client(
       mozc::client::ClientFactory::NewClient());
-  bool server_result = true;
   if (server_client->PingServer()) {
-    server_result = server_client->Shutdown();
+    if (!server_client->Shutdown()) {
+      // This is not fatal as Windows Installer can replace executables even
+      // when they still are running. Just log error then go ahead.
+      LOG_ERROR_FOR_OMAHA();
+    }
   }
   mozc::renderer::RendererClient renderer_client;
-  const bool renderer_result = renderer_client.Shutdown(true);
-  if (!server_result) {
+  if (!renderer_client.Shutdown(true)) {
+    // This is not fatal as Windows Installer can replace executables even when
+    // they are still running. Just log error then go ahead.
     LOG_ERROR_FOR_OMAHA();
-    return ERROR_INSTALL_FAILURE;
   }
-  if (!renderer_result) {
-    LOG_ERROR_FOR_OMAHA();
-    return ERROR_INSTALL_FAILURE;
-  }
-
   return ERROR_SUCCESS;
 }
 
@@ -309,23 +268,6 @@ UINT __stdcall RestoreUserIMEEnvironment(MSIHANDLE msi_handle) {
   return result ? ERROR_SUCCESS : ERROR_INSTALL_FAILURE;
 }
 
-// [Return='ignore']
-UINT __stdcall EnsureIMEIsDisabledForServiceAccount(MSIHANDLE msi_handle) {
-  DEBUG_BREAK_FOR_DEBUGGER();
-  bool is_service = false;
-  if (!mozc::WinUtil::IsServiceAccount(&is_service)) {
-    return ERROR_INSTALL_FAILURE;
-  }
-  if (!is_service) {
-    // Do nothing if this is not a service account.
-    return ERROR_SUCCESS;
-  }
-
-  const bool result =
-      mozc::win32::UninstallHelper::EnsureIMEIsRemovedForCurrentUser(true);
-  return result ? ERROR_SUCCESS : ERROR_INSTALL_FAILURE;
-}
-
 // [Return='ignore']
 // Hides the cancel button on a progress dialog shown by the installer shows.
 // Please see the following page for details.
@@ -361,6 +303,52 @@ UINT __stdcall InitialInstallationCommit(MSIHANDLE msi_handle) {
   return ERROR_SUCCESS;
 }
 
+UINT __stdcall EnableTipProfile(MSIHANDLE msi_handle) {
+  bool is_service = false;
+  if (::mozc::WinUtil::IsServiceAccount(&is_service) && is_service) {
+    // Do nothing if this is a service account.
+    return ERROR_SUCCESS;
+  }
+
+  wchar_t clsid[64] = {};
+  if (!::StringFromGUID2(::mozc::win32::TsfProfile::GetTextServiceGuid(), clsid,
+                         std::size(clsid))) {
+    // Do not care about errors.
+    return ERROR_SUCCESS;
+  }
+  wchar_t profile_id[64] = {};
+  if (!::StringFromGUID2(::mozc::win32::TsfProfile::GetProfileGuid(),
+                         profile_id, std::size(profile_id))) {
+    // Do not care about errors.
+    return ERROR_SUCCESS;
+  }
+
+  // 0x0411 == MAKELANGID(LANG_JAPANESE, SUBLANG_JAPANESE_JAPAN)
+  const auto desc = ::mozc::win32::StrCatW(L"0x0411:", clsid, profile_id);
+
+  // Do not care about errors.
+  ::InstallLayoutOrTip(desc.c_str(), 0);
+  return ERROR_SUCCESS;
+}
+
+UINT __stdcall FixupConfigFilePermission(MSIHANDLE msi_handle) {
+  bool is_service = false;
+  if (::mozc::WinUtil::IsServiceAccount(&is_service) && is_service) {
+    // Do nothing if this is a service account.
+    return ERROR_SUCCESS;
+  }
+
+  // Check the file permission of "config1.db" if exists to ensure that
+  // "ALL APPLICATION PACKAGES" have read access to it.
+  // See https://github.com/google/mozc/issues/1076 for details.
+  ::mozc::config::ConfigHandler::FixupFilePermission();
+
+  // Return always ERROR_SUCCESS regardless of the result, as not being able
+  // to fixup the permission is not problematic enough to block installation
+  // and upgrading.
+  return ERROR_SUCCESS;
+}
+
 UINT __stdcall SaveCustomActionData(MSIHANDLE msi_handle) {
   DEBUG_BREAK_FOR_DEBUGGER();
   // store the CHANNEL value specified in the command line argument for
@@ -457,58 +445,6 @@ UINT __stdcall WriteApValueRollback(MSIHANDLE msi_handle) {
   return ERROR_SUCCESS;
 }
 
-UINT __stdcall InstallIME(MSIHANDLE msi_handle) {
-  DEBUG_BREAK_FOR_DEBUGGER();
-
-  const std::wstring ime_path = mozc::win32::ImmRegistrar::GetFullPathForIME();
-  if (ime_path.empty()) {
-    LOG_ERROR_FOR_OMAHA();
-    return ERROR_INSTALL_FAILURE;
-  }
-  const std::wstring ime_filename =
-      mozc::win32::ImmRegistrar::GetFileNameForIME();
-
-  const std::wstring layout_name = mozc::win32::ImmRegistrar::GetLayoutName();
-  if (layout_name.empty()) {
-    LOG_ERROR_FOR_OMAHA();
-    return ERROR_INSTALL_FAILURE;
-  }
-
-  // Install IME and obtain the corresponding HKL value.
-  HKL hkl = nullptr;
-  HRESULT result = mozc::win32::ImmRegistrar::Register(
-      ime_filename, layout_name, ime_path,
-      mozc::win32::ImmRegistrar::GetLayoutDisplayNameResourceId(), &hkl);
-  if (result != S_OK) {
-    LOG_ERROR_FOR_OMAHA();
-    return ERROR_INSTALL_FAILURE;
-  }
-
-  return ERROR_SUCCESS;
-}
-
-UINT __stdcall InstallIMERollback(MSIHANDLE msi_handle) {
-  DEBUG_BREAK_FOR_DEBUGGER();
-  return RemovePreloadKeyByKLID(mozc::win32::ImmRegistrar::GetKLIDForIME());
-}
-
-UINT __stdcall UninstallIME(MSIHANDLE msi_handle) {
-  DEBUG_BREAK_FOR_DEBUGGER();
-  const std::wstring ime_filename =
-      mozc::win32::ImmRegistrar::GetFileNameForIME();
-  if (ime_filename.empty()) {
-    return ERROR_INSTALL_FAILURE;
-  }
-  mozc::win32::ImmRegistrar::Unregister(ime_filename);
-  return ERROR_SUCCESS;
-}
-
-UINT __stdcall UninstallIMERollback(MSIHANDLE msi_handle) {
-  DEBUG_BREAK_FOR_DEBUGGER();
-
-  return ERROR_SUCCESS;
-}
-
 UINT __stdcall RegisterTIP(MSIHANDLE msi_handle) {
   DEBUG_BREAK_FOR_DEBUGGER();
   mozc::ScopedCOMInitializer com_initializer;
@@ -521,14 +457,6 @@ UINT __stdcall RegisterTIP(MSIHANDLE msi_handle) {
 #error "Unsupported CPU architecture"
 #endif  // _M_X64, _M_IX86, and others
   HRESULT result =
-      mozc::win32::TsfRegistrar::RegisterCOMServer(path.c_str(), path.length());
-  if (FAILED(result)) {
-    LOG_ERROR_FOR_OMAHA();
-    UnregisterTIP(msi_handle);
-    return ERROR_INSTALL_FAILURE;
-  }
-
-  result =
       mozc::win32::TsfRegistrar::RegisterProfiles(path.c_str(), path.length());
   if (FAILED(result)) {
     LOG_ERROR_FOR_OMAHA();
@@ -558,7 +486,6 @@ UINT __stdcall UnregisterTIP(MSIHANDLE msi_handle) {
 
   mozc::win32::TsfRegistrar::UnregisterCategories();
   mozc::win32::TsfRegistrar::UnregisterProfiles();
-  mozc::win32::TsfRegistrar::UnregisterCOMServer();
 
   return ERROR_SUCCESS;
 }
