@@ -29,17 +29,19 @@
 
 #include "win32/tip/tip_candidate_list.h"
 
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
-#include <atlbase.h>
-#include <atlcom.h>
 #include <ctffunc.h>
+#include <oleauto.h>
+#include <wil/com.h>
+#include <wil/resource.h>
 
+#include <cstddef>
+#include <memory>
 #include <string>
+#include <string_view>
 #include <vector>
 
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
+#include "absl/strings/string_view.h"
+#include "testing/gunit.h"
 #include "win32/tip/tip_dll_module.h"
 
 namespace mozc {
@@ -47,11 +49,9 @@ namespace win32 {
 namespace tsf {
 namespace {
 
-using ATL::CComBSTR;
-using ATL::CComPtr;
-using testing::AssertionFailure;
-using testing::AssertionResult;
-using testing::AssertionSuccess;
+using ::testing::AssertionFailure;
+using ::testing::AssertionResult;
+using ::testing::AssertionSuccess;
 
 class TipCandidateListTest : public testing::Test {
  protected:
@@ -61,6 +61,8 @@ class TipCandidateListTest : public testing::Test {
 class MockCallbackResult {
  public:
   MockCallbackResult() : on_finalize_called_(false), index_(0) {}
+  MockCallbackResult(const MockCallbackResult &) = delete;
+  MockCallbackResult &operator=(const MockCallbackResult &) = delete;
 
   void Reset() {
     on_finalize_called_ = false;
@@ -84,31 +86,28 @@ class MockCallbackResult {
   bool on_finalize_called_;
   size_t index_;
   std::wstring candidate_;
-  DISALLOW_COPY_AND_ASSIGN(MockCallbackResult);
 };
 
 class MockCallback : public TipCandidateListCallback {
  public:
   // Does not take ownership of |result|.
   explicit MockCallback(MockCallbackResult *result) : result_(result) {}
+  MockCallback(const MockCallback &) = delete;
+  MockCallback &operator=(const MockCallback &) = delete;
 
  private:
   // TipCandidateListCallback overrides:
-  virtual void OnFinalize(size_t index, const std::wstring &candidate) {
+
+  void OnFinalize(size_t index, const std::wstring &candidate) override {
     result_->OnFinalize(index, candidate);
   }
 
   MockCallbackResult *result_;
-  DISALLOW_COPY_AND_ASSIGN(MockCallback);
 };
 
-std::wstring ToWStr(const CComBSTR &bstr) {
-  return std::wstring(static_cast<const wchar_t *>(bstr), bstr.Length());
-}
-
 AssertionResult ExpectCandidateString(ULONG expected_index,
                                       const std::wstring &expected_text,
-                                      CComPtr<ITfCandidateString> candidate) {
+                                      ITfCandidateString *candidate) {
   if (candidate == nullptr) {
     return AssertionFailure() << "|actual| should be non-null";
   }
@@ -126,13 +125,13 @@ AssertionResult ExpectCandidateString(ULONG expected_index,
     }
   }
   {
-    CComBSTR str;
-    hr = candidate->GetString(&str);
+    wil::unique_bstr str;
+    hr = candidate->GetString(str.put());
     if (FAILED(hr)) {
       return AssertionFailure() << "ITfCandidateString::GetString failed."
                                 << " hr = " << hr;
     }
-    const std::wstring wstr(ToWStr(str));
+    const std::wstring wstr(str.get());
     if (expected_text != wstr) {
       return AssertionFailure()
              << "expected: " << expected_text << ", actual: " << wstr;
@@ -148,27 +147,27 @@ TEST(TipCandidateListTest, EmptyCandidate) {
   MockCallbackResult result;
 
   std::vector<std::wstring> empty;
-  CComPtr<ITfCandidateList> candidate_list(
-      TipCandidateList::New(empty, new MockCallback(&result)));
-  ASSERT_NE(nullptr, candidate_list);
+  wil::com_ptr_nothrow<ITfCandidateList> candidate_list(
+      TipCandidateList::New(empty, std::make_unique<MockCallback>(&result)));
+  ASSERT_NE(candidate_list, nullptr);
 
   ULONG num = 0;
   EXPECT_HRESULT_SUCCEEDED(candidate_list->GetCandidateNum(&num));
-  EXPECT_EQ(0, num);
+  EXPECT_EQ(num, 0);
 
-  CComPtr<ITfCandidateString> str;
+  wil::com_ptr_nothrow<ITfCandidateString> str;
   EXPECT_HRESULT_FAILED(candidate_list->GetCandidate(0, &str));
-  EXPECT_EQ(nullptr, str);
+  EXPECT_EQ(str, nullptr);
 
-  CComPtr<IEnumTfCandidates> enum_candidates;
+  wil::com_ptr_nothrow<IEnumTfCandidates> enum_candidates;
   EXPECT_HRESULT_SUCCEEDED(candidate_list->EnumCandidates(&enum_candidates));
-  ASSERT_NE(nullptr, enum_candidates);
+  ASSERT_NE(enum_candidates, nullptr);
 
   ITfCandidateString *buffer[3] = {};
   ULONG num_fetched = 0;
-  EXPECT_EQ(S_FALSE,
-            enum_candidates->Next(std::size(buffer), buffer, &num_fetched));
-  EXPECT_EQ(0, num_fetched);
+  EXPECT_EQ(enum_candidates->Next(std::size(buffer), buffer, &num_fetched),
+            S_FALSE);
+  EXPECT_EQ(num_fetched, 0);
 
   EXPECT_FALSE(result.on_finalize_called());
 
@@ -184,45 +183,44 @@ TEST(TipCandidateListTest, NonEmptyCandidates) {
   for (wchar_t c = L'A'; c < L'Z'; ++c) {
     source.push_back(std::wstring(c, 1));
   }
-  CComPtr<ITfCandidateList> candidate_list(
-      TipCandidateList::New(source, new MockCallback(&result)));
-  ASSERT_NE(nullptr, candidate_list);
+  wil::com_ptr_nothrow<ITfCandidateList> candidate_list(
+      TipCandidateList::New(source, std::make_unique<MockCallback>(&result)));
+  ASSERT_NE(candidate_list, nullptr);
 
   ULONG num = 0;
   EXPECT_HRESULT_SUCCEEDED(candidate_list->GetCandidateNum(&num));
-  ASSERT_EQ(source.size(), num);
+  ASSERT_EQ(num, source.size());
 
   for (size_t i = 0; i < source.size(); ++i) {
-    CComPtr<ITfCandidateString> candidate_str;
+    wil::com_ptr_nothrow<ITfCandidateString> candidate_str;
     EXPECT_HRESULT_SUCCEEDED(candidate_list->GetCandidate(i, &candidate_str));
-    EXPECT_CANDIDATE_STR(i, source[i], candidate_str);
+    EXPECT_CANDIDATE_STR(i, source[i], candidate_str.get());
   }
 
-  CComPtr<IEnumTfCandidates> enum_candidates;
+  wil::com_ptr_nothrow<IEnumTfCandidates> enum_candidates;
   EXPECT_HRESULT_SUCCEEDED(candidate_list->EnumCandidates(&enum_candidates));
-  EXPECT_NE(nullptr, enum_candidates);
+  EXPECT_NE(enum_candidates, nullptr);
 
-  size_t offset = 0;
-  while (offset < source.size()) {
+  for (size_t offset = 0; offset < source.size();) {
     constexpr size_t kBufferSize = 10;
     ITfCandidateString *buffer[kBufferSize] = {};
-    CComPtr<ITfCandidateString> strings[kBufferSize];
+    wil::com_ptr_nothrow<ITfCandidateString> strings[kBufferSize];
     ULONG num_fetched = 0;
     HRESULT hr = enum_candidates->Next(kBufferSize, buffer, &num_fetched);
     for (size_t i = 0; i < kBufferSize; ++i) {
-      strings[i].Attach(buffer[i]);
+      strings[i].attach(buffer[i]);
     }
     const size_t rest = source.size() - offset;
     if (rest > kBufferSize) {
-      EXPECT_EQ(S_OK, hr);
-      ASSERT_EQ(kBufferSize, num_fetched);
+      EXPECT_EQ(hr, S_OK);
+      ASSERT_EQ(num_fetched, kBufferSize);
     } else {
-      EXPECT_EQ(S_FALSE, hr);
-      ASSERT_EQ(rest, num_fetched);
+      EXPECT_EQ(hr, S_FALSE);
+      ASSERT_EQ(num_fetched, rest);
     }
     for (size_t buffer_index = 0; buffer_index < num_fetched; ++buffer_index) {
       const size_t index = offset + buffer_index;
-      EXPECT_CANDIDATE_STR(index, source[index], strings[buffer_index]);
+      EXPECT_CANDIDATE_STR(index, source[index], strings[buffer_index].get());
     }
     offset += num_fetched;
   }
@@ -235,8 +233,8 @@ TEST(TipCandidateListTest, NonEmptyCandidates) {
 
   EXPECT_HRESULT_SUCCEEDED(candidate_list->SetResult(1, CAND_FINALIZED));
   EXPECT_TRUE(result.on_finalize_called());
-  EXPECT_EQ(1, result.index());
-  EXPECT_EQ(source[1], result.candidate());
+  EXPECT_EQ(result.index(), 1);
+  EXPECT_EQ(result.candidate(), source[1]);
 
   result.Reset();
 
