@@ -30,33 +30,44 @@
 #ifndef MOZC_WIN32_TIP_TIP_DLL_MODULE_H_
 #define MOZC_WIN32_TIP_TIP_DLL_MODULE_H_
 
-#include <Windows.h>
+#include <windows.h>
 
-#include "base/port.h"
+#include "absl/base/call_once.h"
+#include "base/win32/com_implements.h"
 
 namespace mozc {
 namespace win32 {
 namespace tsf {
 
+struct TipComTraits : public ComImplementsTraits {
+  static void OnObjectRelease(ULONG ref);
+};
+
 class TipDllModule {
  public:
-  // Increases and decreases the reference count to this module.
-  // This reference count is used for preventing Windows from unloading
-  // this module.
-  static LONG AddRef();
-  static LONG Release();
+  TipDllModule() = delete;
+  TipDllModule(const TipDllModule&) = delete;
+  TipDllModule& operator=(const TipDllModule&) = delete;
 
-  static bool IsUnloaded();
-  static bool CanUnload();
+  static void PrepareForShutdown();
 
-  static HMODULE module_handle();
+  static bool IsUnloaded() { return unloaded_; }
+  static void Unload() { unloaded_ = true; }
+
+  static void set_module_handle(HMODULE handle) { module_handle_ = handle; }
+  static HMODULE module_handle() { return module_handle_; }
 
   static void InitForUnitTest();
 
  private:
-  DISALLOW_IMPLICIT_CONSTRUCTORS(TipDllModule);
+  static HMODULE module_handle_;
+  static bool unloaded_;
+  static absl::once_flag uninitialize_once_;
 };
 
+template <typename... Interfaces>
+using TipComImplements = ComImplements<TipComTraits, Interfaces...>;
+
 }  // namespace tsf
 }  // namespace win32
 }  // namespace mozc
