@@ -27,49 +27,48 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
-#include <atlbase.h>
-#include <atlcom.h>
-
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "win32/tip/tip_dll_module.h"
 #include "win32/tip/tip_enum_display_attributes.h"
 
+#include <objbase.h>
+#include <wil/com.h>
+
+#include <iterator>
+
+#include "testing/gunit.h"
+#include "win32/tip/tip_dll_module.h"
+
 namespace mozc {
 namespace win32 {
 namespace tsf {
-
-using ATL::CComPtr;
+namespace {
 
 class TipEnumDisplayAttributesTest : public testing::Test {
  protected:
-  static void SetUpTestCase() { TipDllModule::InitForUnitTest(); }
+  TipEnumDisplayAttributesTest() { TipDllModule::InitForUnitTest(); }
 };
 
-TEST(TipEnumDisplayAttributesTest, BasicTest) {
+TEST_F(TipEnumDisplayAttributesTest, BasicTest) {
   TipEnumDisplayAttributes enum_display_attribute;
   ASSERT_TRUE(SUCCEEDED(enum_display_attribute.Reset()));
 
   while (true) {
-    CComPtr<ITfDisplayAttributeInfo> info;
+    wil::com_ptr_nothrow<ITfDisplayAttributeInfo> info;
     ULONG fetched = 0;
     const HRESULT result = enum_display_attribute.Next(1, &info, &fetched);
     EXPECT_TRUE(SUCCEEDED(result));
     if (result == S_OK) {
-      EXPECT_EQ(1, fetched);
-      EXPECT_NE(nullptr, info);
+      EXPECT_EQ(fetched, 1);
+      EXPECT_NE(info, nullptr);
     } else {
-      EXPECT_EQ(S_FALSE, result);
-      EXPECT_EQ(0, fetched);
-      EXPECT_EQ(nullptr, info);
+      EXPECT_EQ(result, S_FALSE);
+      EXPECT_EQ(fetched, 0);
+      EXPECT_EQ(info, nullptr);
       break;
     }
   }
 }
 
-TEST(TipEnumDisplayAttributesTest, NextTest) {
+TEST_F(TipEnumDisplayAttributesTest, NextTest) {
   TipEnumDisplayAttributes enum_display_attribute;
 
   ITfDisplayAttributeInfo *infolist[4] = {};
@@ -77,23 +76,24 @@ TEST(TipEnumDisplayAttributesTest, NextTest) {
   ULONG fetched = 0;
   const HRESULT result =
       enum_display_attribute.Next(std::size(infolist), infolist, &fetched);
-  EXPECT_EQ(S_FALSE, result);
-  EXPECT_EQ(2, fetched);
+  EXPECT_EQ(result, S_FALSE);
+  EXPECT_EQ(fetched, 2);
 
-  EXPECT_NE(nullptr, infolist[0]);
-  EXPECT_NE(nullptr, infolist[1]);
-  EXPECT_EQ(nullptr, infolist[2]);
-  EXPECT_EQ(nullptr, infolist[3]);
+  EXPECT_NE(infolist[0], nullptr);
+  EXPECT_NE(infolist[1], nullptr);
+  EXPECT_EQ(infolist[2], nullptr);
+  EXPECT_EQ(infolist[3], nullptr);
 
   // Clean up.
-  for (size_t i = 0; i < std::size(infolist); ++i) {
-    if (infolist[i] != nullptr) {
-      infolist[i]->Release();
-      infolist[i] = nullptr;
+  for (auto &i : infolist) {
+    if (i != nullptr) {
+      i->Release();
+      i = nullptr;
     }
   }
 }
 
+}  // namespace
 }  // namespace tsf
 }  // namespace win32
 }  // namespace mozc
