@@ -29,34 +29,28 @@
 
 #include "win32/tip/tip_keyevent_handler.h"
 
-#include <Windows.h>
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
-#include <atlbase.h>
-#include <atlcom.h>
 #include <msctf.h>
+#include <wil/com.h>
+#include <windows.h>
 
+#include <cstdint>
 #include <memory>
 #include <string>
 
-#include "base/logging.h"
-#include "base/util.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "base/win32/wide_char.h"
 #include "client/client_interface.h"
 #include "protocol/commands.pb.h"
-#include "win32/base/browser_info.h"
 #include "win32/base/conversion_mode_util.h"
 #include "win32/base/deleter.h"
-#include "win32/base/focus_hierarchy_observer.h"
-#include "win32/base/indicator_visibility_tracker.h"
 #include "win32/base/input_state.h"
 #include "win32/base/keyboard.h"
 #include "win32/base/keyevent_handler.h"
 #include "win32/base/surrogate_pair_observer.h"
-#include "win32/base/win32_window_util.h"
 #include "win32/tip/tip_edit_session.h"
 #include "win32/tip/tip_input_mode_manager.h"
 #include "win32/tip/tip_private_context.h"
-#include "win32/tip/tip_ref_count.h"
 #include "win32/tip/tip_status.h"
 #include "win32/tip/tip_surrounding_text.h"
 #include "win32/tip/tip_text_service.h"
@@ -65,19 +59,16 @@
 namespace mozc {
 namespace win32 {
 namespace tsf {
-
-using ATL::CComPtr;
-using mozc::commands::Context;
-using mozc::commands::Output;
-typedef mozc::commands::CompositionMode CompositionMode;
-typedef mozc::commands::SessionCommand SessionCommand;
-
 namespace {
 
+using ::mozc::commands::CompositionMode;
+using ::mozc::commands::Context;
+using ::mozc::commands::SessionCommand;
+
 // Defined in the following white paper.
 // http://msdn.microsoft.com/en-us/library/windows/apps/hh967425.aspx
-const UINT kTouchKeyboardNextPage = 0xf003;
-const UINT kTouchKeyboardPreviousPage = 0xf004;
+constexpr UINT kTouchKeyboardNextPage = 0xf003;
+constexpr UINT kTouchKeyboardPreviousPage = 0xf004;
 
 // Unlike IMM32 Mozc which is marked as IME_PROP_ACCEPT_WIDE_VKEY,
 // TSF Mozc cannot always receive a VK_PACKET keyevent whose high word consists
@@ -95,7 +86,7 @@ VirtualKey GetVK(WPARAM wparam, const KeyboardStatus &keyboad_status) {
                   std::size(buffer), 0) != 1) {
     return VirtualKey::FromVirtualKey(wparam);
   }
-  const uint32 ucs2 = buffer[0];
+  const uint32_t ucs2 = buffer[0];
   if (ucs2 == L' ') {
     return VirtualKey::FromVirtualKey(VK_SPACE);
   }
@@ -114,7 +105,8 @@ VirtualKey GetVK(WPARAM wparam, const KeyboardStatus &keyboad_status) {
 }
 
 bool GetOpenAndMode(TipTextService *text_service, ITfContext *context,
-                    bool *open, uint32 *logical_mode, uint32 *visible_mode) {
+                    bool *open, uint32_t *logical_mode,
+                    uint32_t *visible_mode) {
   DCHECK(text_service);
   DCHECK(context);
   DCHECK(open);
@@ -149,7 +141,7 @@ void FillMozcContextCommon(TipTextService *text_service, ITfContext *context,
   }
   mozc_context->set_revision(
       text_service->GetThreadContext()->GetFocusRevision());
-  CComPtr<ITfContextView> context_view;
+  wil::com_ptr_nothrow<ITfContextView> context_view;
   if (FAILED(context->GetActiveView(&context_view))) {
     return;
   }
@@ -160,14 +152,6 @@ void FillMozcContextCommon(TipTextService *text_service, ITfContext *context,
   if (FAILED(context_view->GetWnd(&attached_window))) {
     return;
   }
-  const auto *focus_hierarchy_observer =
-      text_service->GetThreadContext()->GetFocusHierarchyObserver();
-  if (focus_hierarchy_observer->IsAbailable()) {
-    if (BrowserInfo::IsOnChromeOmnibox(*focus_hierarchy_observer)) {
-      mozc_context->set_suppress_suggestion(true);
-      mozc_context->add_experimental_features("chrome_omnibox");
-    }
-  }
 }
 
 HRESULT OnTestKey(TipTextService *text_service, ITfContext *context,
@@ -187,8 +171,8 @@ HRESULT OnTestKey(TipTextService *text_service, ITfContext *context,
   }
 
   bool open = false;
-  uint32 logical_mode = 0;
-  uint32 visible_mode = 0;
+  uint32_t logical_mode = 0;
+  uint32_t visible_mode = 0;
   if (!GetOpenAndMode(text_service, context, &open, &logical_mode,
                       &visible_mode)) {
     *eaten = FALSE;
@@ -202,24 +186,24 @@ HRESULT OnTestKey(TipTextService *text_service, ITfContext *context,
   if (open) {
     // Check if this key event is handled by VKBackBasedDeleter to support
     // *deletion_range* rule.
-    const VKBackBasedDeleter::ClientAction vk_back_action =
+    const ClientAction vk_back_action =
         private_context->GetDeleter()->OnKeyEvent(
             vk.virtual_key(), key_info.IsKeyDownInImeProcessKey(), true);
     switch (vk_back_action) {
-      case VKBackBasedDeleter::DO_DEFAULT_ACTION:
+      case ClientAction::DO_DEFAULT_ACTION:
         // do nothing.
         break;
-      case VKBackBasedDeleter::CALL_END_DELETION_THEN_DO_DEFAULT_ACTION:
+      case ClientAction::CALL_END_DELETION_THEN_DO_DEFAULT_ACTION:
         private_context->GetDeleter()->EndDeletion();
         break;
-      case VKBackBasedDeleter::SEND_KEY_TO_APPLICATION:
+      case ClientAction::SEND_KEY_TO_APPLICATION:
         *eaten = FALSE;  // Do not consume this key.
         return S_OK;
-      case VKBackBasedDeleter::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
+      case ClientAction::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
         *eaten = TRUE;  // Consume this key but do not send this key to server.
         return S_OK;
-      case VKBackBasedDeleter::CALL_END_DELETION_BUT_NEVER_SEND_TO_SERVER:
-      case VKBackBasedDeleter::APPLY_PENDING_STATUS:
+      case ClientAction::CALL_END_DELETION_BUT_NEVER_SEND_TO_SERVER:
+      case ClientAction::APPLY_PENDING_STATUS:
       default:
         DLOG(FATAL) << "this action is not applicable to OnTestKey.";
         *eaten = FALSE;
@@ -233,7 +217,7 @@ HRESULT OnTestKey(TipTextService *text_service, ITfContext *context,
       case SurrogatePairObserver::DO_DEFAULT_ACTION:
         break;
       case SurrogatePairObserver::DO_DEFAULT_ACTION_WITH_RETURNED_UCS4:
-        vk = VirtualKey::FromUnicode(surrogate_action.ucs4);
+        vk = VirtualKey::FromUnicode(surrogate_action.codepoint);
         break;
       case SurrogatePairObserver::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
         *eaten = TRUE;
@@ -307,14 +291,10 @@ void FillMozcContextForOnKey(TipTextService *text_service, ITfContext *context,
     return;
   }
   if (info.has_preceding_text) {
-    std::string utf8_preceding_text;
-    Util::WideToUtf8(info.preceding_text, &utf8_preceding_text);
-    mozc_context->set_preceding_text(utf8_preceding_text);
+    mozc_context->set_preceding_text(WideToUtf8(info.preceding_text));
   }
   if (info.has_following_text) {
-    std::string utf8_following_text;
-    Util::WideToUtf8(info.following_text, &utf8_following_text);
-    mozc_context->set_following_text(utf8_following_text);
+    mozc_context->set_following_text(WideToUtf8(info.following_text));
   }
 }
 
@@ -335,8 +315,8 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
   }
 
   bool open = false;
-  uint32 logical_mode = 0;
-  uint32 visible_mode = 0;
+  uint32_t logical_mode = 0;
+  uint32_t visible_mode = 0;
   if (!GetOpenAndMode(text_service, context, &open, &logical_mode,
                       &visible_mode)) {
     *eaten = FALSE;
@@ -347,9 +327,8 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
   const KeyboardStatus keyboard_status(key_state);
   VirtualKey vk = GetVK(wparam, keyboard_status);
 
-  const VKBackBasedDeleter::ClientAction vk_back_action =
-      private_context->GetDeleter()->OnKeyEvent(vk.virtual_key(), is_key_down,
-                                                false);
+  const ClientAction vk_back_action = private_context->GetDeleter()->OnKeyEvent(
+      vk.virtual_key(), is_key_down, false);
 
   // Check if this key event is handled by VKBackBasedDeleter to support
   // *deletion_range* rule.
@@ -357,23 +336,23 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
   bool ignore_this_keyevent = false;
   if (open) {
     switch (vk_back_action) {
-      case VKBackBasedDeleter::DO_DEFAULT_ACTION:
+      case ClientAction::DO_DEFAULT_ACTION:
         // do nothing.
         break;
-      case VKBackBasedDeleter::CALL_END_DELETION_THEN_DO_DEFAULT_ACTION:
+      case ClientAction::CALL_END_DELETION_THEN_DO_DEFAULT_ACTION:
         private_context->GetDeleter()->EndDeletion();
         break;
-      case VKBackBasedDeleter::APPLY_PENDING_STATUS:
+      case ClientAction::APPLY_PENDING_STATUS:
         use_pending_output = true;
         break;
-      case VKBackBasedDeleter::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
+      case ClientAction::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
         ignore_this_keyevent = true;
         break;
-      case VKBackBasedDeleter::CALL_END_DELETION_BUT_NEVER_SEND_TO_SERVER:
+      case ClientAction::CALL_END_DELETION_BUT_NEVER_SEND_TO_SERVER:
         ignore_this_keyevent = true;
         private_context->GetDeleter()->EndDeletion();
         break;
-      case VKBackBasedDeleter::SEND_KEY_TO_APPLICATION:
+      case ClientAction::SEND_KEY_TO_APPLICATION:
       default:
         DLOG(FATAL) << "this action is not applicable to OnKey.";
         break;
@@ -390,7 +369,7 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
       case SurrogatePairObserver::DO_DEFAULT_ACTION:
         break;
       case SurrogatePairObserver::DO_DEFAULT_ACTION_WITH_RETURNED_UCS4:
-        vk = VirtualKey::FromUnicode(surrogate_action.ucs4);
+        vk = VirtualKey::FromUnicode(surrogate_action.codepoint);
         break;
       case SurrogatePairObserver::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
         ignore_this_keyevent = true;
@@ -411,7 +390,7 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
   if (use_pending_output) {
     // In this case, we have a pending output. So no need to call
     // KeyEventHandler::ImeToAsciiEx.
-    temporal_output.CopyFrom(private_context->GetDeleter()->pending_output());
+    temporal_output = private_context->GetDeleter()->pending_output();
   } else if (open && is_key_down &&
              (vk.wide_char() == kTouchKeyboardPreviousPage)) {
     // Handle PrevPage button on the on-screen keyboard.
@@ -435,7 +414,7 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
   } else {
     InputBehavior behavior = private_context->input_behavior();
 
-    // Update On/Off state an conversion mode.
+    // Update On/Off state a conversion mode.
     InputState ime_state;
     ime_state.logical_conversion_mode = logical_mode;
     ime_state.visible_conversion_mode = visible_mode;
@@ -469,7 +448,7 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
     const TipInputModeManager::Action action =
         text_service->GetThreadContext()->GetInputModeManager()->OnKey(
             vk, is_key_down, result.should_be_eaten);
-    if (action == IndicatorVisibilityTracker::kUpdateUI) {
+    if (action == TipInputModeManager::Action::kUpdateUI) {
       text_service->PostUIUpdateMessage();
     }
 
